clear all
clear matrix
set more off
set maxvar 30000

/* TODO IMPORTANT:
  - "Require": [egenmore, outreg2, tabout, estout] packages
  - Rewrite stuff done with "at least on price change" to use adjusted no. of
    price changes
*/

/* - Must run Russia analysis first */
*****************************************************************************
* SET PATHS

cd "/Users/jmorelli/Dropbox/new_sync/replication_files/empirical" // switch here to user's path

global stdata   "data/stata_data"
global output   "data/stata_output"
global regout   "data/stata_output/baseline/regressions"

capture log close
log using "$regout/log_Regressions.txt", text replace

** Run commands
local do_main_regs                     = 1
  local do_baseline                    = 1
  local do_onlysov                     = 1
  local do_nocontrols                  = 1
  local do_samemat                     = 1
  local do_ivshareaig                  = 1
  local do_ivshareaig_1stage           = 1
local do_robust_regs                   = 1
  local do_contemp                     = 1
  local do_fc                          = 1
  local do_lc                          = 1
  local do_onlybanks                   = 1
  local do_banksamd                    = 1
  local do_am                          = 1
  local do_other                       = 1
  local do_tightwin                    = 1
  local do_widewin                     = 1
  local do_post10                      = 1
  local do_post30                      = 1
  local do_post45                      = 1
  local do_onlynonmmk                  = 1
  local do_gfishare                    = 1
  local do_leverage                    = 1
  local do_liquidity1                  = 1
local do_add_regs                      = 1
  local do_yieldres_baseline           = 1
  local do_yieldres_onlysov            = 1
local graphs_main_regs                 = 1
local graphs_add_regs                  = 1
local table_main_regs                  = 1
local table_rob_regs                   = 1
local table_yieldres                   = 1
local table_interactive_regs           = 1

** Relevant locals
local date_lehman           = "15sep2008"
local hmax_pre              = 60
local hpre_chosen           = 10
local hmax_post             = 120
local hmax_post_short       = 60
local hpost_fixed           = 3
local conf_int              = 1.645
local sector_list           = "Government Financial Utilities Communications Industrial Energy"
local list_covariates       = "issuance_resid bamount_resid resid_maturity BA_spread"
local country_sample        = "AR BG BI BR DR EC EG GG GH GO GR GU HR ID IQ KZ LB LL LT LV MA MJ MX NG RR RU SV TH TN VE VN"
local precision             = 5.960e-08

**********************************
**********************************
* Prepare Output for Regressions
**********************************
**********************************

if `do_main_regs' == 1|`do_robust_regs' == 1|`do_add_regs' == 1 {

  gen yvar             = . /* 1 = deltayields, 2 = deltaprice, 3 = delta daily (or two days) yields， 4= delta daily nw */
  gen xvar             = . /* 0 = weighted nw, 1 = only banks nw,  2 = only non-banks nw, 3=separate banks and non-banks nw, 4 = only mmk nw,  5 = only non-mmk nw, 6=separate mmk and non-mmk nw, 7=banks ex. amd nw, 8=banks+insurance nw, 9=bank+insurance nw ex. amd nw, 10= asset managers nw, 11=others nw, 12=dps_m10_p9, 13= dps_m10_p29, 14= dps_m10_p45 */
  gen days_pre         = . /* when negative means days post for delta stock price variable */
  gen days_post        = .
  gen jorda_proj       = . /* 1 = jorda, 2 = contemporaneous elasticity, 3 = iv with jorda (share aig instrument), 4= iv 1st stage */
  gen control_vars     = . /* 0 = 1-sumofgbholdings, 1 = 0 + initialyield BA_spread resid_maturity bamount_resid cac1 payment_rank1, 2 = 1 - initialyield, 3 = 1 - cac1 - payment_rank1, 4=1+leverage1 leveage1*nw, 5=1+liquidity1 liquidity1*nw, 6=1+liquidity2 liquidity2*nw, 7= 1+gfi_share gfi_share * nw , 8 = 1+ net leverage+ net leverage*nw, 9= leverage1 only, 10= liquidity1 only*/
  gen fixed_effects    = . /* 1 = Country-Sector & Currency, 2 = Country & Sector & Currency, , 3 = Country & Sector & Currency & Maturity, 4 = no FE, 5 = Country-Sector,  */
  gen subsample        = . /* 0 = all sample, 1 = only sov, 2 = extended sample (including bonds with no dps data), 3 = by country regs, 4 = country_sample, 5= bonds with high yield, 6 = by country regs only sov, 7= bonds with high yield only sov, 8 = same mat, 9 = by banks, 10 = by banks only sov, 11 = by type of GFI, 12 = by type of GFI only sov, 13=banks and asset managers only, 14=only dollar bond, 15=local currency only, 16=foreign currency onlys*/
  gen nobs             = .
  gen country          = ""
  gen lender           = ""
  gen stock_coef       = .
  gen stock_lb         = .
  gen stock_ub         = .
  gen inter_coef       = .
  gen inter_lb         = .
  gen inter_ub         = .
  gen levliq_coef      = .
  gen levliq_lb        = .
  gen levliq_ub        = .
  gen stock_coef_add   = .
  gen stock_lb_add     = .
  gen stock_ub_add     = .
  gen sd_yvar          = .
  gen av_yvar          = .
  gen sd_res           = .
  gen r2               = .
  gen sd_dyvar         = .
  gen av_dyvar         = .
  gen sd_dres          = .
  gen r2_d             = .
  save "$regout/regression_output.dta", replace

  use "$stdata/beneficiary/regression_variables_extension.dta", clear
  append using "$stdata/beneficiary/regression_variables.dta"
  append using "$stdata/beneficiary/regression_variables_ext2010.dta" 
  
  drop if bond_id == "EC119001Corp"
  drop if bond_id == "EC396225Corp"
  drop if bond_id == "EC396227Corp" 
  drop if bond_id == "EC396229Corp"
  
  keep if CRNCY ~=""

  drop if sector=="#N/A Invalid Security"
  egen countrysector = group(country sector)
  encode country, gen(country_num)
  encode sector, gen(sector_num)
  encode CRNCY, gen(currency_num)
  levelsof country, local(country_list)

  foreach c of local country_list {
    local si =0
    levelsof sector if country=="`c'", local(sector_list)
    foreach s of local sector_list {
        local si = `si' + 1
        gen dcs_`c'_`si' = (country == "`c'" & sector=="`s'")
    }
  }
  levelsof CRNCY, local(cur_list)
  foreach cur of local cur_list {
        gen dcur_`cur' = (CRNCY == "`cur'")
  }

  * Maturity fixed effects
  local matbunch     = 182 /* number of days that will be assigned same mat fixed effect*/
  gen mat_`matbunch' = floor(resid_maturity/`matbunch')
  egen countrymat    = group(country mat_`matbunch')
  egen sectormat     = group(sector mat_`matbunch')


  *Average bond yield pre-Lehman
  gen aux_m62 =0
  local j = 0
  forval i=-60(2)0 {
    local j = `j' + 1
    local absi   = abs(`i')
    local absim2 = abs(`i'-2)
    gen aux_m`absi' =  (aux_m`absim2'*(`j'-1) + bondY_m`absi')/`j'
  }
  rename aux_m0 bondY_prelehman
  sum bondY_prelehman, d
  local bondy_lowerbound = `r(p50)'
  di `bondy_lowerbound'
  drop aux*
  gen highy_ind = 0
  replace highy_ind=1 if bondY_prelehman>.07 & bondY_prelehman!=./*`bondy_lowerbound'*/
  tab highy_ind

  gen outlier_yield = 0
  replace outlier_yield = 1 if bondY_prelehman<.03 | bondY_prelehman>.5

  * Pre-Lehman fixed effects
  local yieldbunch = .01
  gen initialyield_bunch = floor(bondY_prelehman/`yieldbunch')
  replace initialyield_bunch = 0 if initialyield_bunch<0
  tab initialyield_bunch

  * Clean issuance and outstanding from FE
  qui gen ln_bnissnce = ln(bond_issuance_mm)
  qui gen ln_bnamount = ln(bond_amount_mm)
  qui reg ln_bnissnce i.country_num
  predict double issuance_resid, residuals
  qui reg ln_bnamount i.country_num
  predict double bamount_resid, residuals

  * Generate dummy for sample_country
  gen country_sample_ind = .
  local n : word count `country_sample'
  forvalues i = 1(1)`n'     {
       local str = word("`country_sample'", `i')
       replace country_sample_ind = 1 if country=="`str'"
  }
   
   * Categorical cac and payment_rank
   encode cac, gen (cac1)
   encode payment_rank, gen (payment_rank1)

  * Local currency dummy (used to run regressions separately for LC/FC bonds)
  merge m:1 country using "$stdata/country_currency.dta"
  gen lc=1 if CRNCY==local_currency
  replace lc=0 if lc==.
  
  * Standardize liquidity and leverage measures
  sum leverage1_bs
  gen leverage1_zs = (leverage1_bs -`r(mean)')/`r(sd)'
  sum liquidity1_bs
  gen liquidity1_zs = (liquidity1_bs -`r(mean)')/`r(sd)'
  sum liquidity2_bs
  gen liquidity2_zs = (liquidity2_bs -`r(mean)')/`r(sd)'
  gen leverage2_bs = leverage1_bs - 1 /* net leverage */
  sum leverage2_bs
  gen leverage2_zs = (leverage2_bs -`r(mean)')/`r(sd)'
  
  ** Generate dependent variable
  foreach hpost of numlist -`hmax_pre'(2)`hmax_post' {

    if `hpost'<=0 {
      local abshpost   = abs(`hpost')
      gen deltay_bond_m`hpre_chosen'_m`abshpost' = deltay_bond_m`hpre_chosen'_p1 - deltay_bond_m`abshpost'_p1
      local depvar = "deltay_bond_m`hpre_chosen'_m`abshpost'"
    }
    else {
      local hpost = `hpost'-1
      local depvar = "deltay_bond_m`hpre_chosen'_p`hpost'"
    }
  }	
}

**********************************
**********************************
* Run Main Regressions
**********************************
**********************************

if `do_main_regs' == 1 {
 
  * Figure 4/Table 2/Table B10: Yield Regression - Jorda Projection - Baseline
  if `do_baseline' == 1 {
  
     foreach hpost of numlist -`hmax_pre'(2)`hmax_post' {
      
	  if `hpost'<=0 {
        local abshpost   = abs(`hpost')
        local depvar = "deltay_bond_m`hpre_chosen'_m`abshpost'"
      }
      else {
        local hpost = `hpost'-1
        local depvar = "deltay_bond_m`hpre_chosen'_p`hpost'"
      }
	  
      reg `depvar' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1  if rating_num>7 & outlier_yield==0, robust 
	  preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }


  * Table 2: Yield Regression - Jorda Projection - Only sov
  if `do_onlysov' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0 & sector=="Government", robust 
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 1 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }

  * Table 2: Yield Regression - Jorda Projection - No controls
  if `do_nocontrols' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros  i.countrysector i.currency_num if rating_num>7 & outlier_yield==0, robust  
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 0 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }

  * Table 2: Yield Regression - Jorda Projection - Only bonds with same mat
  if `do_samemat' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0 & resid_maturity<1203, robust    
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 8 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }
  
  * Table 2: Yield Regression - IV regression Jorda Projection - Share of bonds held by AIG as instrument
  if `do_ivshareaig   ' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      ivregress 2sls deltay_bond_m`hpre_chosen'_p`hpost' (dps_m`hpre_chosen'_p`hpost_fixed'_bs=share_aig) dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros  i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if (rating_num>7) & outlier_yield==0, robust
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 3 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }

}

******************************wi****
**********************************
* Run Robustness Regressions
**********************************
**********************************

if `do_robust_regs' == 1 {

  * Table 2: Yield Regression - Contemporaneous regressions with all controls
  if `do_contemp' == 1 {
    foreach hpost of numlist 3(2)`hmax_post_short' {
	  local hpost_lag = `hpost'-2
	  sum dps_m`hpre_chosen'_p`hpost'_bs
      if r(N) == 0 {
	    replace dps_m`hpre_chosen'_p`hpost'_bs = dps_m`hpre_chosen'_p`hpost_lag'_bs
	  }
	}
	foreach hpost of numlist 1(2)`hmax_post_short' {
	    reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p`hpost'_bs dps_m`hpre_chosen'_p`hpost'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 /*bamount_resid*/ if rating_num>7 & outlier_yield==0 /*& & resid_maturity>550 resid_maturity>0 */, robust /*, vce(cluster country_num) */
	    preserve
	    use "$regout/regression_output.dta", clear
        local nplus = _N +1
        set obs `nplus'
        replace yvar             = 1 if _n==_N
        replace xvar             = 0 if _n==_N
        replace days_pre          = `hpre_chosen' if _n==_N
        replace days_post        = `hpost' if _n==_N
        replace jorda_proj       = 2 if _n==_N
        replace control_vars     = 1 if _n==_N
        replace fixed_effects    = 1 if _n==_N
        replace subsample        = 0 if _n==_N
        replace nobs             = `e(N)' if _n==_N
        replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost'_bs] if _n==_N
        replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost'_bs] if _n==_N
        replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost'_bs] if _n==_N
        save "$regout/regression_output.dta", replace
        restore
    }
  }
  
   * Table 2: Yield Regression - Jorda Projection - Foreign currency bonds only 
   if `do_fc' == 1{
    foreach hpost of numlist 1 (2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0 & lc==0, robust 
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 16 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }

  * Table 2: Yield Regression - Jorda Projection - Local currency bonds only
  if `do_lc' == 1{
    foreach hpost of numlist 1 (2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0 & lc==1, robust 
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 15 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }
  
  * Table 2: Yield Regression - Jorda Projection - Only banks networth
  if `do_onlybanks' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnk dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros_bnk i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0, robust  
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 1 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnk] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnk]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnk] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnk]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnk] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }
  
  * Table 2: Yield Regression - Jorda Projection - Banks ex. asset management divisons net worth
  if `do_banksamd' == 1 {
     foreach hpost of numlist -`hmax_pre'(2)`hmax_post' {

      if `hpost'<=0 {
        local abshpost   = abs(`hpost')
		drop deltay_bond_m`hpre_chosen'_m`abshpost'
        gen deltay_bond_m`hpre_chosen'_m`abshpost' = deltay_bond_m`hpre_chosen'_p1 - deltay_bond_m`abshpost'_p1
        local depvar = "deltay_bond_m`hpre_chosen'_m`abshpost'"

      }
      else {
        local hpost = `hpost'-1
        local depvar = "deltay_bond_m`hpre_chosen'_p`hpost'"
      }
      
	  reg `depvar' dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnkamd dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros_bnkamd i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0, robust  
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 7 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnkamd] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnkamd]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnkamd] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnkamd]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_bnkamd] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }
  
  * Table 2: Yield Regression - Jorda Projection - Asset managers networth
  if `do_am' == 1 {
     foreach hpost of numlist -`hmax_pre'(2)`hmax_post' {

      if `hpost'<=0 {
        local abshpost   = abs(`hpost')
		drop deltay_bond_m`hpre_chosen'_m`abshpost'
        gen deltay_bond_m`hpre_chosen'_m`abshpost' = deltay_bond_m`hpre_chosen'_p1 - deltay_bond_m`abshpost'_p1
        local depvar = "deltay_bond_m`hpre_chosen'_m`abshpost'"

      }
      else {
        local hpost = `hpost'-1
        local depvar = "deltay_bond_m`hpre_chosen'_p`hpost'"
      }
      
	  reg `depvar' dps_m`hpre_chosen'_p`hpost_fixed'_bs_am dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros_am i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0, robust  
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 10 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_am] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_am]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_am] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_am]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_am] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }

  * Table 2: Yield Regression - Jorda Projection - Others networth
  if `do_other' == 1 {
     foreach hpost of numlist -`hmax_pre'(2)`hmax_post' {

      if `hpost'<=0 {
        local abshpost   = abs(`hpost')
		drop deltay_bond_m`hpre_chosen'_m`abshpost'
        gen deltay_bond_m`hpre_chosen'_m`abshpost' = deltay_bond_m`hpre_chosen'_p1 - deltay_bond_m`abshpost'_p1
        local depvar = "deltay_bond_m`hpre_chosen'_m`abshpost'"

      }
      else {
        local hpost = `hpost'-1
        local depvar = "deltay_bond_m`hpre_chosen'_p`hpost'"
      }
      
	  reg `depvar' dps_m`hpre_chosen'_p`hpost_fixed'_bs_examd dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros_examd i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0, robust 
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 11 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_examd] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_examd]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_examd] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_examd]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_examd] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }
  
  * Table B10: Yield Regression - Alternative Window tighter window
  if `do_tightwin' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m4_p`hpost' dps_m4_p1_bs dps_m4_p1_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0, robust
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = 4 if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m4_p1_bs] if _n==_N
      replace stock_lb         = _b[dps_m4_p1_bs]-`conf_int'*_se[dps_m4_p1_bs] if _n==_N
      replace stock_ub         = _b[dps_m4_p1_bs]+`conf_int'*_se[dps_m4_p1_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }

  * Table B10: Yield Regression - Alternative wider Window
  if `do_widewin' == 1 {

    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m20_p`hpost' dps_m20_p9_bs dps_m20_p9_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1  if rating_num>7 & outlier_yield==0, robust
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = -9 if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m20_p9_bs] if _n==_N
      replace stock_lb         = _b[dps_m20_p9_bs]-`conf_int'*_se[dps_m20_p9_bs] if _n==_N
      replace stock_ub         = _b[dps_m20_p9_bs]+`conf_int'*_se[dps_m20_p9_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore

    }
  }
  
  * Table B10: Yield Regression - Alternative Window 10 days post Lehman
  if `do_post10' == 1 {
     foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p9_bs dps_m`hpre_chosen'_p9_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1  if rating_num>7 & outlier_yield==0, robust
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 12 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m10_p9_bs] if _n==_N
      replace stock_lb         = _b[dps_m10_p9_bs]-`conf_int'*_se[dps_m10_p9_bs] if _n==_N
      replace stock_ub         = _b[dps_m10_p9_bs]+`conf_int'*_se[dps_m10_p9_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
	 }
  }

  * Table B10: Yield Regression - Alternative Window 30 days post Lehman
  if `do_post30' == 1 {
     foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p29_bs dps_m`hpre_chosen'_p29_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1  if rating_num>7 & outlier_yield==0, robust
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 13 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m10_p29_bs] if _n==_N
      replace stock_lb         = _b[dps_m10_p29_bs]-`conf_int'*_se[dps_m10_p29_bs] if _n==_N
      replace stock_ub         = _b[dps_m10_p29_bs]+`conf_int'*_se[dps_m10_p29_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
	 }
  }
  
  * Table B10: Yield Regression - Alternative Window 45 days post Lehman
  if `do_post45' == 1 {
     foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p45_bs dps_m`hpre_chosen'_p45_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1  if rating_num>7 & outlier_yield==0, robust
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 14 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m10_p45_bs] if _n==_N
      replace stock_lb         = _b[dps_m10_p45_bs]-`conf_int'*_se[dps_m10_p45_bs] if _n==_N
      replace stock_ub         = _b[dps_m10_p45_bs]+`conf_int'*_se[dps_m10_p45_bs] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
	 }
  }

  * Table B10: Yield Regression - Jorda Projection - Only non-mmk networth
  if `do_onlynonmmk' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' dps_m`hpre_chosen'_p`hpost_fixed'_bs_nonmmk dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros_nonmmk i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1  if rating_num>7 & outlier_yield==0, robust
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 5 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_nonmmk] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_nonmmk]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_nonmmk] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs_nonmmk]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs_nonmmk] if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }
  
   * Table B11: Yield Regression - Jorda Projection - share held by GFIs 
  if `do_gfishare' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros##c.dps_m`hpre_chosen'_p`hpost_fixed'_bs i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0 , robust  
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 7 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace inter_coef       = -_b[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace inter_lb         = -_b[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace inter_ub         = -_b[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
	  replace levliq_coef      = -_b[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros] if _n==_N
      replace levliq_lb        = -_b[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros]-`conf_int'*_se[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros] if _n==_N
      replace levliq_ub        = -_b[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros]+`conf_int'*_se[c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros] if _n==_N
	  save "$regout/regression_output.dta", replace
      restore
    }
  }
  
  * Table B11: Yield Regression - Jorda Projection - Leverage Interaction 
  if `do_leverage' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' c.leverage1_zs##c.dps_m`hpre_chosen'_p`hpost_fixed'_bs c.leverage1_zs##c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0 , robust  
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 4 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace inter_coef       = _b[c.leverage1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace inter_lb         = _b[c.leverage1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[c.leverage1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace inter_ub         = _b[c.leverage1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[c.leverage1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace levliq_coef      = _b[leverage1_zs] if _n==_N
      replace levliq_lb        = _b[leverage1_zs]-`conf_int'*_se[leverage1_zs] if _n==_N
      replace levliq_ub        = _b[leverage1_zs]+`conf_int'*_se[leverage1_zs] if _n==_N
	  save "$regout/regression_output.dta", replace
      restore
    }
  }
  
  * Table B11: Yield Regression - Jorda Projection - Liquidity Interaction
  if `do_liquidity1' == 1 {
    foreach hpost of numlist 1(2)`hmax_post_short' {
      reg deltay_bond_m`hpre_chosen'_p`hpost' c.liquidity1_zs##c.dps_m`hpre_chosen'_p`hpost_fixed'_bs c.liquidity1_zs##c.dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0, robust 
      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 5 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace inter_coef       = _b[c.liquidity1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace inter_lb         = _b[c.liquidity1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[c.liquidity1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace inter_ub         = _b[c.liquidity1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[c.liquidity1_zs#c.dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace levliq_coef      = _b[liquidity1_zs] if _n==_N
      replace levliq_lb        = _b[liquidity1_zs]-`conf_int'*_se[liquidity1_zs] if _n==_N
      replace levliq_ub        = _b[liquidity1_zs]+`conf_int'*_se[liquidity1_zs] if _n==_N      
	  save "$regout/regression_output.dta", replace
      restore
    }
  }
  
}


**********************************
**********************************
* Run Additional Regressions
**********************************
**********************************

if `do_add_regs' == 1 {

  * Yield Regression on covariates
  if `do_yieldres_baseline' == 1 {

    reg deltay_bond_m`hpre_chosen'_p`hpost_fixed' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0, robust
    capture confirm variable sample_dummy
    if !_rc {
      }
    else {
        gen sample_dummy = 0
        replace sample_dummy = 1 if e(sample)
    }

    forval i=-58(2)`hmax_post' {
      if `i'<=0 {
        local absi   = abs(`i')
        local absim1 = abs(`i'-2)
        local depvar = "bondY_m`absi'"
        local depvarlag = "bondY_m`absim1'"
        gen dbY_m`absi' = `depvar' - `depvarlag'
        local delta_depvar = "dbY_m`absi'"
        local initial_yield = "bondY_m58"

      }
      else {
        local i = `i'-1
        local im1 = `i'-2
        local depvar = "bondY_p`i'"
        local depvarlag = "bondY_p`im1'"
        if `i'==1 {
          local depvarlag = "bondY_m0"
        }
        gen dbY_p`i' = `depvar' - `depvarlag'
        local delta_depvar = "dbY_p`i'"
        local initial_yield = "bondY_m0"

      }

      * Yields on all Covariates
      sum `depvar', d
      local std_yield = r(sd)
      local av_yield = r(mean)
      reg `depvar' i.countrysector i.currency_num BA_spread resid_maturity bamount_resid i.cac1 i.payment_rank1  if sample_dummy==1
      local r2_level = `e(r2)'
      predict `depvar'_res, r
      sum `depvar'_res if sample_dummy==1, d
      local std_yieldres = r(sd)

      sum `delta_depvar', d
      local std_yield_d = r(sd)
      local av_dyield = r(mean)
      reg `delta_depvar' i.countrysector i.currency_num BA_spread resid_maturity bamount_resid i.cac1 i.payment_rank1 if sample_dummy==1
      local r2_delta = `e(r2)'
      predict `delta_depvar'_res_d, r
      sum `delta_depvar'_res_d if sample_dummy==1, d
      local std_yieldres_d = r(sd)

      drop `delta_depvar'_res `depvar'_res

      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 3 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_post        = `i' if _n==_N
      replace jorda_proj       = 2 if _n==_N
      replace control_vars     = 2 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace sd_yvar          = `std_yield' if _n==_N
      replace av_yvar          = `av_yield'  if _n==_N
      replace r2               = `r2_level' if _n==_N
      replace sd_res           = `std_yieldres' if _n==_N
      replace sd_dyvar         = `std_yield_d' if _n==_N
      replace av_dyvar         = `av_dyield' if _n==_N
      replace r2_d             = `r2_delta' if _n==_N
      replace sd_dres          = `std_yieldres_d' if _n==_N
      save "$regout/regression_output.dta", replace
      restore

      * Yields on country-sector FE
      sum `depvar', d
      local std_yield = r(sd)
      local av_yield = r(mean)
      reg `depvar' i.countrysector  if sample_dummy==1
      local r2_level = `e(r2)'
      predict `depvar'_res, r
      sum `depvar'_res if sample_dummy==1, d
      local std_yieldres = r(sd)

      sum `delta_depvar', d
      local std_yield_d = r(sd)
      local av_dyield = r(mean)
      reg `delta_depvar' i.countrysector if sample_dummy==1
      local r2_delta = `e(r2)'
      predict `delta_depvar'_res_d, r
      sum `delta_depvar'_res_d if sample_dummy==1, d
      local std_yieldres_d = r(sd)

      drop `delta_depvar'_res `depvar'_res

      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 3 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_post        = `i' if _n==_N
      replace jorda_proj       = 2 if _n==_N
      replace control_vars     = 0 if _n==_N
      replace fixed_effects    = 5 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace sd_yvar          = `std_yield' if _n==_N
      replace av_yvar          = `av_yield'  if _n==_N
      replace r2               = `r2_level' if _n==_N
      replace sd_res           = `std_yieldres' if _n==_N
      replace sd_dyvar         = `std_yield_d' if _n==_N
      replace av_dyvar         = `av_dyield' if _n==_N
      replace r2_d             = `r2_delta' if _n==_N
      replace sd_dres          = `std_yieldres_d' if _n==_N
      save "$regout/regression_output.dta", replace
      restore

      * Yields on country-sector + currency FE
      sum `depvar', d
      local std_yield = r(sd)
      local av_yield = r(mean)
      reg `depvar' i.countrysector i.currency_num if sample_dummy==1
      local r2_level = `e(r2)'
      predict `depvar'_res, r
      sum `depvar'_res if sample_dummy==1, d
      local std_yieldres = r(sd)

      sum `delta_depvar', d
      local std_yield_d = r(sd)
      local av_dyield = r(mean)
      reg `delta_depvar' i.countrysector i.currency_num if sample_dummy==1
      local r2_delta = `e(r2)'
      predict `delta_depvar'_res_d, r
      sum `delta_depvar'_res_d if sample_dummy==1, d
      local std_yieldres_d = r(sd)

      drop `delta_depvar'_res `depvar'_res

      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 3 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_post        = `i' if _n==_N
      replace jorda_proj       = 2 if _n==_N
      replace control_vars     = 0 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace sd_yvar          = `std_yield' if _n==_N
      replace av_yvar          = `av_yield'  if _n==_N
      replace r2               = `r2_level' if _n==_N
      replace sd_res           = `std_yieldres' if _n==_N
      replace sd_dyvar         = `std_yield_d' if _n==_N
      replace av_dyvar         = `av_dyield' if _n==_N
      replace r2_d             = `r2_delta' if _n==_N
      replace sd_dres          = `std_yieldres_d' if _n==_N
      save "$regout/regression_output.dta", replace
      restore

      * Yields on all Covariates + initial yield
      sum `depvar', d
      local std_yield = r(sd)
      local av_yield = r(mean)
      reg `depvar' i.countrysector i.currency_num BA_spread resid_maturity bamount_resid `initial_yield' i.cac1 i.payment_rank1 if sample_dummy==1
      local r2_level = `e(r2)'
      predict `depvar'_res, r
      sum `depvar'_res if sample_dummy==1, d
      local std_yieldres = r(sd)

      sum `delta_depvar', d
      local std_yield_d = r(sd)
      local av_dyield = r(mean)
      reg `delta_depvar' i.countrysector i.currency_num BA_spread resid_maturity bamount_resid `initial_yield' i.cac1 i.payment_rank1  if sample_dummy==1
      local r2_delta = `e(r2)'
      predict `delta_depvar'_res_d, r
      sum `delta_depvar'_res_d if sample_dummy==1, d
      local std_yieldres_d = r(sd)

      drop `delta_depvar'_res `depvar'_res

      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 3 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_post        = `i' if _n==_N
      replace jorda_proj       = 2 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace sd_yvar          = `std_yield' if _n==_N
      replace av_yvar          = `av_yield'  if _n==_N
      replace r2               = `r2_level' if _n==_N
      replace sd_res           = `std_yieldres' if _n==_N
      replace sd_dyvar         = `std_yield_d' if _n==_N
      replace av_dyvar         = `av_dyield' if _n==_N
      replace r2_d             = `r2_delta' if _n==_N
      replace sd_dres          = `std_yieldres_d' if _n==_N
      save "$regout/regression_output.dta", replace
      restore
    }
  }

  * Yield Regression on covariates - only sov
  if `do_yieldres_onlysov' == 1 {

    reg deltay_bond_m`hpre_chosen'_p`hpost_fixed' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0 & resid_maturity>0 & resid_maturity<4000, robust
    capture confirm variable sample_dummy
    if !_rc {
      }
    else {
        gen sample_dummy = 0
    replace sample_dummy = 1 if e(sample)
    }

    forval i=-58(2)120 {
      if `i'<=0 {
        local absi   = abs(`i')
        local absim1 = abs(`i'-2)
        local depvar = "bondY_m`absi'"
        local depvarlag = "bondY_m`absim1'"
        local delta_depvar = "dbY_m`absi'"
        local initial_yield = "bondY_m58"
      }
      else {
        local i = `i'-1
        local im1 = `i'-2
        local depvar = "bondY_p`i'"
        local depvarlag = "bondY_p`im1'"
        if `i'==1 {
          local depvarlag = "bondY_m0"
        }
        local delta_depvar = "dbY_p`i'"                                                                                                           
        local initial_yield = "bondY_m0"
      }
	  
      * Yields on all covariates
      sum `depvar' if sector=="Government", d
      local std_yield = r(sd)
      local av_yield = r(mean)
      reg `depvar'  i.countrysector i.currency_num BA_spread resid_maturity i.cac1 i.payment_rank1  if sample_dummy==1 & sector=="Government"
      local r2_level = `e(r2)'
      predict `depvar'_res, r
      sum `depvar'_res if sample_dummy==1 & sector=="Government", d
      local std_yieldres = r(sd)

      sum `delta_depvar' if sector=="Government", d
      local std_yield_d = r(sd)
      local av_dyield = r(mean)
      reg `delta_depvar'  i.countrysector i.currency_num BA_spread resid_maturity i.cac1 i.payment_rank1  if sample_dummy==1 & sector=="Government"
      local r2_delta = `e(r2)'
      predict `delta_depvar'_res, r
      sum `delta_depvar'_res if sample_dummy==1 & sector=="Government", d
      local std_yieldres_d = r(sd)

      drop `delta_depvar'_res `depvar'_res

      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 3 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_post        = `i' if _n==_N
      replace jorda_proj       = 2 if _n==_N
      replace control_vars     = 2 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 1 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace sd_yvar          = `std_yield' if _n==_N
      replace av_yvar          = `av_yield'  if _n==_N
      replace r2               = `r2_level' if _n==_N
      replace sd_res           = `std_yieldres' if _n==_N
      replace sd_dyvar         = `std_yield_d' if _n==_N
      replace av_dyvar         = `av_dyield' if _n==_N
      replace r2_d             = `r2_delta' if _n==_N
      replace sd_dres          = `std_yieldres_d' if _n==_N
      save "$regout/regression_output.dta", replace
      restore

      * Yields on country-sector FE
      sum `depvar' if sector=="Government", d
      local std_yield = r(sd)
      local av_yield = r(mean)
      reg `depvar'  i.countrysector  if sample_dummy==1 & sector=="Government"
      local r2_level = `e(r2)'
      predict `depvar'_res, r
      sum `depvar'_res if sample_dummy==1 & sector=="Government", d
      local std_yieldres = r(sd)

      sum `delta_depvar' if sector=="Government", d
      local std_yield_d = r(sd)
      local av_dyield = r(mean)
      reg `delta_depvar'  i.countrysector if sample_dummy==1 & sector=="Government"
      local r2_delta = `e(r2)'
      predict `delta_depvar'_res, r
      sum `delta_depvar'_res if sample_dummy==1 & sector=="Government", d
      local std_yieldres_d = r(sd)

      drop `delta_depvar'_res `depvar'_res

      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 3 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_post        = `i' if _n==_N
      replace jorda_proj       = 2 if _n==_N
      replace control_vars     = 0 if _n==_N
      replace fixed_effects    = 5 if _n==_N
      replace subsample        = 1 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace sd_yvar          = `std_yield' if _n==_N
      replace av_yvar          = `av_yield'  if _n==_N
      replace r2               = `r2_level' if _n==_N
      replace sd_res           = `std_yieldres' if _n==_N
      replace sd_dyvar         = `std_yield_d' if _n==_N
      replace av_dyvar         = `av_dyield' if _n==_N
      replace r2_d             = `r2_delta' if _n==_N
      replace sd_dres          = `std_yieldres_d' if _n==_N
      save "$regout/regression_output.dta", replace
      restore

      * Yields on country-sector + currency FE
      sum `depvar' if sector=="Government", d
      local std_yield = r(sd)
      local av_yield = r(mean)
      reg `depvar'  i.countrysector i.currency_num  if sample_dummy==1 & sector=="Government"/*, vce(cluster country_num) */
      local r2_level = `e(r2)'
      predict `depvar'_res, r
      sum `depvar'_res if sample_dummy==1 & sector=="Government", d
      local std_yieldres = r(sd)

      sum `delta_depvar' if sector=="Government", d
      local std_yield_d = r(sd)
      local av_dyield = r(mean)
      reg `delta_depvar'  i.countrysector i.currency_num  if sample_dummy==1 & sector=="Government"/*, vce(cluster country_num) */
      local r2_delta = `e(r2)'
      predict `delta_depvar'_res, r
      sum `delta_depvar'_res if sample_dummy==1 & sector=="Government", d
      local std_yieldres_d = r(sd)

      drop `delta_depvar'_res `depvar'_res

      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 3 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_post        = `i' if _n==_N
      replace jorda_proj       = 2 if _n==_N
      replace control_vars     = 0 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 1 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace sd_yvar          = `std_yield' if _n==_N
      replace av_yvar          = `av_yield'  if _n==_N
      replace r2               = `r2_level' if _n==_N
      replace sd_res           = `std_yieldres' if _n==_N
      replace sd_dyvar         = `std_yield_d' if _n==_N
      replace av_dyvar         = `av_dyield' if _n==_N
      replace r2_d             = `r2_delta' if _n==_N
      replace sd_dres          = `std_yieldres_d' if _n==_N
      save "$regout/regression_output.dta", replace
      restore

      * Yields on all covariates + initial_yield
      sum `depvar' if sector=="Government", d
      local std_yield = r(sd)
      local av_yield = r(mean)
      reg `depvar'  i.countrysector i.currency_num BA_spread resid_maturity i.cac1 i.payment_rank1 `initial_yield' if sample_dummy==1 & sector=="Government"
      local r2_level = `e(r2)'
      predict `depvar'_res, r
      sum `depvar'_res if sample_dummy==1 & sector=="Government", d
      local std_yieldres = r(sd)

      sum `delta_depvar' if sector=="Government", d
      local std_yield_d = r(sd)
      local av_dyield = r(mean)
      reg `delta_depvar'  i.countrysector i.currency_num BA_spread resid_maturity i.cac1 i.payment_rank1 `initial_yield' if sample_dummy==1 & sector=="Government"
      local r2_delta = `e(r2)'
      predict `delta_depvar'_res, r
      sum `delta_depvar'_res if sample_dummy==1 & sector=="Government", d
      local std_yieldres_d = r(sd)

      drop `delta_depvar'_res `depvar'_res

      preserve
      use "$regout/regression_output.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 3 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_post        = `i' if _n==_N
      replace jorda_proj       = 2 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 1 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace sd_yvar          = `std_yield' if _n==_N
      replace av_yvar          = `av_yield'  if _n==_N
      replace r2               = `r2_level' if _n==_N
      replace sd_res           = `std_yieldres' if _n==_N
      replace sd_dyvar         = `std_yield_d' if _n==_N
      replace av_dyvar         = `av_dyield' if _n==_N
      replace r2_d             = `r2_delta' if _n==_N
      replace sd_dres          = `std_yieldres_d' if _n==_N
      save "$regout/regression_output.dta", replace
      restore

    }
  }

}

**********************************
**********************************
* Graphical Analysis
**********************************
**********************************

if `graphs_main_regs' == 1 {

  use "$regout/regression_output.dta", clear
  levelsof days_pre, local(days_pre_range)

  * Yield Regression - Jorda Projection - Baseline
  if `do_baseline' == 1 {
    local spec "if yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0 & days_post<71 & days_post>-20"
    twoway (line stock_coef days_post `spec' , lcolor(edkblue) lwidth(medthick) xlabel(#7, labsize(small) angle(0)) xscale(range(-20 70)) xlabel(-20[10]70) ylabel(,labsize(small) format(%9.0gc))  /*
    */ graphregion(color(white)) plotregion(color(white)) yline(0, lc(black)) xline(0, lwidth(thin) lcolor(black) lpattern(dash))) /*
    */ (line stock_ub days_post `spec', lcolor(edkblue) lwidth(medthick) lpattern(dash) xlabel(#7, labsize(small) angle(0)) xscale(range(-20 70)) xlabel(-20[10]70)  ylabel(,labsize(small) format(%9.0gc))) /*
    */ (line stock_lb days_post `spec', lcolor(edkblue) lwidth(medthick) lpattern(dash) xlabel(#7, labsize(small) angle(0)) xtitle("Days Post") xscale(range(-20 70)) xlabel(-20[10] 70)  ylabel(,labsize(small) format(%9.0gc)) ytitle("Beta") leg(off))
    graph export "$regout/yield_jorda_baseline.pdf", replace
    sum stock_coef if yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0 & days_post<120 & days_post>=3
  }
}

**********************************
**********************************
* Graphical Analysis Additional
**********************************
**********************************

if `graphs_add_regs' == 1 {

  use "$regout/regression_output.dta", clear

  * Figure C1(b): Delta Yield Regression on covariates - only sov and baseline joint - dispersion
  if `do_yieldres_onlysov' == 1 & `do_yieldres_baseline' == 1 {
    local spec "if yvar==3 & xvar==0 & jorda_proj==2 & control_vars==2 & fixed_effects==1 & subsample==0 & sd_dres>0.001"
    local spec_sov "if yvar==3 & xvar==0 & jorda_proj==2 & control_vars==2 & fixed_effects==1 & subsample==1 & sd_dres>0.001"
    twoway (line sd_dres days_post `spec' , lcolor(edkblue) lwidth(medthick) xlabel(#7, labsize(small) angle(0)) ylabel(,labsize(small) format(%9.0gc))  xline(0, lwidth(thin) lcolor(black) lpattern(dash)) /*
    */ graphregion(color(white)) plotregion(color(white)) ) /*
    */ (line sd_dres days_post `spec_sov', lcolor(dkgreen) lwidth(medthick) xlabel(-60(20)120, labsize(small) angle(0))  ytitle("") xtitle("Days (0=Lehman Bankruptcy)") ylabel(,labsize(small) format(%9.0gc)) leg(lab(1 "All bonds") lab(2 "Only Sovereign") region(lwidth(none)) r(2) ring(0) pos(11) ) )
    graph export "$regout/dyieldres_baselineonlysov.pdf", replace
  }
  
  * Figure C1(a): Delta Yield Avefrage - only sov and baseline joint - average
  if `do_yieldres_onlysov' == 1 & `do_yieldres_baseline' == 1 {
    local spec "if yvar==3 & xvar==0 & jorda_proj==2 & control_vars==2 & fixed_effects==1 & subsample==0 & sd_dres>0.0002"
    local spec_sov "if yvar==3 & xvar==0 & jorda_proj==2 & control_vars==2 & fixed_effects==1 & subsample==1 & sd_dres>0.0002"
    twoway (line av_dyvar days_post `spec' , lcolor(edkblue) lwidth(medthick) xlabel(#7, labsize(small) angle(0)) ylabel(,labsize(small) format(%9.0gc))  xline(0, lwidth(thin) lcolor(black) lpattern(dash)) /*
    */ graphregion(color(white)) plotregion(color(white)) ) /*
    */ (line av_dyvar days_post `spec_sov', lcolor(dkgreen) lwidth(medthick) xlabel(-60(20)120, labsize(small) angle(0))  ytitle("") xtitle("Days (0=Lehman Bankruptcy)") ylabel(,labsize(small) format(%9.0gc)) leg(lab(1 "All bonds") lab(2 "Only Sovereign") region(lwidth(none)) r(2) ring(0) pos(11) ) )
    graph export "$regout/dyieldav_baselineonlysov.pdf", replace
  }


}

*****************************
*****************************
* Tables - Results
*****************************
*****************************
****** Table 2: Main results
if `table_main_regs' == 1 {
  use "$regout/regression_output.dta", clear
  gen crisis = 1 
  append using "$output/russia/regression_output_russia.dta"
  
  local spec_baseline "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0 & crisis == 1 & days_post<61 & days_post>0"
  local spec_nocontrols "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==0 & fixed_effects==1 & subsample==0 & crisis == 1 & days_post<61 & days_post>0"
  local spec_onlysov "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==1 & crisis == 1 & days_post<61 & days_post>0"
  local spec_samemat "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==8  & crisis == 1 & days_post<61 & days_post>0"
  local spec_iv "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==3 & control_vars==1 & fixed_effects==1 & subsample==0  & crisis == 1 & days_post<61 & days_post>0"
  local spec_contemp "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==2 & control_vars==1 & fixed_effects==1 & subsample==0  & crisis == 1 & days_post<61 & days_post>0"
  local spec_fc "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==16 & crisis == 1 & days_post<61 & days_post>0"
  local spec_lc "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==15 & crisis == 1 & days_post<61 & days_post>0"
  local spec_onlybank "yvar==1 & xvar==1 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & crisis == 1 & days_post<61 & days_post>0"
  local spec_bnkamd "yvar==1 & xvar==7 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & crisis == 1 & days_post<61 & days_post>0"
  local spec_am "yvar==1 & xvar==10 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & crisis == 1 & days_post<61 & days_post>0"
  local spec_other "yvar==1 & xvar==11 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & crisis == 1 & days_post<61 & days_post>0"
  local spec_baseline_rus "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0 & crisis == 2 & days_post<31 & days_post>0"
  local spec_nocontrols_rus "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==0 & fixed_effects==1 & subsample==0 & crisis == 2 & days_post<31 & days_post>0"
  local spec_onlysov_rus "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==1 & crisis == 2 & days_post<31 & days_post>0"
  local spec_samemat_rus "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==8  & crisis == 2 & days_post<31 & days_post>0"
  
  gen obs = _n

  foreach reg in baseline nocontrols onlysov samemat contemp fc lc onlybank bnkamd am other iv baseline_rus {

      sum obs if `spec_`reg'' & days_post==3
      if `r(N)'==1 {
        local obs_impact_`reg' = `r(min)'
      }
      local coef_impact_`reg' = stock_coef[`obs_impact_`reg'']
      local se_impact_`reg' = (stock_ub[`obs_impact_`reg'']-stock_coef[`obs_impact_`reg''])/`conf_int'
      local n_`reg' = nobs[`obs_impact_`reg'']
      local t_impact_`reg' = abs(`coef_impact_`reg''/`se_impact_`reg'')


      sum stock_coef if `spec_`reg''
      local coef_peak_`reg' = `r(min)'
      local coef_av_`reg' = `r(mean)'

      sum obs if `spec_`reg'' & stock_coef>`coef_peak_`reg''-`precision' & stock_coef<`coef_peak_`reg''+`precision'
      if `r(N)'==1 {
        local obs_peak_`reg' = `r(min)'
      }
      local coef_peak_`reg' = stock_coef[`obs_peak_`reg'']
      local se_peak_`reg' = (stock_ub[`obs_peak_`reg'']-stock_coef[`obs_peak_`reg''])/`conf_int'
      local day_peak_`reg' = days_post[`obs_peak_`reg'']
      local t_peak_`reg' = abs(`coef_peak_`reg''/`se_peak_`reg'')

      foreach time in impact peak {
        if `t_`time'_`reg''<1.645 {
          local star_`time'_`reg' = ""
        }
        else if `t_`time'_`reg''>1.645 & `t_`time'_`reg''<1.96 {
          local star_`time'_`reg' = "$^{*}$"

        }
        else if `t_`time'_`reg''>1.96 & `t_`time'_`reg''<2.576 {
          local star_`time'_`reg' = "$^{**}$"

        }
        else if `t_`time'_`reg''>2.576 {
          local star_`time'_`reg' = "$^{***}$"
        }
		
      }
  }

  tempname temp_table
  file open temp_table using "$regout/main_regressions.tex", write replace
  file write temp_table "\begin{tabular}{llcccc}" _n "\hline \hline" _n
  file write temp_table " &  & Impact & Peak & Average & Obs.\\" _n
  file write temp_table "\hline" _n
  
  
  foreach reg in baseline nocontrols onlysov samemat contemp fc lc onlybank bnkamd am other iv baseline_rus {
		if "`reg'"=="baseline" {
		  file write temp_table "A. Baseline & & " 
		}
		if "`reg'"=="nocontrols" {
		  file write temp_table "B. Robustness & No Controls &"
		}
		if "`reg'"=="onlysov" {
		  file write temp_table " & Only Sovereign &"
		}
		if "`reg'"=="samemat" {
		  file write temp_table " & Same Maturity&"
		}
		if "`reg'"=="contemp" {
		  file write temp_table " & Contemporaneous &"
		}
		if "`reg'"=="fc" {
		  file write temp_table "C. By currency & Only Foreign& " 
		}
		if "`reg'"=="lc" {
		  file write temp_table "& Only Local & "
		}
		if "`reg'"=="onlybank" {
		  file write temp_table "D. By institution & Banks & " 
		}
		if "`reg'"=="bnkamd" {
		  file write temp_table "& Banks (ex. AMD) & "
		}
		if "`reg'" == "am" {
		  file write temp_table "& Asset Managers &"
		}
		if "`reg'" == "other" {
		  file write temp_table "& Other &"
		}
	    if "`reg'"=="iv" {
		  file write temp_table "E. IV Strategy  & &"
		}
		if "`reg'" == "baseline_rus" {
		 file write temp_table "F. Russian Crisis & &"
		}

       foreach var in coef_impact coef_peak coef_av {
		  file write temp_table (substr(string(``var'_`reg'',"%9.5f"),1,6))
		  if "`var'" == "coef_impact" {
            file write temp_table "`star_impact_`reg'' &"
          }
   	      else if "`var'" == "coef_peak" {
		    file write temp_table "`star_peak_`reg'' &"
		  }
		  else {
		    file write temp_table "&"
		  }
	   }
	   
	  foreach var in n {
	      file write temp_table (substr(string(``var'_`reg'',"%9.0f"),1,3))
		  file write temp_table "\\" _n
	      file write temp_table "&"
	   }
	   
	   foreach var in se_impact se_peak {
	      file write temp_table "&(" (substr(string(``var'_`reg'',"%9.5f"),1,5)) ")"
		  if "`var'" == "se_peak" {
		     file write temp_table "& & \\" _n
		  }
	   }
	}

  file write temp_table "\hline \hline" _n "\end{tabular}"
  file close temp_table

}


***** Table B10: Robustness 

if `table_rob_regs' == 1 {

  use "$regout/regression_output.dta", clear

  local spec_baseline "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0 & days_post<61 & days_post>0"
  local spec_tight "yvar==1 & xvar==0 & days_pre==4 & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & days_post<61 & days_post>0"
  local spec_wide "yvar==1 & xvar==0 & days_pre==-9 & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & days_post<61 & days_post>0"
  local spec_onlynonmmk "yvar==1 & xvar==5 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & days_post<61 & days_post>0"
  local spec_post10 "yvar==1 & xvar==12 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & days_post<61 & days_post>9"
  local spec_post30 "yvar==1 & xvar==13 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & days_post<61 & days_post>29"
  local spec_post45 "yvar==1 & xvar==14 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0  & days_post<61 & days_post>44"
  
  gen obs = _n
  
  foreach reg in baseline tight wide onlynonmmk post10 post30 post45 {
      sum stock_coef if `spec_`reg''
      local coef_peak_`reg' = `r(min)'
      local coef_av_`reg' = `r(mean)'

      sum obs if `spec_`reg'' & stock_coef>`coef_peak_`reg''-`precision' & stock_coef<`coef_peak_`reg''+`precision'
      if `r(N)'==1 {
        local obs_peak_`reg' = `r(min)'
      }
      local coef_peak_`reg' = stock_coef[`obs_peak_`reg'']
      local se_peak_`reg' = (stock_ub[`obs_peak_`reg'']-stock_coef[`obs_peak_`reg''])/`conf_int'
      local day_peak_`reg' = days_post[`obs_peak_`reg'']
      local t_peak_`reg' = abs(`coef_peak_`reg''/`se_peak_`reg'')
	 
    if "`reg'" != "post10" & "`reg'" != "post30" & "`reg'" != "post45" {
      sum obs if `spec_`reg'' & days_post==3
      if `r(N)'==1 {
        local obs_impact_`reg' = `r(min)'
      }
      local coef_impact_`reg' = stock_coef[`obs_impact_`reg'']
      local se_impact_`reg' = (stock_ub[`obs_impact_`reg'']-stock_coef[`obs_impact_`reg''])/`conf_int'
      local n_`reg' = nobs[`obs_impact_`reg'']
      local t_impact_`reg' = abs(`coef_impact_`reg''/`se_impact_`reg'')
	  
	  sum stock_coef if `spec_`reg'' & days_post >= 3 & days_post <= `day_peak_`reg''
	  local coef_av1_`reg' = `r(mean)'
	  }
	
	else if  "`reg'" == "post10" { 
      sum obs if `spec_`reg'' & days_post==11
      if `r(N)'==1 {
        local obs_impact_`reg' = `r(min)'
      }
      local coef_impact_`reg' = stock_coef[`obs_impact_`reg'']
      local se_impact_`reg' = (stock_ub[`obs_impact_`reg'']-stock_coef[`obs_impact_`reg''])/`conf_int'
      local n_`reg' = nobs[`obs_impact_`reg'']
      local t_impact_`reg' = abs(`coef_impact_`reg''/`se_impact_`reg'')
	  
	  sum stock_coef if `spec_`reg'' & days_post >= 11 & days_post <= `day_peak_`reg''
	  local coef_av1_`reg' = `r(mean)'
     }
    else if "`reg'" == "post30" { 
      sum obs if `spec_`reg'' & days_post==31
      if `r(N)'==1 {
        local obs_impact_`reg' = `r(min)'
      }
      local coef_impact_`reg' = stock_coef[`obs_impact_`reg'']
      local se_impact_`reg' = (stock_ub[`obs_impact_`reg'']-stock_coef[`obs_impact_`reg''])/`conf_int'
      local n_`reg' = nobs[`obs_impact_`reg'']
      local t_impact_`reg' = abs(`coef_impact_`reg''/`se_impact_`reg'')
	  
	  sum stock_coef if `spec_`reg'' & days_post >= 31 & days_post <= `day_peak_`reg''
	  local coef_av1_`reg' = `r(mean)'
     }
    else if "`reg'" == "post45" { 
      sum obs if `spec_`reg'' & days_post==45
      if `r(N)'==1 {
        local obs_impact_`reg' = `r(min)'
      }
      local coef_impact_`reg' = stock_coef[`obs_impact_`reg'']
      local se_impact_`reg' = (stock_ub[`obs_impact_`reg'']-stock_coef[`obs_impact_`reg''])/`conf_int'
      local n_`reg' = nobs[`obs_impact_`reg'']
      local t_impact_`reg' = abs(`coef_impact_`reg''/`se_impact_`reg'')
	  
     }
	 
	foreach time in impact peak {
        if `t_`time'_`reg''<1.645 {
          local star_`time'_`reg' = ""
        }
        else if `t_`time'_`reg''>1.645 & `t_`time'_`reg''<1.96 {
          local star_`time'_`reg' = "$^{*}$"

        }
        else if `t_`time'_`reg''>1.96 & `t_`time'_`reg''<2.576 {
          local star_`time'_`reg' = "$^{**}$"

        }
        else if `t_`time'_`reg''>2.576 {
          local star_`time'_`reg' = "$^{***}$"
        }
	}
  }
  

  tempname temp_table
  file open temp_table using "$regout/rob_regressions.tex", write replace
  file write temp_table "\begin{tabular}{llcccc}" _n "\hline \hline" _n
  file write temp_table " &  & Impact & Peak & Average & Obs.\\" _n
  file write temp_table "\hline" _n
  
  foreach reg in baseline tight wide post10 post30 post45 onlynonmmk {
		if "`reg'"=="baseline" {
		  file write temp_table "A. Baseline & & " 
		}
		if "`reg'"=="tight" {
		  file write temp_table "B. Alternative Windows & Tighter &"
		}
		if "`reg'"=="wide" {
		  file write temp_table " & Wider &"
		}
		if "`reg'" == "post10" {
		  file write temp_table "& 10d Post & "
		} 
	    if "`reg'" == "post30" {
		  file write temp_table "& 30d Post & "
		} 
		if "`reg'" == "post45" {
		  file write temp_table "& 45d Post & "
		} 
		if "`reg'"=="onlynonmmk" {
		  file write temp_table " C. Excluding Market Makers & &"
		}


       foreach var in coef_impact coef_peak coef_av {
		  file write temp_table (substr(string(``var'_`reg'',"%9.5f"),1,6))
		  if "`var'" == "coef_impact" {
            file write temp_table "`star_impact_`reg'' &"
          }
   	      else if "`var'" == "coef_peak" {
		    file write temp_table "`star_peak_`reg'' &"
		  }
		  else {
		    file write temp_table "&"
		  }
	   }
	   
	  foreach var in n {
	      file write temp_table (substr(string(``var'_`reg'',"%9.0f"),1,3))
		  file write temp_table "\\" _n
	      file write temp_table "&"
	   }
	   
	   foreach var in se_impact se_peak {
	      file write temp_table "&(" (substr(string(``var'_`reg'',"%9.5f"),1,5)) ")"
		  if "`var'" == "se_peak" {
		     file write temp_table "& & \\" _n
		  }
	   }
	}

  file write temp_table "\hline \hline" _n "\end{tabular}"
  file close temp_table
}

***** Table B3: Yields


if `table_yieldres' == 1 {

  use "$regout/regression_output.dta", clear

  local spec_all_allcont "yvar==3 & xvar==0 & jorda_proj==2 & control_vars==2 & fixed_effects==1 & subsample==0 & sd_res>0.0002"
  local spec_all_csfe "yvar==3 & xvar==0 & jorda_proj==2 & control_vars==0 & fixed_effects==5 & subsample==0 & sd_res>0.0002"
  local spec_all_cscfe "yvar==3 & xvar==0 & jorda_proj==2 & control_vars==0 & fixed_effects==1 & subsample==0 & sd_res>0.0002"
  local spec_all_initytm "yvar==3 & xvar==0 & jorda_proj==2 & control_vars==1 & fixed_effects==1 & subsample==0 & sd_res>0.0002"

  local spec_onlysov_allcont "yvar==3 & xvar==0 & jorda_proj==2 & control_vars==2 & fixed_effects==1 & subsample==1 & sd_res>0.0002"
  local spec_onlysov_csfe "yvar==3 & xvar==0 & jorda_proj==2 & control_vars==0 & fixed_effects==5 & subsample==1 & sd_res>0.001"
  local spec_onlysov_cscfe "yvar==3 & xvar==0 & jorda_proj==2 & control_vars==0 & fixed_effects==1 & subsample==1 & sd_res>0.001"
  local spec_onlysov_initytm "yvar==3 & xvar==0 & jorda_proj==2 & control_vars==1 & fixed_effects==1 & subsample==1 & sd_res>0.0002"

  gen obs = _n
  
    
  foreach subsample in all onlysov {

    foreach control in allcont csfe cscfe initytm {

      foreach var in av_yvar sd_yvar r2 {


        sum `var' if `spec_`subsample'_`control'' & days_post<0 & days_post>-60, d
        local `var'_`subsample'_`control'_pre = r(mean)
        di ``var'_`subsample'_`control'_pre'
        sum `var' if `spec_`subsample'_`control'' & days_post>=0 & days_post<60, d
        local `var'_`subsample'_`control'_post = r(mean)
        di ``var'_`subsample'_`control'_post'
		
      }
    }
  }
  
  foreach var in av_yvar sd_yvar r2 {
    gen `var'_old = `var'
  }
  
 foreach subsample in all onlysov {
    foreach control in allcont csfe cscfe initytm {
	   gen pre_dummy_`subsample'_`control' = 1 if days_post > -60 & days_post<0
       replace pre_dummy_`subsample'_`control' = 0 if days_post <60 & days_post>= 0
       
		
	   foreach var in av_yvar sd_yvar r2 {
	     sum `var' if `spec_`subsample'_`control'', meanonly
         scalar omean_`var' = r(mean)
         sum `var' if `spec_`subsample'_`control'' & pre_dummy_`subsample'_`control' == 1, meanonly
         replace `var' = `var' - r(mean) + scalar(omean_`var') if `spec_`subsample'_`control'' & pre_dummy_`subsample'_`control' == 1
         sum `var' if `spec_`subsample'_`control'' & pre_dummy_`subsample'_`control' == 0, meanonly
         replace `var' = `var' - r(mean) + scalar(omean_`var') if `spec_`subsample'_`control'' & pre_dummy_`subsample'_`control' == 0
         
		set seed 1 
		bootstrap t=r(t), rep(1000) strata(pre_dummy_`subsample'_`control') saving("$regout/bs_dyieldres_`var'_`subsample'_`control'",replace) nodots: ttest `var' if `spec_`subsample'_`control'', by(pre_dummy_`subsample'_`control') unequal
	  }
	}
  }
 
 foreach subsample in all onlysov {
   foreach control in allcont csfe cscfe initytm {        
	   foreach var in av_yvar sd_yvar r2 {
	    ttest `var'_old if `spec_`subsample'_`control'', by(pre_dummy_`subsample'_`control') unequal
	    scalar tobs = r(t)
		
  		preserve
  		 use "$regout/bs_dyieldres_`var'_`subsample'_`control'", clear
           gen indicator = abs(t)>=abs(scalar(tobs))
           sum indicator, meanonly
  		 local `var'_`subsample'_`control'_p = r(mean)
  		restore      
		
      }
	}
 }
  

  tempname temp_table
  file open temp_table using "$regout/yields_sdres.tex", write replace
  file write temp_table "\begin{tabular}{lcccccc}" _n "\hline \hline" _n
  file write temp_table "  & \multicolumn{3}{c}{All Bonds} & \multicolumn{3}{c}{Only Sovereign} \\" _n
  file write temp_table " &  Pre-Lehman & Post-Lehman & Diff. & Pre-Lehman & Post-Lehman & Diff. \\" _n
  file write temp_table "\hline" _n

  local av_yvar_label "Average"
  local sd_yvar_label "Cross-Sec. Std. Deviation"
  
  
  file write temp_table "Average"
    foreach subsample in all onlysov  {
        file write temp_table "&" (substr(string(`av_yvar_`subsample'_allcont_pre'*100,"%9.2f"),1,6)) "\% & " (substr(string(`av_yvar_`subsample'_allcont_post'*100,"%9.2f"),1,6))  "\% & " (substr(string(`av_yvar_`subsample'_allcont_p',"%9.5f"),1,5))
    }
  file write temp_table " \\" _n
  
  file write temp_table "Cross-Sec. Std. Deviation"
    foreach subsample in all onlysov  {
       file write temp_table "&" (substr(string(`sd_yvar_`subsample'_allcont_pre'*100,"%9.2f"),1,6)) "\% & " (substr(string(`sd_yvar_`subsample'_allcont_post'*100,"%9.2f"),1,6))  "\% & " (substr(string(`sd_yvar_`subsample'_allcont_p',"%9.5f"),1,5))
    }
  file write temp_table " \\" _n
  

  file write temp_table "R$^2$ from Yield Regressions \\" _n
  local csfe_label "\quad (1): Country-Sector FE"
  local cscfe_label "\quad (2): (1) + Currency FE"
  local allcont_label "\quad (3): (2) + Add. Controls"
  local initytm_label "\quad (4): (3) + Initial Yield"

  foreach control in  csfe cscfe allcont initytm {

    file write temp_table "``control'_label'"
    foreach subsample in all onlysov  {
        file write temp_table "&" (substr(string(`r2_`subsample'_`control'_pre'*100,"%9.1f"),1,6)) "\% & " (substr(string(`r2_`subsample'_`control'_post'*100,"%9.1f"),1,6)) "\% &" (substr(string(`r2_`subsample'_`control'_p',"%9.5f"),1,5))
    }
    file write temp_table " \\" _n
  }


  file write temp_table "\hline \hline" _n "\end{tabular}"
  file close temp_table


}

***** Table B11: Interactive effects results

if `table_interactive_regs' == 1 {

  use "$regout/regression_output.dta", clear

  local spec_baseline "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0 & days_post<61 & days_post>0"
  local spec_lev1 "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==4 & fixed_effects==1 & subsample==0 & days_post<61 & days_post>0"
  local spec_liq1 "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==5 & fixed_effects==1 & subsample==0 & days_post<61 & days_post>0"
  local spec_gfi  "yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==7 & fixed_effects==1 & subsample==0 & days_post<61 & days_post>0"
  
  gen obs = _n
      
	  sum obs if `spec_baseline' & days_post==3
      if `r(N)'==1 {
        local obs_impact_baseline = `r(min)'
      }
	  local n_baseline = nobs[`obs_impact_baseline']
	  
      sum stock_coef if `spec_baseline'
      local coef_peak_baseline = `r(min)'
	  local coef_av_baseline = `r(mean)'

      sum obs if `spec_baseline' & stock_coef>`coef_peak_baseline'-`precision' & stock_coef<`coef_peak_baseline'+`precision'
      if `r(N)'==1 {
        local obs_peak_baseline = `r(min)'
      }
      local coef_peak_baseline = stock_coef[`obs_peak_baseline']
      local se_peak_baseline = (stock_ub[`obs_peak_baseline']-stock_coef[`obs_peak_baseline'])/`conf_int'
      local day_peak_baseline = days_post[`obs_peak_baseline']
      local t_peak_baseline = abs(`coef_peak_baseline'/`se_peak_baseline')
  
	    if `t_peak_baseline'<1.645 {
          local star_peak_baseline = ""
        }
        else if `t_peak_baseline'>1.645 & `t_peak_baseline'<1.96 {
          local star_peak_baseline = "$^{*}$"

        }
        else if `t_peak_baseline'>1.96 & `t_peak_baseline'<2.576 {
          local star_peak_baseline = "$^{**}$"

        }
        else if `t_peak_baseline'>2.576 {
          local star_peak_baseline = "$^{***}$"
        }  
		
	  sum stock_coef if `spec_baseline' & days_post >= 3 & days_post <= `day_peak_baseline'
	  local coef_av1_baseline = `r(mean)'
  
  foreach reg in lev1 gfi {
  
      sum obs if `spec_`reg'' & days_post==3
      if `r(N)'==1 {
        local obs_impact_`reg' = `r(min)'
      }
	  local n_`reg' = nobs[`obs_impact_`reg'']
	  
	  sum inter_coef if `spec_`reg''
	  local coef_peak_`reg' = `r(min)'

      sum obs if `spec_`reg'' & inter_coef>`coef_peak_`reg''-`precision' & inter_coef<`coef_peak_`reg''+`precision'
      if `r(N)'==1 {
        local obs_peak_`reg' = `r(min)'
      }
      local coef_peak_`reg' = stock_coef[`obs_peak_`reg'']
      local se_peak_`reg' = (stock_ub[`obs_peak_`reg'']-stock_coef[`obs_peak_`reg''])/`conf_int'
      local day_peak_`reg' = days_post[`obs_peak_`reg'']
      local t_peak_`reg' = abs(`coef_peak_`reg''/`se_peak_`reg'')
	  
	  local coef_inter_`reg' = inter_coef[`obs_peak_`reg'']
	  local se_inter_`reg' = (inter_ub[`obs_peak_`reg'']-inter_coef[`obs_peak_`reg''])/`conf_int'
      local t_inter_`reg' = abs(`coef_inter_`reg''/`se_inter_`reg'')
	  
	  local coef_levliq_`reg' = levliq_coef[`obs_peak_`reg'']
	  local se_levliq_`reg' = (levliq_ub[`obs_peak_`reg'']-levliq_coef[`obs_peak_`reg''])/`conf_int'
      local t_levliq_`reg' = abs(`coef_levliq_`reg''/`se_levliq_`reg'')
   
      foreach time in peak inter levliq {
	    if `t_`time'_`reg''<1.645 {
          local star_`time'_`reg' = ""
        }
        else if `t_`time'_`reg''>1.645 & `t_`time'_`reg''<1.96 {
          local star_`time'_`reg' = "$^{*}$"

        }
        else if `t_`time'_`reg''>1.96 & `t_`time'_`reg''<2.576 {
          local star_`time'_`reg' = "$^{**}$"

        }
        else if `t_`time'_`reg''>2.576 {
          local star_`time'_`reg' = "$^{***}$"
        }
	  }
	  
	  sum stock_coef if `spec_`reg''
	  local coef_av_`reg' = `r(mean)'
	  
	  sum stock_coef if `spec_`reg'' & days_post >= 3 & days_post <= `day_peak_`reg''
	  local coef_av1_`reg' = `r(mean)'
  }
  
  sum obs if `spec_liq1' & days_post==3
      if `r(N)'==1 {
        local obs_impact_liq1 = `r(min)'
      }
	  local n_liq1 = nobs[`obs_impact_liq1']
	  
	  sum inter_coef if `spec_liq1'
	  local coef_peak_liq1 = `r(max)'

      sum obs if `spec_liq1' & inter_coef>`coef_peak_liq1'-`precision' & inter_coef<`coef_peak_liq1'+`precision'
      if `r(N)'==1 {
        local obs_peak_liq1 = `r(min)'
      }
      local coef_peak_liq1 = stock_coef[`obs_peak_liq1']
      local se_peak_liq1 = (stock_ub[`obs_peak_liq1']-stock_coef[`obs_peak_liq1'])/`conf_int'
      local day_peak_liq1 = days_post[`obs_peak_liq1']
      local t_peak_liq1 = abs(`coef_peak_liq1'/`se_peak_liq1')
	  
	  local coef_inter_liq1 = inter_coef[`obs_peak_liq1']
	  local se_inter_liq1 = (inter_ub[`obs_peak_liq1']-inter_coef[`obs_peak_liq1'])/`conf_int'
      local t_inter_liq1 = abs(`coef_inter_liq1'/`se_inter_liq1')
	  
	  local coef_levliq_liq1 = levliq_coef[`obs_peak_liq1']
	  local se_levliq_liq1 = (levliq_ub[`obs_peak_liq1']-levliq_coef[`obs_peak_liq1'])/`conf_int'
      local t_levliq_liq1 = abs(`coef_levliq_liq1'/`se_levliq_liq1')
   
      foreach time in peak inter levliq {
	    if `t_`time'_liq1'<1.645 {
          local star_`time'_liq1 = ""
        }
        else if `t_`time'_liq1'>1.645 & `t_`time'_liq1'<1.96 {
          local star_`time'_liq1 = "$^{*}$"

        }
        else if `t_`time'_liq1'>1.96 & `t_`time'_liq1'<2.576 {
          local star_`time'_liq1 = "$^{**}$"

        }
        else if `t_`time'_liq1'>2.576 {
          local star_`time'_liq1 = "$^{***}$"
        }
	  }
	  
	  sum stock_coef if `spec_liq1'
	  local coef_av_liq1 = `r(mean)'
	  
	  sum stock_coef if `spec_liq1' & days_post >= 3 & days_post <= `day_peak_liq1'
	  local coef_av1_liq1 = `r(mean)'
  
  tempname temp_table
  file open temp_table using "$regout/regressions_interactive.tex", write replace
  file write temp_table "\begin{tabular}{lccc}" _n "\hline \hline" _n
  file write temp_table " &  (1) & (2) & (3) \\" _n
  file write temp_table "\hline" _n

  file write temp_table "Share held by GFIs Interaction &"  (substr(string(`coef_inter_gfi',"%9.5f"),1,6)) "`star_inter_gfi' & - & - \\" _n
  file write temp_table  "& ("  (substr(string(`se_inter_gfi',"%9.5f"),1,5)) ") & & \\" _n
    
  file write temp_table "Leverage Interaction & - & "  (substr(string(`coef_inter_lev1',"%9.5f"),1,6)) "`star_inter_lev1' & - \\" _n
  file write temp_table  "& & ("  (substr(string(`se_inter_lev1',"%9.5f"),1,5)) ") & \\" _n
   
  file write temp_table "Liquidity Interaction & - & - &" (substr(string(`coef_inter_liq1',"%9.5f"),1,6)) "`star_inter_liq1' \\" _n
  file write temp_table "& & & (" (substr(string(`se_inter_liq1',"%9.5f"),1,5)) ") \\" _n
  
  file write temp_table "\hline " _n
  file write temp_table "Peak Day"
  foreach reg in  gfi lev1 liq1 {
      file write temp_table "&" (substr(string(`day_peak_`reg'',"%9.0f"),1,6))
  }
  file write temp_table " \\" _n
  
  file write temp_table "N Observations"
  foreach reg in  gfi lev1 liq1 {
      file write temp_table "&" (substr(string(`n_`reg'',"%9.0f"),1,6))
  }
  file write temp_table " \\" _n

  file write temp_table "\hline \hline" _n "\end{tabular}"
  file close temp_table

}

