clear all
clear matrix
set more off
set maxvar 30000
/* ssc install moremata */
/* ssc install carryforward */

cd "/Users/jmorelli/Dropbox/new_sync/replication_files/empirical" // switch here to user's path

global input 	"data/bloomberg"
global stdata 	"data/stata_data"
global stoutput "data/stata_output"

local hpost_fixed = 3
local hpre_chosen = 10
local hmax_post   = 60
local date_lehman = "15sep2008"
local peak_lev    = 53 /*peak day from leverage regression */
local peak_liq    = 35 /*peak day from leverage regression */

** Run commands
local do_table_deltan               = 1
local do_table_statsbycountry       = 1
local do_table_sector_stats         = 1
local do_table_banksnames           = 1
local do_figure_sorting_country		= 1
local do_table_banksstats           = 1
local do_table_sorting_sector       = 1
local do_table_sorting_covariates   = 1
local do_table_covariates_deltanreg = 1
local do_table_holding_auto         = 1

* Prepare regression dataset by computing auxiliary variables used in regression
use "$stdata/beneficiary/regression_variables.dta", clear
append using "$stdata/beneficiary/regression_variables_extension.dta"
append using "$stdata/beneficiary/regression_variables_ext2010.dta" 

drop if bond_id == "EC119001Corp"
drop if bond_id == "EC396225Corp"
drop if bond_id == "EC396227Corp" 
drop if bond_id == "EC396229Corp"
  
* Create variables used in regressions to obtain consistent sample
drop if sector=="#N/A Invalid Security"
egen countrysector = group(country sector)
encode country, gen(country_num)
encode sector, gen(sector_num)
encode CRNCY, gen(currency_num)
levelsof country, local(country_list)

*Average bond yield pre-Lehman
gen aux_m62 =0
local j = 0
forval i=-60(2)0 {
local j = `j' + 1
local absi   = abs(`i')
local absim2 = abs(`i'-2)
gen aux_m`absi' =  (aux_m`absim2'*(`j'-1) + bondY_m`absi')/`j'
}
rename aux_m0 bondY_prelehman
sum bondY_prelehman, d
gen outlier_yield = 0
replace outlier_yield = 1 if bondY_prelehman<.03 | bondY_prelehman>.5

* CAC and payment_rank
encode cac, gen (cac1)
encode payment_rank, gen (payment_rank1)

* adjust for sample
qui reg deltay_bond_m`hpre_chosen'_p`hpost_fixed' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros  i.countrysector i.currency_num if rating_num>7 & outlier_yield==0
qui gen sample_dummy = 0
qui replace sample_dummy = 1 if e(sample)

preserve
clear
import excel "$input/Country_codes_names.xlsx", firstrow
save "$stdata/country_names.dta", replace
restore

merge m:1 country using "$stdata/country_names.dta"
drop if _merge==2
drop _merge

save "$stdata/beneficiary/regression_variables_prepared.dta", replace

** Generate banks names that have data on stock price and positive holdings
clear
use "$stdata/beneficiary/lender_holdings_append.dta", clear
append using "$stdata/beneficiary/lender_holdings_append_ext.dta"
append using "$stdata/beneficiary/lender_holdings_append_ext2010.dta"

append using "$stdata/bank_names.dta" 

levelsof lender_name_cr, local(lender_name_list)
foreach cr_name of local lender_name_list {

tempvar tv1
gene `tv1'=_n if lender_name_cr=="`cr_name'"
summ `tv1', meanonly
local index=r(min)
di `index'
local cr_k1   =lender_keyword1[`index']
local cr_k2   =lender_keyword2[`index']
local cr_k3   =lender_keyword3[`index']
drop `tv1'
di "`cr_name' `cr_k1' `cr_k2' `cr_k3'"
replace lender_name_cr = "`cr_name'" if strpos(lender_name,"`cr_name'")>0
replace lender_name_cr = "`cr_name'" if (strpos(lower(lender_name),"`cr_k1'")>0 & "`cr_k1'"!="") & ("`cr_k2'"=="" | (strpos(lower(lender_name),"`cr_k2'")>0 & "`cr_k2'"!=""))
replace lender_name_cr = "`cr_name'" if strpos(lower(lender_name),"`cr_k3'")>0 & "`cr_k3'"!=""
}

drop if bond_id == ""
drop lender_keyword*


collapse (sum) lender_amount, by(lender_name_cr)

merge 1:m lender_name_cr using "$stdata/stocks_deltap.dta"

drop _merge

collapse (sum) lender_amount (firstnm) dps_m10_p3, by(lender_name_cr)
generate sample_dummy_bank =0
replace sample_dummy_bank = 1 if dps_m10_p3!=. & lender_amount>0
save "$stdata/beneficiary/banks_names_sample_dummy.dta", replace


// ****** TABLE 1: GFI NW and EM Bond Prices During Lehman ****** //
if `do_table_deltan' == 1 {

	** Sumstats table
	use  "$stdata/stocks_deltap.dta", clear
	merge m:1 lender_name_cr using "$stdata/beneficiary/banks_names_sample_dummy.dta", keepusing(sample_dummy_bank)
	keep if _merge==3

	sum dps_m`hpre_chosen'_p`hpost_fixed' if sample_dummy_bank==1, d
	local deltan_stock_mean   = r(mean)*100
	local deltan_stock_median = r(p50)*100
	local deltan_stock_sd     = r(sd)*100
	local deltan_stock_p5     = r(p5)*100
	local deltan_stock_p95    = r(p95)*100
	local deltan_stock_n      = r(N)

	use "$stdata/beneficiary/regression_variables_prepared.dta", clear

	qui gen coverage_banks = 1-dps_m10_p3_bs_zeros if sample_dummy == 1
	qui sum coverage_banks if coverage_banks!=. & sample_dummy == 1, d
	local coverage_mean   = r(mean)*100
	local coverage_median = r(p50)*100
	local coverage_sd     = r(sd)*100
	local coverage_p5     = r(p5)*100
	local coverage_p95    = r(p95)*100
	local coverage_n      = r(N)


	qui gen dps_m10_p3_bond_w   = dps_m10_p3_bs/coverage_banks

	qui sum dps_m10_p3_bond_w if sample_dummy == 1, d
	local deltan_bond_mean   = r(mean)*100
	local deltan_bond_median = r(p50)*100
	local deltan_bond_sd     = r(sd)*100
	local deltan_bond_p5     = r(p5)*100
	local deltan_bond_p95    = r(p95)*100
	local deltan_bond_n      = r(N)
	
	reg dps_m10_p3_bs dps_m10_p3_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman i.cac1 i.payment_rank1 if rating_num>7 & outlier_yield==0, robust 
    predict resid, residuals
	qui sum resid if sample_dummy == 1, d
	local dps_resid_mean   = r(mean)*100
	local dps_resid_median = r(p50)*100
	local dps_resid_sd     = r(sd)*100
	local dps_resid_p5     = r(p5)*100
	local dps_resid_p95    = r(p95)*100
	local dps_resid_n      = r(N)
	
	tempname variables_sumstats
	file open variables_sumstats using "$stoutput/baseline/deltan_sumstats.tex", write replace
	file write variables_sumstats "\begin{tabular}{lcccc}" _n "\hline \hline" _n
	file write variables_sumstats "        & Bank coverage & $\Delta$ Stock price & $\Delta$ Stock price  & $\Delta$ Stock price \\  " _n
	file write variables_sumstats "  & per bond &  bank level  & bond level & residualized \\ \hline " _n

	foreach stat in mean median sd p5 p95 n {
	di "`stat'"
	local suffix= "\%"
	local suffix_bps= "bps"
		if "`stat'"=="mean" {
		  file write variables_sumstats "Mean &"
		}
		if "`stat'"=="median" {
		  file write variables_sumstats "Median &"
		}
		if "`stat'"=="sd" {
		  file write variables_sumstats "Std Deviation &"
		}
		if "`stat'"=="p5" {
		  file write variables_sumstats "5th percentile &"
		}
		if "`stat'"=="p95" {
		  file write variables_sumstats "95th percentile &"
		}
		if "`stat'"=="n" {
		  file write variables_sumstats "Num. Obs. &"
		  local suffix
		  local suffix_bps
		}

		foreach var in coverage deltan_stock  deltan_bond dps_resid {
		  di "`var'"
		  file write variables_sumstats (substr(string(``var'_`stat'',"%5.0f"),1,3))

		  if "`var'"=="dps_resid" {
		    file write variables_sumstats "`suffix' \\"_n
		  }
		  else {
		    file write variables_sumstats "`suffix' &"
		  }
		}
	}
	file write variables_sumstats "\hline \hline" _n "\end{tabular}"
	file close variables_sumstats
}

// ****** TABLE B1: Descriptive Statistics by Country ****** //
if `do_table_statsbycountry' == 1 {

	use "$stdata/beneficiary/regression_variables_prepared.dta", clear

	levelsof country if sample_dummy==1, local(country_list_reg)
	gen BA_spread_win = max(0,BA_spread)

	gen cumshare_bond_m10_p3_nonzero = 1-dps_m10_p3_bs_zeros
	qui gen number_bonds  = .
	qui gen avg_yield     = .
	qui gen avg_maturity  = . /* residual maturity */
	qui gen avg_mat       = . /* maturity */
	qui gen avg_liquidity = .


	qui gen number_bonds_sov  = .
	qui gen avg_yield_sov     = .
	qui gen avg_maturity_sov  = .
	qui gen avg_mat_sov       = .
	qui gen avg_liquidity_sov = .

	tempname temp_table
	file open temp_table using "$stoutput/baseline/table_bonds_by_country.tex", write replace
	file write temp_table "\begin{tabular}{lcccc}" _n "\hline \hline" _n
	file write temp_table " Country & N Bonds & YTM & Residual Maturity & Bid-Ask Spread \\  " _n
	file write temp_table "   \hline " _n

	tempname temp_table5
	file open temp_table5 using "$stoutput/baseline/table_bonds_by_country5.tex", write replace
	file write temp_table5 "\begin{tabular}{lcccc}" _n "\hline \hline" _n
	file write temp_table5 " Country & N Bonds & YTM & Residual Maturity & Bid-Ask Spread \\  " _n
	file write temp_table5 "   \hline " _n

	local Nbonds_avg5  = 0
	local avgYTM_avg5  = 0
	local avgMat_avg5  = 0
	local avgMat1_avg5 = 0
	local avgLiq_avg5  = 0

	local ii = 0
	local i5 = 0

	foreach cty of local country_list_reg {

      local ii = `ii' + 1
      tempvar tv1
      gene `tv1'=_n if country=="`cty'"
      summ `tv1', meanonly
      local index=r(min)
      local cty_name   =country_long[`index']
      drop `tv1'
      
	  
		qui sum bondY_m0 if (country=="`cty'" & sample_dummy==1 /*& cumshare_bond_m10_p3_nonzero!=. & bondY_m0!=. & bondY_m0<=0.25*/), detail
	    qui replace number_bonds = r(N) if _n==`ii'
	    qui replace avg_yield    = r(mean) if _n==`ii'
	    local Nbonds_`cty' = r(N)
	    local avgYTM_`cty' = r(mean)*100

	    qui sum resid_maturity if (country=="`cty'" & sample_dummy==1 /*& cumshare_bond_m10_p3_nonzero!=. & bondY_m0!=. & bondY_m0<=0.25*/), detail
	    qui replace avg_maturity = r(mean) if _n==`ii'
	    local avgMat_`cty' = r(mean)/365
		
		qui sum maturity if (country=="`cty'" & sample_dummy==1 /*& cumshare_bond_m10_p3_nonzero!=. & bondY_m0!=. & bondY_m0<=0.25*/), detail
	    qui replace avg_mat = r(mean) if _n==`ii'
	    local avgMat1_`cty' = r(mean)
		
	    qui sum BA_spread_win if (country=="`cty'" & sample_dummy==1 /*& cumshare_bond_m10_p3_nonzero!=. & bondY_m0!=. & bondY_m0<=0.25*/), detail
	    qui replace avg_liquidity = r(mean) if _n==`ii'
	    local avgLiq_`cty' = r(mean)*100

	    file write temp_table (substr("`cty_name'",1,18)) "&" (substr(string(`Nbonds_`cty'',"%5.0f"),1,4)) "&" (substr(string(`avgYTM_`cty'',"%9.1f"),1,5)) "\%" "&" (substr(string(`avgMat_`cty'',"%9.5f"),1,4)) /*"&" (substr(string(`avgMat1_`cty'',"%5.0f"),1,5))*/ "&" (substr(string(`avgLiq_`cty'',"%9.2f"),1,5)) "\% \\" _n

		local Nbonds_avg = `Nbonds_avg' + `Nbonds_`cty''
		local avgYTM_avg = `avgYTM_avg' + `avgYTM_`cty''
		local avgMat_avg = `avgMat_avg' + `avgMat_`cty''
		local avgMat1_avg = `avgMat1_avg' + `avgMat1_`cty''
		local avgLiq_avg = `avgLiq_avg' + `avgLiq_`cty''

		if `Nbonds_`cty''>4 {
			local i5 = `i5' + 1
			file write temp_table5 (substr("`cty_name'",1,18)) "&" (substr(string(`Nbonds_`cty'',"%5.0f"),1,4)) "&" (substr(string(`avgYTM_`cty'',"%9.1f"),1,5)) "\%" "&" (substr(string(`avgMat_`cty'',"%9.5f"),1,4)) /*"&" (substr(string(`avgMat1_`cty'',"%5.0f"),1,5))*/ "&" (substr(string(`avgLiq_`cty'',"%9.2f"),1,5)) "\% \\" _n

			local Nbonds_avg5 = `Nbonds_avg5' + `Nbonds_`cty''
			local avgYTM_avg5 = `avgYTM_avg5' + `avgYTM_`cty''
			local avgMat_avg5 = `avgMat_avg5' + `avgMat_`cty''
			local avgMat1_avg5 = `avgMat1_avg5' + `avgMat1_`cty''
			local avgLiq_avg5 = `avgLiq_avg' + `avgLiq_`cty''
		}

	    qui sum bondY_m0 if (country=="`cty'" & sample_dummy==1 /*& cumshare_bond_m10_p3_nonzero!=. & bondY_m0!=. & bondY_m0<=0.25*/ & sector=="Government"), detail
	    qui replace number_bonds_sov = r(N) if _n==`ii'
	    qui replace avg_yield_sov    = r(mean) if _n==`ii'

	    qui sum resid_maturity if (country=="`cty'" & sample_dummy==1 /*& cumshare_bond_m10_p3_nonzero!=. & bondY_m0!=. & bondY_m0<=0.25*/ & sector=="Government"), detail
	    qui replace avg_maturity_sov = r(mean) if _n==`ii'
		
	    qui sum maturity if (country=="`cty'" & sample_dummy==1 /*& cumshare_bond_m10_p3_nonzero!=. & bondY_m0!=. & bondY_m0<=0.25*/ & sector=="Government"), detail
	    qui replace avg_mat_sov = r(mean) if _n==`ii'

	    qui sum BA_spread if (country=="`cty'" & sample_dummy==1 /*& cumshare_bond_m10_p3_nonzero!=. & bondY_m0!=. & bondY_m0<=0.25*/ & sector=="Government"), detail
	    qui replace avg_liquidity_sov = r(mean) if _n==`ii'

	}

	local Nbonds_avg = `Nbonds_avg' / `ii'
	local avgYTM_avg = `avgYTM_avg' / `ii'
	local avgMat_avg = `avgMat_avg' / `ii'
    local avgMat1_avg = `avgMat1_avg' / `ii'
	local avgLiq_avg = `avgLiq_avg' / `ii'


	file write temp_table "\hline " _n
	file write temp_table "Average &" (substr(string(`Nbonds_avg',"%5.0f"),1,4)) "&" (substr(string(`avgYTM_avg',"%9.1f"),1,5)) "\%" "&" (substr(string(`avgMat_avg',"%9.5f"),1,4)) "&" /*(substr(string(`avgMat1_avg',"%5.0f"),1,5)) "&"*/ (substr(string(`avgLiq_avg',"%9.2f"),1,5)) "\% \\" _n

	file write temp_table "\hline \hline" _n "\end{tabular}"
	file close temp_table

	local Nbonds_avg5 = `Nbonds_avg5' / `i5'
	local avgYTM_avg5 = `avgYTM_avg5' / `i5'
	local avgMat_avg5 = `avgMat_avg5' / `i5'
    local avgMat1_avg5 = `avgMat1_avg5' / `i5'
	local avgLiq_avg5 = `avgLiq_avg5' / `i5'


	file write temp_table5 "\hline " _n
	file write temp_table5 "Average &" (substr(string(`Nbonds_avg5',"%5.0f"),1,4)) "&" (substr(string(`avgYTM_avg5',"%9.1f"),1,5)) "\%" "&" (substr(string(`avgMat_avg5',"%9.5f"),1,4)) "&" /*(substr(string(`avgMat1_avg5',"%5.0f"),1,5)) "&"*/ (substr(string(`avgLiq_avg5',"%9.2f"),1,5)) "\% \\" _n

	file write temp_table5 "\hline \hline" _n "\end{tabular}"
	file close temp_table5

	mkmat number_bonds avg_yield avg_maturity avg_mat avg_liquidity if (number_bonds & _n<50), matrix(table_all_bonds)
	mkmat number_bonds_sov avg_yield_sov avg_maturity_sov avg_mat_sov avg_liquidity_sov if (number_bonds & _n<50), matrix(table_sov_bonds)

	putexcel set "$stoutput/baseline/tables_descriptive_statistics", sheet(DES_bonds_all) modify
	putexcel B2 = matrix(table_all_bonds)

	putexcel set "$stoutput/baseline/tables_descriptive_statistics", sheet(DES_bonds_svg) modify
	putexcel B2 = matrix(table_sov_bonds)
}

// ****** TABLE B2: Shares by Sector of Bonds Included in the Empirical Analysis ****** //
if `do_table_sector_stats' == 1 {
	use "$stdata/beneficiary/regression_variables_prepared.dta", clear

	local sector_list = "Government Industrial Financial Utilities Communications Energy"

	local list_covariates = "bond_issuance_mm bond_amount_mm issuance_resid bamount_resid resid_maturity BA_spread"

	count if sample_dummy==1
	local denom_val = r(N)

	tempname temp_table
	file open temp_table using "$stoutput/baseline/EM_bonds_sectors.tex", write replace
	file write temp_table "\begin{tabular}{lccccc}" _n "\hline \hline" _n
	file write temp_table " & & Share & YTM & Residual Maturity & Bid-Ask Spread \\  " _n
	file write temp_table "   \hline " _n

	local share_other = 100

	local shareb_avg = 0
	local avgYTM_avg = 0
	local avgMat_avg = 0
	local avgMat1_avg = 0
	local avgLiq_avg = 0

	local ii = 0

	qui gen dum_tmp_sector = 0

	foreach sec_name of local sector_list {

		local ii = `ii' + 1

		qui replace dum_tmp_sector = 1 if sector == "`sec_name'"

		/*qui*/ sum sector_dummy if ( sample_dummy==1 & sector == "`sec_name'")
		local share_`sec_name' = (r(N) / `denom_val')*100
		local share_other = `share_other' - `share_`sec_name''

		/*qui*/ sum bondY_m0 if (sector == "`sec_name'" & sample_dummy==1), d
		local avgYTM_`sec_name' = r(mean)*100

	    /*qui*/ sum resid_maturity if (sector == "`sec_name'" & sample_dummy==1), d
	    local avgMat_`sec_name' = r(mean)/365
		
		 /*qui*/ sum maturity if (sector == "`sec_name'" & sample_dummy==1), d
	    local avgMat1_`sec_name' = r(mean)

	    /*qui*/ sum BA_spread if (sector == "`sec_name'" & sample_dummy==1), d
	    local avgLiq_`sec_name' = r(mean)*100

		if "`sec_name'" == "Government" {
		  file write temp_table "A. Sector &" (substr("`sec_name'",1,15)) "&" (substr(string(`share_`sec_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`sec_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`sec_name'',"%9.5f"),1,4)) /* "&" (substr(string(`avgMat1_`sec_name'',"%5.0f"),1,5))*/ "&" (substr(string(`avgLiq_`sec_name'',"%9.2f"),1,5)) "\% \\" _n
        }
		else {
		  file write temp_table "&" (substr("`sec_name'",1,15)) "&" (substr(string(`share_`sec_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`sec_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`sec_name'',"%9.5f"),1,4)) /* "&" (substr(string(`avgMat1_`sec_name'',"%5.0f"),1,5))*/ "&" (substr(string(`avgLiq_`sec_name'',"%9.2f"),1,5)) "\% \\" _n
		}
		local shareb_avg = `shareb_avg' + `share_`sec_name''
		local avgYTM_avg = `avgYTM_avg' + `avgYTM_`sec_name''
		local avgMat_avg = `avgMat_avg' + `avgMat_`sec_name''
		local avgMat1_avg = `avgMat1_avg' + `avgMat1_`sec_name''
		local avgLiq_avg = `avgLiq_avg' + `avgLiq_`sec_name''

	}

	local share_other = `share_other'

	qui sum bondY_m0 if (sample_dummy==1 & dum_tmp_sector==0), d
	local avgYTM_others = r(mean)*100

	qui sum resid_maturity if (sample_dummy==1 & dum_tmp_sector==0), d
	local residmat_others = r(mean)/365
	
    qui sum maturity if (sample_dummy==1 & dum_tmp_sector==0), d
	local mat_others = r(mean)/365

	qui sum BA_spread if (sample_dummy==1 & dum_tmp_sector==0), d
	local BAspread_others = r(mean)*100
	
	local shareb_avg = `shareb_avg' + `share_other'
    local avgYTM_avg = `avgYTM_avg' + `avgYTM_others'
	local avgMat_avg = `avgMat_avg' + `mat_others'
	local avgLiq_avg = `avgLiq_avg' + `BAspread_others'
	local ii = `ii' + 1

	local shareb_avg = `shareb_avg' / `ii'
	local avgYTM_avg = `avgYTM_avg' / `ii'
	local avgMat_avg = `avgMat_avg' / `ii'
	local avgLiq_avg = `avgLiq_avg' / `ii'
	
	file write temp_table "& Other & " (substr(string(`share_other',"%9.1f"),1,4)) "\% &" (substr(string(`avgYTM_others',"%9.1f"),1,5)) "\%" "&" (substr(string(`residmat_others',"%9.5f"),1,4)) /*"&" (substr(string(`mat_others',"%5.0f"),1,5))*/ "&" (substr(string(`BAspread_others',"%9.2f"),1,5)) "\% \\" _n

	file write temp_table "\hline " _n
	file write temp_table "& Average &" (substr(string(`shareb_avg',"%9.1f"),1,4)) "\% &" (substr(string(`avgYTM_avg',"%9.1f"),1,5)) "\%" "&" (substr(string(`avgMat_avg',"%9.5f"),1,4)) /*"&" (substr(string(`avgMat1_avg',"%5.0f"),1,5))*/ "&" (substr(string(`avgLiq_avg',"%9.2f"),1,5)) "\% \\" _n
    file write temp_table "\hline" _n
	
	local cac_list = "Y N NA"
	
	local ii = 0
	
	local shareb_avg = 0
	local avgYTM_avg = 0
	local avgMat_avg = 0
	local avgMat1_avg = 0
	local avgLiq_avg = 0
	
	qui gen dum_tmp_cac = 0
	
	foreach cac_name of local cac_list {

		local ii = `ii' + 1

		qui replace dum_tmp_cac = 1 if cac == "`cac_name'"

		/*qui*/ sum cac1 if ( sample_dummy==1 & cac == "`cac_name'")
		local share_`cac_name' = (r(N) / `denom_val')*100

		/*qui*/ sum bondY_m0 if (cac == "`cac_name'" & sample_dummy==1), d
		local avgYTM_`cac_name' = r(mean)*100

	    /*qui*/ sum resid_maturity if (cac == "`cac_name'" & sample_dummy==1), d
	    local avgMat_`cac_name' = r(mean)/365
		
	    /*qui*/ sum BA_spread if (cac == "`cac_name'" & sample_dummy==1), d
	    local avgLiq_`cac_name' = r(mean)*100
        if "`cac_name'" == "Y" { 
		   file write temp_table "B. CAC & Yes &" (substr(string(`share_`cac_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`cac_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`cac_name'',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_`cac_name'',"%9.2f"),1,5)) "\% \\" _n
        }
		if "`cac_name'" == "N" { 
		   file write temp_table " & No &" (substr(string(`share_`cac_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`cac_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`cac_name'',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_`cac_name'',"%9.2f"),1,5)) "\% \\" _n
        }
		if "`cac_name'" == "NA" { 
		   file write temp_table " & NA &" (substr(string(`share_`cac_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`cac_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`cac_name'',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_`cac_name'',"%9.2f"),1,5)) "\% \\" _n
        }
		
		local shareb_avg = `shareb_avg' + `share_`cac_name''
		local avgYTM_avg = `avgYTM_avg' + `avgYTM_`cac_name''
		local avgMat_avg = `avgMat_avg' + `avgMat_`cac_name''
		local avgLiq_avg = `avgLiq_avg' + `avgLiq_`cac_name''

	}

	local shareb_avg = `shareb_avg' / `ii'
	local avgYTM_avg = `avgYTM_avg' / `ii'
	local avgMat_avg = `avgMat_avg' / `ii'
	local avgLiq_avg = `avgLiq_avg' / `ii'
	
	file write temp_table "\hline " _n
	file write temp_table "& Average &" (substr(string(`shareb_avg',"%9.1f"),1,4)) "\% &" (substr(string(`avgYTM_avg',"%9.1f"),1,5)) "\%" "&" (substr(string(`avgMat_avg',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_avg',"%9.2f"),1,5)) "\% \\" _n
    file write temp_table "\hline " _n
	
	replace payment_rank = "first_lien" if payment_rank == "1st lien"
	replace payment_rank = "second_lien" if payment_rank == "2nd lien"
	replace payment_rank = "jr_sub" if payment_rank == "Jr Subordinated"
	replace payment_rank = "sr_sub" if payment_rank == "Sr Subordinated"
	replace payment_rank = "sr_unsecured" if payment_rank == "Sr Unsecured"
    local payment_list = "first_lien second_lien Secured sr_unsecured Unsecured sr_sub Subordinated jr_sub"
	
    local ii = 0
	
	local shareb_avg = 0
	local avgYTM_avg = 0
	local avgMat_avg = 0
	local avgMat1_avg = 0
	local avgLiq_avg = 0

	qui gen dum_tmp_payment = 0

	foreach payment_name of local payment_list {

		local ii = `ii' + 1

		qui replace dum_tmp_payment = 1 if payment_rank == "`payment_name'"

		/*qui*/ sum payment_rank1 if ( sample_dummy==1 & payment_rank == "`payment_name'")
		local share_`payment_name' = (r(N) / `denom_val')*100

		/*qui*/ sum bondY_m0 if (payment_rank == "`payment_name'" & sample_dummy==1), d
		local avgYTM_`payment_name' = r(mean)*100

	    /*qui*/ sum resid_maturity if (payment_rank == "`payment_name'" & sample_dummy==1), d
	    local avgMat_`payment_name' = r(mean)/365
		
	    /*qui*/ sum BA_spread if (payment_rank == "`payment_name'" & sample_dummy==1), d
	    local avgLiq_`payment_name' = r(mean)*100
        
		if "`payment_name'" == "first_lien" {
		   file write temp_table "C. Seniority & 1st Lien &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`payment_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`payment_name'',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_`payment_name'',"%9.2f"),1,5)) "\% \\" _n
        }
		else if "`payment_name'" == "second_lien" {
		   file write temp_table " & 2nd Lien &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`payment_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`payment_name'',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_`payment_name'',"%9.2f"),1,5)) "\% \\" _n
        }
		else if "`payment_name'" == "sr_unsecured" {
		   file write temp_table " & Senior Unsecured &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`payment_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`payment_name'',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_`payment_name'',"%9.2f"),1,5)) "\% \\" _n
        }
		else if "`payment_name'" == "sr_sub" {
		   file write temp_table " & Senior Subordinated &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`payment_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`payment_name'',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_`payment_name'',"%9.2f"),1,5)) "\% \\" _n
        }
		else if "`payment_name'" == "jr_sub" {
		   file write temp_table " & Junior Subordinated &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`payment_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`payment_name'',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_`payment_name'',"%9.2f"),1,5)) "\% \\" _n
        }
		else {
		   file write temp_table "&" (substr("`payment_name'",1,15)) "&" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`avgYTM_`payment_name'',"%9.1f"),1,5)) "\% & " (substr(string(`avgMat_`payment_name'',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_`payment_name'',"%9.2f"),1,5)) "\% \\" _n
		}
		
		local shareb_avg = `shareb_avg' + `share_`payment_name''
		local avgYTM_avg = `avgYTM_avg' + `avgYTM_`payment_name''
		local avgMat_avg = `avgMat_avg' + `avgMat_`payment_name''
		local avgLiq_avg = `avgLiq_avg' + `avgLiq_`payment_name''

	}

	local shareb_avg = `shareb_avg' / `ii'
	local avgYTM_avg = `avgYTM_avg' / `ii'
	local avgMat_avg = `avgMat_avg' / `ii'
	local avgLiq_avg = `avgLiq_avg' / `ii'

	file write temp_table "\hline " _n
	file write temp_table "& Average &" (substr(string(`shareb_avg',"%9.1f"),1,4)) "\% &" (substr(string(`avgYTM_avg',"%9.1f"),1,5)) "\%" "&" (substr(string(`avgMat_avg',"%9.5f"),1,4)) "&" (substr(string(`avgLiq_avg',"%9.2f"),1,5)) "\% \\" _n

	file write temp_table "\hline \hline" _n "\end{tabular}"
	file close temp_table
}

// ****** TABLE B4: Financial Institutions Included in the Empirical Analysis ****** //
if `do_table_banksnames' == 1 {


	use "$stdata/beneficiary/banks_names_sample_dummy.dta", clear
	keep if sample_dummy_bank==1
	count
	local num_banks = r(N)
	local num_low_row = 23
	local low_row_last = `num_banks' - `num_low_row'*2

	qui replace lender_name_cr = "M\&T Bank" if lender_name_cr == "M&T Bank"
	qui replace lender_name_cr = "Bb\&T" if lender_name_cr == "Bb&T"
	qui replace lender_name_cr = "Societe Generale" if lender_name_cr == "SocieTe GeNeRale"
	qui replace lender_name_cr = "Allstate Corp" if lender_name_cr == "Allstate Corp/The"
	qui replace lender_name_cr = "Huntington Bancshares" if lender_name_cr == "Huntington Bancshares Inc/OH"
	qui replace lender_name_cr = "Franklin Resources" if lender_name_cr == "franklin resources stock"
	qui replace lender_name_cr = "Loomis Sayles" if lender_name_cr == "loomis sayles"
	qui replace lender_name_cr = "Nomura Holdings" if lender_name_cr == "nomura holdings"

	replace lender_name_cr = subinstr(lender_name_cr, " Inc", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " SpA", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " Corp", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " SA", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " Ltd", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " PLC", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, "Group,", "Group", .)

	sort lender_name_cr
	tempname temp_table
	file open temp_table using "$stoutput/baseline/banks_names.tex", write replace
	file write temp_table "\begin{tabular}{llll}" _n "\hline \hline" _n

	forvalues ii=1(1)`num_low_row'{
		if `ii'<=`low_row_last'{
			local ii2 = `ii'+`num_low_row'
			local ii3 = `ii'+`num_low_row'*2
			local bnk_nm1 = lender_name_cr[`ii']
			local bnk_nm2 = lender_name_cr[`ii2']
			local bnk_nm3 = lender_name_cr[`ii3']
			file write temp_table "`bnk_nm1' & `bnk_nm2' & `bnk_nm3' \\"
		}
		else {
			local ii2 = `ii'+`num_low_row'
			local bnk_nm1 = lender_name_cr[`ii']
			local bnk_nm2 = lender_name_cr[`ii2']
			file write temp_table "`bnk_nm1' & `bnk_nm2' &  \\"
		}

	}


	file write temp_table "\hline \hline" _n "\end{tabular}"
	file close temp_table


}

// ****** Figure B1: Sorting of Financial Institutions into Countries ****** //
if `do_figure_sorting_country'==1 {

	use "$stdata/beneficiary/regression_variables_prepared.dta", clear

	keep if sample_dummy==1
	qui sum dps_m`hpre_chosen'_p`hpost_fixed'_bs if sample_dummy==1, detail
	local med_NW = r(p50)
	local avg_NW = r(mean)
	qui gen index_NW_mean   = ""
	qui replace index_NW_mean   = "blw" if dps_m`hpre_chosen'_p`hpost_fixed'_bs < `avg_NW'
	qui replace index_NW_mean   = "abv" if dps_m`hpre_chosen'_p`hpost_fixed'_bs >= `avg_NW'
	gen index_NW_mean_num = 0 if index_NW_mean   == "blw"
	replace index_NW_mean_num = 1 if index_NW_mean   == "abv"

	fcollapse (firstnm) country_long country index_NW_mean (count) bondY_m0, by(country_num index_NW_mean_num)
	rename bondY_m0 numbonds
	keep numbonds country_long index_NW_mean
	reshape wide numbonds, i(country_long) j(index_NW_mean) s
	replace country_long= "Bulgaria" if country_long==""
	replace numbondsabv = 0 if numbondsabv==.
	replace numbondsblw = 0 if numbondsblw==.
	gen numbonds = numbondsabv + numbondsblw

	foreach var of varlist numbonds* {
		sum `var'
		local tot_`var' = r(sum)
		gen `var'_share = `var'/`tot_`var''
	}

	gen aux = -numbonds_share
	sort aux
	graph hbar numbondsblw_share  numbondsabv_share if numbonds_share>0.01, /*
	*/ over(country_long, label(labsize(small)) sort(numbonds_share) rev)  leg(lab(1 "{&Delta}e < Avg {&Delta}e") lab(2 "{&Delta}e > Avg {&Delta}e") pos(5) ring(0) bplace(seast)) ylabel(,labsize(small) format(%9.0gc)) /*
	*/ bargap(10) bar(1, color(eltblue)) bar(2, color(gs4)) graphregion(color(white)) plotregion(color(white))
	graph export "$stoutput/baseline/sorting_bycountry.pdf", replace
}

// ****** TABLE B5: Descriptive Statistics by Financial Institution ****** //
if `do_table_banksstats' == 1{

	use "$stdata/beneficiary/lender_holdings_append.dta", clear
	append using "$stdata/beneficiary/lender_holdings_append_ext.dta"
	append using "$stdata/beneficiary/lender_holdings_append_ext2010.dta"
	
	append using "$stdata/bank_names.dta"

	levelsof lender_name_cr, local(lender_name_list)
	foreach cr_name of local lender_name_list {
		tempvar tv1
		gene `tv1'=_n if lender_name_cr=="`cr_name'"
		summ `tv1', meanonly
		local index=r(min)
		di `index'
		local cr_k1   =lender_keyword1[`index']
		local cr_k2   =lender_keyword2[`index']
		local cr_k3   =lender_keyword3[`index']
		drop `tv1'
		di "`cr_name' `cr_k1' `cr_k2' `cr_k3'"
		replace lender_name_cr = "`cr_name'" if strpos(lender_name,"`cr_name'")>0
		replace lender_name_cr = "`cr_name'" if (strpos(lower(lender_name),"`cr_k1'")>0 & "`cr_k1'"!="") & ("`cr_k2'"=="" | (strpos(lower(lender_name),"`cr_k2'")>0 & "`cr_k2'"!=""))
		replace lender_name_cr = "`cr_name'" if strpos(lower(lender_name),"`cr_k3'")>0 & "`cr_k3'"!=""
	}

	qui replace lender_name_cr = "M\&T Bank" if lender_name_cr == "M&T Bank"
	qui replace lender_name_cr = "Bb\&T" if lender_name_cr == "Bb&T"
	qui replace lender_name_cr = "Societe Generale" if lender_name_cr == "SocieTe GeNeRale"
	qui replace lender_name_cr = "Allstate Corp" if lender_name_cr == "Allstate Corp/The"
	qui replace lender_name_cr = "Huntington Bancshares" if lender_name_cr == "Huntington Bancshares Inc/OH"
	qui replace lender_name_cr = "Franklin Resources" if lender_name_cr == "franklin resources stock"
	qui replace lender_name_cr = "Loomis Sayles" if lender_name_cr == "loomis sayles"
	qui replace lender_name_cr = "Nomura Holdings" if lender_name_cr == "nomura holdings"

	replace lender_name_cr = subinstr(lender_name_cr, " Inc", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " SpA", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " Corp", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " SA", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " Ltd", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, " PLC", "", .)
	replace lender_name_cr = subinstr(lender_name_cr, "Group,", "Group", .)

	drop if bond_id == ""
	drop lender_keyword*

	merge m:1 lender_name_cr using "$stdata/stocks_deltap.dta"
	keep if _merge==3
	drop _merge

	merge m:1 lender_name_cr using "$stdata/beneficiary/banks_names_sample_dummy.dta", keepusing(sample_dummy_bank)
	keep if sample_dummy_bank==1

	bysort bond_id: egen bond_amount_reported = sum(lender_amount)
	gen share_lender_reported = lender_amount/bond_amount_reported*100

	bysort lender_name_cr country: gen filter_cty = _n==1 
	bysort lender_name_cr: gen Nbonds = _N
	bysort lender_name_cr: egen Ncties = sum(filter_cty)
	bysort lender_name_cr: egen avgcvg = mean(share_lender_reported)
	bysort lender_name_cr: egen avgDNW = mean(dps_m10_p3)


	collapse (firstnm) Nbonds Ncties avgcvg avgDNW, by(lender_name_cr)

	gsort -Nbonds

	local max_rows  = 20

	preserve
	qui gen cumN = _n
	keep if cumN <= `max_rows'
	save "$stdata/list_top_20_banks.dta", replace
	restore

	tempname temp_table
	file open temp_table using "$stoutput/baseline/table_top_20_banks.tex", write replace
	file write temp_table "\begin{tabular}{lcccc}" _n "\hline \hline" _n
	file write temp_table "{Financial Institution} &	{N Bonds} &	{N Countries} &	{Avg Share} &	{$\Delta e_i$} \\" _n
	file write temp_table "\hline" _n

	qui gen cum_N = _n
	qui sum Nbonds if cum_N<=`max_rows'
	local Nbonds_mean = r(mean)

	qui sum Ncties if cum_N<=`max_rows'
	local Ncties_mean = r(mean)

	qui sum avgcvg if cum_N<=`max_rows'
	local avgcvg_mean = r(mean)

	qui sum avgDNW if cum_N<=`max_rows'
	local avgDNW_mean = r(mean)



	forvalues ii=1(1)`max_rows'{
		local bnknm_v = lender_name_cr[`ii']
		local Nbnds_v = Nbonds[`ii']
		local Nctis_v = Ncties[`ii']
		local avgvg_v = avgcvg[`ii']
		local avgNW_v = avgDNW[`ii']

		file write temp_table "`bnknm_v' & " (substr(string(`Nbnds_v',"%5.0f"),1,4)) "&" (substr(string(`Nctis_v',"%5.0f"),1,4)) "&" (substr(string(`avgvg_v',"%9.1f"),1,4)) "\% &" (substr(string(`avgNW_v',"%9.2f"),1,5)) "\\" _n
	}
	file write temp_table "\hline" _n
	file write temp_table "Average & " (substr(string(`Nbonds_mean',"%5.0f"),1,4)) " & " (substr(string(`Ncties_mean',"%5.0f"),1,4)) " & " (substr(string(`avgcvg_mean',"%9.1f"),1,4)) "\% & " (substr(string(`avgDNW_mean',"%9.2f"),1,5)) " \\ " _n
	file write temp_table "\hline \hline" _n "\end{tabular}"
	file close temp_table

}

// ****** TABLE B6: Sorting of Financial Institutions into Sectors ****** //
if `do_table_sorting_sector'==1 {

	use "$stdata/beneficiary/regression_variables_prepared.dta", clear

	qui sum dps_m`hpre_chosen'_p`hpost_fixed'_bs if sample_dummy==1, detail
	local med_NW = r(p50)
	local avg_NW = r(mean)
	qui gen index_NW_mean   = ""
	qui replace index_NW_mean   = "blw" if dps_m`hpre_chosen'_p`hpost_fixed'_bs < `avg_NW'
	qui replace index_NW_mean   = "abv" if dps_m`hpre_chosen'_p`hpost_fixed'_bs >= `avg_NW'

	local pst_moment      = "blw abv"
	local sector_list     = "Government Industrial Financial Utilities Communications Energy"
    local cac_list        = "Y N NA" 
	replace payment_rank = "first_lien" if payment_rank == "1st lien"
	replace payment_rank = "second_lien" if payment_rank == "2nd lien"
	replace payment_rank = "jr_sub" if payment_rank == "Jr Subordinated"
	replace payment_rank = "sr_sub" if payment_rank == "Sr Subordinated"
	replace payment_rank = "sr_unsecured" if payment_rank == "Sr Unsecured"
	local payment_list = "first_lien second_lien Secured sr_unsecured Unsecured sr_sub Subordinated jr_sub"
	
	count if sample_dummy==1
	local n_total     = r(N)
	count if sample_dummy==1 & index_NW_mean=="blw"
	local n_total_blw = r(N)
	count if sample_dummy==1 & index_NW_mean=="abv"
	local n_total_abv = r(N)

	tempname temp_table
	file open temp_table using "$stoutput/baseline/EM_bonds_sector_sorting.tex", write replace
	file write temp_table "\begin{tabular}{lcccc}" _n "\hline \hline" _n
	file write temp_table " & & All bonds & $\Delta e_i  < \Delta \overline{e}_i $ & $\Delta e_i  > \Delta \overline{e}_i $  \\  " _n
	file write temp_table "   \hline " _n

	local cumshare     = 0
	local cumshare_blw = 0
	local cumshare_abv = 0

	foreach sec_name of local sector_list {

		count if sample_dummy==1 & sector=="`sec_name'"
		local n_`sec_name' = r(N)
		local share_`sec_name' = `n_`sec_name''/`n_total'*100

		count if sample_dummy==1 & sector=="`sec_name'" & index_NW_mean=="blw"
		local n_`sec_name'_blw = r(N)
		local share_`sec_name'_blw = `n_`sec_name'_blw'/`n_total_blw'*100

		count if sample_dummy==1 & sector=="`sec_name'" & index_NW_mean=="abv"
		local n_`sec_name'_abv = r(N)
		local share_`sec_name'_abv = `n_`sec_name'_abv'/`n_total_abv'*100
		
		if "`sec_name'" == "Government" {
          file write temp_table "A. Sector & Government &" (substr(string(`share_`sec_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`sec_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`sec_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
		else {
		  file write temp_table " &" (substr("`sec_name'",1,15)) "&" (substr(string(`share_`sec_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`sec_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`sec_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
		}
		
		local cumshare     = `cumshare' + `share_`sec_name''
		local cumshare_blw = `cumshare_blw' + `share_`sec_name'_blw'
		local cumshare_abv = `cumshare_abv' + `share_`sec_name'_abv'

	}

	local share_other     = 100 - `cumshare'
	local share_other_blw = 100 - `cumshare_blw'
	local share_other_abv = 100 - `cumshare_abv'

	file write temp_table "& Other & " (substr(string(`share_other',"%9.1f"),1,4)) "\% & " (substr(string(`share_other_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_other_abv',"%9.1f"),1,5)) "\% \\ "_n
    file write temp_table "\hline " _n
	
	local cumshare     = 0
	local cumshare_blw = 0
	local cumshare_abv = 0

	foreach cac_name of local cac_list {

		count if sample_dummy==1 & cac=="`cac_name'"
		local n_`cac_name' = r(N)
		local share_`cac_name' = `n_`cac_name''/`n_total'*100

		count if sample_dummy==1 & cac=="`cac_name'" & index_NW_mean=="blw"
		local n_`cac_name'_blw = r(N)
		local share_`cac_name'_blw = `n_`cac_name'_blw'/`n_total_blw'*100

		count if sample_dummy==1 & cac=="`cac_name'" & index_NW_mean=="abv"
		local n_`cac_name'_abv = r(N)
		local share_`cac_name'_abv = `n_`cac_name'_abv'/`n_total_abv'*100
		
		if "`cac_name'" == "Y" {
          file write temp_table "B. CAC & Yes &" (substr(string(`share_`cac_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`cac_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`cac_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
	    if "`cac_name'" == "N" {
          file write temp_table " & No &" (substr(string(`share_`cac_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`cac_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`cac_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
		if "`cac_name'" == "NA" {
          file write temp_table " & NA &" (substr(string(`share_`cac_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`cac_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`cac_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
		
		local cumshare     = `cumshare' + `share_`cac_name''
		local cumshare_blw = `cumshare_blw' + `share_`cac_name'_blw'
		local cumshare_abv = `cumshare_abv' + `share_`cac_name'_abv'

	}
	file write temp_table "\hline " _n

	local cumshare     = 0
	local cumshare_blw = 0
	local cumshare_abv = 0

	foreach payment_name of local payment_list {

		count if sample_dummy==1 & payment_rank =="`payment_name'"
		local n_`payment_name' = r(N)
		local share_`payment_name' = `n_`payment_name''/`n_total'*100

		count if sample_dummy==1 & payment_rank=="`payment_name'" & index_NW_mean=="blw"
		local n_`payment_name'_blw = r(N)
		local share_`payment_name'_blw = `n_`payment_name'_blw'/`n_total_blw'*100

		count if sample_dummy==1 & payment_rank=="`payment_name'" & index_NW_mean=="abv"
		local n_`payment_name'_abv = r(N)
		local share_`payment_name'_abv = `n_`payment_name'_abv'/`n_total_abv'*100
        
		if "`payment_name'" == "first_lien" {
		   file write temp_table "C. Seniority & 1st Lien &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`payment_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`payment_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
	    else if "`payment_name'" == "second_lien" {
		   file write temp_table "& 2nd Lien &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`payment_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`payment_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
		else if "`payment_name'" == "sr_unsecured" {
		   file write temp_table "& Senior Unsecured &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`payment_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`payment_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
		else if "`payment_name'" == "sr_sub" {
		   file write temp_table "& Senior Subordinated &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`payment_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`payment_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
		else if "`payment_name'" == "jr_sub" {
		   file write temp_table "& Junior Subordinated &" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`payment_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`payment_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
	    else {
		   file write temp_table "&" (substr("`payment_name'",1,15)) "&" (substr(string(`share_`payment_name'',"%9.1f"),1,4)) "\% & " (substr(string(`share_`payment_name'_blw',"%9.1f"),1,5)) "\% & " (substr(string(`share_`payment_name'_abv',"%9.1f"),1,5)) "\% \\ "_n
        }
		
		local cumshare     = `cumshare' + `share_`payment_name''
		local cumshare_blw = `cumshare_blw' + `share_`payment_name'_blw'
		local cumshare_abv = `cumshare_abv' + `share_`payment_name'_abv'

	}
	
	file write temp_table "\hline \hline" _n "\end{tabular}"
	file close temp_table
}

// ****** TABLE B7: EM Bonds' Characteristic by Holders’ Change in Net Worth ****** //

if `do_table_sorting_covariates' == 1{

	use "$stdata/beneficiary/regression_variables_prepared.dta", clear

	local pst_moment      = "blw abv"
	local list_covariates = "resid_maturity BA_spread bondY_m0 maturity_FE BAspread_FE YTM_FE issuance_FE bamount_FE ln_bnissnce ln_bnamount"

	qui gen cum_n_tmp = _n


	*** CONTROL FOR FIXED EFFECTS
	qui gen ln_bnissnce = ln(AMT_ISSUED)
	qui gen ln_bnamount = ln(bond_amount)
	qui gen ln_residmat = ln(resid_maturity)


	qui reg ln_bnissnce i.countrysector if sample_dummy==1
	predict double issuance_FE if sample_dummy==1, residuals
	qui reg ln_bnamount i.countrysector if sample_dummy==1
	predict double bamount_FE if sample_dummy==1, residuals

	qui reg resid_maturity i.countrysector if sample_dummy==1
	predict double maturity_FE if sample_dummy==1, residuals
	qui reg BA_spread i.countrysector if sample_dummy==1
	predict double BAspread_FE if sample_dummy==1, residuals
	qui reg bondY_m0 i.countrysector if sample_dummy==1
	predict double YTM_FE if sample_dummy==1, residuals

	qui reg ln_bnissnce i.countrysector if sample_dummy==1 & sector=="Government"
	predict double issuance_FE_sov if sample_dummy==1 & sector=="Government", residuals
	qui reg ln_bnamount i.countrysector if sample_dummy==1 & sector=="Government"
	predict double bamount_FE_sov if sample_dummy==1 & sector=="Government", residuals

	qui reg resid_maturity i.countrysector if sample_dummy==1 & sector=="Government"
	predict double maturity_FE_sov if sample_dummy==1 & sector=="Government", residuals
	qui reg BA_spread i.countrysector if sample_dummy==1 & sector=="Government"
	predict double BAspread_FE_sov if sample_dummy==1 & sector=="Government", residuals
	qui reg bondY_m0 i.countrysector if sample_dummy==1 & sector=="Government"
	predict double YTM_FE_sov if sample_dummy==1 & sector=="Government", residuals

	*** Tables: Value of Covariates for change in net worth below/above moment (i.e. median and mean)
	qui sum dps_m`hpre_chosen'_p`hpost_fixed'_bs if sample_dummy==1, detail
	local med_NW = r(p50)
	local avg_NW = r(mean)
	qui gen index_NW_mean   = ""
	qui replace index_NW_mean   = "blw" if dps_m`hpre_chosen'_p`hpost_fixed'_bs < `avg_NW'
	qui replace index_NW_mean   = "abv" if dps_m`hpre_chosen'_p`hpost_fixed'_bs >= `avg_NW'

	foreach covar_name of local list_covariates {

		qui sum `covar_name' if ( sample_dummy==1 & index_NW_mean == "blw" )
		local `covar_name'_avg_blw = r(mean)
		local `covar_name'_se_blw = r(sd)/sqrt(r(N))

		qui sum `covar_name' if ( sample_dummy==1 & index_NW_mean == "abv" )
		local `covar_name'_avg_abv = r(mean)
		local `covar_name'_se_abv = r(sd)/sqrt(r(N))

		qui sum `covar_name' if ( sample_dummy==1 & sector=="Government" & index_NW_mean == "blw" )
		local `covar_name'_avg_blw_sov = r(mean)
		local `covar_name'_se_blw_sov = r(sd)/sqrt(r(N))

		qui sum `covar_name' if ( sample_dummy==1 & sector=="Government" & index_NW_mean == "abv" )
		local `covar_name'_avg_abv_sov = r(mean)
		local `covar_name'_se_abv_sov = r(sd)/sqrt(r(N))

	}

	local BA_spread_avg_blw = `BA_spread_avg_blw'*100
	local BA_spread_se_blw  = `BA_spread_se_blw'*100
	local BA_spread_avg_abv = `BA_spread_avg_abv'*100
	local BA_spread_se_abv  = `BA_spread_se_abv'*100

	local bondY_m0_avg_blw = `bondY_m0_avg_blw'*100
	local bondY_m0_se_blw  = `bondY_m0_se_blw'*100
	local bondY_m0_avg_abv = `bondY_m0_avg_abv'*100
	local bondY_m0_se_abv  = `bondY_m0_se_abv'*100

	local BAspread_FE_avg_blw = `BAspread_FE_avg_blw'*100
	local BAspread_FE_se_blw  = `BAspread_FE_se_blw'*100
	local BAspread_FE_avg_abv = `BAspread_FE_avg_abv'*100
	local BAspread_FE_se_abv  = `BAspread_FE_se_abv'*100

	local YTM_FE_avg_blw = `YTM_FE_avg_blw'*100
	local YTM_FE_se_blw  = `YTM_FE_se_blw'*100
	local YTM_FE_avg_abv = `YTM_FE_avg_abv'*100
	local YTM_FE_se_abv  = `YTM_FE_se_abv'*100

	local BA_spread_avg_blw_sov = `BA_spread_avg_blw_sov'*100
	local BA_spread_se_blw_sov  = `BA_spread_se_blw_sov'*100
	local BA_spread_avg_abv_sov = `BA_spread_avg_abv_sov'*100
	local BA_spread_se_abv_sov  = `BA_spread_se_abv_sov'*100

	local bondY_m0_avg_blw_sov = `bondY_m0_avg_blw_sov'*100
	local bondY_m0_se_blw_sov  = `bondY_m0_se_blw_sov'*100
	local bondY_m0_avg_abv_sov = `bondY_m0_avg_abv_sov'*100
	local bondY_m0_se_abv_sov  = `bondY_m0_se_abv_sov'*100

	local BAspread_FE_avg_blw_sov = `BAspread_FE_avg_blw_sov'*100
	local BAspread_FE_se_blw_sov  = `BAspread_FE_se_blw_sov'*100
	local BAspread_FE_avg_abv_sov = `BAspread_FE_avg_abv_sov'*100
	local BAspread_FE_se_abv_sov  = `BAspread_FE_se_abv_sov'*100

	local YTM_FE_avg_blw_sov = `YTM_FE_avg_blw_sov'*100
	local YTM_FE_se_blw_sov  = `YTM_FE_se_blw_sov'*100
	local YTM_FE_avg_abv_sov = `YTM_FE_avg_abv_sov'*100
	local YTM_FE_se_abv_sov  = `YTM_FE_se_abv_sov'*100

	tempname temp_table3
	file open temp_table3 using "$stoutput/baseline/variables_balancing_table_withFE.tex", write replace
	file write temp_table3 "\begin{tabular}{lcccccccc}" _n "\hline \hline" _n
	file write temp_table3 "     &   \multicolumn{4}{c}{ No Fixed Effects }  & \multicolumn{4}{c}{ Country by Sector FE } \\" _n
	file write temp_table3 "        & & $\Delta e_i  < \Delta \overline{e}_i $ & & $\Delta e_i  > \Delta \overline{e}_i $  & & $\Delta e_i  < \Delta \overline{e}_i $ & & $\Delta e_i  > \Delta \overline{e}_i $ \\  " _n
	file write temp_table3 "   \hline " _n

	file write temp_table3 "   Residual maturity & & " (substr(string(`resid_maturity_avg_blw',"%5.0f"),1,4)) "& &" (substr(string(`resid_maturity_avg_abv',"%5.0f"),1,4))  "& &"  (substr(string(`maturity_FE_avg_blw',"%5.2f"),1,6))  "& &"  (substr(string(`maturity_FE_avg_abv',"%5.2f"),1,5))  "\\" _n
	file write temp_table3 "    & & [" (substr(string(`resid_maturity_se_blw',"%5.0f"),1,3)) "] & & [" (substr(string(`resid_maturity_se_abv',"%5.0f"),1,3))  "] & & [" (substr(string(`maturity_FE_se_blw',"%5.3f"),1,5)) "] & & [" (substr(string(`maturity_FE_se_abv',"%5.3f"),1,5)) "] \\" _n

	file write temp_table3 "   Bid-ask spread & & " (substr(string(`BA_spread_avg_blw',"%5.2f"),1,4)) "\% & &" (substr(string(`BA_spread_avg_abv',"%5.2f"),1,4)) "\% & &" (substr(string(`BAspread_FE_avg_blw',"%5.2f"),1,5)) "\% & &" (substr(string(`BAspread_FE_avg_abv',"%5.2f"),1,5)) "\% \\" _n
	file write temp_table3 "    & & [" (substr(string(`BA_spread_se_blw',"%9.2f"),1,5)) "\%] & & [" (substr(string(`BAspread_FE_se_blw',"%9.2f"),1,5)) "\%] & & [" (substr(string(`BAspread_FE_se_blw',"%9.2f"),1,5)) "\%] & & [" (substr(string(`BAspread_FE_se_abv',"%9.2f"),1,5)) "\%]\\" _n

	file write temp_table3 "   Yield (pre-Lehman) & & " (substr(string(`bondY_m0_avg_blw',"%5.2f"),1,3)) "\% & &" (substr(string(`bondY_m0_avg_abv',"%5.2f"),1,3)) "\% & &" (substr(string(`YTM_FE_avg_blw',"%5.2f"),1,5)) "\% & &" (substr(string(`YTM_FE_avg_abv',"%5.2f"),1,5)) "\% \\" _n
	file write temp_table3 "    & & [" (substr(string(`bondY_m0_se_blw',"%9.2f"),1,5)) "\%] & & [" (substr(string(`bondY_m0_se_abv',"%9.2f"),1,5)) "\%] & & [" (substr(string(`YTM_FE_se_blw',"%9.2f"),1,5)) "\%] & & [" (substr(string(`YTM_FE_se_abv',"%9.2f"),1,5)) "\%] \\" _n

	file write temp_table3 "\hline \hline" _n "\end{tabular}"
	file close temp_table3

}

// ****** TABLE B8 Regression: Regression controls onto delta NW ****** //
if `do_table_covariates_deltanreg' == 1{

	use "$stdata/beneficiary/regression_variables_prepared.dta", clear

	local pst_moment      = "blw abv"
	local list_covariates = "ln_residmat BA_spread bondY_m0 ln_bnamount"

	qui gen cum_n_tmp = _n
	qui gen ln_bnissnce = ln(AMT_ISSUED)
	qui gen ln_bnamount = ln(bond_amount)
	qui gen ln_residmat = ln(resid_maturity)


	foreach covar_name of local list_covariates {
		qui reg `covar_name' dps_m`hpre_chosen'_p`hpost_fixed'_bs  dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector if sample_dummy==1, vce(cluster country_num)
		local beta_`covar_name'_FE = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]
		local se_`covar_name'_FE = _se[dps_m`hpre_chosen'_p`hpost_fixed'_bs]

		qui reg `covar_name' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros if sample_dummy==1, vce(cluster country_num)
		local beta_`covar_name' = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]
		local se_`covar_name' = _se[dps_m`hpre_chosen'_p`hpost_fixed'_bs]

	}

	tempname temp_table1
	file open temp_table1 using "$stoutput/baseline/table_sorting_regression.tex", write replace
	file write temp_table1 "\begin{tabular}{lcccc}" _n "\hline \hline" _n
	file write temp_table1 "& & With FE & & Without FE \\" _n
	file write temp_table1 "   \hline " _n

	file write temp_table1 "   Residual maturity & & " (substr(string(`beta_ln_residmat_FE',"%5.3f"),1,5)) " & & " (substr(string(`beta_ln_residmat',"%5.3f"),1,5)) "\\" _n
	file write temp_table1 "    & & [" (substr(string(`se_ln_residmat_FE',"%5.3f"),1,5)) "] & &[" (substr(string(`se_ln_residmat',"%5.3f"),1,5)) "] \\" _n

	file write temp_table1 "   BA Spread & & " (substr(string(`beta_BA_spread_FE',"%9.3f"),1,5)) " & & " (substr(string(`beta_BA_spread',"%9.3f"),1,5)) " \\" _n
	file write temp_table1 "    & & [" (substr(string(`se_BA_spread_FE',"%9.3f"),1,5)) "] & & [" (substr(string(`se_BA_spread',"%9.3f"),1,5)) "] \\" _n

	file write temp_table1 "   YTM & & " (substr(string(`beta_bondY_m0_FE',"%9.3f"),1,5)) " & & " (substr(string(`beta_bondY_m0',"%9.3f"),1,5)) " \\" _n
	file write temp_table1 "    & & [" (substr(string(`se_bondY_m0_FE',"%9.3f"),1,5)) "] & & [" (substr(string(`se_bondY_m0',"%9.3f"),1,5)) "] \\" _n

	file write temp_table1 "\hline \hline" _n "\end{tabular}"
	file close temp_table1
	
}

***** Table B9: Stickiness of Holdings *****
if `do_table_holding_auto' == 1 {
  
  clear
  local country_list "AR BG BR CN CL CO EC HR HU ID JM LT LV MA MX MY PA PE PH PK PL RU SV TH TR UA VE ZA"
  local country_extension "CR GR IN KR KZ LB NG PK RO TN UY"
  local country_ext2010 "AR BR CL CN CO HR HU ID JM LT LV MA MX MY PA PH PK PL RU SV TH TR UA VE ZA"
  ** Import data
  foreach country of local country_list {
    append using "$stdata/beneficiary/lender_holdings_`country'_ts.dta", force
  }
  save "$stdata/beneficiary/holders.dta",replace
  clear
  
  foreach country of local country_extension {
    append using "$stdata/beneficiary/lender_holdings_`country'_ext_ts.dta", force
  }
  save "$stdata/beneficiary/holders_exten.dta",replace
  clear
  
  foreach country of local country_ext2010 {
    append using "$stdata/beneficiary/lender_holdings_`country'_ext2010_ts.dta", force
  }
  save "$stdata/beneficiary/holders_ext2010.dta",replace
  clear
  
  use "$stdata/beneficiary/holders.dta", clear
  append using "$stdata/beneficiary/holders_exten.dta",force
  append using "$stdata/beneficiary/holders_ext2010.dta",force
  bysort bond_id lender_name: gen obs = _N
  drop if CUSIP== "" & obs == 16
  drop obs

  ** Compute lender's share
  bysort bond_id date: egen tot_lending = total(lender_amount_)
  gen lender_share = lender_amount_/tot_lending
  replace lender_share = 0 if lender_share == .
  
  ** Regressions
  egen id = group(bond_id lender_name)
  egen id1 = group(country lender_name)
  bysort bond_id lender_name date: gen obs = _N
  drop if Country == "" & obs >1
  drop if Country == "MY" & country == "PA" & obs >1
  tsset id date 
  
  reg lender_share L.lender_share, r
  local prev_share_1 = _b[L.lender_share]
  local se_prev_1 = _se[L.lender_share]
  local r2_1 = `e(r2)'
  local nobs_1 = `e(N)'
  
  areg lender_share L.lender_share, absorb(lender_name) vce(robust)
  local prev_share_2 = _b[L.lender_share]
  local se_prev_2 = _se[L.lender_share]
  local r2_2 = `e(r2)'
  local nobs_2 = `e(N)'
  
  areg lender_share L.lender_share, absorb(country) vce(robust)
  local prev_share_3 = _b[L.lender_share]
  local se_prev_3 = _se[L.lender_share]
  local r2_3 = `e(r2)'
  local nobs_3 = `e(N)'
  
  areg lender_share L.lender_share, absorb(id1) r
  local prev_share_4 = _b[L.lender_share]
  local se_prev_4 = _se[L.lender_share]
  local r2_4 = `e(r2)'
  local nobs_4 = `e(N)'
  
  foreach reg of numlist 1(1)4 {
    local t_prev_`reg' = abs(`prev_share_`reg''/`se_prev_`reg'')
  }
  
  foreach reg of numlist 1(1)4 {
    if `t_prev_`reg''<1.645 {
      local star_`reg' = ""
     }
    else if `t_prev_`reg''>1.645 & `t_prev_`reg''<1.96 {
      local star_`reg' = "$^{*}$"
     }
    else if `t_prev_`reg''>1.96 & `t_prev_`reg''<2.576 {
      local star_`reg' = "$^{**}$"
     }
    else if `t_prev_`reg''>2.576 {
      local star_`reg' = "$^{***}$"
     }
  }
  
  tempname temp_table
  file open temp_table using "$stoutput/baseline/holding_regressions.tex", write replace
  file write temp_table "\begin{tabular}{lcccc}" _n "\hline \hline" _n
  file write temp_table " & (1) & (2) & (3) & (4) \\" _n
  file write temp_table "\hline" _n
  
  file write temp_table "Previous Share" 
  foreach reg of numlist 1(1)4 {
     file write temp_table "&" (substr(string(`prev_share_`reg'',"%9.5f"),1,6)) "`star_`reg''"
  }
  file write temp_table "\\" _n
  foreach reg of numlist 1(1)4  {
      file write temp_table "&(" (substr(string(`se_prev_`reg'',"%9.5f"),1,5)) ")"
    }
  file write temp_table " \\" _n
  file write temp_table "Lender FE & No & Yes & No & No \\" _n
  file write temp_table "Country FE & No & No & Yes & No \\" _n
  file write temp_table "Country-lender FE & No & No & No & Yes \\" _n
  file write temp_table "R-squared"
  foreach reg of numlist 1(1)4 {
     file write temp_table "&" (substr(string(`r2_`reg'',"%9.5f"),1,6))
  }
  file write temp_table " \\" _n
  file write temp_table "Observations"
  foreach reg of numlist 1(1)4 {
     file write temp_table "&" (substr(string(`nobs_`reg'',"%9.0f"),1,7))
  }
  file write temp_table " \\" _n
  file write temp_table "\hline \hline" _n "\end{tabular}"
  file close temp_table

}
