clear all
clear matrix
set more off
set maxvar 30000

cd "/Users/jmorelli/Dropbox/new_sync/replication_files/empirical" // switch here to user's path

** Set paths
global stdata   "data/stata_data"
global regout   "data/stata_output/russia"

capture log close
log using "$regout/russia.txt", text replace

** Run commands
local do_main_regs                     = 1
  local do_baseline                    = 1
local graphs_main_regs                 = 1

** Relevant locals
local date_russia           = "17aug1998"
local hmax_pre              = 60
local hpre_chosen           = 10
local hmax_post             = 120
local hmax_post_short       = 60
local hpost_fixed           = 3
local conf_int              = 1.645
local sector_list           = "Government Financial Utilities Communications Industrial Energy"
local list_covariates       = "issuance_resid bamount_resid resid_maturity BA_spread"
local country_sample        = "AR BG BR CL CN CO EC HR HU ID MX MY PA PE PH PK RU TH TR VE ZA"
local precision             = 5.960e-08

** Prepare data
if `do_main_regs' == 1 {
  gen yvar             = . /* 1 = deltayields, 2 = deltaprice, 3 = delta daily (or two days) yields */
  gen xvar             = . /* 0 = weighted nw, 1 = only banks nw,  2 = only non-banks nw, 3=separate banks and non-banks nw, 4 = only mmk nw,  5 = only non-mmk nw, 6=separate mmk and non-mmk nw, 7 = delta_e_10, 8 = delta_e_30, 9 = delta_e_45, 10=delta_e_augw1, 11=delta_e_augw2, 12=delta_e_augw3, 13=delta_e_augw4, 14=delta_e_bnkamd, 15=delta_e_bnknamd, 16=delta_e_amdbnk, 17=delta_e_insamd, 18=delta_e_insnamd, 19=delta_e_nbnkamd, 20=delta_e_nbnknamd, 21=delta_e_amdins, 22=delta_e_bnknbnkamd, 23=delta_e_bnknbnknamd, 24=delta_e_bnkinsamd, 25=delta_e_bnkinsnamd, 26=delta_e_amd*/
  gen days_pre         = . /* when negative means days post for delta stock price variable */
  gen days_post        = .
  gen jorda_proj       = . /* 1 = jorda, 2 = contemporaneous elasticity, 3 = iv with jorda (share aig instrument), 4= iv 1st stage */
  gen control_vars     = . /* 0 = 1-sumofgbholdings, 1 = 0 + initialyield BA_spread resid_maturity bamount_resid, 2 = 1 - initialyield, 6 = credit ratings +1, 7 = credit ratings +0, 8 = Delta_e interact with share of bonds +1, 9 = local currency interaction +1， 10 = global bond holdings interaction +1, 11 = 1-Delta_e, 12 = leverage1_07 interaction +1, 13= liquidity1_07 interaction +1, 14= liquidity2_07 interaction +1 */
  gen fixed_effects    = . /* 1 = Country-Sector & Currency, 2 = Country & Sector & Currency, , 3 = Country & Sector & Currency & Maturity, 4 = no FE, 5 = Country-Sector,  */
  gen subsample        = . /* 0 = all sample, 1 = only sov, 2 = extended sample (including bonds with no dps data), 3 = by country regs, 4 = country_sample, 5= bonds with high yield, 6 = by country regs only sov, 7= bonds with high yield only sov, 8 = same mat, 9 = by banks, 10 = by banks only sov, 11 = by type of GFI, 12 = by type of GFI only sov, 13=banks and asset managers only, 14=only dollar bonds, 15=only foreign currency bonds, 16=only local currency bonds, 17=no controls for baseline, 18=only control for credit rating >7, 19=only control for outlier yield*/
  gen nobs             = .
  gen country          = ""
  gen lender           = ""
  gen stock_coef       = .
  gen stock_lb         = .
  gen stock_ub         = .
  save "$regout/regression_output_russia.dta", replace
  
  use "$stdata/regression_variables_russia.dta",clear
 
  replace dps_m`hpre_chosen'_p`hpost_fixed'_bs = 0 if dps_m`hpre_chosen'_p`hpost_fixed'_bs ==.
  replace dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros = 0 if dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros ==.
  
  
  drop if sector=="#N/A Invalid Security"
  egen countrysector = group(country sector)
  encode country, gen(country_num)
  encode sector, gen(sector_num)
  encode CRNCY, gen(currency_num)
  levelsof country, local(country_list)

  foreach c of local country_list {
    local si =0
    levelsof sector if country=="`c'", local(sector_list)
    foreach s of local sector_list {
        local si = `si' + 1
        gen dcs_`c'_`si' = (country == "`c'" & sector=="`s'")
    }
  }
  levelsof CRNCY, local(cur_list)
  foreach cur of local cur_list {
        gen dcur_`cur' = (CRNCY == "`cur'")
  }

  * Maturity fixed effects
  local matbunch     = 182 /* number of days that will be assigned same mat fixed effect*/
  gen mat_`matbunch' = floor(resid_maturity/`matbunch')
  egen countrymat    = group(country mat_`matbunch')
  egen sectormat     = group(sector mat_`matbunch')


  *Average bond yield pre-Lehman
  gen aux_m62 =0
  local j = 0
  forval i=-60(2)0 {
    local j = `j' + 1
    local absi   = abs(`i')
    local absim2 = abs(`i'-2)
    gen aux_m`absi' =  (aux_m`absim2'*(`j'-1) + bondY_m`absi')/`j'
  }
  rename aux_m0 bondY_prelehman
  sum bondY_prelehman, d
  local bondy_lowerbound = `r(p50)'
  di `bondy_lowerbound'
  drop aux*
  gen highy_ind = 0
  replace highy_ind=1 if bondY_prelehman>.09 & bondY_prelehman!=.
  tab highy_ind

  gen outlier_yield = 0
  replace outlier_yield = 1 if bondY_prelehman<.06 | bondY_prelehman>.5

  * Pre-Lehman fixed effects
  local yieldbunch = .01
  gen initialyield_bunch = floor(bondY_prelehman/`yieldbunch')
  replace initialyield_bunch = 0 if initialyield_bunch<0
  tab initialyield_bunch

  * Clean issuance and outstanding from FE
  qui gen ln_bnissnce = ln(bond_issuance_mm)
  qui gen ln_bnamount = ln(bond_amount_mm)
  qui reg ln_bnissnce i.country_num
  predict double issuance_resid, residuals
  qui reg ln_bnamount i.country_num
  predict double bamount_resid, residuals

  * Generate dummy for sample_country
  gen country_sample_ind = .
  local n : word count `country_sample'
  forvalues i = 1(1)`n'     {
       local str = word("`country_sample'", `i')
       replace country_sample_ind = 1 if country=="`str'"
  }
}
  
***** REGRESSION *****
 if `do_main_regs' == 1 {
 
  * Yield Regression - Jorda Projection - Baseline - Credit ratings > 7 and remove outlier yields
  if `do_baseline' == 1 {
    foreach hpost of numlist -`hmax_pre'(2)`hmax_post' {

      if `hpost'<=0 {
        local abshpost   = abs(`hpost')
        gen deltay_bond_m`hpre_chosen'_m`abshpost' = deltay_bond_m`hpre_chosen'_p1 - deltay_bond_m`abshpost'_p1
        local depvar = "deltay_bond_m`hpre_chosen'_m`abshpost'"

      }
      else {
        local hpost = `hpost'-1
        local depvar = "deltay_bond_m`hpre_chosen'_p`hpost'"
      }

      reg `depvar' dps_m`hpre_chosen'_p`hpost_fixed'_bs dps_m`hpre_chosen'_p`hpost_fixed'_bs_zeros i.countrysector i.currency_num BA_spread resid_maturity bondY_prelehman if rating_num>7 & outlier_yield==0, robust
	  preserve
      use "$regout/regression_output_russia.dta", clear
      local nplus = _N +1
      set obs `nplus'
      replace yvar             = 1 if _n==_N
      replace xvar             = 0 if _n==_N
      replace days_pre         = `hpre_chosen' if _n==_N
      replace days_post        = `hpost' if _n==_N
      replace jorda_proj       = 1 if _n==_N
      replace control_vars     = 1 if _n==_N
      replace fixed_effects    = 1 if _n==_N
      replace subsample        = 0 if _n==_N
      replace nobs             = `e(N)' if _n==_N
      replace stock_coef       = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_lb         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]-`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      replace stock_ub         = _b[dps_m`hpre_chosen'_p`hpost_fixed'_bs]+`conf_int'*_se[dps_m`hpre_chosen'_p`hpost_fixed'_bs] if _n==_N
      save "$regout/regression_output_russia.dta", replace
      restore
    }
  }
  
  use "$regout/regression_output_russia.dta",clear
  gen crisis = 2 
  save "$regout/regression_output_russia.dta",replace

}

***** FIGURES *****
if `graphs_main_regs' == 1 {

  use "$regout/regression_output_russia.dta", clear
  levelsof days_pre, local(days_pre_range)

  * Yield Regression - Jorda Projection - Baseline
  if `do_baseline' == 1 {
    local spec "if yvar==1 & xvar==0 & days_pre==`hpre_chosen' & jorda_proj==1 & control_vars==1 & fixed_effects==1 & subsample==0 & days_post<31 & days_post>0"
    twoway (line stock_coef days_post `spec' , lcolor(edkblue) lwidth(medthick) xlabel(#7, labsize(small) angle(0)) ylabel(,labsize(small) format(%9.0gc))  /*
    */ graphregion(color(white)) plotregion(color(white)) yline(0, lc(black)) xline(0, lwidth(thin) lcolor(black) lpattern(dash))) /*
    */ (line stock_ub days_post `spec', lcolor(edkblue) lwidth(medthick) lpattern(dash) xlabel(#7, labsize(small) angle(0)) ylabel(,labsize(small) format(%9.0gc))) /*
    */ (line stock_lb days_post `spec', lcolor(edkblue) lwidth(medthick) lpattern(dash) xlabel(#7, labsize(small) angle(0)) xtitle("Days Post") ylabel(,labsize(small) format(%9.0gc)) ytitle("Beta") leg(off))
    graph export "$regout/baseline_russia.pdf", replace
	}
}

