clear all
cls

cd "/Users/jmorelli/Dropbox/new_sync/replication_files/empirical" // switch here to user's path
import excel "data/raw_data/KeyItems.xlsx", sheet("out_stata") firstrow
qui gen banks = 0
qui replace banks = 1 if TypeGFI=="BNK"

qui replace Lender = "Principal Financial" if Lender=="Principal Financial Group Inc"
qui replace Lender = "T Rowe Price" if Lender=="T Rowe Price Group Inc"
qui replace Lender = "Intesa Sanpaolo" if Lender=="Intesa Sanpaolo SpA"
qui replace Lender = "HSBC" if Lender=="HSBC Holdings GB"


*** Construct weights
qui sum AssetsUSD
qui gen wgt_all = AssetsUSD / r(sum)

qui sum AssetsUSD if banks==1
qui gen wgt_BNK = AssetsUSD / r(sum) if banks==1

qui sum AssetsUSD if banks==0
qui gen wgt_other = AssetsUSD / r(sum) if banks==0


foreach vv in "BookLVG" "AUMAdjLVG" "ExposureEM" {
	*** Unweighted Averages
	qui sum `vv'
	local avg_`vv'_unw_all = r(mean)

	qui sum `vv' if banks==1
	local avg_`vv'_unw_BNK = r(mean)

	qui sum `vv' if banks==0
	local avg_`vv'_unw_other = r(mean)	

	*** Weighted Averages
	foreach wv in "all" "BNK" "other" {
		qui gen `vv'_wgt_`wv' = `vv' * wgt_`wv'
		qui sum `vv'_wgt_`wv'
		local avg_`vv'_wgt_`wv' = r(sum)
		drop `vv'_wgt_`wv'
	}
}

foreach vv in "BookLVG" "AUMAdjLVG" "ExposureEM" {
	foreach wv in "all" "BNK" "other" {
		di "`vv'_unw_`wv' = `avg_`vv'_unw_`wv''"
		di "`vv'_wgt_`wv' = `avg_`vv'_wgt_`wv''"
	}
}

sort Lender



*** Table on Leverage

preserve
	drop if AUMAdjLVG==.
	local avg_BookLVG_unw_all     = substr(string(`avg_BookLVG_unw_all',"%5.1f"),1,4)
	local avg_AUMAdjLVG_unw_all   = substr(string(`avg_AUMAdjLVG_unw_all',"%5.1f"),1,4)
	local avg_BookLVG_unw_BNK     = substr(string(`avg_BookLVG_unw_BNK',"%5.1f"),1,4)
	local avg_AUMAdjLVG_unw_BNK   = substr(string(`avg_AUMAdjLVG_unw_BNK',"%5.1f"),1,4)
	local avg_BookLVG_unw_other   = substr(string(`avg_BookLVG_unw_other',"%5.1f"),1,4)
	local avg_AUMAdjLVG_unw_other = substr(string(`avg_AUMAdjLVG_unw_other',"%5.1f"),1,4)

	tempname tmp_tab
	file open tmp_tab using "data/stata_output/baseline/list_lenders_leverage.tex", write replace
	file write tmp_tab "\begin{tabular}{lcclcc}" _n "\hline \hline" _n
	file write tmp_tab "\multirow{2}{*}{Lender} & \multicolumn{2}{c}{Leverage} & \multirow{2}{*}{Lender} & \multicolumn{2}{c}{Leverage} \\" _n
	file write tmp_tab " & Book Value & AUM Adjusted & & Book Value & AUM Adjusted \\" _n "\hline" _n
	forvalues ii=1(1)14{
		local lnd_v  = Lender[`ii']
		local book_v = substr(string(BookLVG[`ii'],"%5.1f"),1,4)
		local adjl_v = substr(string(AUMAdjLVG[`ii'],"%5.1f"),1,4)

		local ii2 	= `ii' + 14
			local lnd2_v  = Lender[`ii2']
			local book2_v = substr(string(BookLVG[`ii2'],"%5.1f"),1,4)
			local adjl2_v = substr(string(AUMAdjLVG[`ii2'],"%5.1f"),1,4)
			file write tmp_tab "`lnd_v' & `book_v' & `adjl_v' & `lnd2_v' & `book2_v' & `adjl2_v' \\" _n
	}
	file write tmp_tab "\hline" _n
	file write tmp_tab "\textit{Average} \\" _n
	file write tmp_tab "\quad All lenders & `avg_BookLVG_unw_all' & `avg_AUMAdjLVG_unw_all' \\ " _n
	file write tmp_tab "\quad Banks & `avg_BookLVG_unw_BNK' & `avg_AUMAdjLVG_unw_BNK' \\ " _n
	file write tmp_tab "\quad Other & `avg_BookLVG_unw_other' & `avg_AUMAdjLVG_unw_other' \\ " _n
	file write tmp_tab "\hline \hline" _n "\end{tabular}"
	file close tmp_tab
restore




*** Table on Exposures
keep if ExposureEM >0

qui sum ExposureEM
local pos_v = substr(string(r(mean),"%5.1f"),1,4)
local all_v = substr(string(`avg_ExposureEM_unw_all',"%5.1f"),1,4)


tempname tmp_tab
file open tmp_tab using "data/stata_output/baseline/list_lenders_exposureEM.tex", write replace
file write tmp_tab "\begin{tabular}{lclc}" _n "\hline \hline" _n
file write tmp_tab "Lender & Estimated Exp. & Lender & Estimated Exp. \\" _n "\hline" _n
forvalues ii=1(1)13{
	local lnd_v = Lender[`ii']
	local exp_v = substr(string(ExposureEM[`ii'],"%5.1f"),1,4)

	local ii2 	= `ii' + 13
	if `ii2'<26{
		local lnd2_v = Lender[`ii2']
		local exp2_v = substr(string(ExposureEM[`ii2'],"%5.1f"),1,4)
		file write tmp_tab "`lnd_v' & `exp_v'\%  & `lnd2_v' & `exp2_v'\% \\" _n
	}
	else {
		file write tmp_tab "`lnd_v' & `exp_v'\%  \\" _n
	}
}
file write tmp_tab "\hline" _n
file write tmp_tab "\textit{Average} \\" _n
file write tmp_tab "\quad Positive exposure & `pos_v'\% \\ " _n
file write tmp_tab "\quad All lenders & `all_v'\% \\" _n
file write tmp_tab "\hline \hline" _n "\end{tabular}"
file close tmp_tab


