
function get_sparse_KS(cm::CoreModule, asm::AshocksModuleOBC, YEMv, ω_v, aDMp_v, itp_bp, itp_def)
    # In this function we generate a sparse transition matrix Π_{t},
    # such that Λ_{t+1}=Π_{t}'Λ_{t}.
    # Π_{t} = sparse(I,J,V), where I is a vector indicating the location of the row,
    # J is a vector containing the location of the column, and V is a vector with 
    # the corresponding values to fill in.
    # We will actually be computing Π_{t}'.

    zn_mk_fine = cm.zn_mk_fine # length(cm.zgrid)
    nb_fine    = length(cm.b_grid_fine)
    nΠaggr     = nb_fine * cm.zn_mk_fine * 2

    #== Initialize Vectors for sparse matrix ==#
    Ic    = Array{Int64}(0)
    Ir    = Array{Int64}(0)
    V_vec = Array{Float64}(0)

    # Here we change the output to the transpose.
    """ Compute transition matrix associated to (default, default) """
    ic_b1 = ones(1)
    ir_b1 = ones(1)
    vv_b1 = [1.0 - cm.θ]

    """ Compute transition matrix associated to (default, repay) """
    ic_b2 = ones(1)
    ir_b2 = ones(1)
    vv_b2 = [cm.θ]

    # rows have t+1, columns have t

    for iι = 1:2

        if iι == 1
            for iz = 1:cm.zn_mk_fine

                for iz1 = 1:cm.zn_mk_fine

                    """ (default, default) """
                    valin_b1      = cm.Πz_fine[iz, iz1]*vv_b1
                    col_start     = (iz-1)*nb_fine    # Starting points for columns
                    row_start     = (iz1-1)*nb_fine   # Starting points for rows
           
                    #== Append on indices ==#
                    append!(Ic, ic_b1 + col_start)
                    append!(Ir, ir_b1 + row_start)
                    append!(V_vec, valin_b1 )

                    """ (default, repay) """
                    valin_b2      = cm.Πz_fine[iz, iz1]*vv_b2
                    col_start     = (iz-1)*nb_fine                         # Starting points for columns
                    row_start     = nb_fine*cm.zn_mk_fine + (iz1-1)*nb_fine   # Starting points for rows
           
                    #== Append on indices ==#
                    append!(Ic, ic_b2 + col_start)
                    append!(Ir, ir_b2 + row_start)
                    append!(V_vec, valin_b2 )
                end
            end

        else

            for iz = 1:cm.zn_mk_fine
                # Generate matrix of next period's bonds policy given current state
                z_v = cm.zgrid_fine[iz]

                bprime  = zeros(0)
                def_ind = 0

                for ib=1:nb_fine
                    def_val = min(1.0, max(0.0, itp_def[cm.b_grid_fine[ib], z_v, YEMv, ω_v, aDMp_v]))

                    if def_val<=0.5
                        push!(bprime, itp_bp[cm.b_grid_fine[ib], z_v, YEMv, ω_v, aDMp_v])
                    else
                        def_ind = ib
                        break
                    end
                end

                if def_ind > 1
                    bprime = bprime[1:def_ind-1]

                    """ Compute transition matrix associated to (repay, repay) """
                    ic_b4, ir_b4, vv_b4 = fill_sparse_fun(collect(cm.b_grid_fine), bprime)
                
                    """ Compute transition matrix associated to (repay, default) """
                    ic_b3 = collect(def_ind:nb_fine)
                    ir_b3 = ones(length(ic_b3))         # first point in grid
                    vv_b3 = ones(length(ic_b3))         # mass points

                elseif def_ind == 1
                    ic_b4, ir_b4, vv_b4 = collect(1:1), collect(1:1), zeros(1)

                    ic_b3 = collect(1:nb_fine)
                    ir_b3 = ones(nb_fine)
                    vv_b3 = ones(nb_fine)
                else
                    ic_b4, ir_b4, vv_b4 = fill_sparse_fun(collect(cm.b_grid_fine), bprime)
                    ic_b3, ir_b3, vv_b3 = collect(1:1), collect(1:1), zeros(1)
                end


                for iz1 = 1:cm.zn_mk_fine

                    valin_b3 = cm.Πz_fine[iz, iz1]*vv_b3 # (repay, default)
                    valin_b4 = cm.Πz_fine[iz, iz1]*vv_b4 # (repay, repay)

                    # Note: below multiply by nb_fine*cm.nz_fine to skip part where ι=0

                    """ case (repay, repay) """
                    col_start  = nb_fine * cm.zn_mk_fine + (iz-1) * nb_fine    # Starting points for columns
                    row_start  = nb_fine * cm.zn_mk_fine + (iz1-1) * nb_fine   # Starting points for rows

                    append!(Ic, ic_b4 + col_start)
                    append!(Ir, ir_b4 + row_start)
                    append!(V_vec, valin_b4 )


                    """ case (repay, default) """
                    col_start  = nb_fine * cm.zn_mk_fine + (iz-1) * nb_fine    # Starting points for columns
                    row_start  = (iz1-1) * nb_fine                    # Starting points for rows

                    append!(Ic, ic_b3 + col_start)
                    append!(Ir, ir_b3 + row_start)
                    append!(V_vec, valin_b3 )

                end
            end
        end
    end    

    return sparse(Ir, Ic, V_vec, nΠaggr, nΠaggr)
end


function compute_Btil_Bhat_calib(cm::CoreModule, asm::AshocksModuleOBC, input_state, aDMp_v, Λt, itp_def, itp_q, itp_bp, itp_qDFRSK)

	yEMv, ω_v, aDMv = input_state[1], input_state[2], input_state[3]

    Btilde, Bhat, SP, yield_v, by_avg, by2, SP2, Ct, SPDFRK, TB, C_Y = 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0

    Λt_mkt  = Λt[cm.num_nodes+1:end] # since Λt is SparseVector, so is Λt_mkt.

    min_q = 0.001

    for i_s=1:cm.num_nodes
        b_v   = cm.snodes[i_s, 1]
        ZEM_v = cm.snodes[i_s, 2]
        def_v = itp_def[b_v, ZEM_v, yEMv, ω_v, aDMp_v]
        if def_v.<=0.5
            yv      = exp(ZEM_v + yEMv)            
            bp_v    = max(itp_bp[b_v, ZEM_v, yEMv, ω_v, aDMp_v], cm.bgrid[1])
            qv      = max(min_q, itp_q[bp_v, ZEM_v, yEMv, ω_v, aDMp_v])
            qvDFRK  = max(min_q, itp_qDFRSK[bp_v, ZEM_v, yEMv, ω_v, aDMp_v])
            cv      = max( yv - b_v + qv*(bp_v-cm.ξ*b_v), 1e-14)
            
            SP     += Λt_mkt[i_s] * ( (1.0/qv+cm.ξ-1.0) - (1.0/cm.βLD-1.0) ) # see Arellano-Ramanarayanan (2012)
            SPDFRK += Λt_mkt[i_s] * ( (1.0/qvDFRK+cm.ξ-1.0) - (1.0/cm.βLD-1.0) )
            Bhat   += Λt_mkt[i_s] * qv * bp_v
            Btilde += Λt_mkt[i_s] * b_v*(1.0+cm.ξ*qv)
            Ct     += Λt_mkt[i_s] * cv
            TB     += Λt_mkt[i_s] * (yv - cv)/yv
            C_Y    += Λt_mkt[i_s] * cv/yv

            by_avg += Λt_mkt[i_s] * ( ( qv * bp_v ) / yv )
            by2    += Λt_mkt[i_s] * ( ( qv * bp_v ) / yv ) ^ 2
            SP2    += Λt_mkt[i_s] * ( (1.0/qv+cm.ξ-1.0) - (1.0/cm.βLD-1.0) )^2 # for computation of s.d.

            yield_v += Λt_mkt[i_s] * (1.0/qv+cm.ξ-1.0) # see Arellano-Ramanarayanan (2012)
        end
    end

    Btilde  = Btilde / sum(Λt_mkt)
    Bhat    = Bhat / sum(Λt_mkt)
    SP      = SP / sum(Λt_mkt)
    yield_v = yield_v / sum(Λt_mkt)

    by_avg = by_avg / sum(Λt_mkt)
    by2    = by2 / sum(Λt_mkt)
    SP2    = SP2 / sum(Λt_mkt)

    sdby   = (by2 - by_avg^2)^0.5
    valtmp = SP2 - SP^2
    if valtmp>=0.0
        sdSP   = (SP2 - SP^2)^0.5
    else
        sdSP = 0.0
    end

    Ct     = Ct / sum(Λt_mkt)
    TB     = TB / sum(Λt_mkt)
    SPDFRK = SPDFRK / sum(Λt_mkt)
    C_Y    = C_Y / sum(Λt_mkt)

    return (Btilde, Bhat, SP, yield_v, by_avg, sdby, sdSP, Ct, SPDFRK, TB, C_Y)
end


function first_trial(cm::CoreModule, asm::AshocksModuleOBC, input_state, Λt, aDMp_v, out_obj, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, itp_qDFRSK)

	yEMv, ω_v, aDMv, Dv  = input_state[1], input_state[2], input_state[3], input_state[4]

    min_RDM  = fun_RDM(cm.α, cm.δ, cm.aDMgrid[end], cm.ωgrid[1])
    max_ReDM = maximum(asm.ReDM)*1.1
    min_ReDM = max(minimum(asm.ReDM), min_RDM*1.01)
    max_Red  = maximum(asm.Red)
    min_Red  = minimum(asm.Red)


	# Compute Btil and Bhat
	outBh   = compute_Btil_Bhat_calib(cm, asm, input_state, aDMp_v, Λt, itp_def, itp_q, itp_bp, itp_qDFRSK)
	Btilv   = outBh[1]
	Bhatv   = outBh[2]
    SPv     = outBh[3]
    yield_v = outBh[4]
    by_avg  = outBh[5]
    sdby    = outBh[6]
    sdSP    = outBh[7]
    Ct      = outBh[8]
    SPDFRK  = outBh[9]
    TB      = outBh[10]
    C_Y     = outBh[11]

	### Compute variables from lender    
    ReDM   = min( max(min_ReDM, itp_ReDM[ω_v, aDMp_v]), max_ReDM )
    Red    = min( max(min_Red, itp_Red[ω_v, aDMp_v]), max_Red )
    RDMv   = fun_RDM(cm.α, cm.δ, aDMv, ω_v)
    NW     = max(Btilv*cm.μEM + RDMv*aDMv - Dv, 1e-7)

    divs     = 0.5*NW/cm.phi*(1.0-Red*cm.βLD)
    Dpv      = ( Bhatv*cm.μEM + aDMp_v - ((1.0-cm.σn)*cm.nbar + cm.σn*NW) + divs ) / cm.βLD
    aDMp_out = aDMp_v

	copy!(out_obj, [aDMp_v; Btilv; Bhatv; 0.0; NW; ReDM; SPv; yield_v; by_avg; sdby; sdSP; Ct; SPDFRK; Dpv; aDMp_out; TB; C_Y])

	return Void
end



function find_ReEM_once_calib(cm::CoreModule, asm::AshocksModuleOBC, input_state, Λt, aDMp_v, out_obj, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, itp_qDFRSK)

    yEMv, ω_v, aDMv, Dv  = input_state[1], input_state[2], input_state[3], input_state[4]

    min_RDM  = fun_RDM(cm.α, cm.δ, cm.aDMgrid[end], cm.ωgrid[1])
    max_ReDM = maximum(asm.ReDM)*1.1
    min_ReDM = max(minimum(asm.ReDM), min_RDM*1.01)
    max_Red  = maximum(asm.Red)
    min_Red  = minimum(asm.Red)



    # Compute Btil and Bhat
    outBh   = compute_Btil_Bhat_calib(cm, asm, input_state, aDMp_v, Λt, itp_def, itp_q, itp_bp, itp_qDFRSK)
    Btilv   = outBh[1]
    Bhatv   = outBh[2]
    SPv     = outBh[3]
    yield_v = outBh[4]
    by_avg  = outBh[5]
    sdby    = outBh[6]
    sdSP    = outBh[7]
    Ct      = outBh[8]
    SPDFRK  = outBh[9]
    TB      = outBh[10]
    C_Y     = outBh[11]

    ### Compute variables from lender    
    ReDM   = min( max(min_ReDM, itp_ReDM[ω_v, aDMp_v]), max_ReDM )
    RDMv   = fun_RDM(cm.α, cm.δ, aDMv, ω_v)
    NW     = max(Btilv*cm.μEM + RDMv*aDMv - Dv, 1e-7)

    Dpv      = cm.κ*((1.0-cm.σn)*cm.nbar + cm.σn*NW)/cm.βLD
    aDMpv2   = ((1.0-cm.σn)*cm.nbar + cm.σn*NW) * ( 1.0 + cm.κ + 0.5/cm.phi*(cm.βLD*ReDM-1.0) ) - Bhatv*cm.μEM
    aDMp_out = aDMpv2[1]
    out_v    = aDMpv2[1] - aDMp_v

    copy!(out_obj, [aDMp_v; Btilv; Bhatv; 0.0; NW; ReDM; SPv; yield_v; by_avg; sdby; sdSP; Ct; SPDFRK; Dpv; aDMp_out; TB; C_Y])

    return out_v
end



function solve_system_calib(cm::CoreModule, asm::AshocksModuleOBC, input_state, Λt, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, bounds, itp_qDFRSK)

    lb_ADM, ub_ADM = bounds[1], bounds[2]

	out_obj    = zeros(17)

    # First try unconstrained
    out1  = first_trial(cm, asm, input_state, Λt, ub_ADM, out_obj, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, itp_qDFRSK)
    Dp_v  = out_obj[14]
    NW_v  = ((1.0-cm.σn)*cm.nbar + cm.σn*out_obj[5])
    check = (1.0/cm.βLD)*cm.κ*NW_v - Dp_v

    if check<0
    	res 	 = Optim.optimize(aDMp_v->(find_ReEM_once_calib(cm, asm, input_state, Λt, aDMp_v, out_obj, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, itp_qDFRSK))^2, lb_ADM, ub_ADM, Brent(), iterations=150)
    	aDMp_out = Optim.minimizer(res)
    else
        aDMp_out = ub_ADM
    end

	yEMv, ω_v, aDMv, Dv  = input_state[1], input_state[2], input_state[3], input_state[4]

    Πst 	= get_sparse_KS(cm, asm, yEMv, ω_v, aDMp_out, itp_bp, itp_def)
    Λt1 	= *(Πst, Λt) # this function preserves sparsity type

    Λt_mkt  = Λt[cm.num_nodes+1:end] # since Λt is SparseVector, so is Λt_mkt.
    Λt1_mkt = Λt1[cm.num_nodes+1:end]
    sum_Λ   = sum(Λt_mkt)
    sum_Λ1  = sum(Λt1_mkt)

    Bt1 = 0.0
    for i_s=1:cm.num_nodes
        Bt1 += Λt1_mkt[i_s] * cm.snodes[i_s, 1]
    end

    Bt1 = Bt1 * cm.μEM / sum_Λ1
    out_obj[4] = Bt1

	return (out_obj, Λt1)
end


function time_iteration_calib(cm::CoreModule, asm::AshocksModuleOBC, capT, dropT, YEMvals, ωvals, itp_qDFRSK)

    @printf("Doing Time Iteration\n")

    NS   = size(cm.snodes, 1)

    if capT <=2_000
        bmk_time = 500
    else
        bmk_time = 2_000
    end

    # Create interpolation/extrapolation object
    knots   = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_def = interpolate(knots, asm.defmat,     Gridded(Linear()))
    itp_q   = interpolate(knots, asm.q_mat,  Gridded(Linear()))
    itp_bp  = interpolate(knots, asm.bprime_mat, Gridded(Linear()))

    itp_ReDM = interpolate((cm.ωgrid, cm.aDMgrid), asm.ReDM, Gridded(Linear()))
    itp_Red  = interpolate((cm.ωgrid, cm.aDMgrid), asm.Red, Gridded(Linear()))


    # Bounds for finding ADM
    bounds = [cm.aDMgrid[1]*0.9; asm.ADMmax]


    ## Pre-allocate memory for arrays to save data that is going to be used for updates
    Bvec, ReEMvec, NWvec, Bhatvec, Btildevec, aDMvec = Array{Float64}(capT+1), Array{Float64}(capT), Array{Float64}(capT),
                                                  	 	Array{Float64}(capT), Array{Float64}(capT), Array{Float64}(capT+1)

    aDMvec2  = Array{Float64}(capT+1)

	ReDMvec  = Array{Float64}(capT)
    SPvec    = Array{Float64}(capT)
    YLDvec   = Array{Float64}(capT)
    Ctvec    = Array{Float64}(capT)
    DFRSKvec = Array{Float64}(capT)
    Dvec     = Array{Float64}(capT+1)
    TB_vec   = Array{Float64}(capT)
    C_Y_vec  = Array{Float64}(capT)

    by_vec, sdby_vec, sdSP_vec = copy(ReDMvec), copy(ReDMvec), copy(ReDMvec)


    def_measure = zeros(capT)


    """ Assign initial values """
    ReEMvec[1], Bvec[1], NWvec[1] 	    = cm.REMstar[1], cm.Bstar[1], cm.NWstar[1]
    Bhatvec[1], Btildevec[1], aDMvec[1]	= cm.Bhtstar[1], cm.Btlstar[1], cm.aDMstar[1]
    Dvec[1], aDMvec2[1] = cm.Dstar, cm.aDMstar[1]

    def_measure[1]  = 1.0 - sum(cm.Λstar[NS+1:end])
    Bvec[1] 		= cm.Bstar/sum(cm.Λstar[NS+1:end])


    """ Compute next period's needed aggregates """
    input_state = [YEMvals[1]; ωvals[1]; aDMvec[1]; Dvec[1]]
    out_X       = solve_system_calib(cm, asm, input_state, cm.Λstar, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, bounds, itp_qDFRSK)
#   out_X[1] = [aDMp_v; Btilv; Bhatv; Bt1; NW; ReDM; SPv; yield_v; by_avg; sdby; sdSP; Ct; SPDFRK; Dpv; aDMp_out; TB; C_Y]
#                 1       2      3     4    5    6    7     8         9     10     11  12    13     14     15     16  17

    aDMpv, Btilde, Bhat, Bt1, NW, ReDM, SPv, YLDv = out_X[1][1], out_X[1][2], out_X[1][3], out_X[1][4], out_X[1][5], out_X[1][6], out_X[1][7], out_X[1][8]
    
    Ctvec[1], DFRSKvec[1], Dvec[2], YLDvec[1] = out_X[1][12], out_X[1][13], out_X[1][14], YLDv

    Λt1 = out_X[2]
    

    Bvec[2] 		= Bt1
    aDMvec[2] 		= aDMpv
    aDMvec2[2]      = out_X[1][15]

    Λ_erg   = spzeros(length(cm.Λstar))
    denom_Λ = 0

    RDMvec = zeros(capT)

    for tt=2:capT

    	YEMt 	= YEMvals[tt]
    	Bt 		= Bvec[tt]
    	aDMv 	= aDMvec[tt]
        Λt 		= copy(Λt1)

        input_state = [YEMt; ωvals[tt]; aDMvec[tt]; Dvec[tt]]
        out_X       = solve_system_calib(cm, asm, input_state, Λt, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, bounds, itp_qDFRSK)

        aDMpv, Btilde, Bhat, Bt1, NW, ReDM, SPv, yieldv = out_X[1][1], out_X[1][2], out_X[1][3], out_X[1][4], out_X[1][5], out_X[1][6], out_X[1][7], out_X[1][8]
        by_avg, sdby, sdSP = out_X[1][9], out_X[1][10], out_X[1][11]

	    Λt1 = out_X[2]

        Bhatvec[tt]      = Bhat
        Btildevec[tt]    = Btilde
        NWvec[tt]        = NW
        Bvec[tt+1]       = Bt1
        ReEMvec[tt] 	 = ReDM
        aDMvec[tt+1] 	 = aDMpv
        ReDMvec[tt]		 = ReDM
        SPvec[tt]        = SPv
        YLDvec[tt]       = yieldv

        by_vec[tt]       = by_avg
        sdby_vec[tt]     = sdby
        sdSP_vec[tt]     = sdSP
        Ctvec[tt], DFRSKvec[tt] = out_X[1][12], out_X[1][13]
        Dvec[tt+1]       = out_X[1][14]
        aDMvec2[tt+1]    = out_X[1][15]
        TB_vec[tt]       = out_X[1][16]
        C_Y_vec[tt]      = out_X[1][17]
        
        Λt_mkt  = Λt[NS+1:end]        
        Λt1_mkt = Λt1[NS+1:end]

        def_measure[tt] = 1.0 - sum(Λt1_mkt)

        RDMvec[tt]   = fun_RDM(cm.α, cm.δ, aDMvec[tt], ωvals[tt])

        if tt==dropT+1
            denom_Λ += 1
            Λ_erg = Λ_erg + Λt
        end

        if tt>dropT+1
            Λ_erg = Λ_erg*denom_Λ/(denom_Λ+1) + Λt/(denom_Λ+1)
            denom_Λ += 1
        end

        if tt%bmk_time==0
        	@printf("time t=%d\n", tt)
        end

    end


    @printf("Mean Bhat = %.3g, mean aDM = %.3g, mean ReDM = %.3g\n", mean(Bhatvec[dropT+1:end]), mean(aDMvec[dropT+1:end]), mean(ReEMvec[dropT+1:end]))

    Bvec, aDMvec, ReEMvec, NWvec, Bhatvec, Btildevec, def_measure, ReDMvec, SPvec, YLDvec, = 
        Bvec[dropT+1:end], aDMvec[dropT+1:end], ReEMvec[dropT+1:end], NWvec[dropT+1:end], Bhatvec[dropT+1:end], Btildevec[dropT+1:end], def_measure[dropT+1:end], ReDMvec[dropT+1:end], SPvec[dropT+1:end], YLDvec[dropT+1:end]

    by_vec, sdby_vec, sdSP_vec, Ctvec, DFRSKvec, Dvec, aDMvec2 = by_vec[dropT+1:end], sdby_vec[dropT+1:end], sdSP_vec[dropT+1:end], Ctvec[dropT+1:end], DFRSKvec[dropT+1:end], Dvec[dropT+1:end], aDMvec2[dropT+1:end]
    RDMvec, TB_vec, C_Y_vec = RDMvec[dropT+1:end], TB_vec[dropT+1:end], C_Y_vec[dropT+1:end]

    Λ_erg = Λ_erg / sum(Λ_erg)

    return (Bvec, aDMvec, ReEMvec, NWvec, Bhatvec, Btildevec, def_measure, ReDMvec, SPvec, YLDvec, by_vec, sdby_vec, sdSP_vec, Ctvec, DFRSKvec, Dvec, Λ_erg, aDMvec2, RDMvec, TB_vec, C_Y_vec)
end

# Bvec, aDMvec, ReEMvec, NWvec, Bhatvec, Btildevec, def_measure, ReDMvec, SPvec, YLDvec, by_vec, sdby_vec, sdSP_vec, Ctvec, DFRSKvec, Dvec, Λ_erg, aDMvec2, RDMvec, TB_vec, C_Y_vec
# 1       2       3       4       5         6          7           8         9      10      11       12       13      14        15      16    17     18       19       20      21


function gen_exogen(cm::CoreModule, asm::AshocksModuleOBC, capT)

    YEMvals = Array{Float64}(capT)
    ωvals  = Array{Float64}(capT)

    srand(5678) # Setting the seed

    YEMvals[1]  = 0.0
    ωvals[1]  = 0.0
    rand_mat    = rand(MvNormal([0;0], eye(2)), capT)
    rand_YEM    = vec(rand_mat[1,:])
    rand_ω    = vec(rand_mat[2,:])

    for tt=2:capT
        YEMvals[tt] = YEMvals[tt-1]*cm.ρYEM + cm.σYEM*rand_YEM[tt]
        ωvals[tt] = ωvals[tt-1]*cm.ρω + cm.σω*rand_ω[tt]
    end

    ωvals = exp.(ωvals)
    ωvals = ωvals - mean(ωvals) + cm.Eω

    return YEMvals, ωvals
end

function gen_exogen_correl(cm::CoreModule, asm::AshocksModuleOBC, corr_v, sig_ω, capT)

    YEMvals = Array{Float64}(capT)
    ωvals  = Array{Float64}(capT)

    # Create Quadratures
    Σ_ωYEM = [cm.σYEM^2.0 corr_v*cm.σYEM*sig_ω ; corr_v*cm.σYEM*sig_ω sig_ω^2.0]
    Rmat     = chol(Σ_ωYEM)

    srand(5678) # Setting the seed

    YEMvals[1]  = 0.0
    ωvals[1]  = 0.0
    rand_mat    = rand(MvNormal([0;0], [[1.0 0.0]; [0.0 1.0]]), capT)
    rand_YEM    = vec(rand_mat[1,:])
    rand_ω    = vec(rand_mat[2,:])

    for tt=2:capT
        YEMvals[tt] = YEMvals[tt-1]*cm.ρYEM + Rmat[1,1]*rand_YEM[tt] + Rmat[2,1]*rand_ω[tt]
        ωvals[tt] = ωvals[tt-1]*cm.ρω + Rmat[1,2]*rand_YEM[tt] + Rmat[2,2]*rand_ω[tt]
    end

    ωvals = exp.(ωvals)
    ωvals = ωvals - mean(ωvals) + cm.Eω

    return YEMvals, ωvals
end

function adjust_muEM!(cm::CoreModule, asm::AshocksModuleOBC, dropT, YEMsim, ωsim, itp_qDFRSK)
    iter     = 0
    ind_stop = 0
    share_DM = 0.0
    obj_share_DM = 0.90

    while ind_stop == 0
        iter += 1
        @time out_time  = time_iteration_calib(cm, asm, size(YEMsim, 1), dropT, YEMsim, ωsim, itp_qDFRSK)
        out_aDM_KS  = out_time[2]
        out_ReEM_KS = out_time[3]
        share_DM    = mean(out_time[2][2:end]./(out_time[2][2:end]+out_time[5]*cm.μEM))
        cm.μEM      = ((1.0-obj_share_DM)/obj_share_DM) * mean(out_time[2])/mean(out_time[5])

        if (iter == 3 || abs(share_DM-obj_share_DM)<0.015)
            ind_stop = 1
        end
    end
    @printf("Adjusted μEM to %.1g, with share DM %.2g\n", cm.μEM, share_DM)
end



function update_coeff!(cm::CoreModule, asm::AshocksModuleOBC, capT, dropT, YEMvals, ωvals, out_calib, dist_KS, itp_qDFRSK)

    damp = 0.0

    # Perform full simulation
	out_calib = time_iteration_calib(cm, asm, capT, dropT, YEMvals, ωvals, itp_qDFRSK)
	capT_new  = length(out_calib[1])-1

    YEMsim   = YEMvals[dropT+1:capT]
    ωsim   = ωvals[dropT+1:capT]
    aDMsim   = out_calib[2]           # this is the current value aDM_{t}

    Ahat_old = zeros(capT_new)
    Ahat_new = zeros(capT_new)

    for tt=1:capT_new
        Ahat_old[tt] = projection_ADM(cm, asm, [aDMsim[tt]; ωsim[tt]])
    end

    # Run regression with ADM values below the maximum ADMmax
    ind_flt  = aDMsim[2:end].<asm.ADMmax
    ADMp_vec = aDMsim[2:end]
    ADMt_vec = aDMsim[1:end-1]
    ADMp_flt = ADMp_vec[ind_flt]
    ADMt_flt = ADMt_vec[ind_flt]
    ωt_flt = ωsim[ind_flt]

	XXADM 	 = [ones(size(ADMt_flt)) log.(ADMt_flt) log.(ωt_flt)]
	coeffaDM = XXADM\log.(ADMp_flt)

    coeff_old   = copy(cm.coeffaDM)
    coeff_new   = copy(coeffaDM)
    cm.coeffaDM = copy(coeffaDM)

    for tt=1:capT_new
        Ahat_new[tt] = projection_ADM(cm, asm, [aDMsim[tt]; ωsim[tt]])
    end

    dist_Ahat = norm(Ahat_old-Ahat_new)

    cm.coeffaDM = damp*coeff_old + (1.0-damp)*coeff_new


    dist_KS[1]     = dist_Ahat

end


### ----------------------- MODULE 5: GLOBAL LOOP ---------------------- ###

function global_loop!(cm::CoreModule, asm::AshocksModuleOBC, capT, dropT, YEM_sim, ω_sim, out_calib, dist_KS)

    dist    = 10.0
    iter    = 0
    maxiter = 10
    maxdist = 1e-2

    q_DFRSK     = load("model_data/q_DFRSK.jld", "q_DFRSK")
    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))

    while dist > maxdist && iter < maxiter

        iter += 1

        update_coeff!(cm, asm, capT, dropT, YEM_sim, ω_sim, out_calib, dist_KS, itp_qDFRSK)
        
        dist = dist_KS[1]

        # Update problem of the lender
        compute_vs!(cm, asm)

        # Update problem of the borrower
        @time vfi!(cm, asm)

        # Udate μEM to have share of 0.9 on average
        adjust_muEM!(cm, asm, 100, YEM_sim[1:300], ω_sim[1:300], itp_qDFRSK)

        @printf("iter coeff # %d, dist_aDM = %.3g\n", iter, dist)
        @printf("-----------------------------------------------\n")
        @printf("\n")

    end

    # Compute intermediation-free price
    q_guess = load("model_data/q_DFRSK.jld", "q_DFRSK")
    q_DFRSK = compute_prices_DFRSK(cm, asm, q_guess)
    save("model_data/q_DFRSK.jld", "q_DFRSK", q_DFRSK)

end



### ----------------------- MODULE 6: VARIATIONS IN DISTRIBUTION ---------------------- ###


function change_distribution_bimodal(cm::CoreModule, asm::AshocksModuleOBC, Λin, cf_mn)

    # This function transforms the distribution of debt to get a binomial shape. That way, we get 
    # a higher dispersion and a relevant measure of countries closer to the default threshold.

    Bmean = dot(Λin[cm.num_nodes+1:end], cm.snodes[:,1])
    Bstdv = sqrt(dot(Λin[cm.num_nodes+1:end], (cm.snodes[:,1]-Bmean).^2.0))

    muB      = log( Bmean / sqrt(1.0+Bstdv^2/Bmean^2) )
    sgB      = log(1.0+Bstdv^2/Bmean^2)

    distrib_tmp1 = Distributions.LogNormal(log(Bmean)*(1.0-cf_mn), 0.05)
    distrib_tmp2 = Distributions.LogNormal(log(Bmean)*(1.0+cf_mn), 0.05)

    prob_Vec = 0.5*Distributions.cdf.(distrib_tmp1, cm.b_grid_fine) + 0.5*Distributions.cdf.(distrib_tmp2, cm.b_grid_fine)

    NN = length(cm.b_grid_fine)
    hist_vec = zeros(NN)

    for i_n=1:NN-1
        hist_vec[i_n] = prob_Vec[i_n+1]-prob_Vec[i_n]
    end

    hist_vec = hist_vec / sum(hist_vec)

    sum1   = sum(Λin[1:cm.num_nodes])
    tmp    = cm.Πz_fine^1000
    tmp    = kron( vec(tmp[1,:]), hist_vec)
    tmp    = tmp/sum(tmp)
    Λamplf = [Λin[1:cm.num_nodes]; tmp*(1-sum1)]

    Bmean2 = dot(Λamplf[cm.num_nodes+1:end], cm.snodes[:,1])
    Bstdv2 = sqrt(dot(Λamplf[cm.num_nodes+1:end], (cm.snodes[:,1]-Bmean2).^2.0))

    return (Λamplf, Bmean2, Bstdv2)
end

