
################# ----------------------------------------------------------------------------- ##################
################# -------------------------- *** SOLVE MODEL ONCE *** ------------------------- ##################
################# ----------------------------------------------------------------------------- ##################

# In case changed parameters. No iteration over projection rules.

if solve_once

	cm  = coremodule(β = eval_params[1], d0 = eval_params[2], d1 = eval_params[3], σn = eval_params[4], ρω = eval_params[5], ρEMDM = eval_params[6], σω = eval_params[7], δ = eval_params[8], phi=eval_params[9], κ=eval_params[10])
	
	## Take as guess the uncorrelated shocks
	cmnc 			= load("model_data/NAG_module_structure_start.jld", "cm");
    cm.vf          	= copy(cmnc.vf)
    cm.vd          	= copy(cmnc.vd)
    cm.vc          	= copy(cmnc.vc)
    cm.bprime_mat  	= copy(cmnc.bprime_mat)
    cm.q_mat       	= copy(cmnc.q_mat)
    cm.def_states  	= copy(cmnc.def_states)

	cm.coeffaDM	   = [0.635976; 0.398587; 3.23005]



	### --- Do VFI --- ###
	vfi_nas!(cm)
	compute_ss_root!(cm)

	asm = ashocksmodule(cm)
	

	## Take as guess the uncorrelated shocks
	asmnc 			= load("model_data/ASM_module_structure_start.jld", "asm");
    asm.vf          = copy(asmnc.vf)
    asm.vd          = copy(asmnc.vd)
    asm.vc          = copy(asmnc.vc)
    asm.bprime_mat  = copy(asmnc.bprime_mat)
    asm.q_mat       = copy(asmnc.q_mat)
    asm.defmat      = copy(asmnc.defmat)
   

	compute_vs!(cm, asm)
	@time vfi!(cm, asm)

	q_guess = load("model_data/q_DFRSK_start.jld", "q_DFRSK");
    q_DFRSK = compute_prices_DFRSK(cm, asm, q_guess)
    save("model_data/q_DFRSK.jld", "q_DFRSK", q_DFRSK)

    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))


	### --- Simulate Once and Adjust μEM --- ###

	capT    = 500
	YEM_sim, ω_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)

	# Run short simulation to adjust μEM to get average DM share of 90%
	adjust_muEM!(cm, asm, 100, YEM_sim, ω_sim, itp_qDFRSK)


	### --- Save Model --- ###
	if save_models
		save("model_data/NAG_module_structure.jld", "cm", cm)
		save("model_data/ASM_module_structure.jld", "asm", asm)
	end

end



################# ----------------------------------------------------------------------------- ##################
################# ---------------------- *** ITERATE ON COEFFICIENTS *** ---------------------- ##################
################# ----------------------------------------------------------------------------- ##################

if iterate_coeffs == 1

	cm  = load("model_data/NAG_module_structure.jld", "cm");
	asm = load("model_data/ASM_module_structure.jld", "asm");

	capT    = 3_000
	dropT 	= 100
	YEM_sim, ω_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)

	out_calib = []
	dist_KS   = [0.0]
	global_loop!(cm, asm, capT, dropT, YEM_sim, ω_sim, out_calib, dist_KS)


	if save_models
		save("model_data/NAG_module_structure.jld", "cm", cm)
		save("model_data/ASM_module_structure.jld", "asm", asm)
	end

end
