

################# ----------------------------------------------------------------------------- ##################
################# --------------- *** CONDITIONAL AND UNCONDITIONAL MOMENTS *** --------------- ##################
################# ----------------------------------------------------------------------------- ##################

if do_calib_mms

	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  ----------------------------------     BLOCK 1: LOAD MARKET STRUCTURE    ---------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###
	cm  = load("model_data/NAG_module_structure.jld", "cm");
	asm = load("model_data/ASM_module_structure.jld", "asm");

	capT    = 10_000
	dropT 	= 100
	YEM_sim, ω_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)

    q_DFRSK 	= load("model_data/q_DFRSK.jld", "q_DFRSK")
    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))

    # Auxilary function
    corr_fun(x1, x2) = cov(x1, x2) / ( std(x1)*std(x2) )




	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  ----------------------------------     BLOCK 2: SIMULATION AND MOMENTS    ---------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###

	out_calib 	= time_iteration_calib(cm, asm, capT, dropT, YEM_sim, ω_sim, itp_qDFRSK)


	## Simulate High Yield using simulation results
	HYSPmat  = compute_HYSP(cm, asm)
	itp_HYSP = interpolate((cm.ωgrid, cm.aDMgrid), HYSPmat, Gridded(Linear()))

	function simul_HYSP_fun(itp_HYSP, ωvals, ADMpvals)
		HYSPsim = zeros(length(ωvals))
		for t=1:length(ωvals)
			HYSPsim[t] = itp_HYSP[ωvals[t], ADMpvals[t]]
		end
		return HYSPsim
	end


	###  EXTRACT ELEMENTS FROM THE SIMULATION
	# The time iteration for calibration returns:
	# 	out_calib =	Bvec, aDMvec, ReEMvec, NWvec, Bhatvec, Btildevec, def_measure, ReDMvec, SPvec, YLDvec, by_vec, sdby_vec, sdSP_vec, Ctvec, DFRSKvec, Dvec, Λ_erg, aDMvec2, RDMvec, TB_vec, C_Y_vec
	# 				 1       2       3       4       5         6          7           8        9      10      11       12       13      14        15      16    17     18       19       20      21	

	## Extract simulations
	YEM_calib 		= YEM_sim[dropT+1:end]
	ω_calib 		= ω_sim[dropT+1:end]
	ADM_calib 		= out_calib[2] 			# ADM_t  goes until T+1
	ReDM_calib 		= out_calib[3]
	NW_calib 		= out_calib[4] # this is the NW prior to exit, it's not the NW variable of the banking sector
	Bhat_calib 		= out_calib[5]
	Btil_calib 		= out_calib[6]
	SP_calib 		= out_calib[9]
	YLD_calib 		= out_calib[10]
	Ct_calib 		= out_calib[14]
	SPDF_calib 		= out_calib[15]
	Dt_calib 		= out_calib[16]
	RDM_calib 	    = out_calib[19]
	TB_calib 		= out_calib[20]		
	XX_calib 		= Btil_calib*cm.μEM + RDM_calib.*ADM_calib[1:end-1]


	# Compute Asset to Equity Ratio
	Asset_NW 	  = (cm.μEM*Bhat_calib + ADM_calib[2:end])./NW_calib
	avg_ANW_ratio = mean(Asset_NW)

	misc_moments = [avg_ANW_ratio]

	open("simulated_moments/misc_moments.txt", "w") do io
	    writedlm(io, misc_moments, ',')
	end;


	# Fraciton of times constraint is not binding
	frac_bind = (1-sum(ADM_calib[2:end].==asm.ADMmax)/length(ADM_calib[2:end]))*100
	@printf("Fraction of times constraint binding %.1f\n", frac_bind)

	
    diff_rates      = asm.ReDM - asm.Red
    itp_diff_rates  = interpolate((cm.ωgrid, cm.aDMgrid), diff_rates, Gridded(Linear()))

	diff_rates_sim = zeros(size(NW_calib)) 
    for t=1:size(YEM_calib,1)
    	diff_rates_sim[t] = itp_diff_rates[ω_calib[t], ADM_calib[t+1]]
    end


	# Fraction of times ADM is above/below limit of grid
	pt_ADM_abv  = sum(ADM_calib.>cm.aDMgrid[end])/length(ADM_calib) * 100
	pt_ADM_blw  = sum(ADM_calib.<cm.aDMgrid[1])/length(ADM_calib) * 100
	@printf("Frac ADM above range = %.3g, below = %.3g\n", pt_ADM_abv, pt_ADM_blw)



	### ADDITIONAL SIMULATIONS
	## Simulate Market Value
	mktval_calib = simulate_mktval(cm, asm, ω_calib, ADM_calib, NW_calib)

	## Simulate High Yield and Total Assets
	HYSP_calib 	= simul_HYSP_fun(itp_HYSP, ω_calib, ADM_calib[2:end])
	At_calib 	= cm.μEM*Bhat_calib + ADM_calib[2:end]
	

	### COMPUTE MOMENTS FROM HISTOGRAM AND AGGREGATE SIMULATION

	## Aggregate consumption moments
	sd_lnCt = std(log.(Ct_calib))*100


	## Compute spread decomposition (default risk vs intermediation premium)
	EMSP_TOT = (mean(SP_calib))*10_000
	EMSP_DRK = (mean(SPDF_calib))*10_000
	EMSP_IMD = (EMSP_TOT - EMSP_DRK)

	# Risk-free priced by intermediary
	q_IR 	= compute_prices_nodefault(cm, asm,  reshape(asm.q_mat[1,end,:,:,:], asm.nYEM, cm.nω, cm.naDM));
	itp_qIR = interpolate((asm.YEMgrid, cm.ωgrid, cm.aDMgrid), q_IR, Gridded(Linear()))

	SP_IR = zeros(capT-dropT)
	for t=1:(capT-dropT)
		qIR_val  = itp_qIR[YEM_calib[t], ω_calib[t], ADM_calib[t+1]]
		SP_IR[t] = (1.0/qIR_val+cm.ξ-1.0) - (1.0/cm.βLD-1.0)
	end


	SP_IR_avg   = mean(SP_IR)*10_000
	SP_PureRisk = EMSP_IMD - SP_IR_avg

	# Save spread moments into matrix
	spread_dcp_hist = [EMSP_DRK; EMSP_TOT; EMSP_IMD; SP_IR_avg; SP_PureRisk; 0.0]


	## Additional moments
	mu_DM = mean(out_calib[2])/(mean(out_calib[2])+mean(out_calib[5])*cm.μEM)
	corr_HYSP_EMSP 	= corr_fun(HYSP_calib, SP_calib)


	## Save moments into matrix
	moments_histogram = [EMSP_TOT; mu_DM; corr_HYSP_EMSP]


	# Auxiliary function to compute further correlations
	function fun_temp_NW(Xin, SPin, HYSPin, YEMin)
		out1 = std(Xin)
		out2 = corr_fun(Xin, SPin)
		out3 = corr_fun(Xin, HYSPin)
		out4 = corr_fun(Xin, YEMin)
		return [out1; out2; out3; out4]
	end

	out_mm_NW 	 	= fun_temp_NW(NW_calib, SP_calib, HYSP_calib, YEM_calib) 			# Net Worth
	out_mm_mktvl 	= fun_temp_NW(mktval_calib, SP_calib, HYSP_calib, YEM_calib) 		# Market Value
	out_mm_lnNW	 	= fun_temp_NW(log.(NW_calib), SP_calib, HYSP_calib, YEM_calib) 		# Net Worth (in logs)
	out_mm_lnmktvl 	= fun_temp_NW(log.(mktval_calib), SP_calib, HYSP_calib, YEM_calib) 	# Market Value (in logs)

	out_mms_NW   = [out_mm_NW out_mm_mktvl out_mm_lnNW out_mm_lnmktvl]
	# it's 3x4, rows being std, corr(X,SP), corr(X,RDM); columns being each variable NW and mktval, in levels and in logs


	### Simulate panel
	capN = 1_000
	panel_vars = simul_panel_fun(cm, asm, [YEM_calib ω_calib ADM_calib[2:end]], capN, 0)

	out_panel_calib   = compute_moments_panel_calib(cm, asm, YEM_calib, ω_calib, ADM_calib, panel_vars, q_DFRSK, q_IR)
	# Output is:
	#						 1 			2			3				4			5				6				7				8				9				10					11				12				13		
	# out_cross_section = [cs_sd_by; cs_sd_SP; cs_corr_by_y; cs_corr_SP_y; cs_corr_sdby_y; cs_corr_sdSP_y; cs_corr_by_lny; cs_corr_SP_lny; cs_corr_sdby_lny; cs_corr_sdSP_lny; cs_corr_TB_lny; corr_spi_lnci; corr_spi_TBi]
	# 						1			2		3			4			5			6				7			8			9			10			11				12		13			14			16				16			17				18					19
	# out_time_series   = [mean_DY; DF_mean; mean_spread; std_spread; std_tb; mean(corr_sp_Y)*100; sdci_sdyi; corr_ci_yi; sd_TBi_yi; corr_TBi_yi; corr_spi_SP; std_tb2; sd_TBi_yi2; corr_TBi_yi2; ln_sdci_sdyi; ln_sd_TBi_yi; ln_corr_ci_yi; ln_corr_TBi_yi; ln_corr_SP_yi]
	# out_decomp        = [mean_SP_DFRSK/capN*10_000; mean_SP_TOTAL/capN*10_000; mean_SP_IMRSK/capN*10_000; std_SP_DFRSK/capN*10_000; std_SP_TOTAL/capN*10_000; std_SP_IMRSK/capN*10_000]
	# out_dcmp_interm   = [mean_SP_IR/capN*10_000; mean_SP_PR/capN*10_000; std_SP_IR/capN*10_000; std_SP_PR/capN*10_000; 0.0; 0.0]

	# use: out_decomp = [mean_SP_DFRSK/capN*10_000; mean_SP_TOTAL/capN*10_000; mean_SP_IMRSK/capN*10_000; std_SP_DFRSK/capN*10_000; std_SP_TOTAL/capN*10_000; std_SP_IMRSK/capN*10_000]


	###  WRITE TABLES WITH SIMULATED MOMENTS INTO TEXT FILES

	open("simulated_moments/NW_moments_calib.txt", "w") do io
	writedlm(io, out_mms_NW, ',')
	end

	open("simulated_moments/moments_histogram.txt", "w") do io
	    writedlm(io, moments_histogram, ',')
	end;

	open("simulated_moments/out_panel_moments_ts.txt", "w") do io
	    writedlm(io, out_panel_calib[2], ',')
	end;

	open("simulated_moments/out_panel_moments_cs.txt", "w") do io
	    writedlm(io, out_panel_calib[1], ',')
	end;

	open("simulated_moments/out_spread_decomp.txt", "w") do io
	    writedlm(io, [out_panel_calib[3] spread_dcp_hist out_panel_calib[4]], ',')
	end;



	### COMPUTE ELASTICITY
    ADMpcalib = ADM_calib[2:end]
    mean_ADM  = mean(ADMpcalib[ADMpcalib.<asm.ADMmax])
    mean_Dt   = mean(Dt_calib[1:end-1][ADMpcalib.<asm.ADMmax])

    YEM_path  = asm.YEMgrid[3]*ones(2)

    init_val = [mean_ADM; mean_Dt] # ADM and D

	init_ω   = 0.98

	function fun_tmp(omg_val)
		omg_path = [init_ω; omg_val]
		out_tmp  = compute_IRF_phi(cm, asm, init_val, omg_path, YEM_path, out_calib[17], itp_qDFRSK, itp_qIR)
		return (out_tmp[2]+0.12)^2.0
	end

	res   = Optim.optimize(fun_tmp, cm.ωgrid[1], cm.ωgrid[end], Brent(), iterations=50)
	ω_out = Optim.minimizer(res)

	omg_path = [init_ω; ω_out]
	out_beta = compute_IRF_phi(cm, asm, init_val, omg_path, YEM_path, out_calib[17], itp_qDFRSK, itp_qIR)
	# [ΔNW; ΔMV; ΔSP; ΔYLD; beta; ΔSPIR; ΔSPDF; ΔBt1]

	beta_IR  = out_beta[6]./out_beta[2] # Elasticity with no default risk

	open("simulated_moments/elasticity_moments.txt", "w") do io
	    writedlm(io, [out_beta; beta_IR], ',')
	end;



	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  ----------------------------------     BLOCK 3: LEHMAN BROTHERS EPISODE    --------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###


	## Initial values for ADM, Dt and the distribution
	init_state_LHM = [mean_ADM; mean_Dt]

	ΛLHMN = out_calib[17]


	## Data Counterparts 2007-2014

	# Non-normalized version
	YEM_data 	= [5.589304864; 5.766800791; -0.927928742; -0.640567159; 0.468040397; 0.189046504; -0.221100566; -1.792559773]/100
	lnmkv_data 	= [0.44876; -0.02947; -0.58661; -0.27721; -0.27322; -0.19787; 0.11443; 0.32736]
	Ct_data 	= [0.051559; 0.053934; -0.00144; -0.00297; 0.006019; 0.006464; 0.003013; -0.01444]
	EMSP_data 	= [173; 410; 575; 339; 364; 391; 347; 336]
	TB_data 	= [-0.023381745; -0.041299142; -0.005394627; -0.002458823; -0.01618504; -0.029529748; -0.029412709; -0.026419831]
	C_Y_data 	= [0.774198592; 0.777449429; 0.784081459; 0.776056767; 0.781374872; 0.78555429; 0.787168086; 0.78638947]
	data_in 	= [YEM_data EMSP_data lnmkv_data Ct_data TB_data*100 C_Y_data*100]


	μYEM_data, σYEM_data 		= -0.00113, 0.02877
	μlnmkv_data, σlnmkv_data 	= 0.0, 0.27819
	μCt_data, σCt_data 			= 0.0016, 0.03077
	μTB_data, σTB_data 			= -0.01859, 0.01058


	μYEM_model, σYEM_model 		= 0.0, sqrt(cm.σYEM^2/(1.0-cm.ρYEM^2))
	μlnmkv_model, σlnmkv_model 	= mean(log.(mktval_calib)), std(log.(mktval_calib))
	μCt_model, σCt_model 		= mean(log.(Ct_calib)), std(log.(Ct_calib))
	μTB_model, σTB_model 		= mean(TB_calib), std(TB_calib)
	μNW_model, σNW_model 		= mean(log.(NW_calib)), std(log.(NW_calib))

	mms_in_data  = [ [μYEM_data σYEM_data]; [μlnmkv_data σlnmkv_data]; [μCt_data σCt_data]; [μTB_data σTB_data] ]
	mms_in_model = [ [μYEM_model σYEM_model]; [μlnmkv_model σlnmkv_model]; [μCt_model σCt_model]; [μTB_model σTB_model] ]

	YEM_path 	= (YEM_data-μYEM_data)*(σYEM_model/σYEM_data)
	capT_Lehman = size(YEM_path, 1)

	omg_path 	  = ones(capT_Lehman)
	omg_path[1:4] = [cm.ωgrid[end-1]*1.08; cm.ωgrid[end-2]*0.92; cm.ωgrid[1]*1.064; cm.ωgrid[3]*0.98]
	omg_path[5:8] = [cm.ωgrid[3]*0.95; cm.ωgrid[3]*1.01; cm.ωgrid[4]; cm.ωgrid[4]]

	ΔYEM_dat 	= (YEM_path[3]-YEM_path[1])/σYEM_model
	Δmktval_dat = (lnmkv_data[3]-lnmkv_data[1])/σlnmkv_data

	pyplot()

	# Joint shocks YEM and ω
	# Btil_path Bhat_path yield_path EMSP_path HYSP_path NW_path MV_path ADM_path Ct_path TB_path
	LHM_pxy 	= compute_Lehman_proxy(cm, asm, init_state_LHM, omg_path, YEM_path, ΛLHMN, data_in, mms_in_model, mms_in_data, itp_HYSP, itp_qDFRSK);
	ΔSP_LHM 	= (LHM_pxy[3,4]-LHM_pxy[1,4])*10_000
	ΔCt_LHM_stn = (LHM_pxy[3,9]-LHM_pxy[1,9])
	ΔTB_LHM 	= (LHM_pxy[3,10]-LHM_pxy[1,10])
	ΔCt_LHM_dev = (LHM_pxy[3,13]-LHM_pxy[1,13])
	ΔYEM_LHM 	= ΔYEM_dat
	Δmktval_LHM = (LHM_pxy[3,7]-LHM_pxy[1,7])
	ΔNW_LHM 	= (LHM_pxy[3,6]-LHM_pxy[1,6])/σNW_model
	ΔSPDF_LHM 	= (LHM_pxy[3,14]-LHM_pxy[1,14])*10_000

	open("simulated_moments/out_Lehman.txt", "w") do io
	    writedlm(io, LHM_pxy, ',')
	end;


	# Only YEM shock
	YEMonly_pxy = compute_Lehman_proxy(cm, asm, init_state_LHM, ones(capT_Lehman), YEM_path, ΛLHMN, data_in, mms_in_model, mms_in_data, itp_HYSP, itp_qDFRSK, type_shock=1);
	ΔSP_YEM 	= (YEMonly_pxy[3,4]-YEMonly_pxy[1,4])*10_000
	ΔCt_YEM_stn = (YEMonly_pxy[3,9]-YEMonly_pxy[1,9])
	ΔTB_YEM 	= (YEMonly_pxy[3,10]-YEMonly_pxy[1,10])
	ΔCt_YEM_dev = (YEMonly_pxy[3,13]-YEMonly_pxy[1,13])
	ΔYEM_YEM 	= ΔYEM_dat
	Δmktval_YEM = (YEMonly_pxy[3,7]-YEMonly_pxy[1,7])
	ΔNW_YEM 	= (YEMonly_pxy[3,6]-YEMonly_pxy[1,6])/σNW_model
	ΔSPDF_YEM 	= (YEMonly_pxy[3,14]-YEMonly_pxy[1,14])*10_000

	# Only ωshock
	OMGonly_pxy = compute_Lehman_proxy(cm, asm, init_state_LHM, omg_path, zeros(capT_Lehman), ΛLHMN, data_in, mms_in_model, mms_in_data, itp_HYSP, itp_qDFRSK, type_shock=2);
	ΔSP_OMG 	= (OMGonly_pxy[3,4]-OMGonly_pxy[1,4])*10_000
	ΔCt_OMG_stn = (OMGonly_pxy[3,9]-OMGonly_pxy[1,9])
	ΔTB_OMG 	= (OMGonly_pxy[3,10]-OMGonly_pxy[1,10])
	ΔCt_OMG_dev = (OMGonly_pxy[3,13]-OMGonly_pxy[1,13])
	ΔYEM_OMG 	= 0.0
	Δmktval_OMG = (OMGonly_pxy[3,7]-OMGonly_pxy[1,7])
	ΔNW_OMG 	= (OMGonly_pxy[3,6]-OMGonly_pxy[1,6])/σNW_model
	ΔSPDF_OMG 	= (OMGonly_pxy[3,14]-OMGonly_pxy[1,14])*10_000


	# Addiotional moments
	Ct_data_stn = (Ct_data-μCt_data)/σCt_data
	Ct_data_dev = Ct_data-μCt_data

	ΔSP_dat 	= EMSP_data[3] - EMSP_data[1]
	ΔTB_dat 	= (TB_data[3] - TB_data[1])*100
	ΔCt_dat_stn = Ct_data_stn[3] - Ct_data_stn[1]
	ΔCt_dat_dev = Ct_data_dev[3] - Ct_data_dev[1]


	## Save moments for Lehman episode
	spread_LHM_decomp = [ [ΔSP_dat; ΔSP_LHM; ΔSP_YEM; ΔSP_OMG] [ΔTB_dat; ΔTB_LHM; ΔTB_YEM; ΔTB_OMG] [ΔCt_dat_stn; ΔCt_LHM_stn; ΔCt_YEM_stn; ΔCt_OMG_stn] [ΔCt_dat_dev; ΔCt_LHM_dev; ΔCt_YEM_dev; ΔCt_OMG_dev]*100 [ΔYEM_dat; ΔYEM_LHM; ΔYEM_YEM; ΔYEM_OMG] [Δmktval_dat; Δmktval_LHM; Δmktval_YEM; Δmktval_OMG] [0.0; ΔSPDF_LHM; ΔSPDF_YEM; ΔSPDF_OMG] ]

	open("simulated_moments/out_spread_LHM_decomp.txt", "w") do io
	    writedlm(io, spread_LHM_decomp, ',')
	end;

end




################# ----------------------------------------------------------------------------- ##################
################# --------------------- *** LIQUIDITY POLICY EXERCISE *** --------------------- ##################
################# ----------------------------------------------------------------------------- ##################

if do_fall_omega


	cm  = load("model_data/NAG_module_structure.jld", "cm");
	asm = load("model_data/ASM_module_structure.jld", "asm");


	capT    = 1_000
	dropT 	= 100
	YEM_sim, ω_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)

    q_DFRSK 	= load("model_data/q_DFRSK.jld", "q_DFRSK")
    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))

    out_calib 	= time_iteration_calib(cm, asm, capT, dropT, YEM_sim, ω_sim, itp_qDFRSK)

	Dt_calib   = out_calib[16]
	ADM_calib  = out_calib[2] 	

	# Initial conditions
    ADMpcalib = ADM_calib[2:end]
    mean_ADM  = mean(ADMpcalib[ADMpcalib.<asm.ADMmax])
    mean_Dt   = mean(Dt_calib[1:end-1][ADMpcalib.<asm.ADMmax])	

	capT_ωFall = 5

	init_state = [mean_ADM; mean_Dt]
	
	omg_path 	= ones(capT_ωFall)
	YEM_path 	= zeros(capT_ωFall)
	omg_path[1] = 1.06
	omg_path[2] = omg_path[1] -2.0*cm.σω	


	Λ_init = out_calib[17]


	# Only ωshock
	OMGfall_out = compute_fall_omega(cm, asm, init_state, omg_path, YEM_path, Λ_init, itp_qDFRSK)

	open("simulated_moments/OMGfall_out.txt", "w") do io
	    writedlm(io, OMGfall_out, ',')
	end;

	open("simulated_moments/OMGfall_init_cond.txt", "w") do io
	    writedlm(io, init_state, ',')
	end;

	open("simulated_moments/OMGfall_init_distrib.txt", "w") do io
	    writedlm(io, Λ_init, ',')
	end;

end





################# ----------------------------------------------------------------------------- ##################
################# ------------- *** AMPLIFICATION UNDER HIGH AND LOW EXPOSURE *** ------------- ##################
################# ----------------------------------------------------------------------------- ##################


if do_amplif_mod


	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  ------------------------------------    BLOCK 1: LOAD MODEL AND SIMULATE    -------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###

	cm  = load("model_data/NAG_module_structure.jld", "cm");
	asm = load("model_data/ASM_module_structure.jld", "asm");

    q_DFRSK 	= load("model_data/q_DFRSK.jld", "q_DFRSK")
    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))

	capT    = 1_000
	dropT 	= 100

	YEM_sim, ω_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)


    capT_Lehman = 8

	# Choose path for output. It will be taken by YEM or ZEM depending on source of shock.
	Ypath_amplf    = zeros(capT_Lehman)
	Ypath_amplf[1] = -0.02 #-0.03 #asm.YEMgrid[3]
	Ypath_amplf[2] = Ypath_amplf[1]-2.5*cm.σYEM

	for t=3:capT_Lehman
		Ypath_amplf[t]=Ypath_amplf[t-1]*cm.ρYEM
	end

	YEM_amplf_path = ones(capT_Lehman)*Ypath_amplf[1] # path taken by YEM when ZEM shock
	ZEM_amplf_path = ones(capT_Lehman)*Ypath_amplf[1] # path taken by ZEM when YEM shock


	ωpath_amplf_LE  = ones(capT_Lehman)*cm.ωgrid[3]
	ωpath_amplf_HE  = ones(capT_Lehman)*cm.ωgrid[3]
	ωpath_amplf_mus = ones(capT_Lehman)*cm.ωgrid[3]



	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  ------------------------------------------    BLOCK 2: LOW EXPOSURE    ------------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###


	### SIMULATION: LOW EXPOSURE - LOW VARIANCE ###
	out_calib_LE    = time_iteration_calib(cm, asm, capT, dropT, YEM_sim, ω_sim, itp_qDFRSK)
	exposure_LE     = mean(cm.μEM*out_calib_LE[5]./(out_calib_LE[2][2:end]+cm.μEM*out_calib_LE[5]))
    Bmean_org_LE    = dot(out_calib_LE[17][cm.num_nodes+1:end], cm.snodes[:,1])
    Bstdv_org_LE    = sqrt(dot(out_calib_LE[17][cm.num_nodes+1:end], (cm.snodes[:,1]-Bmean_org_LE).^2.0))
	hit_bound_LE 	= sum(out_calib_LE[4].<=1e-6)

	ADM_calib_LE 	= out_calib_LE[2]
	D_calib_LE 		= out_calib_LE[16]
	ADMp0_amplf_LE 	= mean(ADM_calib_LE[ADM_calib_LE.<asm.ADMmax])
	D0_amplf_LE 	= mean(D_calib_LE[ADM_calib_LE.<asm.ADMmax])


	# Compute IRF
	out_amplf_YEM_LE = compute_amplification_proxy(cm, asm, [ADMp0_amplf_LE; D0_amplf_LE], ωpath_amplf_LE, Ypath_amplf, ZEM_amplf_path, out_calib_LE[17], itp_qDFRSK)
	out_amplf_ZEM_LE = compute_amplification_proxy(cm, asm, [ADMp0_amplf_LE; D0_amplf_LE], ωpath_amplf_LE, YEM_amplf_path, Ypath_amplf, out_calib_LE[17], itp_qDFRSK)





	# LOW EXPOSURE - CHANGE IN DISTRIBUTION
	avgcf   	= linspace(0.0,0.08,6)
	Nsd     	= length(avgcf)
	amp_mat_LE 	= zeros(Nsd, 4)

	# out_dist_tmp = []
	out_ampf_LE, out_ADM_LE, out_NW_LE, out_RDM_LE 	 = [], [], [], []
	out_ct_LE, out_Btil_LE, out_SP_LE  = [], [], []
	
	for ii=1:Nsd
		out_chng_dist = change_distribution_bimodal(cm, asm, out_calib_LE[17], avgcf[ii])

	    Tvec_plot = collect(2007:1:2014)

		# Btil_path Bhat_path DF_path EMSP_path NW_path ADM_path
		out_amplf_YEM = compute_amplification_proxy(cm, asm, [ADMp0_amplf_LE; D0_amplf_LE], ωpath_amplf_LE, Ypath_amplf, ZEM_amplf_path, out_chng_dist[1], itp_qDFRSK)
		out_amplf_ZEM = compute_amplification_proxy(cm, asm, [ADMp0_amplf_LE; D0_amplf_LE], ωpath_amplf_LE, YEM_amplf_path, Ypath_amplf, out_chng_dist[1], itp_qDFRSK)
		out_amplf_NS  = compute_amplification_proxy(cm, asm, [ADMp0_amplf_LE; D0_amplf_LE], ωpath_amplf_LE, YEM_amplf_path, ZEM_amplf_path, out_chng_dist[1], itp_qDFRSK)

		amp_mat_LE[ii, 1] = (out_amplf_YEM[2,4]-out_amplf_ZEM[2,4])*10_000
		amp_mat_LE[ii, 2] = out_chng_dist[2]
		amp_mat_LE[ii, 3] = out_chng_dist[3]
		amp_mat_LE[ii, 4] = maximum(out_amplf_YEM[:,6])

		# push!(out_dist_tmp, out_chng_dist[1])
		push!(out_ampf_LE, [out_amplf_YEM[:,4] out_amplf_ZEM[:,4] out_amplf_NS[:,4]])
		push!(out_ADM_LE, [out_amplf_YEM[:,7] out_amplf_ZEM[:,7] out_amplf_NS[:,7]])		
		push!(out_RDM_LE, [out_amplf_YEM[:,6] out_amplf_ZEM[:,6] out_amplf_NS[:,6]])
		push!(out_ct_LE, [out_amplf_YEM[:,9] out_amplf_ZEM[:,9] out_amplf_NS[:,9]])
		push!(out_Btil_LE, [out_amplf_YEM[:,1] out_amplf_ZEM[:,1] out_amplf_NS[:,1]])
		push!(out_NW_LE, [out_amplf_YEM[:,5] out_amplf_ZEM[:,5] out_amplf_NS[:,5]])
		push!(out_SP_LE, [out_amplf_YEM[:,4] out_amplf_ZEM[:,4] out_amplf_NS[:,4]])
	end

	SD_relative_LE = amp_mat_LE[:,3]/Bstdv_org_LE

	ind_HV_LE = 5





	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  -----------------------------------------    BLOCK 3: HIGH EXPOSURE    ------------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###

    cm.μEM = 9.2;
	out_calib_HE   = time_iteration_calib(cm, asm, capT, dropT, YEM_sim, ω_sim, itp_qDFRSK)
	exposure_HE    = mean(cm.μEM*out_calib_HE[5]./(out_calib_HE[2][2:end]+cm.μEM*out_calib_HE[5]))
    Bmean_org_HE   = dot(out_calib_HE[17][cm.num_nodes+1:end], cm.snodes[:,1])
    Bstdv_org_HE   = sqrt(dot(out_calib_HE[17][cm.num_nodes+1:end], (cm.snodes[:,1]-Bmean_org_HE).^2.0))
	
	exposure_val_HE = mean(cm.μEM*out_calib_HE[5]./(out_calib_HE[2][2:end]+cm.μEM*out_calib_HE[5]))
	hit_bound_HE 	= sum(out_calib_HE[4].<=1e-6)

	ADM_calib_HE 	= out_calib_HE[2]
	D_calib_HE      = out_calib_HE[16]
	
	ADMp0_amplf_HE 	  = mean(ADM_calib_HE[ADM_calib_HE.<asm.ADMmax]);
	D0_amplf_HE 	  = mean(D_calib_HE[ADM_calib_HE.<asm.ADMmax])
	ωpath_amplf_HE    = ones(capT_Lehman)*cm.ωgrid[3];

	out_amplf_YEM_HE = compute_amplification_proxy(cm, asm, [ADMp0_amplf_HE; D0_amplf_HE], ωpath_amplf_HE, Ypath_amplf, ZEM_amplf_path, out_calib_HE[17], itp_qDFRSK)
	out_amplf_ZEM_HE = compute_amplification_proxy(cm, asm, [ADMp0_amplf_HE; D0_amplf_HE], ωpath_amplf_HE, YEM_amplf_path, Ypath_amplf, out_calib_HE[17], itp_qDFRSK)


	# HIGH EXPOSURE - CHANGE IN DISTRIBUTION
	avgcf   = linspace(0.0,0.12,6)
	Nsd     = length(avgcf)
	amp_mat_HE = zeros(Nsd, 4)

	# out_dist_tmp = []
	out_ampf_HE, out_ADM_HE, out_NW_HE, out_RDM_HE 	 = [], [], [], []
	out_ct_HE, out_Btil_HE, out_SP_HE  = [], [], []
	
	for ii=1:Nsd
		out_chng_dist = change_distribution_bimodal(cm, asm, out_calib_HE[17], avgcf[ii])

	    Tvec_plot = collect(2007:1:2014)

		# Btil_path Bhat_path DF_path EMSP_path NW_path ADM_path
		out_amplf_YEM = compute_amplification_proxy(cm, asm, [ADMp0_amplf_HE; D0_amplf_HE], ωpath_amplf_HE, Ypath_amplf, ZEM_amplf_path, out_chng_dist[1], itp_qDFRSK)
		out_amplf_ZEM = compute_amplification_proxy(cm, asm, [ADMp0_amplf_HE; D0_amplf_HE], ωpath_amplf_HE, YEM_amplf_path, Ypath_amplf, out_chng_dist[1], itp_qDFRSK)
		out_amplf_NS  = compute_amplification_proxy(cm, asm, [ADMp0_amplf_HE; D0_amplf_HE], ωpath_amplf_HE, YEM_amplf_path, ZEM_amplf_path, out_chng_dist[1], itp_qDFRSK)

		amp_mat_HE[ii, 1] = (out_amplf_YEM[2,4]-out_amplf_ZEM[2,4])*10_000
		amp_mat_HE[ii, 2] = out_chng_dist[2]
		amp_mat_HE[ii, 3] = out_chng_dist[3]
		amp_mat_HE[ii, 4] = maximum(out_amplf_YEM[:,6])

		# push!(out_dist_tmp, out_chng_dist[1])
		push!(out_ampf_HE, [out_amplf_YEM[:,4] out_amplf_ZEM[:,4] out_amplf_NS[:,4]])
		push!(out_ADM_HE, [out_amplf_YEM[:,7] out_amplf_ZEM[:,7] out_amplf_NS[:,7]])		
		push!(out_RDM_HE, [out_amplf_YEM[:,6] out_amplf_ZEM[:,6] out_amplf_NS[:,6]])
		push!(out_ct_HE, [out_amplf_YEM[:,9] out_amplf_ZEM[:,9] out_amplf_NS[:,9]])
		push!(out_Btil_HE, [out_amplf_YEM[:,1] out_amplf_ZEM[:,1] out_amplf_NS[:,1]])
		push!(out_NW_HE, [out_amplf_YEM[:,5] out_amplf_ZEM[:,5] out_amplf_NS[:,5]])
		push!(out_SP_HE, [out_amplf_YEM[:,4] out_amplf_ZEM[:,4] out_amplf_NS[:,4]])
	end

	SD_relative_HE = amp_mat_HE[:,3]/Bstdv_org_HE

	ind_HV_HE = 5



	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  -----------------------------------------    BLOCK 4: PLOTS FOR PAPER    ----------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###

	pyplot()

    ## Amplification by dispersion of distribution
    fig, ax = PyPlot.subplots(figsize=(9,5))
    ax[:plot](SD_relative_LE, amp_mat_LE[:, 1], linestyle="-", linewidth=2.5, label="Low Exposure")
    ax[:plot](SD_relative_LE, amp_mat_HE[:, 1], color="black", linestyle="--", linewidth=2.5, label="High Exposure")
    ax[:legend](loc=2, frameon=false)
    ax[:set_xlabel]("x Ergodic Dispersion of EMs' Debt Distribution")
    PyPlot.grid("on", alpha=0.3)
    PyPlot.tight_layout()
	PyPlot.savefig("figures/amplification/LEvsHE_bySD.pdf")


end




