
###  -----------------------------------------------------------------------------------------------------------------  ###
###  ----------------------------------     BLOCK 1: LOAD MARKET STRUCTURE    ---------------------------------------  ###
###  -----------------------------------------------------------------------------------------------------------------  ###
cm  = load("model_data/NAG_module_structure.jld", "cm")
asm = load("model_data/ASM_module_structure.jld", "asm")


capT    = 1_000
dropT 	= 100
YEM_sim, ω_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)

q_DFRSK 	= load("model_data/q_DFRSK.jld", "q_DFRSK")
knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))


###  -----------------------------------------------------------------------------------------------------------------  ###
###  ----------------------------------------     BLOCK 2: SIMULATION    ---------------------------------------------  ###
###  -----------------------------------------------------------------------------------------------------------------  ###

out_calib 	= time_iteration_calib(cm, asm, capT, dropT, YEM_sim, ω_sim, itp_qDFRSK)

## Simulate High Yield using simulation results
HYSPmat  = compute_HYSP(cm, asm)
itp_HYSP = interpolate((cm.ωgrid, cm.aDMgrid), HYSPmat, Gridded(Linear()))

function simul_HYSP_fun(itp_HYSP, ωvals, ADMpvals)
	HYSPsim = zeros(length(ωvals))
	for t=1:length(ωvals)
		HYSPsim[t] = itp_HYSP[ωvals[t], ADMpvals[t]]
	end
	return HYSPsim
end


###  EXTRACT ELEMENTS FROM THE SIMULATION
# The time iteration for calibration returns:
# 	out_calib =	Bvec, aDMvec, ReEMvec, NWvec, Bhatvec, Btildevec, def_measure, ReDMvec, SPvec, YLDvec, by_vec, sdby_vec, sdSP_vec, Ctvec, DFRSKvec, Dvec, Λ_erg, aDMvec2, RDMvec, TB_vec, C_Y_vec
# 				 1       2       3       4       5         6          7           8        9      10      11       12       13      14        15      16    17     18       19       20      21	


## Extract simulations
YEM_calib 		= YEM_sim[dropT+1:end]
ω_calib 		= ω_sim[dropT+1:end]
ADM_calib 		= out_calib[2] 			# ADM_t  goes until T+1
NW_calib 		= out_calib[4]
Bhat_calib 		= out_calib[5]
Btil_calib 		= out_calib[6]
Ct_calib 		= out_calib[14]
RDM_calib 	    = out_calib[19]
TB_calib 		= out_calib[20]
SP_calib 		= out_calib[9]
Dt_calib 		= out_calib[16]
ReDM_calib 		= out_calib[3]


### ADDITIONAL SIMULATIONS
## Simulate Market Value
mktval_calib = simulate_mktval(cm, asm, ω_calib, ADM_calib, NW_calib)



###  -----------------------------------------------------------------------------------------------------------------  ###
###  ----------------------------------     BLOCK 3: LEHMAN BROTHERS EPISODE    --------------------------------------  ###
###  -----------------------------------------------------------------------------------------------------------------  ###


ΛLHMN = out_calib[17]

# Boom and bust period

# Non-normalized version (2004-2014)
YEM_data 	= [-0.929749943; 0.953744911; 3.145456314; 5.589304864; 5.766800791; -0.927928742; -0.640567159; 0.468040397; 0.189046504; -0.221100566; -1.792559773]/100
lnmkv_data 	= [0.19204; 0.25271; 0.39928; 0.44876; -0.02947; -0.58661; -0.27721; -0.27322; -0.19787; 0.11443; 0.32736]
Ct_data 	= [-0.01724; 0.00805; 0.02524; 0.051559; 0.053934; -0.00144; -0.00297; 0.006019; 0.006464; 0.003013; -0.01444]
EMSP_data 	= [255; 202; 165; 173; 410; 575; 339; 364; 391; 347; 336]
TB_data 	= [-0.006915747; -0.019959807; -0.012992907; -0.023381745; -0.041299142; -0.005394627; -0.002458823; -0.01618504; -0.029529748; -0.029412709; -0.026419831]
C_Y_data 	= [0.761126220; 0.765791476; 0.764668882; 0.774198592; 0.777449429; 0.784081459; 0.776056767; 0.781374872; 0.78555429; 0.787168086; 0.78638947]
data_in 	= [YEM_data EMSP_data lnmkv_data Ct_data TB_data*100 C_Y_data*100]



μYEM_data, σYEM_data 		= -0.00113, 0.02877
μlnmkv_data, σlnmkv_data 	= 0.0, 0.27819
μCt_data, σCt_data 			= 0.0016, 0.03077
μTB_data, σTB_data 			= -0.01859, 0.01058


μYEM_model, σYEM_model 		= 0.0, sqrt(cm.σYEM^2/(1.0-cm.ρYEM^2))
μlnmkv_model, σlnmkv_model 	= mean(log.(mktval_calib)), std(log.(mktval_calib))
μCt_model, σCt_model 		= mean(log.(Ct_calib)), std(log.(Ct_calib))
μTB_model, σTB_model 		= mean(TB_calib), std(TB_calib)
μNW_model, σNW_model 		= mean(log.(NW_calib)), std(log.(NW_calib))

mms_in_data  = [ [μYEM_data σYEM_data]; [μlnmkv_data σlnmkv_data]; [μCt_data σCt_data]; [μTB_data σTB_data] ]
mms_in_model = [ [μYEM_model σYEM_model]; [μlnmkv_model σlnmkv_model]; [μCt_model σCt_model]; [μTB_model σTB_model] ]

YEM_path 	= (YEM_data-μYEM_data)*(σYEM_model/σYEM_data)
capT_Lehman = size(YEM_path, 1)

omg_path 	   = ones(capT_Lehman)
omg_path[1:3]  = [cm.ωgrid[end-1]*0.99; cm.ωgrid[end-1]*0.98; cm.ωgrid[end-1]*0.98]
omg_path[4:7]  = [cm.ωgrid[end-1]*0.965; cm.ωgrid[end-2]*0.9; cm.ωgrid[end-2]*0.865; cm.ωgrid[end-2]*0.98]
omg_path[8:11] = [cm.ωgrid[end-2]*0.95; cm.ωgrid[end-2]*0.955; cm.ωgrid[end-2]*0.955; cm.ωgrid[end-2]*0.955]


ΔYEM_dat 	= (YEM_path[6]-YEM_path[4])/σYEM_model
Δmktval_dat = (lnmkv_data[6]-lnmkv_data[4])/σlnmkv_data

init_state_LHM = [cm.aDMgrid[3]; mean(Dt_calib)]

pyplot()
# Joint shocks YEM and ω
# Btil_path Bhat_path yield_path EMSP_path HYSP_path NW_path MV_path ADM_path Ct_path TB_path
LHM_pxy 	= compute_Lehman_proxy(cm, asm, init_state_LHM, omg_path, YEM_path, ΛLHMN, data_in, mms_in_model, mms_in_data, itp_HYSP, itp_qDFRSK, full_dyn=true);
ΔSP_LHM 	= (LHM_pxy[6,4]-LHM_pxy[4,4])*10_000
ΔCt_LHM_stn = (LHM_pxy[6,9]-LHM_pxy[4,9])
ΔTB_LHM 	= (LHM_pxy[6,10]-LHM_pxy[4,10])
ΔCt_LHM_dev = (LHM_pxy[6,13]-LHM_pxy[4,13])
ΔYEM_LHM 	= ΔYEM_dat
Δmktval_LHM = (LHM_pxy[6,7]-LHM_pxy[4,7])
ΔNW_LHM 	= (LHM_pxy[6,6]-LHM_pxy[4,6])/σNW_model
ΔSPDF_LHM 	= (LHM_pxy[6,14]-LHM_pxy[4,14])*10_000


# Only YEM shock
YEMonly_pxy = compute_Lehman_proxy(cm, asm, init_state_LHM, ones(capT_Lehman), YEM_path, ΛLHMN, data_in, mms_in_model, mms_in_data, itp_HYSP, itp_qDFRSK, type_shock=1, full_dyn=true);
ΔSP_YEM 	= (YEMonly_pxy[6,4]-YEMonly_pxy[4,4])*10_000
ΔCt_YEM_stn = (YEMonly_pxy[6,9]-YEMonly_pxy[4,9])
ΔTB_YEM 	= (YEMonly_pxy[6,10]-YEMonly_pxy[4,10])
ΔCt_YEM_dev = (YEMonly_pxy[6,13]-YEMonly_pxy[4,13])
ΔYEM_YEM 	= ΔYEM_dat
Δmktval_YEM = (YEMonly_pxy[6,7]-YEMonly_pxy[4,7])
ΔNW_YEM 	= (YEMonly_pxy[6,6]-YEMonly_pxy[4,6])/σNW_model
ΔSPDF_YEM 	= (YEMonly_pxy[6,14]-YEMonly_pxy[4,14])*10_000

# Only ωshock
OMGonly_pxy = compute_Lehman_proxy(cm, asm, init_state_LHM, omg_path, zeros(capT_Lehman), ΛLHMN, data_in, mms_in_model, mms_in_data, itp_HYSP, itp_qDFRSK, type_shock=2, full_dyn=true);
ΔSP_OMG 	= (OMGonly_pxy[6,4]-OMGonly_pxy[4,4])*10_000
ΔCt_OMG_stn = (OMGonly_pxy[6,9]-OMGonly_pxy[4,9])
ΔTB_OMG 	= (OMGonly_pxy[6,10]-OMGonly_pxy[4,10])
ΔCt_OMG_dev = (OMGonly_pxy[6,13]-OMGonly_pxy[4,13])
ΔYEM_OMG 	= 0.0
Δmktval_OMG = (OMGonly_pxy[6,7]-OMGonly_pxy[4,7])
ΔNW_OMG 	= (OMGonly_pxy[6,6]-OMGonly_pxy[4,6])/σNW_model
ΔSPDF_OMG 	= (OMGonly_pxy[6,14]-OMGonly_pxy[4,14])*10_000



# Addiotional moments
Ct_data_stn = (Ct_data-μCt_data)/σCt_data
Ct_data_dev = Ct_data-μCt_data

ΔSP_dat 	= EMSP_data[3] - EMSP_data[1]
ΔTB_dat 	= (TB_data[3] - TB_data[1])*100
ΔCt_dat_stn = Ct_data_stn[3] - Ct_data_stn[1]
ΔCt_dat_dev = Ct_data_dev[3] - Ct_data_dev[1]



## Save moments for Lehman episode
spread_LHM_decomp = [ [ΔSP_dat; ΔSP_LHM; ΔSP_YEM; ΔSP_OMG] [ΔTB_dat; ΔTB_LHM; ΔTB_YEM; ΔTB_OMG] [ΔCt_dat_stn; ΔCt_LHM_stn; ΔCt_YEM_stn; ΔCt_OMG_stn] [ΔCt_dat_dev; ΔCt_LHM_dev; ΔCt_YEM_dev; ΔCt_OMG_dev]*100 [ΔYEM_dat; ΔYEM_LHM; ΔYEM_YEM; ΔYEM_OMG] [Δmktval_dat; Δmktval_LHM; Δmktval_YEM; Δmktval_OMG] [0.0; ΔSPDF_LHM; ΔSPDF_YEM; ΔSPDF_OMG] ]

open("simulated_moments/out_spread_LHM_decomp_BoomBust.txt", "w") do io
    writedlm(io, spread_LHM_decomp, ',')
end;

open("simulated_moments/out_Lehman_BoomBust.txt", "w") do io
    writedlm(io, LHM_pxy, ',')
end;
