
###  -----------------------------------------------------------------------------------------------------------------  ###
###  ------------------------------------     BLOCK 1: LOAD MODEL AND SIMULATE    ------------------------------------  ###
###  -----------------------------------------------------------------------------------------------------------------  ###

cm  = load("model_data/NAG_module_structure.jld", "cm");
asm = load("model_data/ASM_module_structure.jld", "asm");

capT    = 1_000
dropT 	= 100
YEM_sim, ω_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)

q_DFRSK 	= load("model_data/q_DFRSK.jld", "q_DFRSK")
knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))


out_simul 	= time_iteration_calib(cm, asm, capT, dropT, YEM_sim, ω_sim, itp_qDFRSK)


# #  EXTRACT ELEMENTS FROM THE SIMULATION
# The time iteration for calibration returns:
# Bvec, aDMvec, ReEMvec, NWvec, Bhatvec, Btildevec, frac_rep, def_measure, ReDMvec, SPvec, DFpbvec, by_vec, sdby_vec, sdSP_vec, Ctvec, DFRSKvec, Dvec, Λ_erg, aDMvec2, RDMvec, TB_vec, C_Y_vec
# 1       2       3       4       5         6         7           8           9      10       11       12       13      14        15      16      17    18     19       20       21      22

## Extract simulations
YEM_simul 		= YEM_sim[dropT+1:end]
ω_simul 		= ω_sim[dropT+1:end]
ADM_simul 		= out_simul[2] # ADM_t  goes until T+1
NW_simul  		= out_simul[4] # this is the NW prior to exit, it's not the NW variable of the banking sector
ReDM_simul 		= out_simul[3]
avg_assets_NW 	= mean((cm.μEM*out_simul[5] + ADM_simul[2:end])./NW_simul)
mktval_simul  	= simulate_mktval(cm, asm, ω_simul, ADM_simul, NW_simul)


# Simulation of SDF
m_simul = zeros(length(ω_simul)) # log(Bank_SDF)
itp_vs  = interpolate((cm.ωgrid, cm.aDMgrid), asm.vs, Gridded(Linear()))

for t=2:length(ω_simul)
	nu_v 		= itp_vs[ω_simul[t], ADM_simul[t+1]]
    m_simul[t] 	= log(nu_v/ReDM_simul[t-1])
end

m_simul[1] = log(cm.βLD)



XX_mkt   = [ones(size(m_simul[2:end])) (log.(mktval_simul[2:end])-log.(mktval_simul[1:end-1]))]



##  -----------------------------------------------------------------------------------------------------------------  ###
##  ----------------------------------------     BLOCK 2: FORM PORTFOLIOS    ----------------------------------------  ###
##  -----------------------------------------------------------------------------------------------------------------  ###


capN_pn    = 1_000
dropT_pn   = 100
panel_vars = simul_panel_fun(cm, asm, [YEM_simul ω_simul ADM_simul[2:end]], capN_pn, dropT_pn)
# (zsim, bsim, def_sim, spsim, ysim, csim, bpy_sim, retEM)

capT_pn  = length(panel_vars[1][:,1])
wnd_roll = 5 # for rolling beta
rebal_T  = 5 # for portfolio rebalancing frequency


roll_beta = zeros(capT_pn, capN_pn)


Rfactor = [0; log.(mktval_simul[2:end]) - log.(mktval_simul[1:end-1])]
Rfactor = Rfactor[dropT_pn+1:end]


## Rolling betas
for ii=1:capN_pn
	retEM = panel_vars[8][:,ii]-1/cm.βLD # excess returns
	dfsim = panel_vars[3][:,ii]

	for tau=0:capT_pn-wnd_roll
		init_t = 1+tau
		last_t = wnd_roll+tau		
		if ( sum(dfsim[init_t:last_t])==0 & sum(isnan.(Rfactor[init_t:last_t]))==0 )
			XX_mkt = [ones(wnd_roll) Rfactor[init_t:last_t]]
			roll_beta[last_t, ii] = (inv(XX_mkt'*XX_mkt)*XX_mkt'*retEM[init_t:last_t])[2]
		else
			roll_beta[last_t, ii] = NaN
		end
	end
end



avg_ptf = zeros(capT_pn,6); std_ptf = zeros(capT_pn,6);


retEM_panel = panel_vars[8]-1/cm.βLD  # excess returns
b_panel 	= panel_vars[2]
sp_panel 	= panel_vars[4]

ptf1_p1, ptf2_p1, ptf3_p1, ptf4_p1, ptf5_p1, ptf6_p1 = [], [], [], [], [], []
ptf1, ptf2, ptf3, ptf4, ptf5, ptf6 = [], [], [], [], [], []

betas_cell = []

x_std_sp = 1.0

tau = 0

for tt=wnd_roll:(capT_pn-wnd_roll)

	tau +=1

	beta_vals  = vec(roll_beta[tt, :])
	retEM_vals = vec(retEM_panel[tt,:]) # excess returns
	sp_vals    = vec(sp_panel[tt,:])

	cond1 = beta_vals.!=NaN
	cond2 = 100.>beta_vals.>=-0.0
	cond3 = -0.8.<retEM_vals.<0.8
	cond_all = cond1.*cond2.*cond3

	push!(betas_cell, beta_vals[cond1.*(100.>beta_vals.>=-100.0)])

	beta_filt = beta_vals[cond_all]
	avg_beta  = mean(beta_filt)
	std_beta  = std(beta_filt)

	# Portfolio re-balancing
	if tt==wnd_roll		

		ptf_beta  = beta_vals.<avg_beta+0.6*std_beta

		avg_SP_Lbeta = mean(sp_vals[cond_all.*(ptf_beta.==1)])
		avg_SP_Hbeta = mean(sp_vals[cond_all.*(ptf_beta.==0)])
		std_SP_Lbeta = std(sp_vals[cond_all.*(ptf_beta.==1)])
		std_SP_Hbeta = std(sp_vals[cond_all.*(ptf_beta.==0)])

		ptf1_p1 = (ptf_beta.==1) .* (sp_vals.<(avg_SP_Lbeta-x_std_sp*std_SP_Lbeta))
		ptf2_p1 = (ptf_beta.==1) .* (sp_vals.>=(avg_SP_Lbeta-x_std_sp*std_SP_Lbeta)) .* (sp_vals.<=(avg_SP_Lbeta+x_std_sp*std_SP_Lbeta))
		ptf3_p1 = (ptf_beta.==1) .* (sp_vals.>(avg_SP_Lbeta+x_std_sp*std_SP_Lbeta))
		ptf4_p1 = (ptf_beta.==0) .* (sp_vals.<(avg_SP_Hbeta-x_std_sp*std_SP_Hbeta))
		ptf5_p1 = (ptf_beta.==0) .* (sp_vals.>=(avg_SP_Hbeta-x_std_sp*std_SP_Hbeta)) .* (sp_vals.<=(avg_SP_Hbeta+x_std_sp*std_SP_Hbeta))
		ptf6_p1 = (ptf_beta.==0) .* (sp_vals.>(avg_SP_Hbeta+x_std_sp*std_SP_Hbeta))

		ptf1 = copy(ptf1_p1)
		ptf2 = copy(ptf2_p1)
		ptf3 = copy(ptf3_p1)
		ptf4 = copy(ptf4_p1)
		ptf5 = copy(ptf5_p1)
		ptf6 = copy(ptf6_p1)

	end
	if tau == rebal_T
		ptf_beta  = beta_vals.<avg_beta+0.6*std_beta

		avg_SP_Lbeta = mean(sp_vals[cond_all.*(ptf_beta.==1)])
		avg_SP_Hbeta = mean(sp_vals[cond_all.*(ptf_beta.==0)])
		std_SP_Lbeta = std(sp_vals[cond_all.*(ptf_beta.==1)])
		std_SP_Hbeta = std(sp_vals[cond_all.*(ptf_beta.==0)])

		# Portfolios to be used on the next rebalancing
		ptf1_p1 = (ptf_beta.==1) .* (sp_vals.<(avg_SP_Lbeta-x_std_sp*std_SP_Lbeta))
		ptf2_p1 = (ptf_beta.==1) .* (sp_vals.>=(avg_SP_Lbeta-x_std_sp*std_SP_Lbeta)) .* (sp_vals.<=(avg_SP_Lbeta+x_std_sp*std_SP_Lbeta))
		ptf3_p1 = (ptf_beta.==1) .* (sp_vals.>(avg_SP_Lbeta+x_std_sp*std_SP_Lbeta))
		ptf4_p1 = (ptf_beta.==0) .* (sp_vals.<(avg_SP_Hbeta-x_std_sp*std_SP_Hbeta))
		ptf5_p1 = (ptf_beta.==0) .* (sp_vals.>=(avg_SP_Hbeta-x_std_sp*std_SP_Hbeta)) .* (sp_vals.<=(avg_SP_Hbeta+x_std_sp*std_SP_Hbeta))
		ptf6_p1 = (ptf_beta.==0) .* (sp_vals.>(avg_SP_Hbeta+x_std_sp*std_SP_Hbeta))

		tau = 0

	end

	# Filter for those who defaulted (or have atypical betas)
	ptf1_f = ptf1.*cond_all
	ptf2_f = ptf2.*cond_all
	ptf3_f = ptf3.*cond_all
	ptf4_f = ptf4.*cond_all
	ptf5_f = ptf5.*cond_all
	ptf6_f = ptf6.*cond_all

	avg_ptf[tt, :] = [mean(retEM_vals[ptf1_f.==1]) mean(retEM_vals[ptf2_f.==1]) mean(retEM_vals[ptf3_f.==1]) mean(retEM_vals[ptf4_f.==1]) mean(retEM_vals[ptf5_f.==1]) mean(retEM_vals[ptf6_f.==1])]
	std_ptf[tt, :] = [std(retEM_vals[ptf1_f.==1]) std(retEM_vals[ptf2_f.==1]) std(retEM_vals[ptf3_f.==1]) std(retEM_vals[ptf4_f.==1]) std(retEM_vals[ptf5_f.==1]) std(retEM_vals[ptf6_f.==1])]

	ptf1 = copy(ptf1_p1)
	ptf2 = copy(ptf2_p1)
	ptf3 = copy(ptf3_p1)
	ptf4 = copy(ptf4_p1)
	ptf5 = copy(ptf5_p1)
	ptf6 = copy(ptf6_p1)

end



###  -----------------------------------------------------------------------------------------------------------------  ###
###  ---------------------------------     BLOCK 3: REGRESSIONS USING PORTFOLIOS    ----------------------------------  ###
###  -----------------------------------------------------------------------------------------------------------------  ###

YEM_AP  	= YEM_simul[dropT_pn+wnd_roll+1:end]
ADMp_AP 	= ADM_simul[dropT_pn+wnd_roll+2:end]
ω_AP  	= ω_simul[dropT_pn+wnd_roll+1:end]
Rfactor_AP 	= Rfactor[wnd_roll+1:end]
avg_ptf_AP  = avg_ptf[wnd_roll+1:end,:]


# Index for possible NaN in main risk factor (ln(V(N)))
cond_noNaNRF   = isnan.(Rfactor).==false;
cond_RFNaN     = vec(cond_noNaNRF[wnd_roll+1:end])

# Index for binding constraint
index_bind = Rfactor_AP.<median(Rfactor_AP)


# De-mean risk factors (these will be overwritten in next lines!)
RF_YEM_noNaN 	= YEM_AP-mean(YEM_AP)
RF_ω_noNaN 	= log.(ω_AP)-mean(log.(ω_AP))
RF_ADMp_noNaN 	= log.(ADMp_AP)-mean(log.(ADMp_AP))


# Filter for possible NaN in main risk factor (ln(V(N))) [reduction in length]
Rfactor_noNaN 		= Rfactor_AP[cond_RFNaN]
avg_ptf_noNaN 		= avg_ptf_AP[cond_RFNaN,:]
index_bind_noNaN 	= index_bind[cond_RFNaN]

RF_YEM_noNaN 		= RF_YEM_noNaN[cond_RFNaN]
RF_ω_noNaN 		= RF_ω_noNaN[cond_RFNaN]
RF_ADMp_noNaN 		= RF_ADMp_noNaN[cond_RFNaN]
RF_ADMpbind_noNaN 	= RF_ADMp_noNaN[index_bind_noNaN] # note: this is of smaller length than RF_ADMp_noNaN
Rfactor_bind_noNaN 	= Rfactor_noNaN[index_bind_noNaN] # note: this is of smaller length than Rfactor_noNaN
avg_ptf_bind  = avg_ptf_noNaN[index_bind_noNaN,:] # note: this is of smaller length than avg_ptf_noNaN

# Form matrices with risk factors as regressors (demaned and no NaN)
XXv1_FF = [ones(size(Rfactor_noNaN)) RF_YEM_noNaN RF_ω_noNaN RF_ADMp_noNaN]
XXv2_FF = [ones(size(Rfactor_noNaN)) Rfactor_noNaN]
XXv3_FF = [ones(size(Rfactor_noNaN)) Rfactor_noNaN RF_YEM_noNaN]

corr_fun(x,y) = cov(x,y)/(std(x)*std(y))


# Compute cross-sectional returns (average across time)
avg_ret_byptf, cv_REM_RF1, cv_REM_RF2, std_ret_byptf = zeros(6), zeros(6), zeros(6), zeros(6)
cond_ptf_vec  = falses(size(avg_ptf_noNaN,1), 6)

for ii=1:6
	cond_tmp = isnan.(avg_ptf_noNaN[:, ii]).==false # filter for possible NaN in returns
	cond_ptf_vec[:,ii] 	= cond_tmp
	avg_ret_byptf[ii]  	= mean(avg_ptf_noNaN[cond_tmp,ii])
	std_ret_byptf[ii]  	= std(avg_ptf_noNaN[cond_tmp,ii])
	cv_REM_RF1[ii] 		= corr_fun(avg_ptf_noNaN[cond_tmp,ii], Rfactor_noNaN[cond_tmp])
	cv_REM_RF2[ii] 		= corr_fun(avg_ptf_noNaN[cond_tmp,ii], RF_YEM_noNaN[cond_tmp])
end


PR_cell, EMRh_cell, PR_wc_cell, EMRh_wc_cell, betas_cell = [], [], [], [], []
R2_PR_vec, RMSE_AP_vec, R2_PR_wc_vec, RMSE_AP_wc_vec, intercept_vec = zeros(3), zeros(3), zeros(3), zeros(3), zeros(3)

cf_1st = zeros(2,6)

for i_F = 1:3

	if i_F==1
		XX_FF = copy(XXv1_FF)
	elseif i_F==2
		XX_FF = copy(XXv2_FF)
	else
		XX_FF = copy(XXv3_FF)
	end

	### REGRESSION OF PORTFOLIO RETURNS ONTO FACTORS (panel data, first stage regression)
	N_RF = size(XX_FF,2)-1

	cf_all = zeros(6, N_RF)
	for ii=1:6
		cf_ptf 	 = inv(XX_FF[cond_ptf_vec[:,ii],:]'*XX_FF[cond_ptf_vec[:,ii],:])*XX_FF[cond_ptf_vec[:,ii],:]'*avg_ptf_noNaN[cond_ptf_vec[:,ii],ii]		
		cf_all[ii,:] = cf_ptf[2:end]' # do not keep intercept
		if i_F==2
			cf_1st[:,ii] = cf_ptf
		end
	end

	push!(betas_cell, cf_all)


	### ESTIMATION PRICE OF RISK (NO INTERCEPT) (cross-section, second stage regression)
	price_risk    = inv(cf_all'*cf_all)*cf_all'*avg_ret_byptf
	EM_PtfRet_hat = cf_all*price_risk

	RMSE_AP  = sqrt(mean((avg_ret_byptf-EM_PtfRet_hat).^2))
	SStot    = sum(avg_ret_byptf.^2) # special case for model without intercept! See Woolridge's intro econometrics section 2.6
	R2_Prisk = 1 - sum((avg_ret_byptf-EM_PtfRet_hat).^2)/SStot



	### ESTIMATION PRICE OF RISK (WITH INTERCEPT)
	XX_wc   = [ones(6) cf_all]
	cf_tmp  = inv(XX_wc'*XX_wc)*XX_wc'*avg_ret_byptf
	EM_PtfRet_hat_wc = XX_wc*cf_tmp
	price_risk_wc    = cf_tmp[2:end]
	intercept_val 	 = cf_tmp[1]

	RMSE_AP_wc  = sqrt(mean((avg_ret_byptf-EM_PtfRet_hat_wc).^2))
	SStot       = sum((avg_ret_byptf-mean(avg_ret_byptf)).^2)
	R2_Prisk_wc = 1 - sum((avg_ret_byptf-EM_PtfRet_hat_wc).^2)/SStot #* (6-1)/(6-1-N_RF)


	# Save results
	push!(PR_cell, price_risk)
	push!(EMRh_cell, EM_PtfRet_hat)
	R2_PR_vec[i_F]   = R2_Prisk
	RMSE_AP_vec[i_F] = RMSE_AP

	push!(PR_wc_cell, price_risk_wc)
	push!(EMRh_wc_cell, EM_PtfRet_hat_wc)
	R2_PR_wc_vec[i_F]   = R2_Prisk_wc
	RMSE_AP_wc_vec[i_F] = RMSE_AP_wc

	intercept_vec[i_F] = intercept_val

end


# Take common sample across portfolios
ind_min = findmin(sum(cond_ptf_vec,1))
β_hat   = cf_1st[2,:]							# factor loadings
α_hat   = avg_ret_byptf - β_hat*PR_cell[2][1] 	# pricing errors
eps_t   = zeros(ind_min[1], 6) 					# errors from first stage regression

for ii=1:6
	eps_t[:,ii] = avg_ptf_noNaN[cond_ptf_vec[:,ind_min[2]],ii] - XXv2_FF[cond_ptf_vec[:,ind_min[2]],:]*cf_1st[:,ii]
end

std_f   = std(XXv2_FF[cond_ptf_vec[:,ind_min[2]],2]) # s.d. of factor
SIG_hat = cov(eps_t)
cov_α   = (1/size(eps_t,1)) * (eye(6)-β_hat*inv(β_hat'*β_hat)*β_hat') * SIG_hat * (eye(6)-β_hat*inv(β_hat'*β_hat)*β_hat') * (1+(PR_cell[2][1]/std_f)^2) # see equation 12.20
chi_val = α_hat'*inv(cov_α)*α_hat
p_val   = cdf(Chi(size(eps_t,2)-1), chi_val)
chi_vec  = zeros(3); chi_vec[2]  = chi_val
pval_vec = zeros(3); pval_vec[2] = p_val


### Save Results
open("simulated_moments/crossection_AP_rets.txt", "w") do io
    writedlm(io, [RMSE_AP_vec RMSE_AP_wc_vec R2_PR_vec intercept_vec chi_vec pval_vec], ',')
end;
