
# Code that generates tables

## ------------------------------------------------------------------- ##
## -------------------------   UPLOAD DATA   ------------------------- ##
## ------------------------------------------------------------------- ##

 
# Upload model for calibrated parameters
cm  = load("model_data/NAG_module_structure.jld", "cm")
asm = load("model_data/ASM_module_structure.jld", "asm")

# Upload simulated moments
hist_mms = readdlm("simulated_moments/moments_histogram.txt", ',', Float64) 	# [[avg_by; df_prb; avg_SP; std_SP; cs_sd_by; cs_sd_SP; 0.0] [corr_by_YEM; corr_SP_YEM; corr_sdby_YEM; corr_sdSP_YEM; corr_XX_YEM; corr_NWact_YEM; corr_NWpxy_YEM]]
panel_cs = readdlm("simulated_moments/out_panel_moments_cs.txt", ',', Float64) 	# [cs_sd_by; cs_sd_SP; cs_corr_by_y; cs_corr_SP_y; cs_corr_sdby_y; cs_corr_sdSP_y] 6 moments
panel_ts = readdlm("simulated_moments/out_panel_moments_ts.txt", ',', Float64) 	# [mean_DY; DF_mean; mean_spread; std_spread; std_tb; mean(corr_sp_Y)*100; sdci_sdyi; corr_ci_yi; sd_TBi_yi; corr_TBi_yi; corr_spi_SP]
SP_decom = readdlm("simulated_moments/out_spread_decomp.txt", ',', Float64) 	# [mean_SP_DFRSK/capN*10_000; mean_SP_TOTAL/capN*10_000; mean_SP_IMRSK/capN*10_000; std_SP_DFRSK/capN*10_000; std_SP_TOTAL/capN*10_000; std_SP_IMRSK/capN*10_000]
NW_addms = readdlm("simulated_moments/NW_moments_calib.txt", ',', Float64) 		# it's 3x5, rows being std, corr(X,SP), corr(X,RDM); columns being each variable NWpxy, NWact, mktvalpxy, mktvalact, XX

elast_mm = readdlm("simulated_moments/elasticity_moments.txt", ',', Float64) 	# [ΔNW; ΔMV; ΔSP; ΔYLD; beta; ΔSPIR; ΔSPDF; ΔBt1]

SPLHMdcm = readdlm("simulated_moments/out_spread_LHM_decomp.txt", ',', Float64) # [ [ΔSP_LHM; ΔSP_YEM; ΔSP_OMG] [ΔTB_LHM; ΔTB_YEM; ΔTB_OMG] ]
misc_mms = readdlm("simulated_moments/misc_moments.txt", ',', Float64)


### Unconditional Spread Decomposition
SP_decom_v1 = readdlm("robustness/FM_low_elast/simulated_moments/out_spread_decomp.txt", ',', Float64)
SP_decom_v2 = readdlm("robustness/FM_diff_process/simulated_moments/out_spread_decomp.txt", ',', Float64)
SP_decom_v3 = readdlm("robustness/FM_kappa0_benchmark/simulated_moments/out_spread_decomp.txt", ',', Float64)
SP_decom_v4 = readdlm("robustness/SC_phi_smooth/simulated_moments/out_spread_decomp.txt", ',', Float64)
SP_decom_v5 = readdlm("robustness/FM_high_lvg/simulated_moments/out_spread_decomp.txt", ',', Float64)


### Upload AP moments (data and model)
good_fit_md  	= readdlm("simulated_moments/crossection_AP_rets.txt", ',', Float64) # [RMSE_AP_vec RMSE_AP_wc_vec R2_PR_vec intercept_vec chi_v pval]
good_fit_dt  	= readdlm("../../Results1/BV/goodness_fit.txt", ',', Float64) 	 # r2_data rmse_data chi_data pval_data r2_modimp rmse_modimp chi_modimp pval_modimp


### Lehman Episode
out_LHM    = readdlm("simulated_moments/out_Lehman.txt", ',', Float64)
out_LHM_BB = readdlm("simulated_moments/out_Lehman_BoomBust.txt", ',', Float64)


### Spreads and Consumption Dynamics During Lehman
SPLHMdcm_v1 = readdlm("robustness/FM_low_elast/simulated_moments/out_spread_LHM_decomp.txt", ',', Float64)
SPLHMdcm_v2 = readdlm("robustness/FM_diff_process/simulated_moments/out_spread_LHM_decomp.txt", ',', Float64)
SPLHMdcm_v3 = readdlm("robustness/FM_kappa0_benchmark/simulated_moments/out_spread_LHM_decomp.txt", ',', Float64)
SPLHMdcm_v4 = readdlm("simulated_moments/out_spread_LHM_decomp_BoomBust.txt", ',', Float64)
SPLHMdcm_v5 = readdlm("robustness/SC_phi_smooth/simulated_moments/out_spread_LHM_decomp.txt", ',', Float64)
SPLHMdcm_v6 = readdlm("robustness/FM_high_lvg/simulated_moments/out_spread_LHM_decomp.txt", ',', Float64)



################# ------------------------------------------------------------------------------------- #################
################# -------------------------------- *** MAIN TABLES *** -------------------------------- #################
################# ------------------------------------------------------------------------------------- #################


### Table 3. Parameter Values of the Baseline Calibration
open("tables/table_params.txt", "w") do io
write(io, "\\begin{table}[H] \n")
write(io, "\\centering \n")
write(io, "\\begin{tabular}{clcclc} \n")
write(io, "\\hline \\hline \n")
write(io, " \\multicolumn{3}{c}{Panel A: Fixed Parameters} & \\multicolumn{3}{c}{Panel B: Calibrated Parameters} \\\\ \n")
write(io, "\\hline \n")
write(io, " Param. &  \\hspace{1.1cm} Description  &   Value  & Param. &   \\hspace{1.1cm} Description  &  Value  \\\\\ \n ")
write(io, "\\hline \n")
write(io, " \ \$\\gamma\$ 														&  Risk aversion  							& \$$(string(round(cm.γ,2),0))\$   	& 	\ \$\\beta_{{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}}\$ 		&  Discount rate of EMs						& \$$(string(round(cm.β,2),0))\$  	\\\\  \n")
write(io, " \ \$\\theta\$ 														&  Reentry probability 						& \$$(round(cm.θ,2))\$   			& 	\ \$d_0\$ 					 											&  Default cost---level 					& \$$(round(cm.d0,4))\$  			\\\\  \n")
write(io, " \ \$\\xi\$ 															&  Debt duration 							& \$$(round(cm.ξ,2))\$   			& 	\ \$d_1\$ 					 											&  Default cost---curvature  				& \$$(round(cm.d1,2))\$  			\\\\  \n")
write(io, " \ \$\\rho_{{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}}\$ 	&  EM endowment, autocorrelation  			& \$$(round(cm.ρYEM,2))\$  			& 	\ \$\\sigma\$ 				 											&  Bank survival rate  						& \$$(round(cm.σn,2))\$  			\\\\  \n")
write(io, " \ \$\\sigma_{{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}}\$ 	&  EM endowment, shock volatility  			& \$$(round(cm.σYEM,2))\$   		& 	\ \$\\phi\$ 				 											&  Marginal cost of raising equity		 	& \$$(string(round(cm.phi,2),0))\$  \\\\  \n")
write(io, " \ \$\\beta_{{\\ensuremath{\\text{\\miniscule\\normalfont DM}}}}\$ 	&  Discount rate of DM  					& \$$(round(cm.βLD,2))\$  			& 	\ \$\\mu_{{{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}}}\$ 		&  Mass of EM economies 					& \$$(round(cm.μEM,2))\$  			\\\\  \n")
write(io, " \ \$\\alpha\$ 														&  Share of capital  						& \$$(round(cm.α,2))\$  			& 	\ \$\\sigma_{{\\ensuremath{\\text{\\miniscule\\normalfont DM}}}}\$ 	 	&  Volatility of DM shock					& \$$(round(cm.σω,4))\$  			\\\\  \n")
write(io, " \ \$\\delta\$ 														&  Depreciation  	 						& \$$(round(cm.δ,2))\$  			& 	\ \$\\kappa\$ 				 											&  Debt to net worth ratio  				& \$$(string(round(cm.κ,2),0))\$  	\\\\  \n")
write(io, " \ 																	& 											& 									& 	\ \$\\bar{n}\$ 				 											&  Net worth of new entrants  	 			& \$$(round(cm.nbar,2))\$ 			\\\\  \n")
write(io, " \ 																	& 											& 									& 	\ \$\\rho_{{\\ensuremath{\\text{\\miniscule\\normalfont DM,EM}}}}\$ 	&  Correlation of exogenous shocks 			& \$$(round(cm.ρEMDM,4))\$  		\\\\  \n")
write(io, "\\hline \\hline \n")
write(io, "\\end{tabular} \n")
write(io, "\\end{table} \n")
end




### Table 4. Targeted Moments
open("tables/table_target_paper.txt", "w") do io
write(io, "\\begin{table}[H] \n")
write(io, "\\centering \n")
write(io, "\\begin{tabular}{llcc} \n")
write(io, "\\hline \\hline \n")
write(io, " Target & Description & Data & Model \\\\ \n")
write(io, "\\hline \n")
write(io, "\$\\mathbb{E}[D_i/Y_i]\$ 				& Average EM debt											& \$15.0\\%\$ 	& \$$(round(panel_ts[1],1))\\%\$  		\\\\ \n")
write(io, "\$\\mathbb{P}[DF_i]\$ 					& EM default frequency										& \$1.5\\%\$  	&  \$$(round(panel_ts[2],1))\\%\$ 		 \\\\ \n")
write(io, "\$\\mathbb{E}[SP_i]\$ 					& Average EM-bond spreads									& 410bp 	  	&  $(round(Int, hist_mms[1, 1]))bp   			\\\\ \n")
write(io, "\$\\sigma(SP_i)\$ 						& Volatility EM-bond spreads								& 173bp 	  	&  $(round(Int, panel_ts[4]))bp   			\\\\ \n")
write(io, "\$\\text{corr}(SP_i,\\text{log}Y_i)\$ 	& Correlation EM-bond spreads \\& endowment					& \$-31\\%\$  	&  \$$(round(Int, panel_ts[19]*100))\\%\$ 	 \\\\ \n")
write(io, "\$\\sigma(\\text{log}V(N))\$				& Volatility global banks\' net worth (NW)					& \$0.28\$    	&  $(round(NW_addms[1, 4],2)) 				 \\\\ \n")
write(io, "corr(log\$V(N)\$, \\text{log}\$Y_{{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}}\$)				& Correlation banks\' NW \\& systemic EM endowment	& \$40\\%\$   	& \$$(round(Int,NW_addms[4, 4]*100))\\%\$ 	 \\\\ \n")
write(io, "\$\\mathbb{E}[A_{{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}}/(A_{{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}}+A_{{\\ensuremath{\\text{\\miniscule\\normalfont DM}}}})]\$ 						& Global banks\' exposure to EMs	& \$10\\%\$   					&  \$$(round(Int, (1-hist_mms[2, 1])*100))\\%\$ 	 \\\\ \n")
write(io, "\$\\eta_{{\\ensuremath{\\text{\\miniscule\\normalfont EM,N}}}}\$ 							& Elasticity EM spreads to banks\' NW 				& 0.056   		&  $(round(-elast_mm[9],3)) 	 \\\\ \n")
write(io, "\$\\mathbb{E}[(A_{{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}}+A_{{\\ensuremath{\\text{\\miniscule\\normalfont DM}}}})/NW]\$ 						& Total assets to equity ratio										& 3.8 	  		&  $(round(misc_mms[1], 1))   			\\\\ \n")
write(io, "\\hline \\hline \n")
write(io, "\\end{tabular} \n")
write(io, "\\end{table} \n")
end


### Table 5. International Comovements: Data and Model
R2_md     = good_fit_md[2, 3]
RMSE_md   = good_fit_md[2, 1]
chi_md    = good_fit_md[2, 5]
pval_md   = good_fit_md[2, 6]
R2_dtDT   = good_fit_dt[1]
RMSE_dtDT = good_fit_dt[2]
chi_dtDT  = good_fit_dt[3]
pval_dtDT = good_fit_dt[4]
R2_dtMI   = good_fit_dt[5]
RMSE_dtMI = good_fit_dt[6]
chi_dtMI  = good_fit_dt[7]
pval_dtMI = good_fit_dt[8]

open("tables/table_IntCom_AP.txt", "w") do io
write(io, "\\begin{table}[H] \n")
write(io, "\\centering \n")
write(io, "\\begin{tabular}{lcccccccc} \n")
write(io, "\\hline \\hline \n")
write(io, " & \\multicolumn{2}{c}{Correlation of \$SP_{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}\$} with & & \\multicolumn{2}{c}{Correlation of \\text{log}V(N) with} & & \\multicolumn{2}{c}{Factor Models}  \\\\ \\cline{2-3} \\cline{5-6} \\cline{8-9} \n")
write(io, " & \$SP_{\\ensuremath{\\text{\\miniscule\\normalfont EM,}i}}\$ & \$SP_{\\ensuremath{\\text{\\miniscule\\normalfont DM}}}\$ & & \$SP_{\\ensuremath{\\text{\\miniscule\\normalfont EM}}}\$ &  \$SP_{\\ensuremath{\\text{\\miniscule\\normalfont DM}}}\$ & & \$R^2\$ 	& RMSE \\\\  \\cline{2-3} \\cline{5-6} \\cline{8-9} \n")
write(io, "\\ Data 								& 0.69 							& 0.51  							& & -0.57 								& -0.79 								& & $(round(R2_dtDT*100,1))\\%	& $(round(RMSE_dtDT*100,2))\\% 		\\\\ \n")
write(io, "\\ Model 							& $(round(panel_ts[11], 2)) 	& $(round(hist_mms[3,1], 2)) 		& & $(round(NW_addms[2,4],2)) 			& $(string(round(NW_addms[3,4], 2),0)) 	& & $(round(R2_md*100,1))\\%	& $(round(RMSE_md*100,2))\\% 		\\\\ \n")
write(io, "\\ Model \$ \\lambda_f\$				& 								& 									& & 									& 										& & $(round(R2_dtMI*100,1))\\%	& $(round(RMSE_dtMI*100,2))\\% 		\\\\ \n")
write(io, "\\hline \\hline \n")
write(io, "\\end{tabular} \n")
write(io, "\\end{table} \n")
end



### Table 6. Unconditional Decomposition of EM-Bond Spreads
open("tables/table_spread_decomp_all.txt", "w") do io
write(io, "\\begin{table}[H] \n")
write(io, "\\centering \n")
write(io, "\\begin{tabular}{lccccccc} \n")
write(io, "\\hline \\hline \n")
write(io, "\\ 										& \\multicolumn{3}{c}{Average} 		& & \\multicolumn{3}{c}{Standard Deviation} 	\\\\ \\cline{2-8} \n")
write(io, "\\ 										& \\multirow{2}{*}{Total} 				& \\% Default													& \\% Interm. 													& & \\multirow{2}{*}{Total} 				& \\% Default 													& \\% Interm. 												\\\\  \n")
write(io, "\\ 										& 						 				& Premium 														& Premium 														& & 						 				& Premium 														& Premium 													\\\\ \\cline{2-8} \n")
write(io, " Data									& 410 									& 																& 																& & 173 									& 																& 															\\\\ \n")
write(io, " Baseline Model							& $(round(Int, SP_decom[2,2])) 			& $(round(Int, SP_decom[1,2]/SP_decom[2,2]*100))\\%				& $(round(Int, SP_decom[4,2]/SP_decom[2,2]*100))\\%  			& & $(round(Int, SP_decom[5,1])) 			& $(round(Int, SP_decom[4,1]/SP_decom[5,1]*100))\\%				& $(round(Int, SP_decom[3,3]/SP_decom[5,1]*100))\\%  		\\\\ \n")
write(io, " Robustness  \\\\ \n")& 				
write(io, " \\ i. Alternative Elasticity			& $(round(Int, SP_decom_v1[2,2])) 		& $(round(Int, SP_decom_v1[1,2]/SP_decom_v1[2,2]*100))\\%		& $(round(Int, SP_decom_v1[4,2]/SP_decom_v1[2,2]*100))\\%  		& & $(round(Int, SP_decom_v1[5,1])) 		& $(round(Int, SP_decom_v1[4,1]/SP_decom_v1[5,1]*100))\\%		& $(round(Int, SP_decom_v1[3,3]/SP_decom_v1[5,1]*100))\\%  		\\\\ \n")
write(io, " \\ ii. Measured Income Process			& $(round(Int, SP_decom_v2[2,2])) 		& $(round(Int, SP_decom_v2[1,2]/SP_decom_v2[2,2]*100))\\%		& $(round(Int, SP_decom_v2[4,2]/SP_decom_v2[2,2]*100))\\%  		& & $(round(Int, SP_decom_v2[5,1])) 		& $(round(Int, SP_decom_v2[4,1]/SP_decom_v2[5,1]*100))\\%		& $(round(Int, SP_decom_v2[3,3]/SP_decom_v2[5,1]*100))\\%  		\\\\ \n")
write(io, " \\ iii. Asset Managers					& $(round(Int, SP_decom_v3[2,2])) 		& $(round(Int, SP_decom_v3[1,2]/SP_decom_v3[2,2]*100))\\%		& $(round(Int, SP_decom_v3[4,2]/SP_decom_v3[2,2]*100))\\%  		& & $(round(Int, SP_decom_v3[5,1])) 		& $(round(Int, SP_decom_v3[4,1]/SP_decom_v3[5,1]*100))\\%		& $(round(Int, SP_decom_v3[3,3]/SP_decom_v3[5,1]*100))\\%  		\\\\ \n")
write(io, " \\ iv. High Leverge						& $(round(Int, SP_decom_v5[2,2])) 		& $(round(Int, SP_decom_v5[1,2]/SP_decom_v5[2,2]*100))\\%		& $(round(Int, SP_decom_v5[4,2]/SP_decom_v5[2,2]*100))\\%  		& & $(round(Int, SP_decom_v5[5,1])) 		& $(round(Int, SP_decom_v5[4,1]/SP_decom_v5[5,1]*100))\\%		& $(round(Int, SP_decom_v5[3,3]/SP_decom_v5[5,1]*100))\\%  		\\\\ \n")
write(io, " \\ v. Time-varying \$\\phi\$			& $(round(Int, SP_decom_v4[2,2])) 		& $(round(Int, SP_decom_v4[1,2]/SP_decom_v4[2,2]*100))\\%		& $(round(Int, SP_decom_v4[4,2]/SP_decom_v4[2,2]*100))\\%  		& & $(round(Int, SP_decom_v4[5,1])) 		& $(round(Int, SP_decom_v4[4,1]/SP_decom_v4[5,1]*100))\\%		& $(round(Int, SP_decom_v4[3,3]/SP_decom_v4[5,1]*100))\\%  		\\\\ \n")
write(io, "\\hline \\hline \n")
write(io, "\\end{tabular} \n")
write(io, "\\end{table} \n")
end



################# ------------------------------------------------------------------------------------- #################
################# -------------------------------- *** MAIN FIGURES *** ------------------------------- #################
################# ------------------------------------------------------------------------------------- #################

using PyPlot

Ct_dt 	= [0.051559; 0.053934; -0.00144; -0.00297; 0.006019; 0.006464; 0.003013; -0.01444]
EMSP_dt = [173; 410; 575; 339; 364; 391; 347; 336]

μCt_dt, σCt_dt 	 = 0.0016, 0.03077

plot_Ct_md 	 = out_LHM[1:end-1, 9]
plot_Ct_dt   = (Ct_dt-μCt_dt)/σCt_dt
plot_EMSP_md = out_LHM[1:end-1, 4]

end_T = 5
timeT = collect(2007:1:2014)

### Figure 5. EM-Bond Spreads and Consumption Dynamics During the Global Financial Crisis: Data and Model

## (a) EM-Bond Spreads
fig, ax = PyPlot.subplots(figsize=(6,6))
ax[:plot](timeT[1:end_T], plot_EMSP_md[1:end_T]*10_000, linestyle="-", linewidth=2.5, label="Model")
ax[:plot](timeT[1:end_T], EMSP_dt[1:end_T], color="black", linestyle="--", linewidth=2.5, label="Data")
ax[:legend](loc=4, frameon=false)
PyPlot.xticks(timeT[1:end_T])
PyPlot.grid("on", alpha=0.3)
PyPlot.tight_layout()
PyPlot.savefig("figures/IRF_plots/Lehman_EMSP.pdf")
PyPlot.close("all")


## (b) EMs' Consumption Dynamics
fig, ax = PyPlot.subplots(figsize=(6,6))
ax[:plot](timeT[1:end_T], plot_Ct_md[1:end_T], linestyle="-", linewidth=2.5, label="Model")
ax[:plot](timeT[1:end_T], plot_Ct_dt[1:end_T], color="black", linestyle="--", linewidth=2.5, label="Data")
ax[:legend](loc=4, frameon=false)
PyPlot.xticks(timeT[1:end_T])
PyPlot.grid("on", alpha=0.3)
PyPlot.tight_layout()
PyPlot.savefig("figures/IRF_plots/Lehman_Ct_stn.pdf")
PyPlot.close("all")


# Erase objects
out_LHM=[]; Ct_dt=[]; EMSP_dt=[]; μCt_dt=[]; σCt_dt=[];
plot_Ct_md=[]; plot_Ct_dt=[]; plot_EMSP_md=[];
end_T=[]; timeT=[]; fig=[]; ax=[];




################# ------------------------------------------------------------------------------------- #################
################# ------------------------------ *** APPENDIX TABLES *** ------------------------------ #################
################# ------------------------------------------------------------------------------------- #################


### TABLE: BUSINESS CYCLE MOMENTS
open("tables/table_bcmms_paper.txt", "w") do io
write(io, "\\begin{table}[H] \n")
write(io, "\\centering \n")
write(io, "\\begin{tabular}{llcc} \n")
write(io, "\\hline \\hline \n")
write(io, " Target & Description & Data & Model \\\\ \n")
write(io, "\\hline \n")
write(io, "\$\\sigma(\\text{log}C_i)/\\sigma(\\text{log}Y_i)\$ 	& Excess Volatility of Consumption 					& 1.14  & $(round(panel_ts[15],2))  \\\\ \n")
write(io, "\$\\text{corr}(\\text{log}C_i,\\text{log}Y_i)\$ 		& Cyclicality of Consumption						& 0.90  & $(round(panel_ts[17],2))  \\\\ \n")
write(io, "\$\\sigma(TB_i/Y_i)\$								& Volatility of the Trade-balance-to-output Ratio	& 0.04  & $(round(panel_ts[5], 2))  \\\\ \n")
write(io, "\$\\text{corr}(TB_i/Y_i,\\text{log}Y_i)\$			& Cyclicality of the Trade-balance-to-output Ratio	& -0.31 & $(round(panel_cs[11], 2))  \\\\ \n")
write(io, "\\hline \\hline \n")
write(io, "\\end{tabular} \n")
write(io, "\\end{table} \n")
end



### TABLE: DECOMPOSING EMS' SPREAD DYNAMICS
open("tables/table_spread_TB_Lehman.txt", "w") do io
write(io, "\\begin{table}[H] \n")
write(io, "\\centering \n")
write(io, "\\begin{tabular}{lccccc} \n")
write(io, "\\hline \\hline \n")
write(io, "\\ 							& \\multicolumn{2}{c}{ \$\\Delta\$ Spread } & \\multicolumn{2}{c}{ \$\\Delta\$ Consumption } \\\\ \\cline{2-5} \n")
write(io, "\\ 							& Joint Shocks 						& DM Contribution 									& Joint Shocks 					& 	DM Contribution 								\\\\ \\cline{2-5} \n")
write(io, "\\ Data 						&  $(round(Int, SPLHMdcm[1,1]))  	& 												 	& \$$(round(SPLHMdcm[1,3],2))\$ & 													\\\\ \n")
write(io, "\\ Baseline Model 			&  $(round(Int, SPLHMdcm[2,1]))  	& $(round(SPLHMdcm[4,1]/SPLHMdcm[2,1]*100, 1))\\% 	& \$$(round(SPLHMdcm[2,3],2))\$ & 	$(round(SPLHMdcm[4,3]/SPLHMdcm[2,3]*100,1))\\%	\\\\ \n")
write(io, "\\ Robustness  \\\\ \n")
write(io, "\\ \\ i. Alternative Elasticity  	&  $(round(Int, SPLHMdcm_v1[2,1]))  	& $(round(SPLHMdcm_v1[4,1]/SPLHMdcm_v1[2,1]*100, 1))\\% 	& \$$(round(SPLHMdcm_v1[2,3],2))\$ & 	$(round(SPLHMdcm_v1[4,3]/SPLHMdcm_v1[2,3]*100,1))\\%	\\\\ \n")
write(io, "\\ \\ ii. Measured Income Process  	&  $(round(Int, SPLHMdcm_v2[2,1]))  	& $(round(SPLHMdcm_v2[4,1]/SPLHMdcm_v2[2,1]*100, 1))\\% 	& \$$(round(SPLHMdcm_v2[2,3],2))\$ & 	$(round(SPLHMdcm_v2[4,3]/SPLHMdcm_v2[2,3]*100,1))\\%	\\\\ \n")
write(io, "\\ \\ iii. Asset Managers  			&  $(round(Int, SPLHMdcm_v3[2,1]))  	& $(round(SPLHMdcm_v3[4,1]/SPLHMdcm_v3[2,1]*100, 1))\\% 	& \$$(round(SPLHMdcm_v3[2,3],2))\$ & 	$(round(SPLHMdcm_v3[4,3]/SPLHMdcm_v3[2,3]*100,1))\\%	\\\\ \n")
write(io, "\\ \\ iv. High Leverge  				&  $(round(Int, SPLHMdcm_v6[2,1]))  	& $(round(SPLHMdcm_v6[4,1]/SPLHMdcm_v6[2,1]*100, 1))\\% 	& \$$(round(SPLHMdcm_v6[2,3],2))\$ & 	$(round(SPLHMdcm_v6[4,3]/SPLHMdcm_v6[2,3]*100,1))\\%	\\\\ \n")
write(io, "\\ \\ v. Risk Buildup  				&  $(round(Int, SPLHMdcm_v4[2,1]))  	& $(round(SPLHMdcm_v4[4,1]/SPLHMdcm_v4[2,1]*100, 1))\\% 	& \$$(round(SPLHMdcm_v4[2,3],2))\$ & 	$(round(SPLHMdcm_v4[4,3]/SPLHMdcm_v4[2,3]*100,1))\\%	\\\\ \n")
write(io, "\\ \\ vi. Time-varying \$\\phi\$ 		&  $(round(Int, SPLHMdcm_v5[2,1]))  	& $(round(SPLHMdcm_v5[4,1]/SPLHMdcm_v5[2,1]*100, 1))\\% 	& \$$(round(SPLHMdcm_v5[2,3],2))\$ & 	$(round(SPLHMdcm_v5[4,3]/SPLHMdcm_v5[2,3]*100,1))\\%	\\\\ \n")
write(io, "\\hline \\hline \n")
write(io, "\\end{tabular} \n")
write(io, "\\end{table} \n")
end




################# ------------------------------------------------------------------------------------- #################
################# ----------------------------- *** APPENDIX FIGURES *** ------------------------------ #################
################# ------------------------------------------------------------------------------------- #################

using PyPlot

Ct_dt 	= [-0.01724; 0.00805; 0.02524; 0.051559; 0.053934; -0.00144; -0.00297; 0.006019; 0.006464; 0.003013; -0.01444]
EMSP_dt = [255; 202; 165; 173; 410; 575; 339; 364; 391; 347; 336]

μCt_dt, σCt_dt 	 = 0.0016, 0.03077

plot_Ct_md 	 = out_LHM_BB[1:end-1, 9]
plot_Ct_dt   = (Ct_dt-μCt_dt)/σCt_dt
plot_EMSP_md = out_LHM_BB[1:end-1, 4]

end_T = 8
timeT = collect(2004:1:2014)

### Figure C2. Boom and Bust: Spreads and Consumption

## (a) EM-Bond Spreads
fig, ax = PyPlot.subplots(figsize=(6,6))
ax[:plot](timeT[1:end_T], plot_EMSP_md[1:end_T]*10_000, linestyle="-", linewidth=2.5, label="Model")
ax[:plot](timeT[1:end_T], EMSP_dt[1:end_T], color="black", linestyle="--", linewidth=2.5, label="Data")
ax[:legend](loc=4, frameon=false)
PyPlot.xticks(timeT[1:end_T])
PyPlot.grid("on", alpha=0.3)
PyPlot.tight_layout()
PyPlot.savefig("figures/IRF_plots/Lehman_EMSP_FD.pdf")
PyPlot.close("all")


## (b) EMs' Consumption Dynamics
fig, ax = PyPlot.subplots(figsize=(6,6))
ax[:plot](timeT[1:end_T], plot_Ct_md[1:end_T], linestyle="-", linewidth=2.5, label="Model")
ax[:plot](timeT[1:end_T], plot_Ct_dt[1:end_T], color="black", linestyle="--", linewidth=2.5, label="Data")
ax[:legend](loc=4, frameon=false)
PyPlot.xticks(timeT[1:end_T])
PyPlot.grid("on", alpha=0.3)
PyPlot.tight_layout()
PyPlot.savefig("figures/IRF_plots/Lehman_Ct_stn_FD.pdf")
PyPlot.close("all")


# Erase objects
out_LHM_BB=[]; Ct_dt=[]; EMSP_dt=[]; μCt_dt=[]; σCt_dt=[];
plot_Ct_md=[]; plot_Ct_dt=[]; plot_EMSP_md=[];
end_T=[]; timeT=[]; fig=[]; ax=[];


