
using JLD


do_module   = 0

# =1 to solve the model; =2 for baseline moments; =3 for boom-bust episode; =4 for XS asset pricing;

#=
NOTE:
To fully replicate results, set do_module=1 and set solve_once=true and iterate_coeffs=true.
The initial guess for the model has to be the one ending on "_start" and the LoM coefficients
as the appear on the "solve_once".

One can avoid solving + LoM iteration by just setting solve_once=false, iterate_coeffs=false, do_calib_mms=true.
=#


### Options for Solving Model (do one at a time)
save_models    	 = true
solve_once 	   	 = true
iterate_coeffs 	 = true

### Options for Baseline Moments
do_calib_mms   	 = true
do_fall_omega 	 = false
do_amplif_mod 	 = false


# params: β, d0, d1, σn, ρZDM, ρEMDM, σZDM, δ, phi, κ
eval_params = [0.9049; 0.03206; 14.0; 0.71; 0.0; 0.45; 0.068; 0.15; 2.5; 3.5];




################# -------------- UPLOAD FUNCTIONS -------------- ##################
include("00_tauchen.jl")
include("00_define_types.jl") 
include("00_structure_NAG.jl") 				# load module of model without aggregate shocks
include("00_structure_ASM.jl") 				# load module of model with aggregate shocks
include("00_global_loop.jl")
include("00_simulation_functions.jl")


if do_module == 1
	include("01_solve_model.jl")
elseif do_module == 2
	include("02_baseline_moments.jl")
elseif do_module == 3
	include("03_boom_bust.jl")	
elseif do_module == 4
	include("04_crossecAP_returns.jl")
end

include("create_tables.jl")

