
function fun_RDM(α, δ, ADMv, ωv)
    ωv .* ( α*ADMv.^(α-1) + (1.0-δ) )
end

function coremodule(;β   = 0.85,
                   βLD   = 0.98,
                   α 	 = 0.35,
                   δ 	 = 0.15,
                   γ     = 2.0,
                   ρz    = 0.82,
                   σz    = 0.025,
                   ρYEM  = 0.68,
                   σYEM  = 0.03,
                   θ     = 1.0/4.0,
                   d0    = 0.0298,
                   d1    = 1.6,
                   nz    = 5,
                   nb    = 81,
                   nω  = 5,
                   naDM  = 5,
                   ρEMDM = 0.522,
                   ρω  = 0.0,
    			   σω  = 0.01,
                   Eω  = 1.0,
                   n_ut  = 7,
                   κ     = 4.5,
                   phi   = 5.0,
                   σn    = 0.70,
                   μEM   = 4.905,
                   σM    = 0.02,
                   ξ     = 0.8,
                   maxit = 150,
                   damp  = 0.5,
                   vftol = 1e-3)


    RDMstar = 1.035
    naDM    = 5
    aDMmean = ( α/(RDMstar-(1.0-δ)) ) ^ (1.0/(1.0-α))
    Eω    = RDMstar / ( α*aDMmean^(α-1) + 1.0-δ )

    ADMmax = ( α/(1/(Eω*βLD)-(1.0-δ)) ) ^ (1.0/(1.0-α))

    ### Construction of grids
    aDMgrid = collect(linspace(1.5, ADMmax, naDM))
  

    ωgrid, Πω = tauchen_fun(nω, ρω, σω)
    μω    = log(Eω)- 0.5*σω^2.0
    ωgrid = exp.(ωgrid + μω)
    Πωinv = (Πω^1000)[1, :]
    Eω    = dot(ωgrid, vec(Πωinv))

    bgrid     = linspace(0.0, .50, nb)
    zgrid, Πz = tauchen_fun(nz, ρz, σz)
    


    ### Pre-comoute arrays
    vf          = zeros(nb, nz, naDM)
    vd          = zeros(nz, naDM)
    vc          = zeros(nb, nz, naDM)
    bprime_mat  = zeros(nb, nz, naDM)
    q_mat       = ones(nb, nz, naDM)/(1.0+0.04)
    def_states  = zeros(vf)

    zero_ind    = searchsortedfirst(bgrid, 0.0)

    # Create Quadratures
    quad_x, quad_w = gausshermite(n_ut)

    zn_mk_fine = 21 # 31
    zgrid_fine = linspace(zgrid[1], zgrid[end], zn_mk_fine)
    Πz_fine    = zeros(zn_mk_fine, zn_mk_fine)

    for (iz, zv) in enumerate(zgrid_fine)

        for i_e=1:n_ut
            z_next = sqrt(2)*σz*quad_x[i_e]+ ρz*zv
            prob_z = (1/sqrt(pi)) * quad_w[i_e]

            indj   = min(max( searchsortedfirst(zgrid_fine, z_next), 2),length(zgrid_fine))  ## NOTE: 1st value ≥ in grid
            z_next = max( min(z_next, zgrid_fine[end]), zgrid_fine[1])
            w_r    = ( z_next - zgrid_fine[indj-1] ) / ( zgrid_fine[indj] - zgrid_fine[indj-1] )
            Πz_fine[iz, indj]   = w_r*prob_z + Πz_fine[iz, indj]
            Πz_fine[iz, indj-1] = (1.0-w_r)*prob_z + Πz_fine[iz, indj-1]
        end
    end

    b_grid_fine = linspace(bgrid[1],0.25,min(2*nb, 101) )
    snodes      = [repmat(b_grid_fine,zn_mk_fine,1) kron(zgrid_fine,ones(length(b_grid_fine)))]
    num_nodes   = size(snodes,1)

    coeffaDM    =  [0.14016; 0.874739; 0.914663]


    ### Compute steady state aggregates
    aDMstar     = aDMmean
    Astar       = 10.0/9.0 * aDMstar

    Dstar       = 0.0
    NWstar      = 0.0
    nbar        = 0.0
    Btlstar 	= 0.0
    Bhtstar 	= 0.0
	REMstar 	= 0.0
    nustar      = 0.0
	Bstar 		= 0.0
	Λstar 		= spzeros(2*num_nodes)
	iotas_star 	= spzeros(num_nodes)


    ### Compute excess return for prices
    # In steady state, ReDM = RDM since there is no uncertainty. Also, since ν is constant,
    # ν term in numerator and denominator in EM bond price equation will cancel out. So q=E[iota]/RDM.
	ReDM        = Eω*(α*aDMgrid.^(α-1) + (1.0-δ))

    return CoreModule( β, βLD, α, δ, γ, ρz, σz, ρω, σω, ρYEM, σYEM, ρEMDM, θ, d0, d1, κ, σn, phi, Eω, μEM, μω, σM, ξ, nz, zn_mk_fine, 
                      nb, n_ut, num_nodes, naDM, nω, bgrid, zgrid, zgrid_fine, aDMgrid, ωgrid, Πz, Πω, Πz_fine, zero_ind, 
                      vf, vd, vc, bprime_mat, def_states, q_mat, ReDM, quad_x, quad_w, b_grid_fine, snodes, nbar, Btlstar, Bhtstar, aDMstar, Astar, 
                      NWstar, Dstar, REMstar, RDMstar, nustar, Bstar, Λstar, iotas_star, coeffaDM, maxit, damp, vftol )

end



### ----------------------- MODULE 2: BORROWER ---------------------- ###

ufun(cm::CoreModule, c) = c.^(1.0 - cm.γ) ./ (1.0 - cm.γ)

function y_fun_nas(cm::CoreModule, z_v, bp_v, aDM_v, itp_vc, itp_vd)

    # Given y_{t}, compute the maximum and minimum possible values for y_{t+1}
    y_max = sqrt(2)*cm.σz*cm.quad_x[end] + cm.ρz*z_v
    y_min = sqrt(2)*cm.σz*cm.quad_x[1]   + cm.ρz*z_v

    diff_fun_tmp(zp_v) = itp_vc[bp_v, zp_v, aDM_v] - itp_vd[zp_v, aDM_v]
   
    # If dif>0, not defaulting yields a higher value.
    dif_right = diff_fun_tmp(y_max)
    dif_left  = diff_fun_tmp(y_min)
    
    if (dif_right * dif_left < 0.0) # THERE IS AN INTERIOR ROOT

        left  = y_min
        right = y_max
        
        dif_x_root = fzero(diff_fun_tmp, [left, right])
        y_fun_val  = dif_x_root
        
    elseif dif_left > 0.0 
        y_fun_val = y_min
        
    else # dif_right<0
        y_fun_val = y_max
    end
    
    return y_fun_val
end

function one_step_update!(cm::CoreModule, EV, EVd, Udef)

    bgrid  = cm.bgrid
    β      = cm.β

    knots  = (cm.bgrid, cm.zgrid, cm.aDMgrid)
    itp_EV = interpolate(knots, EV,    		Gridded(Linear()))
    itp_q  = interpolate(knots, cm.q_mat, 	Gridded(Linear()))

    for (iaDM, aDMv) in enumerate(cm.aDMgrid)
        for (iz, zv) in enumerate(cm.zgrid)
            yt     = exp(zv)
            defval = Udef[iz] + cm.β*(cm.θ*EV[cm.zero_ind, iz, iaDM] + (1.0-cm.θ)*EVd[iz, iaDM])
            cm.vd[iz, iaDM] = defval

            for (ib, bt) in enumerate(cm.bgrid)

                rep_v(bp_v) = -ufun(cm, max(yt + max(0.0, itp_q[bp_v, zv, aDMv]*bp_v) - bt, 1e-14)[1] ) - β * itp_EV[bp_v, zv, aDMv]
                res         = Optim.optimize(rep_v, cm.bgrid[1], cm.bgrid[end], GoldenSection(), iterations=25)
                pol_val     = Optim.minimizer(res)
                current_max = -Optim.minimum(res)

                # Update value and policy functions
                cm.vc[ib, iz, iaDM]           = current_max
                cm.bprime_mat[ib, iz, iaDM]   = pol_val
                cm.vf[ib, iz, iaDM]           = defval > current_max ? defval: current_max
            end
        end
    end
    Void
end

function compute_prices(cm::CoreModule)

    itp_vc = interpolate((cm.bgrid, cm.zgrid, cm.aDMgrid), cm.vc, Gridded(Linear()))
    itp_vd = interpolate((cm.zgrid, cm.aDMgrid), cm.vd, Gridded(Linear()))
    q_out  = Array{Float64}(cm.nb, cm.nz, cm.naDM)

    for (i_aDM, aDM_v) in enumerate(cm.aDMgrid)

    	ReEM_v = cm.ReDM[i_aDM]

        for (i_z, z_v) in enumerate(cm.zgrid)
        	for (i_bp, bp_v) in enumerate(cm.bgrid)

	        	ydef    = y_fun_nas(cm, z_v, bp_v, aDM_v, itp_vc, itp_vd)
	        	eps_def = ( ydef - cm.ρz*z_v ) / ( sqrt(2)*cm.σz )

	            if (cdf(Normal(0.0, 1.0), eps_def) >= cdf(Normal(0.0, 1.0), cm.quad_x[end])-1.0e-10)   # default for sure
	                exp_v = 0.0
	            elseif (cdf(Normal(0.0, 1.0), eps_def) <= cdf(Normal(0.0, 1.0), cm.quad_x[1])+1.0e-10) # no default for sure
	                exp_v = 1.0
	            else
	                exp_v = (1.0-cdf(Normal(0.0, 1.0), eps_def))
	            end

	            q_out[i_bp, i_z, i_aDM]  = exp_v / ReEM_v
	        end
        end
    end

    return q_out
end

function compute_EV(cm::CoreModule, Vmat; full_dim::Bool = true)

    EVmat = Array{Float64}(size(Vmat))

    if full_dim

        itp_v = interpolate((cm.bgrid, cm.zgrid, cm.aDMgrid), Vmat, Gridded(Linear()))

        for (i_aDM, aDM_v) in enumerate(cm.aDMgrid)
            for (i_z, z_v) in enumerate(cm.zgrid)
                for (i_bp, bp_v) in enumerate(cm.bgrid)
                
                    exp_v = 0.0

                    for i_e=1:cm.n_ut
                        z_next   = sqrt(2)*cm.σz*cm.quad_x[i_e]+ cm.ρz*z_v
                        val_next = itp_v[bp_v, z_next, aDM_v]
                        exp_v   += (1/sqrt(pi)) * cm.quad_w[i_e] * val_next
                    end

                    EVmat[i_bp, i_z, i_aDM] = exp_v
                end
            end
        end
    else

        itp_v = interpolate((cm.zgrid, cm.aDMgrid), Vmat, Gridded(Linear()))
                
        for (i_aDM, aDM_v) in enumerate(cm.aDMgrid)
            for (i_z, z_v) in enumerate(cm.zgrid)

                exp_v = 0.0

                for i_e=1:cm.n_ut
                    z_next   = sqrt(2)*cm.σz*cm.quad_x[i_e]+ cm.ρz*z_v
                    val_next = itp_v[z_next, aDM_v]
                    exp_v   += (1/sqrt(pi))*cm.quad_w[i_e]*val_next
                end

                EVmat[i_z, i_aDM] = exp_v
            end
        end
    end

    return EVmat

end

function compute_defstates!(cm::CoreModule)
    for (iaDM, aDMv) in enumerate(cm.aDMgrid)
        for (iz, zv) in enumerate(cm.zgrid)
            for (ib, bt) in enumerate(cm.bgrid)
                cm.def_states[ib, iz, iaDM] = cm.vd[iz, iaDM] > cm.vc[ib, iz, iaDM]
            end
        end
    end
end

function vfi_nas!(cm::CoreModule)

    damp_price = 0.9

    # Iteration stuff
    it   = 0
    dist = 10.0

    # Allocate memory for update
    V_upd = zeros(cm.vf)    
    Udef  = zeros(cm.zgrid)

    for (i_z, z_v) in enumerate(cm.zgrid)
        y_v       = exp(z_v)
        ydf       = max(y_v*(1.0- cm.d0*y_v^cm.d1), 1e-14)
        Udef[i_z] = ufun(cm, ydf)
    end

    while dist > cm.vftol && it < cm.maxit

        it += 1

        # Compute expectations for this iterations
        # (We need Π' because of order value function dimensions)
        copy!(V_upd, cm.vf)

        EV  = compute_EV(cm, cm.vf)
        EVd = compute_EV(cm, cm.vd; full_dim=false)

        # Update Value Function
        one_step_update!(cm, EV, EVd, Udef)

        # Update prices
        q_old    = copy(cm.q_mat)
        q_out    = compute_prices(cm)
        distq    = maximum(abs, q_old - q_out)
        cm.q_mat = damp_price*q_old + (1.0-damp_price)*q_out

        distV    = maximum(abs, V_upd - cm.vf)
        dist     = max(distV, distq)

        if it%50==0
            @printf("it # %d with distV = %.3g and distq = %.3g\n", it, distV, distq)
        end

    end
    
    compute_defstates!(cm)

    Void
end



### ----------------------- MODULE 3: STEADY STATE ---------------------- ###


function get_sparse_am(cm::CoreModule, aDMval)
    # In this function we generate a sparse transition matrix Π_{t},
    # such that Λ_{t+1}=Π_{t}'Λ_{t}.
    # Π_{t} = sparse(I,J,V), where I is a vector indicating the location of the row,
    # J is a vector containing the location of the column, and V is a vector with 
    # the corresponding values to fill in.
    # We will actually be computing Π_{t}'.

    zn_mk_fine = cm.zn_mk_fine # length(cm.zgrid)
    nb_fine    = length(cm.b_grid_fine)
    nΠaggr     = nb_fine * cm.zn_mk_fine * 2

    #== Initialize Vectors for sparse matrix ==#
    Ic    = Array{Int64}(0)
    Ir    = Array{Int64}(0)
    V_vec = Array{Float64}(0)

    knots 		= (cm.bgrid, cm.zgrid, cm.aDMgrid)
    itp_obj_bi 	= interpolate(knots, cm.bprime_mat, Gridded(Linear()))
    itp_obj_vc 	= interpolate(knots, cm.vc, Gridded(Linear()))
    itp_obj_vd 	= interpolate((cm.zgrid, cm.aDMgrid), cm.vd, Gridded(Linear()))


    # Here we change the output to the transpose.
    """ Compute transition matrix associated to (default, default) """
    ic_b1 = ones(1)
    ir_b1 = ones(1)
    vv_b1 = [1.0 - cm.θ]

    """ Compute transition matrix associated to (default, repay) """
    ic_b2 = ones(1)
    ir_b2 = ones(1)
    vv_b2 = [cm.θ]

    # rows have t+1, columns have t

    for iι = 1:2

        if iι == 1
            for iz = 1:cm.zn_mk_fine

                for iz1 = 1:cm.zn_mk_fine

                    """ (default, default) """
                    valin_b1      = cm.Πz_fine[iz, iz1]*vv_b1
                    col_start     = (iz-1)*nb_fine    # Starting points for columns
                    row_start     = (iz1-1)*nb_fine   # Starting points for rows
           
                    #== Append on indices ==#
                    append!(Ic, ic_b1 + col_start)
                    append!(Ir, ir_b1 + row_start)
                    append!(V_vec, valin_b1 )

                    """ (default, repay) """
                    valin_b2      = cm.Πz_fine[iz, iz1]*vv_b2
                    col_start     = (iz-1)*nb_fine                         # Starting points for columns
                    row_start     = nb_fine*cm.zn_mk_fine + (iz1-1)*nb_fine   # Starting points for rows
           
                    #== Append on indices ==#
                    append!(Ic, ic_b2 + col_start)
                    append!(Ir, ir_b2 + row_start)
                    append!(V_vec, valin_b2 )
                end
            end

        else

            for iz = 1:cm.zn_mk_fine
                # Generate matrix of next period's bonds policy given current state
                z_v = cm.zgrid_fine[iz]

                bprime  = zeros(0)
                def_ind = 0

                for ib=1:nb_fine
                    repay_val  = itp_obj_vc[cm.b_grid_fine[ib], z_v, aDMval] > itp_obj_vd[z_v, aDMval]

                    if repay_val
                        push!(bprime, itp_obj_bi[cm.b_grid_fine[ib], z_v, aDMval])
                    else
                        def_ind = ib
                        break
                    end
                end

                if def_ind > 1
                    bprime = bprime[1:def_ind-1]

                    """ Compute transition matrix associated to (repay, repay) """
                    ic_b4, ir_b4, vv_b4 = fill_sparse_fun(collect(cm.b_grid_fine), bprime)
                
                    """ Compute transition matrix associated to (repay, default) """
                    ic_b3 = collect(def_ind:nb_fine)
                    ir_b3 = ones(length(ic_b3))         # first point in grid
                    vv_b3 = ones(length(ic_b3))         # mass points

                elseif def_ind == 1
                    ic_b4, ir_b4, vv_b4 = collect(1:1), collect(1:1), zeros(1)

                    ic_b3 = collect(1:nb_fine)
                    ir_b3 = ones(nb_fine)
                    vv_b3 = ones(nb_fine)
                else
                    ic_b4, ir_b4, vv_b4 = fill_sparse_fun(collect(cm.b_grid_fine), bprime)
                    ic_b3, ir_b3, vv_b3 = collect(1:1), collect(1:1), zeros(1)
                end


                for iz1 = 1:cm.zn_mk_fine

                    valin_b3 = cm.Πz_fine[iz, iz1]*vv_b3 # (repay, default)
                    valin_b4 = cm.Πz_fine[iz, iz1]*vv_b4 # (repay, repay)

                    # Note: below multiply by nb_fine*cm.nz_fine to skip part where ι=0

                    """ case (repay, repay) """
                    col_start  = nb_fine * cm.zn_mk_fine + (iz-1) * nb_fine    # Starting points for columns
                    row_start  = nb_fine * cm.zn_mk_fine + (iz1-1) * nb_fine   # Starting points for rows

                    append!(Ic, ic_b4 + col_start)
                    append!(Ir, ir_b4 + row_start)
                    append!(V_vec, valin_b4 )


                    """ case (repay, default) """
                    col_start  = nb_fine * cm.zn_mk_fine + (iz-1) * nb_fine    # Starting points for columns
                    row_start  = (iz1-1) * nb_fine                    # Starting points for rows

                    append!(Ic, ic_b3 + col_start)
                    append!(Ir, ir_b3 + row_start)
                    append!(V_vec, valin_b3 )

                end
            end
        end
    end    

    return sparse(Ir, Ic, V_vec, nΠaggr, nΠaggr)
end


function compute_SS_once(cm::CoreModule, itp_bi, itp_qi, itp_vc, itp_vd)

    # Compute Bhat and Btilde
    NS          = size(cm.snodes, 1)
    iotas_star  = spzeros(NS)
    Bhat_star   = 0.0
    Btilde_star = 0.0

    Πst       = get_sparse_am(cm, cm.aDMstar)
    _, eigenv = eigs(Πst, nev = 1);
    eigenv    = real( squeeze(eigenv,2) )
    λstar      = eigenv ./ sum(eigenv);

    #== check all elements are positive ==#
    minimum(λstar) > - 1e-8 || throw(error("Negative histogram weight %.8f",minimum(λstar)))
    λstar[λstar .< 1e-12] = 0.0
    λstar = λstar ./ sum(λstar)

    #== CHECK consistency ==#
    err_invDist = maximum( abs.( Πst*λstar - λstar) );
    err_invDist<1e-8 || throw(error("Invariant distribution is incorrect"))

    Λ_mkt = λstar[NS+1:end]

    # Compute the aggregates
    for i_s=1:NS
        bprime      = max(cm.bgrid[1], itp_bi[cm.snodes[i_s,1], cm.snodes[i_s,2], cm.aDMstar])
        qprice      = max(0.0, itp_qi[bprime, cm.snodes[i_s,2], cm.aDMstar])
        iotas_star[i_s]  = itp_vc[cm.snodes[i_s,1], cm.snodes[i_s,2], cm.aDMstar] >= itp_vd[cm.snodes[i_s,2], cm.aDMstar]
        Btilde_star += Λ_mkt[i_s]*iotas_star[i_s]*cm.snodes[i_s,1]
        Bhat_star   += bprime*qprice*Λ_mkt[i_s]*iotas_star[i_s]
    end

    Btilv   = Btilde_star[1]
    Bhatv   = Bhat_star[1]
    
    return (λstar, Btilv, Bhatv, iotas_star)

end


function compute_ss_root!(cm::CoreModule)

    knots   = (cm.bgrid, cm.zgrid, cm.aDMgrid)
    itp_bi  = interpolate(knots, cm.bprime_mat, Gridded(Linear()))
    itp_qi  = interpolate(knots, cm.q_mat, Gridded(Linear()))
    itp_vc  = interpolate(knots, cm.vc, Gridded(Linear()))
    itp_vd  = interpolate((cm.zgrid, cm.aDMgrid), cm.vd, Gridded(Linear()))

	itp_ReDM = interpolate((cm.aDMgrid,), cm.ReDM, Gridded(Linear()))

    outSS    = compute_SS_once(cm, itp_bi, itp_qi, itp_vc, itp_vd)

	cm.Λstar, cm.Btlstar, cm.Bhtstar, cm.iotas_star = outSS[1], outSS[2], outSS[3], outSS[4]
	
    cm.μEM     = cm.aDMstar/(9.0*cm.Bhtstar)
    cm.RDMstar = fun_RDM(cm.α, cm.δ, cm.aDMstar, cm.Eω)
	cm.REMstar = itp_ReDM[cm.aDMstar]
    cm.nbar    = cm.Astar*( (1.0-cm.σn*cm.RDMstar)-(1.0-cm.σn/cm.βLD)*cm.κ/(1+cm.κ) ) / (1.0-cm.σn)
    cm.nbar    = 0.35

    cm.NWstar  = (cm.RDMstar*cm.Astar-cm.κ*(1-cm.σn)*cm.nbar)/(1+cm.κ/cm.βLD)
    cm.Dstar   = cm.κ*(cm.σn*cm.NWstar+(1-cm.σn)*cm.nbar)

    Λ_mkt = cm.Λstar[cm.num_nodes+1:end]
    Bstar = 0.0

    # Compute the aggregates
    for i_s=1:cm.num_nodes
    	Bstar 		+= cm.snodes[i_s, 1] * Λ_mkt[i_s]
    end

    cm.Bstar = Bstar / sum(Λ_mkt) * cm.μEM


end

