
################ ------------------- FUNCTIONS SIMULATION ------------------- ################

function simul_panel_fun(cm::CoreModuleSC, asm::AshocksModuleSC, agg_state, capN, dropT)

    srand(5678) # Setting the seed

    YEMsim  = agg_state[:, 1]
    ωsim  = agg_state[:, 2]
    aDMpsim = agg_state[:, 3]
    capT    = size(YEMsim, 1)

    ### Simulate Idiosyncratic Shocks and Variables
    shocks_z  = rand(Normal(0.0, cm.σz), capT-1, capN)
    zsim      = zeros(capT,   capN)
    bsim      = zeros(capT+1, capN)
    ysim      = zeros(capT,   capN)
    def_sim   = zeros(capT+1, capN)
    spsim     = zeros(capT,   capN)
    csim      = zeros(capT, capN)
    bpy_sim   = zeros(capT,   capN)
    retEM     = zeros(capT,   capN)

    max_q = maximum(asm.q_mat)
    min_q = minimum(asm.q_mat)
    min_b = cm.bgrid[1]
    
    for ic=1:capN
        for tt=1:capT-1
            zsim[tt+1, ic] = cm.ρz*zsim[tt, ic] + shocks_z[tt, ic]
        end
    end

    knots   = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_q   = interpolate(knots, asm.q_mat,  Gridded(Linear()))
    itp_bp  = interpolate(knots, asm.bprime_mat, Gridded(Linear()))
    itp_def = interpolate(knots, asm.defmat,     Gridded(Linear()))

    for ic=1:capN

        qm1_v = 1.0

        for tt=1:capT
            z_v     = zsim[tt, ic]
            b_v     = bsim[tt, ic]
            YEMv    = YEMsim[tt]
            ωv    = ωsim[tt]
            aDMp_v  = aDMpsim[tt] # it is a_{DM,t+1}!
            rf_v    = sc_RF_fun(cm, ωv)

            if def_sim[tt, ic] == 0.0
                df_prob = min( max(0.0, itp_def[b_v, z_v, YEMv, ωv, aDMp_v]), 1.0)
                rand_v  = rand()
                df_chce = 1*(rand_v<df_prob)

                if df_chce == 0.0
                    y_v      = exp(z_v+YEMv)
                    bp_v     = max(min_b, itp_bp[b_v, z_v, YEMv, ωv, aDMp_v] )
                    q_v      = min( max( min_q, itp_q[bp_v, z_v, YEMv, ωv, aDMp_v] ), max_q)
                    sp_v     = (1.0/q_v+cm.ξ-1.0) - rf_v
                    c_v      = max(y_v - b_v + q_v*(bp_v-cm.ξ*b_v), 1e-14)
                    def_sim[tt+1, ic] = 0.0
                    bpy_sim[tt, ic]   = bp_v*q_v / y_v
                else
                    y_v  = max( exp(z_v+YEMv)*(1.0 - cm.d0*exp(z_v+YEMv)^cm.d1), 1e-14 )
                    bp_v = 0.0
                    q_v  = 0.0
                    sp_v = 0.0
                    c_v  = y_v
                    def_sim[tt, ic]   = 1.0
                    def_sim[tt+1, ic] = 1.0
                    bpy_sim[tt, ic]   = 0.0
                end
            else
                rand_v = rand(Uniform(0.0 ,1.0))
                if rand_v <= cm.θ # out of default today
                    y_v      = exp(z_v+YEMv)
                    bp_v     = max(min_b, itp_bp[b_v, z_v, YEMv, ωv, aDMp_v] )
                    q_v      = min( max( min_q, itp_q[bp_v, z_v, YEMv, ωv, aDMp_v] ), max_q)
                    sp_v     = (1.0/q_v+cm.ξ-1.0) - rf_v
                    c_v      = max(y_v - b_v + q_v*(bp_v-cm.ξ*b_v), 1e-14)
                    def_sim[tt, ic]   = 0.0
                    def_sim[tt+1, ic] = 0.0
                    bpy_sim[tt, ic]   = bp_v*q_v / y_v
                else
                    y_v  = max( exp(z_v+YEMv)*(1.0 - cm.d0*exp(z_v+YEMv)^cm.d1), 1e-14 )
                    bp_v = 0.0
                    q_v  = 0.0
                    sp_v = 0.0
                    c_v  = y_v
                    def_sim[tt+1, ic] = 1.0
                    bpy_sim[tt, ic]   = 0.0
                end
            end

            ysim[tt, ic]   = y_v
            bsim[tt+1, ic] = bp_v
            spsim[tt, ic]  = sp_v
            csim[tt, ic]   = c_v
            retEM[tt, ic]  = (1-def_sim[tt, ic])*(1.0+cm.ξ*q_v)/qm1_v

            qm1_v = q_v
        end
    end

    return(zsim[dropT+1:capT, :], bsim[dropT+1:capT, :], def_sim[dropT+1:capT, :], spsim[dropT+1:capT, :], ysim[dropT+1:capT, :], csim[dropT+1:capT, :], bpy_sim[dropT+1:capT, :], retEM[dropT+1:capT, :])
end


function simul_country_fun(cm::CoreModuleSC, asm::AshocksModuleSC, agg_state, dropT; simul_zEM = true)

    srand(5678) # Setting the seed

    YEMsim  = agg_state[:, 1]
    ωsim  = agg_state[:, 2]
    aDMpsim = agg_state[:, 3]
    capT    = size(YEMsim, 1)

    ### Simulate Idiosyncratic Shocks and Variables
    shocks_z  = rand(Normal(0.0, cm.σz), capT-1)
    zsim      = zeros(capT)
    bsim      = zeros(capT+1)
    ysim      = zeros(capT)
    def_sim   = zeros(capT+1)
    spsim     = zeros(capT)
    csim      = zeros(capT)
    bpy_sim   = zeros(capT)
    
    if simul_zEM==true
        for tt=1:capT-1
            zsim[tt+1] = cm.ρz*zsim[tt] + shocks_z[tt]
        end
    end

    max_q = maximum(asm.q_mat)
    min_q = minimum(asm.q_mat)
    min_b = cm.bgrid[1]

    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_q       = interpolate(knots, asm.q_mat,  Gridded(Linear()))
    itp_bp      = interpolate(knots, asm.bprime_mat, Gridded(Linear()))
    itp_def     = interpolate(knots, asm.defmat,     Gridded(Linear()))

    for tt=1:capT
        z_v     = zsim[tt]
        b_v     = bsim[tt]
        YEMv    = YEMsim[tt]
        ωv    = ωsim[tt]
        aDMp_v  = aDMpsim[tt] # it is a_{DM,t+1}!
        rf_v    = sc_RF_fun(cm, ωv)

        if def_sim[tt] == 0.0
            df_prob = min( max(0.0, itp_def[b_v, z_v, YEMv, ωv, aDMp_v]), 1.0)
            rand_v  = rand()
            df_chce = 1*(rand_v<df_prob)

            # df_chce = 1*(vctil_sim[tt]<vd_sim[tt])

            if df_chce == 0.0
                y_v   = exp(z_v+YEMv)
                bp_v  = max(min_b, itp_bp[b_v, z_v, YEMv, ωv, aDMp_v] )
                q_v   = min( max( min_q, itp_q[bp_v, z_v, YEMv, ωv, aDMp_v] ), max_q)
                sp_v  = (1.0/q_v+cm.ξ-1.0) - rf_v
                c_v   = max(y_v - b_v + q_v*(bp_v-cm.ξ*b_v), 1e-14)
                def_sim[tt+1] = 0.0
                bpy_sim[tt]   = bp_v*q_v / y_v

            else
                y_v  = max( exp(z_v+YEMv)*(1.0 - cm.d0*exp(z_v+YEMv)^cm.d1), 1e-14 )
                bp_v = 0.0
                q_v  = 0.0
                sp_v = 0.0
                c_v  = y_v
                def_sim[tt]   = 1.0
                def_sim[tt+1] = 1.0
                bpy_sim[tt]   = 0.0
            end

        else

            rand_v = rand(Uniform(0.0 ,1.0))
            if rand_v <= cm.θ # out of default today
                y_v   = exp(z_v+YEMv)
                bp_v  = max(min_b, itp_bp[b_v, z_v, YEMv, ωv, aDMp_v] )
                q_v   = min( max( min_q, itp_q[bp_v, z_v, YEMv, ωv, aDMp_v] ), max_q)
                sp_v  = (1.0/q_v+cm.ξ-1.0) - rf_v
                c_v   = max(y_v - b_v + q_v*(bp_v-cm.ξ*b_v), 1e-14)
                def_sim[tt]   = 0.0
                def_sim[tt+1] = 0.0
                bpy_sim[tt]   = bp_v*q_v / y_v
            else
                y_v  = max( exp(z_v+YEMv)*(1.0 - cm.d0*exp(z_v+YEMv)^cm.d1), 1e-14 )
                bp_v = 0.0
                q_v  = 0.0
                sp_v = 0.0
                c_v  = y_v
                def_sim[tt+1] = 1.0
                bpy_sim[tt]   = 0.0
            end
        end

        ysim[tt]   = y_v
        bsim[tt+1] = bp_v
        spsim[tt]  = sp_v
        csim[tt]   = c_v
    end

    return [zsim[dropT+1:capT] bsim[dropT+1:capT] def_sim[dropT+1:capT] spsim[dropT+1:capT] ysim[dropT+1:capT] csim[dropT+1:capT] bpy_sim[dropT+1:capT]]
end



################ ------------------- FUNCTIONS COMPUTE MOMENTS ------------------- ################
function compute_moments_country(cm::CoreModuleSC, asm::AshocksModuleSC, country_vars)

    capT = size(country_vars, 1)

    zEMsim  = country_vars[:, 1] # Z_{EM,t}
    btsim   = country_vars[:, 2] # b_{t}
    defsim  = country_vars[:, 3]
    spsim   = country_vars[:, 4]
    ytsim   = country_vars[:, 5]
    bpy_sim = country_vars[:, 7]

    numer = 0
    denom = 0

    for t=2:capT
        if defsim[t]==1 && defsim[t]!=defsim[t-1]
            numer += 1
        end
        if defsim[t]==0 && defsim[t]==defsim[t-1]
            denom += 1
        end
    end    

    def_freq = numer/denom * 100

    by_ratio = btsim./ytsim

    mean_sp  = mean(min.(spsim[defsim.==0.0], 5))*10_000
    std_sp   = std(min.(spsim[defsim.==0.0], 5))*10_000
    mean_by  = mean(bpy_sim[defsim.==0.0])*100
    corr_spY = cov(spsim[defsim.==0.0], ytsim[defsim.==0.0]) / (std(spsim[defsim.==0.0])*std(ytsim[defsim.==0.0])) * 100
    
    return [mean_sp; std_sp; mean_by; corr_spY; def_freq]
end


function compute_moments_panel(YEMsim, ωsim, panel_vars, ReDM, RDM)

    # moments to compute: Avg_B/Y, def rate, corr(SP_i,GDP_i), Avg_SP, std_SP, beta_sys, beta_idi, corr(SP_i, SP)

    capT = size(YEMsim, 1)
    capN = size(panel_vars[1], 2)

    ### Extract simulated aggregate variables
    # Bvec, Dvec, aDMsim, ReEMvec, ntvec, Bhatvec, Btildevec, frac_rep, leverage_vec, def_measure = 
    #     agg_vars[:, 1], agg_vars[:, 2], agg_vars[:, 3], agg_vars[:, 4], agg_vars[:, 5], agg_vars[:, 6], agg_vars[:, 7], agg_vars[:, 8], agg_vars[:, 9], agg_vars[:, 10]

    # Bvec, Dvec, aDMsim, ReEMvec, ntvec, Bhatvec, Btildevec, frac_rep, leverage_vec, def_measure = 
    # Bvec[1:end], Dvec[1:end], aDMsim[1:end], ReEMvec[1:end-1], ntvec[1:end-1], Bhatvec[1:end-1], Btildevec[1:end-1], frac_rep[1:end-1], leverage_vec[1:end-1], def_measure[1:end-1]

    zsim, bsim, def_sim, spsim, ysim, csim, bpy_sim = panel_vars[1], panel_vars[2], panel_vars[3], panel_vars[4], panel_vars[5], panel_vars[6], panel_vars[7]

    corr_fun(x1, x2) = cov(x1, x2) / ( std(x1)*std(x2) )

    ind_tmp1 = (def_sim.==0.0).*(spsim.<1.0)


    ### CROSS-SECTIONAL MOMENTS

    csec_mean_DY = zeros(capT-1)
    csec_std_yt, csec_std_ct, csec_std_tb  = copy(csec_mean_DY), copy(csec_mean_DY), copy(csec_mean_DY)
    csec_std_DY, csec_mean_SP, csec_std_SP = copy(csec_mean_DY), copy(csec_mean_DY), copy(csec_mean_DY)

    aggreg_sp = copy(csec_mean_DY)

    for tt=1:capT-1
        yt_vals = vec(ysim[tt, :])
        by_vals = vec(bpy_sim[tt, :])
        sp_vals = vec(spsim[tt, :])
        ct_vals = vec(csim[tt, :])
        tb_vals = (yt_vals-ct_vals)./yt_vals
        ind_tmp = vec(ind_tmp1[tt,:])#.*vec(sp_vals.>=SPp5[tt]).*vec(sp_vals.<=SPp95[tt])

        yt_vals = yt_vals[ind_tmp]
        by_vals = by_vals[ind_tmp]
        sp_vals = sp_vals[ind_tmp]
        ct_vals = ct_vals[ind_tmp]
        tb_vals = tb_vals[ind_tmp]

        csec_std_yt[tt]  = std(yt_vals)
        csec_std_ct[tt]  = std(ct_vals)
        csec_std_tb[tt]  = std(tb_vals)*100

        csec_mean_DY[tt] = mean(by_vals)*100
        csec_std_DY[tt]  = std(by_vals)*100
        csec_mean_SP[tt] = mean(sp_vals)*10_000
        csec_std_SP[tt]  = std(sp_vals)*10_000

        aggreg_sp[tt] = mean(sp_vals)
    end

    avg_csec_std_yt = mean(csec_std_yt)
    avg_csec_std_ct = mean(csec_std_ct)
    avg_csec_std_tb = mean(csec_std_tb)
    avg_DY = mean(csec_mean_DY)
    avg_SP = mean(csec_mean_SP)

    avg_csec_std_DY = mean(csec_std_DY)
    avg_csec_std_SP = mean(csec_std_SP)

    corr_csec_avg_DY_YEM = corr_fun(csec_mean_DY, YEMsim[1:end-1])*100
    corr_csec_avg_SP_YEM = corr_fun(csec_mean_SP, YEMsim[1:end-1])*100
    corr_csec_std_DY_YEM = corr_fun(csec_std_DY, YEMsim[1:end-1])*100
    corr_csec_std_SP_YEM = corr_fun(csec_std_SP, YEMsim[1:end-1])*100   

    corr_csec_avg_DY_ReDM = corr_fun(csec_mean_DY, ReDM[1:end-1])*100
    corr_csec_avg_SP_ReDM = corr_fun(csec_mean_SP, ReDM[1:end-1])*100
    corr_csec_std_DY_ReDM = corr_fun(csec_std_DY, ReDM[1:end-1])*100
    corr_csec_std_SP_ReDM = corr_fun(csec_std_SP, ReDM[1:end-1])*100

    out1   = [avg_csec_std_DY; corr_csec_avg_DY_YEM; corr_csec_std_DY_YEM]
    out2   = [avg_csec_std_SP; corr_csec_avg_SP_YEM; corr_csec_std_SP_YEM]
    outYEM = [out1 out2]

    out1    = [corr_csec_avg_DY_ReDM; corr_csec_std_DY_ReDM]
    out2    = [corr_csec_avg_SP_ReDM; corr_csec_std_SP_ReDM]
    outReEM = [out1 out2]

    out_cross_section = (outYEM, outReEM)


    # fig, axis = subplots(1, 2)
    # ax = axis[1]; ax[:scatter](csec_mean_SP, ωsim[1:end-1])
    # ax = axis[2]; ax[:plot](YEMsim[1:end-1])


    ### TIME SERIES MOMENTS

    tmp_vec  = collect(1:1:capN)

    corr_sp_Y = zeros(capN)
    def_freq  = zeros(capN)

    # def_sim = def_sim[1:capT-1, :]
    bpsim   = bsim[2:capT, :]
    btsim   = bsim[1:capT-1, :]


    corr_SPi_SPEM = Array{Float64}(capN)
    corr_SPi_ReDM = Array{Float64}(capN)
    corr_SPi_RDM  = Array{Float64}(capN)
    cf_mat        = Array{Float64}(4, capN)
    se_mat        = Array{Float64}(4, capN)
    R2_mat        = Array{Float64}(capN)
    sp_hat        = zeros(0)
    sp_obs        = zeros(0)    
    sp_country    = zeros(capN)
    std_yt, std_ct, std_tb = zeros(capN), zeros(capN), zeros(capN)
    corr_bp_yi, corr_ci_yi, corr_tb_yi = zeros(capN), zeros(capN), zeros(capN)
    rho_ct, rho_tb = zeros(capN), zeros(capN)


    mean_DY     = 0.0
    mean_spread = 0.0
    std_spread  = 0.0

    ind_tmp1 = ind_tmp1[1:capT-1, :]

    for ic=1:capN
        sp_vals  = vec(spsim[1:capT-1, ic])
        z_vals   = vec(zsim[1:capT-1, ic])
        yt_vals  = vec(ysim[1:capT-1, ic])
        ct_vals  = vec(csim[1:capT-1, ic])
        b_vals   = vec(btsim[:, ic])
        by_vals  = vec(bpy_sim[:, ic])
        bp_vals  = vec(bpsim[:, ic])
        tb_vals  = (yt_vals-ct_vals)./yt_vals

        ReDM_vals = ReDM[1:capT-1]
        RDM_vals  = RDM[1:capT-1]

        ind_tmp  = ind_tmp1[:, ic]#.*vec(sp_vals.>=SPp5_tmp).*vec(sp_vals.<=SPp95_tmp)


        yt_vals = yt_vals[ind_tmp]
        by_vals = by_vals[ind_tmp]
        bp_vals = bp_vals[ind_tmp]
        sp_vals = sp_vals[ind_tmp]
        ct_vals = ct_vals[ind_tmp]
        tb_vals = tb_vals[ind_tmp]

        aggSP_vals = aggreg_sp[ind_tmp]
        ReDM_vals  = ReDM_vals[ind_tmp]
        RDM_vals   = RDM_vals[ind_tmp]



        # ind_tmp  = (def_sim[1:capT-1, ic].==0.0).*(sp_vals.<1.0)
        # ind_tmp2 = (ind_tmp[2:end].==1.0).*(ind_tmp[1:end-1].==1.0)

        # XX_ct      = [ones(capT-1) c_vals[1:end-1]].*ind_tmp2
        # XX_tb      = [ones(capT-1) tb_vals[1:end-1]].*ind_tmp2
        # coeff_ct   = inv(XX_ct'*XX_ct)*XX_ct'*(c_vals[2:end].*ind_tmp2)
        # coeff_tb   = inv(XX_tb'*XX_tb)*XX_tb'*(tb_vals[2:end].*ind_tmp2)
        # rho_ct[ic] = coeff_ct[2]
        # rho_tb[ic] = coeff_tb[2]
        

        std_yt[ic] = std(yt_vals)
        std_ct[ic] = std(ct_vals)
        std_tb[ic] = std(tb_vals)


        corr_bp_yi[ic] = corr_fun(yt_vals, by_vals)
        corr_ci_yi[ic] = corr_fun(yt_vals, ct_vals)
        corr_tb_yi[ic] = corr_fun(yt_vals, tb_vals)

        sp_country[ic] = mean(sp_vals) 

        # ### Regression of spreads onto YEM, ZEM and ω
        # XX       = [ones(sp_vals[ind_tmp]) YEMsim[ind_tmp] z_vals[ind_tmp] ωsim[ind_tmp]]

        # coeffs   = vec(inv(XX'*XX)*XX'*sp_vals[ind_tmp])
        # cf_mat[:, ic] = coeffs

        # sp_hat_tmp = vec(XX*coeffs)
        # sp_obs_tmp = sp_vals[ind_tmp]
        # sp_res_tmp = sp_hat_tmp - sp_obs_tmp

        # sp_obs_mean = mean(sp_obs_tmp)
        # SStot_sp    = sum((sp_obs_tmp-sp_obs_mean).^2)
        # SSres_sp    = sum(sp_res_tmp.^2)
        # R2          = 1.0 - SSres_sp / SStot_sp

        # Qxx      = inv(XX'*XX)
        # s2       = SSres_sp/(length(sp_res_tmp)-4)
        # se_coeff = s2*diag(Qxx)

        # R2_mat[ic]    = R2
        # se_mat[:, ic] = se_coeff

        # append!(sp_hat, sp_hat_tmp)
        # append!(sp_obs, sp_obs_tmp)      

        # # XX       = [ones(sp_vals[rep_vals.==1.0]) YEMsim[rep_vals.==1.0] z_vals[rep_vals.==1.0]]
        # # coeffs   = vec(inv(XX'*XX)*XX'*sp_vals[rep_vals.==1.0])
        # # cf_mat[:, ic] = [coeffs; 0.0]


        ### Moments for Spreads
        mean_spread += mean(sp_vals)
        std_spread  += std(sp_vals)


        corr_sp_Y[ic] = cov(sp_vals, yt_vals)/(std(sp_vals)*std(yt_vals))
        
        cum_num = 0.0; cum_den = 0.0

        for tt=2:capT-1
            if (def_sim[tt-1, ic] == 0.0 && def_sim[tt, ic] == 1.0)
                cum_num += 1.0
            end
            if def_sim[tt-1, ic]==0.0
                cum_den += 1.0
            end
        end

        def_freq[ic] = cum_num / cum_den

        mean_DY += mean(by_vals)
    
        corr_SPi_SPEM[ic] = cov(sp_vals, aggSP_vals) / (std(sp_vals)*std(aggSP_vals))
        corr_SPi_ReDM[ic] = cov(sp_vals, ReDM_vals) / (std(sp_vals)*std(ReDM_vals))
        corr_SPi_RDM[ic]  = cov(sp_vals, RDM_vals)  / (std(sp_vals)*std(RDM_vals))

    end

    # Moments - Country Average
    mean_DY     = mean_DY / capN*100
    mean_spread = mean_spread/capN*10_000
    std_spread  = std_spread/capN*10_000
    DF_mean     = mean(def_freq)*100
    cf_mean     = mean(cf_mat, 2)
    cf_median   = median(cf_mat, 2)


    # # POLS
    # sp_vals     = spsim[def_sim.==0]
    # z_vals      = zsim[def_sim.==0]
    # intercp     = ones(sp_vals)
    # YEM_vals    = repmat(YEMsim, 1, capN)
    # ω_vals    = repmat(ωsim, 1, capN)
    # YEM_vals    = YEM_vals[def_sim.==0]
    # ω_vals    = ω_vals[def_sim.==0]
    # XX          = [intercp YEM_vals z_vals ω_vals]
    # coeff_POLS  = inv(XX'*XX)*XX'*sp_vals



    out_basic_mms   = [mean_DY; mean_spread; std_spread; DF_mean]
    out_ts_country  = [mean(std_ct./std_yt); mean(std_tb); mean(corr_bp_yi)*100; mean(corr_ci_yi)*100; mean(corr_tb_yi)*100; mean(corr_sp_Y)*100]
    out_ts_corrs    = [mean(corr_SPi_SPEM)*100; mean(corr_SPi_ReDM)*100; mean(corr_SPi_RDM)*100 ]
    out_time_series = (out_basic_mms, out_ts_country, out_ts_corrs, cf_mean, cf_median)

    return(out_cross_section, out_time_series)

end


function compute_moments_panel_calib(cm::CoreModuleSC, asm::AshocksModuleSC, YEMsim, ωsim, ADMsim, panel_vars, q_DFRSK, q_IR) # this function is reduced (just for calibration)

    # moments to compute: Avg_B/Y, def rate, Avg_SP, std_SP (by country), corr(SP_i,GDP_i), sd(Ci)/sd(Yi), corr(Ci,Yi), sd(TBi/Yi), corr(TBi/Yi, Yi), corr(SP_i, SP)
    # cross-sectional: c.s. sd(D/Y), c.s. sd(SP), corr(E[D/Y],E[Y]), corr(E[SP],E[Y]), corr(c.s.sd(D/Y), E[Y]), corr(c.s.sd(SP), E[Y])

    capT = size(YEMsim, 1)
    capN = size(panel_vars[1], 2)

    zsim, bsim, def_sim, spsim, ysim, csim, bpy_sim = panel_vars[1], panel_vars[2], panel_vars[3], panel_vars[4], panel_vars[5], panel_vars[6], panel_vars[7]

    corr_fun(x1, x2) = cov(x1, x2) / ( std(x1)*std(x2) )

    ind_tmp1 = (def_sim.==0.0).*(spsim.<10_000/10_000)



    knots      = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK = interpolate(knots, q_DFRSK, Gridded(Linear()))
    itp_qIR    = interpolate((asm.YEMgrid, cm.ωgrid, cm.aDMgrid), q_IR, Gridded(Linear()))


    ### CROSS-SECTIONAL MOMENTS

    csec_mean_DY = zeros(capT-1)
    csec_std_yt, csec_std_ct, csec_std_tb  = copy(csec_mean_DY), copy(csec_mean_DY), copy(csec_mean_DY)
    csec_std_DY, csec_mean_SP, csec_std_SP = copy(csec_mean_DY), copy(csec_mean_DY), copy(csec_mean_DY)
    csec_mean_TB, csec_mean_CA  = copy(csec_mean_DY), copy(csec_mean_DY)
    csec_mean_Y   = copy(csec_mean_DY)
    csec_mean_lnY = copy(csec_mean_DY)

    aggreg_sp = copy(csec_mean_DY)

    for tt=1:capT-1
        yy_vals = ysim[tt, :]
        by_vals = vec(bpy_sim[tt, :])
        sp_vals = vec(spsim[tt, :])
        cc_vals = vec(csim[tt, :])
        ind_tmp = vec(ind_tmp1[tt,:])

        yy_vals = yy_vals[ind_tmp]
        by_vals = by_vals[ind_tmp]
        sp_vals = sp_vals[ind_tmp]
        cc_vals = cc_vals[ind_tmp]
        tb_vals = (yy_vals-cc_vals)./yy_vals

        csec_mean_Y[tt]  = mean(yy_vals)
        csec_mean_DY[tt] = mean(by_vals)
        csec_mean_SP[tt] = mean(sp_vals)
        csec_mean_TB[tt] = mean(tb_vals)
        csec_mean_CA[tt]
        csec_std_DY[tt]  = std(by_vals)
        csec_std_SP[tt]  = std(sp_vals)
        aggreg_sp[tt]    = mean(sp_vals)

        csec_mean_lnY[tt] = mean(log.(yy_vals))
    end

    cs_sd_by       = mean(csec_std_DY)*100
    cs_sd_SP       = mean(csec_std_SP)*10_000
    cs_corr_by_y   = corr_fun(csec_mean_DY, csec_mean_Y)
    cs_corr_SP_y   = corr_fun(csec_mean_SP, csec_mean_Y)
    cs_corr_sdby_y = corr_fun(csec_std_DY, csec_mean_Y)
    cs_corr_sdSP_y = corr_fun(csec_std_SP, csec_mean_Y)

    cs_corr_by_lny   = corr_fun(csec_mean_DY, csec_mean_lnY)
    cs_corr_SP_lny   = corr_fun(csec_mean_SP, csec_mean_lnY)
    cs_corr_sdby_lny = corr_fun(csec_std_DY, csec_mean_lnY)
    cs_corr_sdSP_lny = corr_fun(csec_std_SP, csec_mean_lnY)
    cs_corr_TB_lny   = corr_fun(csec_mean_TB, csec_mean_lnY)

    out_cross_section = [cs_sd_by; cs_sd_SP; cs_corr_by_y; cs_corr_SP_y; cs_corr_sdby_y; cs_corr_sdSP_y; cs_corr_by_lny; cs_corr_SP_lny; cs_corr_sdby_lny; cs_corr_sdSP_lny; cs_corr_TB_lny]



    ### TIME SERIES MOMENTS
    # Avg_B/Y, def rate, Avg_SP, std_SP (by country), corr(SP_i,GDP_i), sd(Ci)/sd(Yi), corr(Ci,Yi), sd(TBi/Yi), corr(TBi/Yi, Yi), corr(SP_i, SP)

    corr_sp_Y = zeros(capN)
    def_freq  = zeros(capN)

    # def_sim = def_sim[1:capT-1, :]

    mean_DY, mean_spread, std_spread, std_tb, sdci_sdyi, corr_ci_yi, sd_TBi_yi, corr_TBi_yi, corr_spi_SP = 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0

    mean_SP_DFRSK, mean_SP_TOTAL, mean_SP_IMRSK, std_SP_DFRSK, std_SP_TOTAL, std_SP_IMRSK = 0.0, 0.0, 0.0, 0.0, 0.0, 0.0
    mean_SP_IR, mean_SP_PR, std_SP_IR, std_SP_PR = 0.0, 0.0, 0.0, 0.0
    std_tb2, sd_TBi_yi2, corr_TBi_yi2, corr_spi_lnci, corr_spi_TBi = 0.0, 0.0, 0.0, 0.0, 0.0
    ln_sdci_sdyi, ln_sd_TBi_yi, ln_corr_ci_yi, ln_corr_TBi_yi, ln_corr_SP_yi = 0.0, 0.0, 0.0, 0.0, 0.0

    ind_tmp1    = ind_tmp1[1:capT-1, :]

    for ic=1:capN
        sp_vals  = vec(spsim[1:capT-1, ic])
        yt_vals  = vec(ysim[1:capT-1, ic])
        b_vals   = vec(bsim[1:capT-1, ic])
        by_vals  = vec(bpy_sim[1:capT-1, ic])
        ct_vals  = vec(csim[1:capT-1, ic])
        tb_vals  = (yt_vals-ct_vals)./yt_vals

        ind_tmp  = ind_tmp1[:, ic]

        yt_vals  = yt_vals[ind_tmp]
        by_vals  = by_vals[ind_tmp]
        sp_vals  = sp_vals[ind_tmp]
        ct_vals  = ct_vals[ind_tmp]
        tb_vals  = tb_vals[ind_tmp]
        tb_vals2 = (yt_vals-ct_vals)./yt_vals

        mean_DY     += mean(by_vals)
        mean_spread += mean(sp_vals)
        std_spread  += std(sp_vals)
        std_tb      += std(tb_vals)
        sdci_sdyi   += std(ct_vals) / std(yt_vals)
        sd_TBi_yi   += std(tb_vals) / std(yt_vals)
        corr_ci_yi  += corr_fun(ct_vals, yt_vals)
        corr_TBi_yi += corr_fun(tb_vals, yt_vals)
        corr_spi_SP += corr_fun(sp_vals, aggreg_sp[ind_tmp])

        ln_sdci_sdyi   += std(log.(ct_vals)) / std(log.(yt_vals))
        ln_sd_TBi_yi   += std(tb_vals) / std(log.(yt_vals))
        ln_corr_ci_yi  += corr_fun(log.(ct_vals), log.(yt_vals))
        ln_corr_TBi_yi += corr_fun(tb_vals, log.(yt_vals))
        ln_corr_SP_yi  += corr_fun(sp_vals, log.(yt_vals))

        corr_spi_lnci  += corr_fun(sp_vals, log.(ct_vals))
        corr_spi_TBi   += corr_fun(sp_vals, tb_vals)

        std_tb2       += std(tb_vals2)
        sd_TBi_yi2    += std(tb_vals2) / std(yt_vals)
        corr_TBi_yi2  += corr_fun(tb_vals2, yt_vals)

        corr_sp_Y[ic] = corr_fun(sp_vals, yt_vals)
        
        cum_num = 0.0; cum_den = 0.0

        SPTOT_nodef, SPDFRSK_nodef, SPIR_nodef = [], [], []

        for tt=2:capT-1
            defm1  = def_sim[tt-1, ic]
            deft   = def_sim[tt, ic]
            beta_v = 1.0/(1.0+sc_RF_fun(cm, ωsim[tt]))

            if (defm1 == 0.0 && deft == 1.0)
                cum_num += 1.0
            end
            if defm1==0.0
                cum_den += 1.0
            end

            if deft==0.0
                bp_v = bsim[tt+1, ic]
                z_v  = zsim[tt, ic]
                qDFRSK_v     = itp_qDFRSK[bp_v, z_v, YEMsim[tt], ωsim[tt], ADMsim[tt+1]]
                qIR_v        = itp_qIR[YEMsim[tt], ωsim[tt], ADMsim[tt+1]]
                spDFRSK_v    = (1.0/qDFRSK_v + cm.ξ).*beta_v - 1.0
                spIR_v       = (1.0/qIR_v + cm.ξ).*beta_v - 1.0
                if spDFRSK_v < 10_000/10_000
                    push!(SPDFRSK_nodef, spDFRSK_v)
                    push!(SPIR_nodef, spIR_v)
                    push!(SPTOT_nodef, spsim[tt, ic])
                end
            end
        end

        def_freq[ic] = cum_num / cum_den

        SPDFRSK_nodef = convert(Array{Float64}, SPDFRSK_nodef)
        SPTOT_nodef   = convert(Array{Float64}, SPTOT_nodef)
        SPIR_nodef    = convert(Array{Float64}, SPIR_nodef)
        SPIMRSK_nodef = SPTOT_nodef - SPDFRSK_nodef
        SP_PureRisk   = SPIMRSK_nodef - SPIR_nodef

        mean_SP_DFRSK += mean(SPDFRSK_nodef)
        mean_SP_TOTAL += mean(SPTOT_nodef)
        mean_SP_IMRSK += mean(SPIMRSK_nodef)
        mean_SP_IR    += mean(SPIR_nodef)
        mean_SP_PR    += mean(SP_PureRisk)
        std_SP_DFRSK  += std(SPDFRSK_nodef)
        std_SP_TOTAL  += std(SPTOT_nodef)
        std_SP_IMRSK  += std(SPIMRSK_nodef)
        std_SP_IR     += std(SPIR_nodef)
        std_SP_PR     += std(SP_PureRisk)
    end

    # Moments - Country Average
    mean_DY     = mean_DY / capN*100
    mean_spread = mean_spread/capN*10_000
    std_spread  = std_spread/capN*10_000
    std_tb      = std_tb/capN
    DF_mean     = mean(def_freq)*100
    sdci_sdyi   = sdci_sdyi / capN
    corr_ci_yi  = corr_ci_yi / capN
    sd_TBi_yi   = sd_TBi_yi / capN
    corr_TBi_yi = corr_TBi_yi / capN
    corr_spi_SP = corr_spi_SP / capN

    std_tb2      = std_tb2/capN
    sd_TBi_yi2   = sd_TBi_yi2 / capN
    corr_TBi_yi2 = corr_TBi_yi2 / capN    

    ln_sdci_sdyi   = ln_sdci_sdyi  /capN
    ln_sd_TBi_yi   = ln_sd_TBi_yi  /capN
    ln_corr_ci_yi  = ln_corr_ci_yi /capN
    ln_corr_TBi_yi = ln_corr_TBi_yi/capN
    ln_corr_SP_yi  = ln_corr_SP_yi/capN

    corr_spi_lnci = corr_spi_lnci / capN
    corr_spi_TBi  = corr_spi_TBi / capN

    out_time_series = [mean_DY; DF_mean; mean_spread; std_spread; std_tb; mean(corr_sp_Y)*100; sdci_sdyi; corr_ci_yi; sd_TBi_yi; corr_TBi_yi; corr_spi_SP; std_tb2; sd_TBi_yi2; corr_TBi_yi2; ln_sdci_sdyi; ln_sd_TBi_yi; ln_corr_ci_yi; ln_corr_TBi_yi; ln_corr_SP_yi; corr_spi_lnci; corr_spi_TBi]
    out_decomp      = [mean_SP_DFRSK/capN*10_000; mean_SP_TOTAL/capN*10_000; mean_SP_IMRSK/capN*10_000; std_SP_DFRSK/capN*10_000; std_SP_TOTAL/capN*10_000; std_SP_IMRSK/capN*10_000]
    out_dcmp_interm = [mean_SP_IR/capN*10_000; mean_SP_PR/capN*10_000; std_SP_IR/capN*10_000; std_SP_PR/capN*10_000; 0.0; 0.0]

    return(out_cross_section, out_time_series, out_decomp, out_dcmp_interm)

    

end


function compute_moments_country_calib(cm::CoreModuleSC, asm::AshocksModuleSC, YEMsim, ωsim, ADMsim, EMSP, country_vars, q_DFRSK) # this function is reduced (just for calibration)

    # moments to compute: Avg_B/Y, def rate, Avg_SP, std_SP (by country), corr(SP_i,GDP_i), sd(Ci)/sd(Yi), corr(Ci,Yi), sd(TBi/Yi), corr(TBi/Yi, Yi), corr(SP_i, SP)
    # cross-sectional: c.s. sd(D/Y), c.s. sd(SP), corr(E[D/Y],E[Y]), corr(E[SP],E[Y]), corr(c.s.sd(D/Y), E[Y]), corr(c.s.sd(SP), E[Y])

    capT = size(YEMsim, 1)

    zsim, bsim, def_sim, spsim, ysim, csim, bpy_sim = country_vars[:, 1], country_vars[:, 2], country_vars[:, 3], country_vars[:, 4], country_vars[:, 5], country_vars[:, 6], country_vars[:, 7]
    tbsim  = (ysim-csim)./ysim

    corr_fun(x1, x2) = cov(x1, x2) / ( std(x1)*std(x2) )


    ind_tmp1   = (def_sim.==0.0).*(spsim.<5_000/10_000)    

    knots      = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK = interpolate(knots, q_DFRSK, Gridded(Linear()))


    cum_num, cum_den = 0.0, 0.0
    SPTOT_nodef, SPDFRSK_nodef = [], []

    for t=2:capT-1
        defm1  = def_sim[t-1]
        deft   = def_sim[t]
        beta_v = 1.0/(1.0+sc_RF_fun(cm, ωsim[t]))
        if (defm1 == 0.0 && deft == 1.0)
            cum_num += 1.0
        end
        if defm1==0.0
            cum_den += 1.0
        end
        if deft==0.0
            bp_v = bsim[t+1]
            z_v  = zsim[t]
            qDFRSK_v     = itp_qDFRSK[bp_v, z_v, YEMsim[t], ωsim[t], ADMsim[t+1]]
            spDFRSK_v    = (1.0/qDFRSK_v + cm.ξ).*beta_v - 1.0
            if spDFRSK_v < 1_500/10_000
                push!(SPDFRSK_nodef, spDFRSK_v)
                push!(SPTOT_nodef, spsim[t])
            end
        end
    end    

    def_freq = cum_num/cum_den * 100

    mean_sp    = mean(spsim[ind_tmp1])*10_000
    std_sp     = std(spsim[ind_tmp1])*10_000
    mean_by    = mean(bpy_sim[ind_tmp1])*100
    corr_spY   = corr_fun(spsim[ind_tmp1], ysim[ind_tmp1])
    stdc_stdY  = std(csim[ind_tmp1]) / std(ysim[ind_tmp1])
    corr_cY    = corr_fun(csim[ind_tmp1], ysim[ind_tmp1])
    std_tb     = std(tbsim[ind_tmp1])
    corr_tbY   = corr_fun(tbsim[ind_tmp1], ysim[ind_tmp1])
    corr_spiSP = corr_fun(spsim[ind_tmp1], EMSP[ind_tmp1])
    std_tb2    = std(tbsim)
    corr_tbY2  = corr_fun(tbsim, ysim)

    ln_corr_spY   = corr_fun(spsim[ind_tmp1], log.(ysim[ind_tmp1]))
    ln_stdc_stdY  = std(log.(csim[ind_tmp1])) / std(log.(ysim[ind_tmp1]))
    ln_corr_cY    = corr_fun(log.(csim[ind_tmp1]), log.(ysim[ind_tmp1]))
    ln_corr_tbY   = corr_fun(tbsim[ind_tmp1], log.(ysim[ind_tmp1]))


    SPDFRSK_nodef = convert(Array{Float64}, SPDFRSK_nodef)
    SPTOT_nodef   = convert(Array{Float64}, SPTOT_nodef)
    SPIMRSK_nodef = SPTOT_nodef - SPDFRSK_nodef

    mean_SP_DFRSK = mean(SPDFRSK_nodef)*10_000
    mean_SP_TOTAL = mean(SPTOT_nodef)*10_000
    mean_SP_IMRSK = mean(SPIMRSK_nodef)*10_000
    std_SP_DFRSK  = std(SPDFRSK_nodef)*10_000
    std_SP_TOTAL  = std(SPTOT_nodef)*10_000
    std_SP_IMRSK  = std(SPIMRSK_nodef)*10_000


    out_moment = [mean_by; def_freq; mean_sp; std_sp; corr_spY; stdc_stdY; corr_cY; std_tb; corr_tbY; corr_spiSP; std_tb2; corr_tbY2; ln_corr_spY; ln_stdc_stdY; ln_corr_cY; ln_corr_tbY]
    out_decomp = [mean_SP_DFRSK; mean_SP_TOTAL; mean_SP_IMRSK; std_SP_DFRSK; std_SP_TOTAL; std_SP_IMRSK]

    out_for_HP = [tbsim[ind_tmp1] ysim[ind_tmp1] YEMsim[ind_tmp1]]

    return(out_moment, out_decomp, out_for_HP)

end


function simulate_mktval(cm::CoreModuleSC, asm::AshocksModuleSC, ω_hist, ADM_hist, NW_hist)

    vs_calib = zeros(length(ω_hist))

    itp_vs    = interpolate((cm.ωgrid, cm.aDMgrid), asm.vs, Gridded(Linear()))

    for t=1:length(ω_hist)
        vs_calib[t]         = itp_vs[ω_hist[t], ADM_hist[t+1]]
    end

    mktval  = vs_calib.*NW_hist

    return mktval
end


function default_windows(capT_w, simul_country)

    interval  = round(Int, (capT_w-1)/2)

    def_sim   = simul_country[:, 3]
    vc_sim    = simul_country[:, 6]
    vd_sim    = simul_country[:, 7]
    vctil_sim = simul_country[:, 8]

    iter = 0
    capT = size(vc_sim, 1)

    vcsim_mat    = Array{Float64}(capT_w, round(Int, capT/capT_w))
    vdsim_mat    = copy(vcsim_mat)
    vctilsim_mat = copy(vcsim_mat)
    defsim_mat   = copy(vcsim_mat)

    ind_events = []

    for tt=interval+1:capT-interval-1
        if (def_sim[tt] == 1.0) && (def_sim[tt-1] == 0.0)
            push!(ind_events, tt)
        end
    end

    ind_t_high = 0
    ind_t_low  = 0


    while (ind_t_low<capT-capT_w)

        if isempty(ind_events)
            break
        end

        ind_t_low  = ind_events[1] - interval

        if ( ind_t_low + capT_w - 1 > capT )
            break
        end

        ind_t_high = ind_events[1] + interval

        if ( iter+1 > capT/capT_w )
            break
        end

        iter += 1

        vcsim_mat[:, iter]    = vc_sim[ind_t_low:ind_t_high]
        vdsim_mat[:, iter]    = vd_sim[ind_t_low:ind_t_high]
        vctilsim_mat[:, iter] = vctil_sim[ind_t_low:ind_t_high]
        defsim_mat[:, iter]   = def_sim[ind_t_low:ind_t_high]


        if length(ind_events)==1; break; end

        if ind_events[2] > ind_t_high
            ind_events = ind_events[2:end]
        else
            ind_tmp    = searchsortedfirst(ind_events, ind_t_high)

            if isempty(ind_tmp)
                break
            end
            ind_events = ind_events[ind_tmp:end]
        end
    end

    return(vcsim_mat[:, 1:iter], vdsim_mat[:, 1:iter], vctilsim_mat[:, 1:iter], defsim_mat[:, 1:iter])
end


function compute_Lehman_proxy(cm::CoreModuleSC, asm::AshocksModuleSC, init_val, omg_path, YEM_path, Λerg, data_in, mms_in_mod, mms_in_dat, itp_HYSP, itp_qDFRSK; type_shock=3, full_dyn=false)

    # Create interpolation/extrapolation object
    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_def     = interpolate(knots, asm.defmat,     Gridded(Linear()))
    itp_q       = interpolate(knots, asm.q_mat,  Gridded(Linear()))
    itp_bp      = interpolate(knots, asm.bprime_mat, Gridded(Linear()))
    itp_ReDM    = interpolate((cm.ωgrid, cm.aDMgrid), asm.ReDM, Gridded(Linear()))
    itp_Red     = interpolate((cm.ωgrid, cm.aDMgrid), asm.Red, Gridded(Linear()))


    # Compute difference E[nu'RDM'] - E[nu'Rd]
    diff_rates      = asm.ReDM - asm.Red
    itp_diff_rates  = interpolate((cm.ωgrid, cm.aDMgrid), diff_rates, Gridded(Linear()))
    

    capT  = size(omg_path, 1)
    NS    = size(cm.snodes, 1)
    

    ### Simulation in absence of shocks
    Btil_path_NS, Bhat_path_NS, yield_path_NS, EMSP_path_NS, HYSP_path_NS, NW_path_NS, Ct_path_NS = zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT)
    TB_path_NS, C_Y_path_NS, SPDF_path_NS, defmeas_path_NS = zeros(capT), zeros(capT), zeros(capT), zeros(capT)
    Bt_path_NS = zeros(capT+1)
    rf_path_NS = zeros(capT)

    ADM_path_NS    = zeros(capT+1) # time t
    D_path_NS      = zeros(capT+1) # time t
    ADM_path_NS[1] = init_val[1]
    D_path_NS[1]   = init_val[2]

    Λerg_mkt = Λerg[cm.num_nodes+1:end]
    sum_Λ   = sum(Λerg_mkt)

    Bt = 0.0
    for i_s=1:cm.num_nodes
        Bt    += Λerg_mkt[i_s] * cm.snodes[i_s, 1]
    end

    Bt_path_NS[1] = Bt * cm.μEM / sum_Λ

    YEM_path_NS = zeros(YEM_path)
    omg_path_NS = ones(capT)

    Λt = copy(Λerg)


    # Bounds for finding ADM
    bounds = [cm.aDMgrid[1]*0.9; asm.ADMmax]

    for t=1:capT

        rf_val  = sc_RF_fun(cm, omg_path_NS[t])
        ub_ADM  = fzero(x->itp_diff_rates[omg_path_NS[t],x], cm.aDMgrid[end])
        bounds  = [cm.aDMgrid[1]*0.9; ub_ADM]   # Bounds for finding ADM

        in_state_t  = [YEM_path_NS[t]; omg_path_NS[t]; ADM_path_NS[t]; D_path_NS[t]; rf_val]
        out_X   = solve_system_calib(cm, asm, in_state_t, Λt, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, bounds, itp_qDFRSK) # (out_obj, Λt1)
        # out_obj = [aDMp_v; Btilv; Bhatv; Dt1; NW; ReDM; SPv; yield_v; by_avg; sdby; sdSP; Ct; SPDFRK; Dpv; aDMpv2[1]; TB; C_Y]

        aDMpv, Btilde, Bhat, Bt1, NW, ReDM, SPv, yieldv = out_X[1][1], out_X[1][2], out_X[1][3], out_X[1][4], out_X[1][5], out_X[1][6], out_X[1][7], out_X[1][8]
        by_avg, sdby, sdSP, Ct, TB, C_Y = out_X[1][9], out_X[1][10], out_X[1][11], out_X[1][12], out_X[1][16], out_X[1][17]
        D_path_NS[t+1] = out_X[1][14]
        
        Λt1 = out_X[2]

        rf_path_NS[t]    = rf_val
        yield_path_NS[t] = yieldv
        NW_path_NS[t]    = NW
        Btil_path_NS[t]  = Btilde
        Bhat_path_NS[t]  = Bhat
        EMSP_path_NS[t]  = SPv
        Ct_path_NS[t]    = Ct
        TB_path_NS[t]    = TB*100
        C_Y_path_NS[t]   = C_Y*100
        HYSP_path_NS[t]  = itp_HYSP[omg_path[t], aDMpv]
        Bt_path_NS[t+1]  = Bt1
        SPDF_path_NS[t]  = out_X[1][13]

        ADM_path_NS[t+1] = aDMpv

        Λt1_mkt = Λt1[cm.num_nodes+1:end]
        defmeas_path_NS[t] = 1.0 - sum(Λt1_mkt)

        Λt = copy(Λt1)

    end

    MV_path_NS = simulate_mktval(cm, asm, omg_path_NS, ADM_path_NS, NW_path_NS)


    ### Simulation given shocks
    Btil_path, Bhat_path, yield_path, EMSP_path, HYSP_path, NW_path, Ct_path = zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT)
    TB_path, C_Y_path, SPDF_path, defmeas_path = zeros(capT), zeros(capT), zeros(capT), zeros(capT)
    Bt_path = zeros(capT+1)
    rf_path = zeros(capT)

    RF_path = zeros(capT)

    ADM_path    = zeros(capT+1) # time t
    D_path      = zeros(capT+1) # time t
    ADM_path[1] = init_val[1]
    D_path[1]   = init_val[2]

    Bt_path[1]  = Bt_path_NS[1]

    Λt = copy(Λerg)

    for t=1:capT
        rf_val  = sc_RF_fun(cm, omg_path[t])
        ub_ADM  = fzero(x->itp_diff_rates[omg_path[t],x], cm.aDMgrid[end])
        bounds  = [cm.aDMgrid[1]*0.9; ub_ADM]   # Bounds for finding ADM

        in_state_t = [YEM_path[t]; omg_path[t]; ADM_path[t]; D_path[t]; rf_val]

        out_X   = solve_system_calib(cm, asm, in_state_t, Λt, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, bounds, itp_qDFRSK) # (out_obj, Λt1)


        aDMpv, Btilde, Bhat, Bt1, NW, ReDM, SPv, yieldv = out_X[1][1], out_X[1][2], out_X[1][3], out_X[1][4], out_X[1][5], out_X[1][6], out_X[1][7], out_X[1][8]
        by_avg, sdby, sdSP, Ct, TB, C_Y = out_X[1][9], out_X[1][10], out_X[1][11], out_X[1][12], out_X[1][16], out_X[1][17]
        SPDF_path[t] = out_X[1][13]
        D_path[t+1]  = out_X[1][14]

        Λt1 = out_X[2]

        rf_path[t]    = rf_val
        yield_path[t] = yieldv
        NW_path[t]    = NW
        Btil_path[t]  = Btilde
        Bhat_path[t]  = Bhat
        EMSP_path[t]  = SPv
        Ct_path[t]    = Ct
        TB_path[t]    = TB*100
        C_Y_path[t]   = C_Y*100
        HYSP_path[t]  = itp_HYSP[omg_path[t], aDMpv]
        RF_path[t]    = rf_val
        Bt_path[t+1]  = Bt1

        ADM_path[t+1] = aDMpv

        Λt1_mkt = Λt1[cm.num_nodes+1:end]
        defmeas_path[t] = 1.0 - sum(Λt1_mkt)

        Λt = copy(Λt1)

    end

    MV_path = simulate_mktval(cm, asm, omg_path, ADM_path, NW_path)

    plot_YEM_md_stn  = YEM_path/mms_in_mod[1,2]
    plot_omg_md_stn  = log.(omg_path)/cm.σω
    plot_MV_md_stn   = (log.(MV_path)-log.(MV_path_NS)+log.(MV_path_NS[end])-mms_in_mod[2,1]) / mms_in_mod[2,2]
    plot_Ct_md_stn   = (log.(Ct_path)-log.(Ct_path_NS)+log.(Ct_path_NS[end])-mms_in_mod[3,1]) / mms_in_mod[3,2]       

    plot_YEM_dt_stn  = YEM_path/mms_in_mod[1,2]
    plot_MV_dt_stn   = (data_in[:, 3]-mms_in_dat[2,1])/mms_in_dat[2,2]
    plot_Ct_dt_stn   = (data_in[:, 4]-mms_in_dat[3,1])/mms_in_dat[3,2]

    plot_YEM_md_dev  = YEM_path - mms_in_mod[1,1]
    plot_MV_md_dev   = log.(MV_path)-log.(MV_path_NS)+log.(MV_path_NS[end]) - mms_in_mod[2,1]
    plot_Ct_md_dev   = log.(Ct_path)-log.(Ct_path_NS)+log.(Ct_path_NS[end]) - mms_in_mod[3,1]

    plot_YEM_dt_dev  = YEM_path*(mms_in_dat[1,2]/mms_in_mod[1,2]) - mms_in_dat[1,1]
    plot_MV_dt_dev   = data_in[:, 3]-mms_in_dat[2,1]
    plot_Ct_dt_dev   = data_in[:, 4]-mms_in_dat[3,1]

    plot_EMSP    = EMSP_path-EMSP_path_NS+EMSP_path_NS[end]
    plot_NW      = log.(NW_path)#-log.(NW_path_NS)+log.(NW_path_NS[1])
    plot_ADM     = ADM_path-ADM_path_NS+ADM_path_NS[end]
    plot_D       = D_path-D_path_NS+D_path_NS[end]
    plot_SPDF    = SPDF_path-SPDF_path_NS+SPDF_path_NS[end]

    plot_Bhat    = Bhat_path-Bhat_path_NS+Bhat_path_NS[end]
    plot_Btil    = Btil_path-Btil_path_NS+Btil_path_NS[end]
    plot_HYSP    = HYSP_path-HYSP_path_NS+HYSP_path_NS[end]
    plot_Bt      = Bt_path-Bt_path_NS+Bt_path_NS[end]

    plot_TB      = TB_path-TB_path_NS+TB_path_NS[end]
    plot_C_Y     = C_Y_path-C_Y_path_NS+C_Y_path_NS[end]

    plot_yield   = yield_path-yield_path_NS+yield_path_NS[end]
    plot_defmeas = defmeas_path-defmeas_path_NS+defmeas_path_NS[end]


    Nvars = 16
    out_mat = zeros(capT+1, Nvars)

    out_mat[1:capT, 1:7] = [Btil_path Bhat_path plot_yield plot_EMSP HYSP_path plot_NW plot_MV_md_stn]
    out_mat[:, 8]        = ADM_path
    out_mat[1:capT, 9]   = plot_Ct_md_stn
    out_mat[1:capT, 10]  = plot_TB
    out_mat[1:capT, 11]  = plot_Bt[1:end-1]
    out_mat[1:capT, 12]  = plot_MV_md_dev
    out_mat[1:capT, 13]  = plot_Ct_md_dev
    out_mat[1:capT, 14]  = plot_SPDF
    out_mat[1:capT, 15]  = plot_defmeas
    out_mat[1:capT, 16]  = rf_path
    


    if type_shock==3

        if full_dyn == false
            end_t = 5
            Tvec_plot = collect(2007:1:2014)
        else
            end_t = 8
            Tvec_plot = collect(2004:1:2014)
        end

        data_1      = [plot_YEM_md_stn[1:end_t] plot_YEM_dt_stn[1:end_t]]
        line_type1  = [:solid :dash]
        labels1     = ["Model" "Data"]
        colors1     = [Colors.RGBA(27/255, 129/255, 197/255, 1) :black]
        size1       = (600, 600)
        p1          = plot(Tvec_plot[1:end_t], data_1, linestyle=line_type1, label=labels1, color=colors1, linewidth=3.5, fg_legend = :transparent, legend=:topright, size=size1, xtickfont=font(20), ytickfont=font(20), legendfont=font(20));
        if full_dyn == false
            savefig(p1, "figures/IRF_plots/Lehman_YEM_stn.pdf");
        else
            savefig(p1, "figures/IRF_plots/Lehman_YEM_stn_FD.pdf");
        end        


        size1       = (600, 600)
        p1          = plot(Tvec_plot[1:end_t], plot_omg_md_stn[1:end_t], linestyle=:solid, linewidth=3.5, fg_legend = :transparent, legend=:topright, size=size1, xtickfont=font(20), ytickfont=font(20), legendfont=font(20));        
        if full_dyn == false
            savefig(p1, "figures/IRF_plots/Lehman_omg_stn.pdf");
        else
            savefig(p1, "figures/IRF_plots/Lehman_omg_stn_FD.pdf");
        end 


        size1       = (600, 600)
        p1          = plot(Tvec_plot[1:end_t], RF_path[1:end_t], linestyle=:solid, linewidth=3.5, fg_legend = :transparent, legend=:topright, size=size1, xtickfont=font(20), ytickfont=font(20), legendfont=font(20));        
        if full_dyn == false
            savefig(p1, "figures/IRF_plots/Lehman_RF.pdf");
        else
            savefig(p1, "figures/IRF_plots/Lehman_RF_FD.pdf");
        end 


        data_1      = [plot_EMSP[1:end_t]*10_000 data_in[1:end_t, 2]]
        line_type1  = [:solid :dash]
        labels1     = ["Model" "Data"]
        colors1     = [Colors.RGBA(27/255, 129/255, 197/255, 1) :black]
        size1       = (600, 600)
        p1          = plot(Tvec_plot[1:end_t], data_1, linestyle=line_type1, label=labels1, color=colors1, linewidth=3.5, fg_legend = :transparent, legend=:bottomright, size=size1, xtickfont=font(20), ytickfont=font(20), legendfont=font(20));        
        if full_dyn == false
            savefig(p1, "figures/IRF_plots/Lehman_EMSP.pdf");
        else
            savefig(p1, "figures/IRF_plots/Lehman_EMSP_FD.pdf");
        end 


        data_1      = [plot_MV_md_stn[1:end_t] plot_MV_dt_stn[1:end_t]]
        line_type1  = [:solid :dash]
        labels1     = ["Model" "Data"]
        colors1     = [Colors.RGBA(27/255, 129/255, 197/255, 1) :black]
        size1       = (600, 600)
        p1          = plot(Tvec_plot[1:end_t], data_1, linestyle=line_type1, label=labels1, color=colors1, linewidth=3.5, fg_legend = :transparent, legend=:topright, size=size1, xtickfont=font(20), ytickfont=font(20), legendfont=font(20));        
        if full_dyn == false
            savefig(p1, "figures/IRF_plots/Lehman_mktval_stn.pdf");
        else
            savefig(p1, "figures/IRF_plots/Lehman_mktval_stn_FD.pdf");
        end


        data_1      = [plot_Ct_md_stn[1:end_t] plot_Ct_dt_stn[1:end_t]]
        line_type1  = [:solid :dash]
        labels1     = ["Model" "Data"]
        colors1     = [Colors.RGBA(27/255, 129/255, 197/255, 1) :black]
        size1       = (600, 600)
        p1          = plot(Tvec_plot[1:end_t], data_1, linestyle=line_type1, label=labels1, color=colors1, linewidth=3.5, fg_legend = :transparent, legend=:topright, size=size1, xtickfont=font(20), ytickfont=font(20), legendfont=font(20));        
        if full_dyn == false
            savefig(p1, "figures/IRF_plots/Lehman_Ct_stn.pdf");
        else
            savefig(p1, "figures/IRF_plots/Lehman_Ct_stn_FD.pdf");
        end

    end

    return out_mat

end




function compute_IRF_phi(cm::CoreModuleSC, asm::AshocksModuleSC, init_val, omg_path, YEM_path, Λerg, itp_qDFRSK, itp_qIR; secmkt = false)

    # Create interpolation/extrapolation object
    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_def     = interpolate(knots, asm.defmat,     Gridded(Linear()))
    itp_q       = interpolate(knots, asm.q_mat,  Gridded(Linear()))
    itp_bp      = interpolate(knots, asm.bprime_mat, Gridded(Linear()))
    itp_ReDM    = interpolate((cm.ωgrid, cm.aDMgrid), asm.ReDM, Gridded(Linear()))
    itp_Red     = interpolate((cm.ωgrid, cm.aDMgrid), asm.Red, Gridded(Linear()))
    itp_ADMmax  = interpolate((cm.ωgrid,), asm.ADMmax, Gridded(Linear()))

    # Compute difference E[nu'RDM'] - E[nu'Rd]
    diff_rates      = asm.ReDM - asm.Red
    itp_diff_rates  = interpolate((cm.ωgrid, cm.aDMgrid), diff_rates, Gridded(Linear()))
    

    capT  = size(omg_path, 1)
    Nvars = 10

    Btil_path, Bhat_path, YLD_path, EMSP_path, HYSP_path, NW_path, Ct_path = zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT)
    SPIR_path = zeros(capT)

    ADM_path    = zeros(capT+1) # time t
    D_path      = zeros(capT+1) # time t
    ADM_path[1] = init_val[1]
    D_path[1]   = init_val[2]

    Λt   = copy(Λerg)
    Λtm1 = copy(Λt)


    # Bounds for finding ADM
    bounds = [cm.aDMgrid[1]*0.9; asm.ADMmax]


    for t=1:capT

        rf_val  = sc_RF_fun(cm, omg_path[t])
        
        ub_ADM  = fzero(x->itp_diff_rates[omg_path[t],x], cm.aDMgrid[end])
        bounds  = [cm.aDMgrid[1]*0.9; ub_ADM]   # Bounds for finding ADM 
        out_X   = solve_system_calib(cm, asm, [YEM_path[t]; omg_path[t]; ADM_path[t]; D_path[t]; rf_val], Λt, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, bounds, itp_qDFRSK)
        
        aDMpv, Btilde, Bhat, Bt1, NW, ReDM, SPv, yieldv = out_X[1][1], out_X[1][2], out_X[1][3], out_X[1][4], out_X[1][5], out_X[1][6], out_X[1][7], out_X[1][8]
        by_avg, sdby, sdSP, Ct = out_X[1][9], out_X[1][10], out_X[1][11], out_X[1][12]
        D_path[t+1] = out_X[1][14]

        Λt1 = out_X[2]

        qIR_val        = itp_qIR[YEM_path[t], omg_path[t], aDMpv]     # price without default risk
        SPIR_path[t]   = (1.0/qIR_val+cm.ξ-1.0) - rf_val              # spread without default risk

        YLD_path[t]  = yieldv
        NW_path[t]   = NW
        Btil_path[t] = Btilde
        Bhat_path[t] = Bhat
        EMSP_path[t] = SPv
        Ct_path[t]   = Ct

        ADM_path[t+1] = aDMpv

        Λtm1 = copy(Λt)
        Λt   = copy(Λt1)

    end

    MV_path = simulate_mktval(cm, asm, omg_path, ADM_path, NW_path)

    ΔMV  = log(MV_path[2])-log(MV_path[1])
    ΔSP  = EMSP_path[2] - EMSP_path[1]
    ΔNW  = log(NW_path[2]) - log(NW_path[1])
    ΔYLD = YLD_path[2] - YLD_path[1]

    ΔSPIR = SPIR_path[2] - SPIR_path[1]

    beta = ΔSP / ΔMV

    if secmkt==false
        out = [ΔNW; ΔMV; ΔSP; ΔYLD; beta; ΔSPIR]
    else
        out = ([ADM_path[2]; ADM_path[end]; Btil_path[end]; D_path[end-1]], Λtm1)
    end

    return out

end



function compute_fall_omega(cm::CoreModuleSC, asm::AshocksModuleSC, init_val, omg_path, YEM_path, Λerg, itp_qDFRSK)

    # Create interpolation/extrapolation object
    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_def     = interpolate(knots, asm.defmat,     Gridded(Linear()))
    itp_q       = interpolate(knots, asm.q_mat,  Gridded(Linear()))
    itp_bp      = interpolate(knots, asm.bprime_mat, Gridded(Linear()))
    itp_ReDM    = interpolate((cm.ωgrid, cm.aDMgrid), asm.ReDM, Gridded(Linear()))
    itp_Red     = interpolate((cm.ωgrid, cm.aDMgrid), asm.Red, Gridded(Linear()))


    # Compute difference E[nu'RDM'] - E[nu'Rd]
    diff_rates      = asm.ReDM - asm.Red
    itp_diff_rates  = interpolate((cm.ωgrid, cm.aDMgrid), diff_rates, Gridded(Linear()))
    

    capT  = size(omg_path, 1)
    NS    = size(cm.snodes, 1)
    

    ### Simulation in absence of shocks
    Btil_path_NS, Bhat_path_NS, yield_path_NS, EMSP_path_NS, NW_path_NS, Ct_path_NS = zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT)
    TB_path_NS, C_Y_path_NS, SPDF_path_NS, defmeas_path_NS = zeros(capT), zeros(capT), zeros(capT), zeros(capT)
    Bt_path_NS = zeros(capT+1)
    rf_path_NS = zeros(capT)

    ADM_path_NS    = zeros(capT+1) # time t
    D_path_NS      = zeros(capT+1) # time t
    ADM_path_NS[1] = init_val[1]
    D_path_NS[1]   = init_val[2]

    Λerg_mkt = Λerg[cm.num_nodes+1:end]
    sum_Λ   = sum(Λerg_mkt)

    Bt = 0.0
    for i_s=1:cm.num_nodes
        Bt    += Λerg_mkt[i_s] * cm.snodes[i_s, 1]
    end

    Bt_path_NS[1] = Bt * cm.μEM / sum_Λ

    YEM_path_NS = zeros(YEM_path)
    omg_path_NS = ones(capT)

    Λt = copy(Λerg)


    # Bounds for finding ADM
    bounds = [cm.aDMgrid[1]*0.9; asm.ADMmax]

    for t=1:capT

        rf_val  = sc_RF_fun(cm, omg_path_NS[t])
        ub_ADM  = fzero(x->itp_diff_rates[omg_path_NS[t],x], cm.aDMgrid[end])
        bounds  = [cm.aDMgrid[1]*0.9; ub_ADM]   # Bounds for finding ADM

        in_state_t  = [YEM_path_NS[t]; omg_path_NS[t]; ADM_path_NS[t]; D_path_NS[t]; rf_val]
        out_X   = solve_system_calib(cm, asm, in_state_t, Λt, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, bounds, itp_qDFRSK) # (out_obj, Λt1)
        # out_obj = [aDMp_v; Btilv; Bhatv; Dt1; NW; ReDM; SPv; yield_v; by_avg; sdby; sdSP; Ct; SPDFRK; Dpv; aDMpv2[1]; TB; C_Y]

        aDMpv, Btilde, Bhat, Bt1, NW, ReDM, SPv, yieldv = out_X[1][1], out_X[1][2], out_X[1][3], out_X[1][4], out_X[1][5], out_X[1][6], out_X[1][7], out_X[1][8]
        by_avg, sdby, sdSP, Ct, TB, C_Y = out_X[1][9], out_X[1][10], out_X[1][11], out_X[1][12], out_X[1][16], out_X[1][17]
        D_path_NS[t+1] = out_X[1][14]
        
        Λt1 = out_X[2]

        rf_path_NS[t]    = rf_val
        yield_path_NS[t] = yieldv
        NW_path_NS[t]    = NW
        Btil_path_NS[t]  = Btilde
        Bhat_path_NS[t]  = Bhat
        EMSP_path_NS[t]  = SPv
        Ct_path_NS[t]    = Ct
        TB_path_NS[t]    = TB*100
        C_Y_path_NS[t]   = C_Y*100
        Bt_path_NS[t+1]  = Bt1
        SPDF_path_NS[t]  = out_X[1][13]

        ADM_path_NS[t+1] = aDMpv

        Λt1_mkt = Λt1[cm.num_nodes+1:end]
        defmeas_path_NS[t] = 1.0 - sum(Λt1_mkt)

        Λt = copy(Λt1)

    end


    MV_path_NS = simulate_mktval(cm, asm, omg_path_NS, ADM_path_NS, NW_path_NS)


    ### Simulation given shocks
    Btil_path, Bhat_path, yield_path, EMSP_path, NW_path, Ct_path = zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT), zeros(capT)
    TB_path, C_Y_path, SPDF_path, defmeas_path = zeros(capT), zeros(capT), zeros(capT), zeros(capT)
    Bt_path = zeros(capT+1)
    rf_path = zeros(capT)

    ADM_path    = zeros(capT+1) # time t
    D_path      = zeros(capT+1) # time t
    ADM_path[1] = init_val[1]
    D_path[1]   = init_val[2]

    Bt_path[1]  = Bt_path_NS[1]

    Λt = copy(Λerg)

    for t=1:capT
        rf_val  = sc_RF_fun(cm, omg_path[t])
        ub_ADM  = fzero(x->itp_diff_rates[omg_path[t],x], cm.aDMgrid[end])
        bounds  = [cm.aDMgrid[1]*0.9; ub_ADM]   # Bounds for finding ADM

        in_state_t = [YEM_path[t]; omg_path[t]; ADM_path[t]; D_path[t]; rf_val]

        out_X   = solve_system_calib(cm, asm, in_state_t, Λt, itp_def, itp_q, itp_bp, itp_ReDM, itp_Red, bounds, itp_qDFRSK) # (out_obj, Λt1)


        aDMpv, Btilde, Bhat, Bt1, NW, ReDM, SPv, yieldv = out_X[1][1], out_X[1][2], out_X[1][3], out_X[1][4], out_X[1][5], out_X[1][6], out_X[1][7], out_X[1][8]
        by_avg, sdby, sdSP, Ct, TB, C_Y = out_X[1][9], out_X[1][10], out_X[1][11], out_X[1][12], out_X[1][16], out_X[1][17]
        SPDF_path[t] = out_X[1][13]
        D_path[t+1]  = out_X[1][14]

        Λt1 = out_X[2]

        rf_path[t]    = rf_val
        yield_path[t] = yieldv
        NW_path[t]    = NW
        Btil_path[t]  = Btilde
        Bhat_path[t]  = Bhat
        EMSP_path[t]  = SPv
        Ct_path[t]    = Ct
        TB_path[t]    = TB*100
        C_Y_path[t]   = C_Y*100
        Bt_path[t+1]  = Bt1

        ADM_path[t+1] = aDMpv

        Λt1_mkt = Λt1[cm.num_nodes+1:end]
        defmeas_path[t] = 1.0 - sum(Λt1_mkt)

        Λt = copy(Λt1)

    end

    MV_path = simulate_mktval(cm, asm, omg_path, ADM_path, NW_path)


    plot_mv      = log.(MV_path)-log.(MV_path_NS)+log.(MV_path_NS[end])
    plot_Ct      = log.(Ct_path)-log.(Ct_path_NS)+log.(Ct_path_NS[end])
    plot_EMSP    = EMSP_path-EMSP_path_NS+EMSP_path_NS[end]
    plot_ADM     = ADM_path-ADM_path_NS+ADM_path_NS[end]
    plot_D       = D_path-D_path_NS+D_path_NS[end]
    plot_SPDF    = SPDF_path-SPDF_path_NS+SPDF_path_NS[end]

    plot_Bhat    = Bhat_path-Bhat_path_NS+Bhat_path_NS[end]
    plot_Btil    = Btil_path-Btil_path_NS+Btil_path_NS[end]
    plot_Bt      = Bt_path-Bt_path_NS+Bt_path_NS[end]

    plot_TB      = TB_path-TB_path_NS+TB_path_NS[end]
    plot_C_Y     = C_Y_path-C_Y_path_NS+C_Y_path_NS[end]

    plot_yield   = yield_path-yield_path_NS+yield_path_NS[end]
    plot_defmeas = defmeas_path-defmeas_path_NS+defmeas_path_NS[end]


    Nvars = 12
    out_mat = zeros(capT+1, Nvars)

    out_mat[1:capT, 1:5]  = [plot_Btil plot_Bhat plot_yield plot_EMSP plot_mv]
    out_mat[1:capT+1, 6]  = plot_ADM
    out_mat[1:capT, 7:12] = [plot_Ct plot_TB plot_Bt[1:end-1] plot_SPDF plot_defmeas rf_path]


    return out_mat

end



