
function ashocksmodule( cm::CoreModuleSC;
                        nYEM      = 5,
                        n_et      = 5)

    # Note: CoreModule is needed to have initial guess for value functions and prices

    ρYEM     = cm.ρz
    σYEM     = cm.σz

    YEMgrid, ΠYEM   = tauchen_fun(nYEM, ρYEM, σYEM)


    vf          = reshape(kron(ones(nYEM*cm.nω*cm.naDM), vec(cm.vf[:,:,1])), cm.nb, cm.nz, nYEM, cm.nω, cm.naDM)
    vc          = reshape(kron(ones(nYEM*cm.nω*cm.naDM), vec(cm.vc[:,:,1])), cm.nb, cm.nz, nYEM, cm.nω, cm.naDM)
    q_mat       = reshape(repmat(vec(cm.q_mat[:,:,1]), nYEM*cm.nω*cm.naDM),  cm.nb, cm.nz, nYEM, cm.nω, cm.naDM)
    vd          = reshape(kron(ones(nYEM*cm.nω*cm.naDM), vec(cm.vd[:,1])),   cm.nz, nYEM, cm.nω, cm.naDM)
    defmat      = zeros(vf)
    bprime_mat  = zeros(vf)


    # Create Quadratures
    Σ_ωYEM = [cm.σYEM^2.0 cm.ρEMDM*cm.σYEM*cm.σω ; cm.ρEMDM*cm.σYEM*cm.σω cm.σω^2.0]
    R        = chol(Σ_ωYEM)
    quad_x, quad_w = gausshermite(n_et)
    
    yt_vec = exp.(kron(ones(nYEM),cm.zgrid) + kron(YEMgrid,ones(cm.nz)))

    vs     = zeros(cm.nω, cm.naDM)
    ReDM   = zeros(cm.nω, cm.naDM)
    Red    = zeros(cm.nω, cm.naDM)


    diff_rates = zeros(cm.nω, cm.naDM)
    for (iaDMp, aDMpv) in enumerate(cm.aDMgrid) # policy today, a_{DM,t+1}
        for (iω, ωv) in enumerate(cm.ωgrid)

            Rd_v   = (1.0+sc_RF_fun(cm, ωv)) # time-t interest rate on deposits (promised for next period!)
            ERDM_v = 0.0

            for (iωp, ωpv) in enumerate(cm.ωgrid)
                prob   = cm.Πω[iω, iωp]
                RDMpv  = fun_RDM(cm.α, cm.δ, aDMpv, ωpv)
                ERDM_v += RDMpv * prob
            end
            diff_rates[iω, iaDMp] = ERDM_v - Rd_v
        end
    end


    itp_diff_rates  = interpolate((cm.ωgrid, cm.aDMgrid), diff_rates, Gridded(Linear()))


    ADMmax = ones(cm.nω)*cm.aDMgrid[end] # this will be updated once computed vs
    for (iω, ωv) in enumerate(cm.ωgrid)
        ADMmax[iω] = fzero(x->itp_diff_rates[ωv,x], cm.aDMgrid[3])
    end

    return AshocksModuleSC( nYEM, ρYEM, σYEM, n_et, ADMmax, YEMgrid, yt_vec, vf, vd, vc, bprime_mat, q_mat, defmat, vs, ReDM, Red, ΠYEM, R, quad_x, quad_w)

end


### ------------------------ MODULE 1: LENDER BLOCK ------------------------ ###


function projection_ADM(cm::CoreModuleSC, Xt, ADMmax)
    # Xt contains [aDM_t ω_t]
    # Xmat is [c, aDM_t ω_t]
    val1 = exp( dot( cm.coeffaDM, [1.0; log(Xt[1]); log(Xt[2])] )[1] )
    val  = min( max(val1, cm.aDMgrid[1]*0.7), ADMmax )    

end

#=
Note: if ADMp>=ADMmax then the iteration over vs will make the correct adjustments by choosing
the correct specification for vs_t. When this happens, the value of RDM' does not affect vs so
it doesn't matter that ADMp>ADMmax. This way, we make sure that the value of vs is the correct one.
=#

function iterate_once_vs!(cm::CoreModuleSC, asm::AshocksModuleSC)

    for (iaDM, aDMv) in enumerate(cm.aDMgrid) # policy today, a_{DM,t+1}
        for (iω, ωv) in enumerate(cm.ωgrid)

            beta_v  = 1.0/(1.0+sc_RF_fun(cm, ωv))

            # Note: asm.Red = E[nu(s')]*Rd(s)

            if (asm.ReDM[iω, iaDM]-asm.Red[iω, iaDM]) > 0
                val_iss = (0.25/cm.phi) * ( beta_v * asm.ReDM[iω, iaDM] - 1.0 )^2
                val_inv = beta_v * ( asm.ReDM[iω, iaDM]*(1+cm.κ) - cm.κ*asm.Red[iω, iaDM] )
                asm.vs[iω, iaDM] = (1.0-cm.σn) + cm.σn*( val_iss + val_inv )
            else
                val_1 = (1-asm.Red[iω, iaDM]*beta_v) * (1-0.5*(1-asm.Red[iω, iaDM]*beta_v))
                val_2 = asm.Red[iω, iaDM]*beta_v * (1-asm.Red[iω, iaDM]*beta_v-2*cm.phi)
                asm.vs[iω, iaDM] = (1.0-cm.σn) + cm.σn/(2*cm.phi)*( val_1 - val_2 )
            end
        end
    end
end


function compute_ERHS!(cm::CoreModuleSC, asm::AshocksModuleSC)

    Enu         = zeros(cm.nω, cm.naDM)
    EnuRDM      = zeros(cm.nω, cm.naDM)
    itp_vs      = interpolate((cm.ωgrid, cm.aDMgrid), asm.vs, Gridded(Linear()))

    max_vs = maximum(asm.vs)
    min_vs = minimum(asm.vs)


    for (iω, ωv) in enumerate(cm.ωgrid)

        Rd_v = (1.0+sc_RF_fun(cm, ωv)) # time-t interest rate on deposits (promised for next period!)

        for (iaDMp, aDMpv) in enumerate(cm.aDMgrid) # policy chosen today, a_{DM,t+1}

            Enu_v, EnuRDM_v = 0.0, 0.0

            for (iωp, ωpv) in enumerate(cm.ωgrid)
                aDMppv    = projection_ADM(cm, [aDMpv; ωpv], asm.ADMmax[iωp])
                prob      = cm.Πω[iω, iωp]
                RDMpv     = fun_RDM(cm.α, cm.δ, aDMpv, ωpv) # not limited by ADMmax because this is instead adjusted when computing vs above

                vs_v      = min(max(itp_vs[ωpv, aDMppv],min_vs),max_vs)  # note: this depends on a_{DM,t+2}. Limited by ADMmax.
                
                Enu_v     += vs_v * prob
                EnuRDM_v  += vs_v * RDMpv * prob
            end

            Enu[iω, iaDMp]      = Enu_v * Rd_v
            EnuRDM[iω, iaDMp]   = EnuRDM_v

        end
    end

    copy!(asm.ReDM, EnuRDM)
    copy!(asm.Red,  Enu)

end



function compute_vs!(cm::CoreModuleSC, asm::AshocksModuleSC)

    dist = 10.0
    iter = 0

    while iter < 500 && dist > cm.vftol
        iter += 1
        vsold = copy(asm.vs)
        compute_ERHS!(cm, asm)
        iterate_once_vs!(cm, asm)
        dist = maximum(abs, vsold - asm.vs)
    end
    
    diff_rates      = asm.ReDM - asm.Red
    itp_diff_rates  = interpolate((cm.ωgrid, cm.aDMgrid), diff_rates, Gridded(Linear()))

    for (iω, ωv) in enumerate(cm.ωgrid)
        asm.ADMmax[iω] = fzero(x->itp_diff_rates[ωv,x], cm.aDMgrid[end])
    end

end





### ------------------------ MODULE 2: BORROWER BLOCK ------------------------ ###

function fun_bp(cm::CoreModuleSC, asm::AshocksModuleSC, bp_v, b_v, y_v, q_v, Expec)  # Expec is the continuation value
    c_v = max(y_v - b_v + q_v*(bp_v - cm.ξ*b_v), 1e-14)
    V_v = -ufun(cm, c_v) - cm.β*Expec
    return V_v
end


function compute_def_prob(cm::CoreModuleSC, asm::AshocksModuleSC, z_v, b_v, YEM_v, ω_v, aDMp_v, itp_vc, itp_vd)

    diff_val = itp_vd[z_v, YEM_v, ω_v, aDMp_v] - itp_vc[b_v, z_v, YEM_v, ω_v, aDMp_v]
    prob_def = cdf( Normal(0.0, cm.σM), diff_val)

    return prob_def
end


function one_step_update!(cm::CoreModuleSC, asm::AshocksModuleSC, EV, EVd, Udef)

    bgrid  = cm.bgrid
    β      = cm.β
    yt_vec = asm.yt_vec
    knots  = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)

    itp_EV = interpolate(knots, EV,    Gridded(Linear()))
    itp_q  = interpolate(knots, asm.q_mat, Gridded(Linear()))

    # Allocate memory to objects
    vc_new      = zeros(asm.vc)    
    vf_new      = zeros(asm.vf)
    bp_new      = zeros(asm.bprime_mat)
    vd_new      = zeros(asm.vd)

    for (i_aDMp, aDMp_v) in enumerate(cm.aDMgrid)
        for (i_ω, ω_v) in enumerate(cm.ωgrid)

            aDMp_v = min(asm.ADMmax[i_ω], aDMp_v)

            for (iYEM, YEMv) in enumerate(asm.YEMgrid)
                for (iz, zv) in enumerate(cm.zgrid)

                    iy = (iYEM-1)*cm.nz + iz
                    yt = yt_vec[iy]

                    # Value of being in default with income y
                    defval = Udef[iy] + cm.β*(cm.θ*EV[cm.zero_ind, iz, iYEM, i_ω, i_aDMp] + (1-cm.θ)*EVd[iz, iYEM, i_ω, i_aDMp])
                    vd_new[iz, iYEM, i_ω, i_aDMp] = defval

                    for (ib, bt) in enumerate(cm.bgrid)                    

                        rep_v(bp_v) = fun_bp(cm, asm, bp_v, bt, yt, itp_q[bp_v, zv, YEMv, ω_v, aDMp_v], itp_EV[bp_v, zv, YEMv, ω_v, aDMp_v])
                        res         = Optim.optimize(rep_v, cm.bgrid[1], cm.bgrid[end], GoldenSection())#, iterations=50)
                        pol_val     = Optim.minimizer(res)
                        current_max = -Optim.minimum(res)

                        # Update value and policy functions
                        vc_new[ib, iz, iYEM, i_ω, i_aDMp]  = current_max
                        bp_new[ib, iz, iYEM, i_ω, i_aDMp]  = pol_val
                        vf_new[ib, iz, iYEM, i_ω, i_aDMp]  = defval > current_max ? defval: current_max
                    end
                end
            end
        end
    end

    asm.vc = copy(vc_new); asm.vf = copy(vf_new); asm.vd = copy(vd_new); asm.bprime_mat = copy(bp_new)

    Void
end


function compute_def_states!(cm::CoreModuleSC, asm::AshocksModuleSC)

    knots  = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    knotsd = (cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_vc = interpolate(knots,  asm.vc, Gridded(Linear()))
    itp_vd = interpolate(knotsd, asm.vd, Gridded(Linear()))

    vf_new      = zeros(asm.vf)
    defmat_new  = zeros(asm.defmat)

    for (i_aDMp, aDMp_v) in enumerate(cm.aDMgrid)
        for (i_ω, ω_v) in enumerate(cm.ωgrid)

            aDMp_v = min(asm.ADMmax[i_ω], aDMp_v)

            for (iYEM, YEMv) in enumerate(asm.YEMgrid)
                for (iz, zv) in enumerate(cm.zgrid)
                    for (ib, bt) in enumerate(cm.bgrid)
                        probdf = compute_def_prob(cm, asm, zv, bt, YEMv, ω_v, aDMp_v, itp_vc, itp_vd)
                        defmat_new[ib, iz, iYEM, i_ω, i_aDMp] = probdf
                        vf_new[ib, iz, iYEM, i_ω, i_aDMp]     = probdf * asm.vd[iz, iYEM, i_ω, i_aDMp] +  (1-probdf) * asm.vc[ib, iz, iYEM, i_ω, i_aDMp]
                    end
                end
            end
        end
    end

    asm.vf = copy(vf_new); asm.defmat = copy(defmat_new)

end




function compute_prices(cm::CoreModuleSC, asm::AshocksModuleSC)

    knots  = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_vc = interpolate(knots, asm.vc, Gridded(Linear()))
    itp_q  = interpolate(knots, asm.q_mat, Gridded(Linear()))
    itp_bp = interpolate(knots, asm.bprime_mat, Gridded(Linear()))
    itp_df = interpolate(knots, asm.defmat, Gridded(Linear()))
    itp_vd = interpolate((cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid), asm.vd, Gridded(Linear()))

    itp_vs = interpolate((cm.ωgrid, cm.aDMgrid), asm.vs, Gridded(Linear()))

    itp_ADMmax = interpolate((cm.ωgrid,), asm.ADMmax, Gridded(Linear()))

    q_out  = Array{Float64}(cm.nb, cm.nz, asm.nYEM, cm.nω, cm.naDM)

    for (i_aDMp, aDMp_v) in enumerate(cm.aDMgrid)
        for (i_ω, ω_v) in enumerate(cm.ωgrid)

            aDMp_v = min(asm.ADMmax[i_ω], aDMp_v)

            for (i_YEM, YEM_v) in enumerate(asm.YEMgrid)
                for (i_z, z_v) in enumerate(cm.zgrid)
                    for (i_bp, bp_v) in enumerate(cm.bgrid)

                        val_price = 0.0
                        ERDM_v    = 0.0

                        for i_eω=1:asm.n_et
                            for i_eYEM=1:asm.n_et
                                YEMp_v  = cm.ρYEM*YEM_v + asm.R[1,1] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,1] * sqrt(2) * asm.quad_x[i_eω]
                                ωp_v  = exp(cm.μω + asm.R[1,2] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,2] * sqrt(2) * asm.quad_x[i_eω])
                                probZY  = (1/pi) * asm.quad_w[i_eYEM] * asm.quad_w[i_eω]

                                ADMmax   = itp_ADMmax[ωp_v]
                                aDMpp_v  = projection_ADM(cm, [aDMp_v; ωp_v], ADMmax)
                                nup_v    = itp_vs[ωp_v, aDMpp_v]
                                
                                ERDM_v  += ωp_v * nup_v * ( cm.α * aDMp_v ^(cm.α-1.0) + (1.0-cm.δ) ) * probZY

                                for i_e=1:cm.n_ut
                                    zp_v   = sqrt(2)*cm.σz*cm.quad_x[i_e]+ cm.ρz*z_v
                                    prob_z = (1/sqrt(pi)) * cm.quad_w[i_e]

                                    probdf = min( max(itp_df[bp_v, zp_v, YEMp_v, ωp_v, aDMpp_v], 0.0), 1.0 )

                                    # Future bond policies (b'')
                                    bpp_v = min( max(cm.bgrid[1], itp_bp[bp_v, zp_v, YEMp_v, ωp_v, aDMpp_v]), cm.bgrid[end] )
                                    qp_v  = max(0.01, itp_q[bpp_v, zp_v, YEMp_v, ωp_v, aDMpp_v])
                                    val_price += (1.0-probdf) * nup_v * ( 1.0 + cm.ξ*qp_v ) * (prob_z*probZY)
                                end
                            end
                        end

                        q_out[i_bp, i_z, i_YEM, i_ω, i_aDMp] =  val_price / ERDM_v
                    end
                end
            end
        end
    end

    return q_out
end


function compute_EV_asm(cm::CoreModuleSC, asm::AshocksModuleSC, Vmat; full_dim::Bool = true)

    EVmat = Array{Float64}(size(Vmat))
    itp_ADMmax = interpolate((cm.ωgrid,), asm.ADMmax, Gridded(Linear()))

    if full_dim
        knots = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
        itp_v = interpolate(knots, Vmat, Gridded(Linear()))

        for (i_aDMp, aDMp_v) in enumerate(cm.aDMgrid)
            for (i_ω, ω_v) in enumerate(cm.ωgrid)

                aDMp_v = min(asm.ADMmax[i_ω], aDMp_v)

                for (i_YEM, YEM_v) in enumerate(asm.YEMgrid)
                    for (i_bp, bp_v) in enumerate(cm.bgrid)
                        for (i_z, z_v) in enumerate(cm.zgrid)

                            exp_ADMYEMz = 0.0

                            for i_eω=1:asm.n_et
                                for i_eYEM=1:asm.n_et
                                    YEM_next   = cm.ρYEM*YEM_v + asm.R[1,1] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,1] * sqrt(2) * asm.quad_x[i_eω]
                                    ωp_v     = exp(cm.μω + asm.R[1,2] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,2] * sqrt(2) * asm.quad_x[i_eω])
                                    probZY     = (1/pi) * asm.quad_w[i_eYEM] * asm.quad_w[i_eω]

                                    ADMmax     = itp_ADMmax[ωp_v]
                                    aDMpp_v    = projection_ADM(cm, [aDMp_v; ωp_v], ADMmax)
                                    
                                    for i_e=1:cm.n_ut
                                        z_next   = sqrt(2)*cm.σz*cm.quad_x[i_e]+ cm.ρz*z_v
                                        val_next = itp_v[bp_v, z_next, YEM_next, ωp_v, aDMpp_v]
                                        exp_ADMYEMz   += (1/sqrt(pi)) * cm.quad_w[i_e] * val_next * probZY
                                    end
                                end
                            end

                            EVmat[i_bp, i_z, i_YEM, i_ω, i_aDMp] = exp_ADMYEMz
                        end
                    end
                end
            end
        end
    else

        knots = (cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
        itp_v = interpolate(knots, Vmat, Gridded(Linear()))

        for (i_aDMp, aDMp_v) in enumerate(cm.aDMgrid)
            for (i_ω, ω_v) in enumerate(cm.ωgrid)  

                aDMp_v = min(asm.ADMmax[i_ω], aDMp_v)

                for (i_YEM, YEM_v) in enumerate(asm.YEMgrid)
                    for (i_z, z_v) in enumerate(cm.zgrid)

                        exp_ADMYEMz = 0.0

                        for i_eω=1:asm.n_et
                            for i_eYEM=1:asm.n_et
                                YEM_next   = cm.ρYEM*YEM_v + asm.R[1,1] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,1] * sqrt(2) * asm.quad_x[i_eω]
                                ωp_v     = exp(cm.μω + asm.R[1,2] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,2] * sqrt(2) * asm.quad_x[i_eω])
                                probZY     = (1/pi) * asm.quad_w[i_eYEM] * asm.quad_w[i_eω]

                                ADMmax     = itp_ADMmax[ωp_v]
                                aDMpp_v    = projection_ADM(cm, [aDMp_v; ωp_v], ADMmax)
                                    
                                for i_e=1:cm.n_ut
                                    z_next   = sqrt(2)*cm.σz*cm.quad_x[i_e]+ cm.ρz*z_v
                                    val_next = itp_v[z_next, YEM_next, ωp_v, aDMpp_v]
                                    exp_ADMYEMz   += (1/sqrt(pi)) * cm.quad_w[i_e] * val_next * probZY
                                end
                            end
                        end

                        EVmat[i_z, i_YEM, i_ω, i_aDMp] = exp_ADMYEMz
                    end
                end
            end
        end
    end

    return EVmat

end


function vfi!(cm::CoreModuleSC, asm::AshocksModuleSC)

    damp_price = cm.damp

    @printf("Starting ASM VF\n")

    # get_transition_mat_asm!(cm, asm)

    # Iteration stuff
    it   = 0
    dist = 10.0

    # Allocate memory for update
    V_upd  = zeros(asm.vf)

    Udef   = copy(asm.yt_vec)
    for ii=1:length(asm.yt_vec)
        ydef     = max( asm.yt_vec[ii]*(1.0 - cm.d0*asm.yt_vec[ii]^cm.d1), 1e-14 )
        Udef[ii] = ufun(cm, ydef)
    end


    while dist > cm.vftol && it < 100 #cm.maxit

        it += 1

        # Compute expectations for this iterations
        # (We need Π' because of order value function dimensions)
        copy!(V_upd, asm.vf)

        EV  = compute_EV_asm(cm, asm, asm.vf)
        EVd = compute_EV_asm(cm, asm, asm.vd; full_dim=false)

        # Update Value Function
        one_step_update!(cm, asm, EV, EVd, Udef)
        compute_def_states!(cm, asm);

        # Update prices
        q_old     = copy(asm.q_mat)
        q_out     = compute_prices(cm, asm)
        distq     = maximum(abs, q_old - q_out)
        asm.q_mat = damp_price*q_old + (1.0-damp_price)*q_out

        distV     = maximum(abs, V_upd - asm.vf)
        # dist      = max(distV, distq)
        dist      = distV

        if it%10==0
            @printf("it # %d with distV = %.3g and distq = %.3g\n", it, distV, distq)
        end

    end

    # @printf("Finished VFI AG with distance %.3g\n", dist)

    Void
end



### ------------------------ MODULE 3: POST SOLUTION OBJECTS ------------------------ ###


### Bond price without intermediation component

function compute_once_DFRSK(cm::CoreModuleSC, asm::AshocksModuleSC, q_in, itp_vc, itp_bp, itp_df, itp_vd)

    knots  = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_q  = interpolate(knots, q_in, Gridded(Linear()))

    itp_ADMmax = interpolate((cm.ωgrid,), asm.ADMmax, Gridded(Linear()))

    q_out  = Array{Float64}(cm.nb, cm.nz, asm.nYEM, cm.nω, cm.naDM)

    for (i_aDMp, aDMp_v) in enumerate(cm.aDMgrid)
        for (i_ω, ω_v) in enumerate(cm.ωgrid)
            
            aDMp_v = min(asm.ADMmax[i_ω], aDMp_v)

            for (i_YEM, YEM_v) in enumerate(asm.YEMgrid)
                for (i_z, z_v) in enumerate(cm.zgrid)
                    for (i_bp, bp_v) in enumerate(cm.bgrid)

                        val_price = 0.0
                        ERDM_v    = 0.0

                        for i_eω=1:asm.n_et
                            for i_eYEM=1:asm.n_et
                                YEMp_v   = cm.ρYEM*YEM_v + asm.R[1,1] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,1] * sqrt(2) * asm.quad_x[i_eω]
                                ωp_v   = exp(cm.μω + asm.R[1,2] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,2] * sqrt(2) * asm.quad_x[i_eω])
                                probZY   = (1/pi) * asm.quad_w[i_eYEM] * asm.quad_w[i_eω]

                                ADMmax   = itp_ADMmax[ωp_v]
                                aDMpp_v  = projection_ADM(cm, [aDMp_v; ωp_v], ADMmax)
                                    
                                for i_e=1:cm.n_ut
                                    zp_v   = sqrt(2)*cm.σz*cm.quad_x[i_e]+ cm.ρz*z_v
                                    prob_z = (1/sqrt(pi)) * cm.quad_w[i_e]

                                    probdf = min( max(itp_df[bp_v, zp_v, YEMp_v, ωp_v, aDMpp_v], 0.0), 1.0 )

                                    # Future bond policies (b'')
                                    bpp_v = min( max(cm.bgrid[1], itp_bp[bp_v, zp_v, YEMp_v, ωp_v, aDMpp_v]), cm.bgrid[end] )
                                    qp_v  = max(0.01, itp_q[bpp_v, zp_v, YEMp_v, ωp_v, aDMpp_v])
                                    val_price += (1.0-probdf)*( cm.ξ*qp_v + 1.0 ) * (prob_z*probZY)
                                end
                            end
                        end

                        q_out[i_bp, i_z, i_YEM, i_ω, i_aDMp] =  val_price / (1.0+sc_RF_fun(cm, ω_v))

                    end
                end
            end
        end
    end

    return q_out
end

function compute_prices_DFRSK(cm::CoreModuleSC, asm::AshocksModuleSC, q_guess)

    q_DFRSK = copy(q_guess)

    iter = 0
    dist = 10.0
    damp = 0.0

    knots  = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_vc = interpolate(knots, asm.vc, Gridded(Linear()))
    itp_bp = interpolate(knots, asm.bprime_mat, Gridded(Linear()))
    itp_df = interpolate(knots, asm.defmat, Gridded(Linear()))
    itp_vd = interpolate((cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid), asm.vd, Gridded(Linear())) 

    while iter<50 && dist>5e-5
        iter += 1

        qout    = compute_once_DFRSK(cm, asm, q_DFRSK, itp_vc, itp_bp, itp_df, itp_vd)
        dist    = maximum(abs, q_DFRSK - qout)
        q_DFRSK = damp*q_DFRSK + (1.0-damp)*qout

        if iter%5==0
            @printf("Iter %d with dist = %.3g\n", iter, dist)
        end
    end

    @printf("Iterations on DFRSK price %d, distance %.3g\n", iter, dist)

    return q_DFRSK

end


### Bond price without default risk

function compute_once_nodefault(cm::CoreModuleSC, asm::AshocksModuleSC, q0_IR)

    knots      = (asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qIR    = interpolate(knots, q0_IR, Gridded(Linear()))
    itp_vs     = interpolate((cm.ωgrid, cm.aDMgrid), asm.vs, Gridded(Linear()))
    itp_ADMmax = interpolate((cm.ωgrid,), asm.ADMmax, Gridded(Linear()))

    diff_rates      = asm.ReDM - asm.Red
    itp_diff_rates  = interpolate((cm.ωgrid, cm.aDMgrid), diff_rates, Gridded(Linear()))

    q_IR  = Array{Float64}(asm.nYEM, cm.nω, cm.naDM)

    for (i_aDMp, aDMp_v) in enumerate(cm.aDMgrid)
        for (i_ω, ω_v) in enumerate(cm.ωgrid)

            aDMp_v = min(asm.ADMmax[i_ω], aDMp_v)

            for (i_YEM, YEM_v) in enumerate(asm.YEMgrid)

                val_nodef = 0.0
                ERDM_v = 0.0

                for i_eω=1:asm.n_et
                    for i_eYEM=1:asm.n_et
                        YEMp_v  = cm.ρYEM*YEM_v + asm.R[1,1] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,1] * sqrt(2) * asm.quad_x[i_eω]
                        ωp_v  = exp(cm.μω + asm.R[1,2] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,2] * sqrt(2) * asm.quad_x[i_eω])
                        probZY  = (1/pi) * asm.quad_w[i_eYEM] * asm.quad_w[i_eω]

                        ADMmax   = fzero(x->itp_diff_rates[ωp_v,x], cm.aDMgrid[end]) #itp_ADMmax[ωp_v]
                        aDMpp_v  = projection_ADM(cm, [aDMp_v; ωp_v], ADMmax)
                        nup_v    = itp_vs[ωp_v, aDMpp_v]
                        
                        ERDM_v   += ωp_v * nup_v * ( cm.α * aDMp_v ^(cm.α-1.0) + (1.0-cm.δ) ) * probZY

                        qp_nodef  = max(0.01, itp_qIR[YEMp_v, ωp_v, aDMpp_v])

                        val_nodef += nup_v * ( 1.0 + cm.ξ*qp_nodef ) * probZY

                    end
                end

                q_IR[i_YEM, i_ω, i_aDMp] =  val_nodef / ERDM_v
            end
        end
    end

    return q_IR
end


function compute_prices_nodefault(cm::CoreModuleSC, asm::AshocksModuleSC, qg_IR)

    q0_IR = copy(qg_IR)

    iter = 0
    dist = 10.0
    damp = 0.8

    while iter<200 && dist>5e-5
        iter += 1

        qout  = compute_once_nodefault(cm, asm, q0_IR)
        dist  = maximum(abs, q0_IR - qout)
        q0_IR = damp*q0_IR + (1.0-damp)*qout
    end

    # @printf("Iterations on qnodef price %d, dist %.3g\n", iter, dist)

    return q0_IR

end



### Compute High Yield Spread

function compute_HYSP(cm::CoreModuleSC, asm::AshocksModuleSC)

    itp_vs = interpolate((cm.ωgrid, cm.aDMgrid), asm.vs, Gridded(Linear()))
    HYSP   = Array{Float64}(cm.nω, cm.naDM)

    itp_ADMmax = interpolate((cm.ωgrid,), asm.ADMmax, Gridded(Linear()))

    for (iω, ωv) in enumerate(cm.ωgrid)
        for (iaDMp, aDMpv) in enumerate(cm.aDMgrid) # policy chosen today, a_{DM,t+1}

            numer, denom = 0.0, 0.0

            for i_eω=1:asm.n_et
                for i_eYEM=1:asm.n_et
                    ωp_v   = exp(cm.μω + asm.R[1,2] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,2] * sqrt(2) * asm.quad_x[i_eω])
                    prob = (1/pi) * asm.quad_w[i_eYEM] * asm.quad_w[i_eω]

                    ADMmax   = itp_ADMmax[ωp_v]
                    aDMpp_v  = projection_ADM(cm, [aDMpv; ωp_v], ADMmax)

                    vsp_v  = itp_vs[ωp_v, aDMpp_v]  # note: this depends on a_{DM,t+2}
                    RDMp_v = fun_RDM(cm.α, cm.δ, aDMpp_v, ωp_v)

                    numer += vsp_v*RDMp_v*prob
                    denom += vsp_v*ωp_v/cm.ωgrid[end]*prob
                end
            end

            HYSP[iω, iaDMp] = numer / denom - (1.0+sc_RF_fun(cm, ωv))
        end
    end

    return HYSP
end


### Compute bond price without nu term [not used, just for understanding]

function compute_price_nonu(cm::CoreModuleSC, asm::AshocksModuleSC, q_in)

    knots  = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_vc = interpolate(knots, asm.vc, Gridded(Linear()))
    itp_q  = interpolate(knots, q_in, Gridded(Linear()))
    itp_bp = interpolate(knots, asm.bprime_mat, Gridded(Linear()))
    itp_df = interpolate(knots, asm.defmat, Gridded(Linear()))
    itp_vd = interpolate((cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid), asm.vd, Gridded(Linear()))

    itp_ADMmax = interpolate((cm.ωgrid,), asm.ADMmax, Gridded(Linear()))

    q_out  = Array{Float64}(cm.nb, cm.nz, asm.nYEM, cm.nω, cm.naDM)

    for (i_aDMp, aDMp_v) in enumerate(cm.aDMgrid)
        for (i_ω, ω_v) in enumerate(cm.ωgrid)

            aDMp_v = min(asm.ADMmax[i_ω], aDMp_v)

            for (i_YEM, YEM_v) in enumerate(asm.YEMgrid)
                for (i_z, z_v) in enumerate(cm.zgrid)
                    for (i_bp, bp_v) in enumerate(cm.bgrid)

                        val_price = 0.0
                        ERDM_v    = 0.0

                        for i_eω=1:asm.n_et
                            for i_eYEM=1:asm.n_et
                                YEMp_v   = cm.ρYEM*YEM_v + asm.R[1,1] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,1] * sqrt(2) * asm.quad_x[i_eω]
                                ωp_v   = exp(cm.μω + asm.R[1,2] * sqrt(2) * asm.quad_x[i_eYEM] + asm.R[2,2] * sqrt(2) * asm.quad_x[i_eω])
                                prob = (1/pi) * asm.quad_w[i_eYEM] * asm.quad_w[i_eω]

                                ADMmax   = itp_ADMmax[ωp_v]
                                aDMpp_v  = projection_ADM(cm, [aDMp_v; ωp_v], ADMmax)

                                ERDM_v  += ωp_v * ( cm.α * aDMp_v ^(cm.α-1.0) + (1.0-cm.δ) ) * prob
                                    
                                for i_e=1:cm.n_ut
                                    zp_v   = sqrt(2)*cm.σz*cm.quad_x[i_e]+ cm.ρz*z_v
                                    prob_z = (1/sqrt(pi)) * cm.quad_w[i_e]

                                    probdf = min( max(itp_df[bp_v, zp_v, YEMp_v, ωp_v, aDMpp_v], 0.0), 1.0 )

                                    # Future bond policies (b'')
                                    bpp_v = min( max(cm.bgrid[1], itp_bp[bp_v, zp_v, YEMp_v, ωp_v, aDMpp_v]), cm.bgrid[end] )
                                    qp_v  = max(0.01, itp_q[bpp_v, zp_v, YEMp_v, ωp_v, aDMpp_v])
                                    val_price += (1.0-probdf) * ( 1.0 + cm.ξ*qp_v ) * (prob_z*prob)
                                end
                            end
                        end

                        q_out[i_bp, i_z, i_YEM, i_ω, i_aDMp] =  val_price / ERDM_v

                    end
                end
            end
        end
    end

    return q_out
end

function iterate_price_nonu(cm::CoreModuleSC, asm::AshocksModuleSC)

    q_nonu = copy(asm.q_mat)

    iter = 0
    dist = 10.0
    damp = 0.8

    while iter<50 && dist>5e-3
        iter += 1

        qout    = compute_price_nonu(cm, asm, q_nonu)
        dist    = maximum(abs, q_nonu - qout)
        q_nonu = damp*q_nonu + (1.0-damp)*qout

        @printf("Iter %d with dist = %.3g\n", iter, dist)
    end

    @printf("Iterations on DFRSK price %d, distance %.3g\n", iter, dist)

    return q_nonu

end

