
################# ----------------------------------------------------------------------------- ##################
################# -------------------------- *** SOLVE MODEL ONCE *** ------------------------- ##################
################# ----------------------------------------------------------------------------- ##################


if solve_bmk

	cm  = coremodule(β = eval_params[1], d0 = eval_params[2], d1 = eval_params[3], σn = eval_params[4], ρω = eval_params[5], ρEMDM = eval_params[6], σω = eval_params[7], δ = eval_params[8], phi=eval_params[9], κ=eval_params[10], ph0=eval_params[11], ph1=eval_params[12])
	
	## Take as guess the uncorrelated shocks
	cmnc = load("../../model_data/NAG_module_structure.jld", "cm");
    cm.vf          = copy(cmnc.vf)
    cm.vd          = copy(cmnc.vd)
    cm.vc          = copy(cmnc.vc)
    cm.bprime_mat  = copy(cmnc.bprime_mat)
    cm.q_mat       = copy(cmnc.q_mat)
    cm.def_states  = copy(cmnc.def_states)

    
    cm.coeffaDM = cmnc.coeffaDM


	### --- Do VFI --- ###
	vfi_nas!(cm)
	compute_ss_root!(cm)

	
	asm = ashocksmodule(cm)
	

	## Take as guess the uncorrelated shocks
	asmnc = load("../../model_data/ASM_module_structure.jld", "asm");
    asm.vf          = copy(asmnc.vf)
    asm.vd          = copy(asmnc.vd)
    asm.vc          = copy(asmnc.vc)
    asm.bprime_mat  = copy(asmnc.bprime_mat)
    asm.q_mat       = copy(asmnc.q_mat)
    asm.defmat      = copy(asmnc.defmat)
   

	compute_vs!(cm, asm)
	@time vfi!(cm, asm)


    q_DFRSK = load("../../model_data/q_DFRSK.jld", "q_DFRSK");
    save("model_data/q_DFRSK.jld", "q_DFRSK", q_DFRSK)

    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))


	### --- Simulate Once and Adjust μEM --- ###

	capT    = 500
	YEM_sim, ω_sim, rf_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)

	# Run short simulation to adjust μEM to get average DM share of 90%
	adjust_muEM!(cm, asm, 100, YEM_sim, ω_sim, rf_sim, itp_qDFRSK)


	### --- Save Model --- ###
	if save_models
		save("model_data/NAG_module_structure.jld", "cm", cm)
		save("model_data/ASM_module_structure.jld", "asm", asm)
	end

end



################# ----------------------------------------------------------------------------- ##################
################# --------------- *** CONDITIONAL AND UNCONDITIONAL MOMENTS *** --------------- ##################
################# ----------------------------------------------------------------------------- ##################

if do_calib_mms

	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  ----------------------------------     BLOCK 1: LOAD MARKET STRUCTURE    ---------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###
	cm  = load("model_data/NAG_module_structure.jld", "cm");
	asm = load("model_data/ASM_module_structure.jld", "asm");

	capT    = 1_000
	dropT 	= 100
	YEM_sim, ω_sim, rf_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)

    q_DFRSK 	= load("model_data/q_DFRSK.jld", "q_DFRSK")
    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))

    # Auxilary function
    corr_fun(x1, x2) = cov(x1, x2) / ( std(x1)*std(x2) )
	

	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  ----------------------------------     BLOCK 2: SIMULATION AND MOMENTS    ---------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###

	out_calib 	= time_iteration_calib(cm, asm, capT, dropT, YEM_sim, ω_sim, rf_sim, itp_qDFRSK)

	## Simulate High Yield using simulation results
	HYSPmat  = compute_HYSP(cm, asm)
	itp_HYSP = interpolate((cm.ωgrid, cm.aDMgrid), HYSPmat, Gridded(Linear()))

	function simul_HYSP_fun(itp_HYSP, ωvals, ADMpvals)
		HYSPsim = zeros(length(ωvals))
		for t=1:length(ωvals)
			HYSPsim[t] = itp_HYSP[ωvals[t], ADMpvals[t]]
		end
		return HYSPsim
	end



	###  EXTRACT ELEMENTS FROM THE SIMULATION
	# The time iteration for calibration returns:
	# 	out_calib =	Bvec, aDMvec, ReEMvec, NWvec, Bhatvec, Btildevec, def_measure, ReDMvec, SPvec, YLDvec, by_vec, sdby_vec, sdSP_vec, Ctvec, DFRSKvec, Dvec, Λ_erg, aDMvec2, RDMvec, TB_vec, C_Y_vec
	# 				 1       2       3       4       5         6          7           8        9      10      11       12       13      14        15      16    17     18       19       20      21	

	## Extract simulations
	YEM_calib 		= YEM_sim[dropT+1:end]
	ω_calib 		= ω_sim[dropT+1:end]
	rf_calib 	    = rf_sim[dropT+1:end]
	ADM_calib 		= out_calib[2] 			# ADM_t  goes until T+1
	ReDM_calib 		= out_calib[3]
	NW_calib 		= out_calib[4] # this is the NW prior to exit, it's not the NW variable of the banking sector
	Bhat_calib 		= out_calib[5]
	Btil_calib 		= out_calib[6]
	SP_calib 		= out_calib[9]
	YLD_calib 		= out_calib[10]
	Ct_calib 		= out_calib[14]
	SPDF_calib 		= out_calib[15]
	Dt_calib 		= out_calib[16]
	RDM_calib 	    = out_calib[19]
	TB_calib 		= out_calib[20]		
	XX_calib 		= Btil_calib*cm.μEM + RDM_calib.*ADM_calib[1:end-1]


	# Compute Asset to Equity Ratio
	Asset_NW 	  = (cm.μEM*Bhat_calib + ADM_calib[2:end])./NW_calib;
	avg_ANW_ratio = mean(Asset_NW)

	

	### ADDITIONAL SIMULATIONS
	## Simulate Market Value
	mktval_calib = simulate_mktval(cm, asm, ω_calib, ADM_calib, NW_calib)

	## Simulate High Yield and Total Assets
	HYSP_calib 	= simul_HYSP_fun(itp_HYSP, ω_calib, ADM_calib[2:end])
	At_calib 	= cm.μEM*Bhat_calib + ADM_calib[2:end]
	


	### COMPUTE MOMENTS FROM HISTOGRAM AND AGGREGATE SIMULATION

	## Aggregate consumption moments
	sd_lnCt     = std(log.(Ct_calib))*100

	## sd and correlation for NW and market value
	sd_NW = std(log.(NW_calib)); sd_mktval = std(log.(mktval_calib));
	corr_NW_mktval = corr_fun(log.(NW_calib), log.(mktval_calib));
	vols_NW_mktval = [sd_NW; sd_mktval; corr_NW_mktval];

	## Compute spread decomposition (default risk vs intermediation premium)
	EMSP_TOT = (mean(SP_calib))*10_000
	EMSP_DRK = (mean(SPDF_calib))*10_000
	EMSP_IMD = (EMSP_TOT - EMSP_DRK)

	# Risk-free priced by intermediary
	q_IR 	= compute_prices_nodefault(cm, asm,  reshape(asm.q_mat[1,end,:,:,:], asm.nYEM, cm.nω, cm.naDM));
	itp_qIR = interpolate((asm.YEMgrid, cm.ωgrid, cm.aDMgrid), q_IR, Gridded(Linear()))

	SP_IR = zeros(capT-dropT)
	min_qIR = minimum(q_IR); max_qIR = maximum(q_IR)
	for t=1:(capT-dropT)
		# qIR_val  = min(max(itp_qIR[YEM_calib[t], ω_calib[t], ADM_calib[t+1]],minimum(q_IR)),maximum(q_IR))
		qIR_val  = min(max(min_qIR, itp_qIR[YEM_calib[t], ω_calib[t], ADM_calib[t+1]]), max_qIR)
		SP_IR[t] = (1.0/qIR_val+cm.ξ-1.0) - rf_calib[t]
	end

	frac_neg_EMSP = sum(SP_calib.<0.0)/length(SP_calib)*100
	frac_neg_SPIR = sum(SP_IR.<0.0)/length(SP_IR)*100


	SP_IR_avg   = mean(SP_IR)*10_000

	# Save spread moments into matrix
	spread_dcp_hist = [EMSP_DRK; EMSP_TOT; EMSP_IMD; SP_IR_avg; 0.0; 0.0]


	# Additional targets
	avg_RF 	   = mean(rf_calib*100)
	std_RF 	   = std(rf_calib*100)
	cov_RF_XLF = cov(rf_calib*100, log.(mktval_calib))
	cor_RF_XLF = corr_fun(rf_calib*100, log.(mktval_calib))


	## Additional moments
	mu_DM = mean(out_calib[2])/(mean(out_calib[2])+mean(out_calib[5])*cm.μEM)
	corr_HYSP_EMSP 	= corr_fun(HYSP_calib, SP_calib)


	## Save moments into matrix
	moments_histogram = [EMSP_TOT; mu_DM; corr_HYSP_EMSP]


	# Auxiliary function to compute further correlations
	function fun_temp_NW(Xin, SPin, HYSPin, YEMin)
		out1 = std(Xin)
		out2 = corr_fun(Xin, SPin)
		out3 = corr_fun(Xin, HYSPin)
		out4 = corr_fun(Xin, YEMin)
		return [out1; out2; out3; out4]
	end

	out_mm_NW 	 	= fun_temp_NW(NW_calib, SP_calib, HYSP_calib, YEM_calib) 			# Net Worth
	out_mm_mktvl 	= fun_temp_NW(mktval_calib, SP_calib, HYSP_calib, YEM_calib) 		# Market Value
	out_mm_lnNW	 	= fun_temp_NW(log.(NW_calib), SP_calib, HYSP_calib, YEM_calib) 		# Net Worth (in logs)
	out_mm_lnmktvl 	= fun_temp_NW(log.(mktval_calib), SP_calib, HYSP_calib, YEM_calib) 	# Market Value (in logs)

	out_mms_NW   = [out_mm_NW out_mm_mktvl out_mm_lnNW out_mm_lnmktvl]
	# it's 3x4, rows being std, corr(X,SP), corr(X,RDM); columns being each variable NW and mktval, in levels and in logs


	### SIMULATE SINGLE COUNTRY
	country_vars = simul_country_fun(cm, asm, [YEM_calib ω_calib ADM_calib[2:end]], 0)

	# Simulate panel
	capN = 1_000
	panel_vars = simul_panel_fun(cm, asm, [YEM_calib ω_calib ADM_calib[2:end]], capN, 0)

	out_country_calib = compute_moments_country_calib(cm, asm, YEM_calib, ω_calib, ADM_calib, SP_calib, country_vars, q_DFRSK)
	# Output is:
	# out_moment = [mean_by; def_freq; mean_sp; std_sp; corr_spY; stdc_stdY; corr_cY; std_tb; corr_tbY; corr_spiSP; std_tb2; corr_tbY2; ln_corr_spY; ln_stdc_stdY; ln_corr_cY; ln_corr_tbY]
	# out_decomp = [mean_SP_DFRSK; mean_SP_TOTAL; mean_SP_IMRSK; std_SP_DFRSK; std_SP_TOTAL; std_SP_IMRSK] 6 moments

	out_panel_calib   = compute_moments_panel_calib(cm, asm, YEM_calib, ω_calib, ADM_calib, panel_vars, q_DFRSK, q_IR)
	# Output is:
	#						 1 			2			3				4			5				6				7				8				9				10					11				12				13		
	# out_cross_section = [cs_sd_by; cs_sd_SP; cs_corr_by_y; cs_corr_SP_y; cs_corr_sdby_y; cs_corr_sdSP_y; cs_corr_by_lny; cs_corr_SP_lny; cs_corr_sdby_lny; cs_corr_sdSP_lny; cs_corr_TB_lny; corr_spi_lnci; corr_spi_TBi]
	# 						1			2		3			4			5			6				7			8			9			10			11				12		13			14			16				16			17				18					19
	# out_time_series   = [mean_DY; DF_mean; mean_spread; std_spread; std_tb; mean(corr_sp_Y)*100; sdci_sdyi; corr_ci_yi; sd_TBi_yi; corr_TBi_yi; corr_spi_SP; std_tb2; sd_TBi_yi2; corr_TBi_yi2; ln_sdci_sdyi; ln_sd_TBi_yi; ln_corr_ci_yi; ln_corr_TBi_yi; ln_corr_SP_yi]
	# out_decomp        = [mean_SP_DFRSK/capN*10_000; mean_SP_TOTAL/capN*10_000; mean_SP_IMRSK/capN*10_000; std_SP_DFRSK/capN*10_000; std_SP_TOTAL/capN*10_000; std_SP_IMRSK/capN*10_000]
	# out_dcmp_interm   = [mean_SP_IR/capN*10_000; mean_SP_PR/capN*10_000; std_SP_IR/capN*10_000; std_SP_PR/capN*10_000; 0.0; 0.0]


	###  WRITE TABLES WITH SIMULATED MOMENTS INTO TEXT FILES

	open("simulated_moments/out_spread_decomp.txt", "w") do io
	    writedlm(io, [out_country_calib[2] out_panel_calib[3] spread_dcp_hist out_panel_calib[4]], ',')
	end;


	### COMPUTE ELASTICITY
    ADMmax_sim = zeros(length(ADM_calib)-1)
    itp_ADMmax = interpolate((cm.ωgrid,), asm.ADMmax, Gridded(Linear()))

    for tt=1:length(ADM_calib)-1
        ADMmax_sim[tt] = itp_ADMmax[ω_calib[tt]]
    end

    ADMpcalib = ADM_calib[2:end]
    mean_ADM  = mean(ADMpcalib[ADMpcalib.<ADMmax_sim])
    mean_Dt   = mean(Dt_calib[1:end-1][ADMpcalib.<ADMmax_sim])

	init_val  = [mean(ADM_calib); mean(Dt_calib)] # ADM and D

	init_ω    = cm.ωgrid[3]*0.98

	function fun_tmp(omg_val)
		omg_path = [init_ω; omg_val]
		out_tmp  = compute_IRF_phi(cm, asm, init_val, omg_path, asm.YEMgrid[3]*ones(2), out_calib[17], itp_qDFRSK, itp_qIR)
		return (out_tmp[2]+0.12)^2.0
	end

	res   = Optim.optimize(fun_tmp, cm.ωgrid[1], cm.ωgrid[end], Brent(), iterations=50)
	ω_out = Optim.minimizer(res)

	omg_path = [init_ω; ω_out]
	out_beta = compute_IRF_phi(cm, asm, init_val, omg_path, asm.YEMgrid[3]*ones(2), out_calib[17], itp_qDFRSK, itp_qIR)
	# [ΔNW; ΔMV; ΔSP; ΔYLD; beta; ΔSPIR]

	beta_NW  = out_beta[3]./out_beta[1]
	beta_IR  = out_beta[6]./out_beta[2] # elasticity with no default risk

	open("simulated_moments/elasticity_moments.txt", "w") do io
	    writedlm(io, [out_beta; beta_IR], ',')
	end;


	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  ---------------------------------------     BLOCK 4: EXPORT MOMENTS    ------------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###

	# β; d0; d1; σn; μEM; σω; κ; nbar; ρEMDM; phi; ph0; ph1
	params_out = [cm.β; cm.d0; cm.d1; cm.σn; cm.μEM; cm.σω; cm.κ; cm.nbar; cm.ρEMDM; cm.phi; cm.ph0; cm.ph1]
	open("model_data/params_out.txt", "w") do io
	    writedlm(io, params_out, ',')
	end;

	# targets_out = [ E[D/Y]; DF; E[SP]; sd(SP); corr(SP_i,Y_i); sd(ln(V(N))); corr(ln(V(N),YEM)); eta_EM; A/NW; E(RF); cov(ln(V(N),RF) ]
	targets_out = [out_panel_calib[2][1]; out_panel_calib[2][2]; EMSP_TOT; out_panel_calib[2][4]; out_panel_calib[2][19]*100; out_mms_NW[1, 4]; out_mms_NW[4, 4]*100; -beta_IR; avg_ANW_ratio; avg_RF; cov_RF_XLF]
	open("simulated_moments/targets_out.txt", "w") do io
	    writedlm(io, targets_out, ',')
	end;


end



if do_fall_omega

	###  -----------------------------------------------------------------------------------------------------------------  ###
	###  ----------------------------------     BLOCK 1: LOAD MARKET STRUCTURE    ---------------------------------------  ###
	###  -----------------------------------------------------------------------------------------------------------------  ###
	cm  = load("model_data/NAG_module_structure.jld", "cm");
	asm = load("model_data/ASM_module_structure.jld", "asm");

	capT    = 1_000
	dropT 	= 100
	YEM_sim, ω_sim, rf_sim = gen_exogen_correl(cm, asm, cm.ρEMDM, cm.σω, capT)

    q_DFRSK 	= load("model_data/q_DFRSK.jld", "q_DFRSK")
    knots       = (cm.bgrid, cm.zgrid, asm.YEMgrid, cm.ωgrid, cm.aDMgrid)
    itp_qDFRSK  = interpolate(knots, q_DFRSK,  Gridded(Linear()))

    # Auxilary function
    corr_fun(x1, x2) = cov(x1, x2) / ( std(x1)*std(x2) )	

	out_calib 	= time_iteration_calib(cm, asm, capT, dropT, YEM_sim, ω_sim, rf_sim, itp_qDFRSK)


	# Upload results from Benchmark model
	OMGfall_bmk = readdlm("../../simulated_moments/OMGfall_out.txt", ',', Float64)
	
	init_state 	= readdlm("../../simulated_moments/OMGfall_init_cond.txt", ',', Float64)
	Λ_init 		= readdlm("../../simulated_moments/OMGfall_init_distrib.txt", ',', Float64)

	capT_ωFall = 5

	omg_path 	= ones(capT_ωFall)
	YEM_path 	= zeros(capT_ωFall)
	omg_path[1] = 1.06#cm.ωgrid[4]
	omg_path[2] = omg_path[1] -2.0*cm.σω	


	# Only ωshock
	OMGfall_out = compute_fall_omega(cm, asm, init_state, omg_path, YEM_path, Λ_init, itp_qDFRSK)




	gr()
	size1       = (600, 600)
	line_type1  = [:solid :dash]
	colors1     = [Colors.RGBA(27/255, 129/255, 197/255, 1) :black]
	labels1     = ["Liquidity policy" "Benchmark"]

	pyplot()
    ## Yields: benchmark vs liquidity provision
    fig, ax = PyPlot.subplots(figsize=(6,6))
    ax[:plot](OMGfall_out[1:end-1,3]-OMGfall_out[1,3], linestyle="-", linewidth=2.5, label="Liquidity policy")
    ax[:plot](OMGfall_bmk[1:end-1,3]-OMGfall_bmk[1,3], color="black", linestyle="--", linewidth=2.5, label="Benchmark")
    ax[:legend](loc=1, frameon=false)
    PyPlot.xticks(collect(0:4))
    PyPlot.grid("on", alpha=0.3)
    PyPlot.tight_layout()
	PyPlot.savefig("figures/fall_omega/fall_omega_YLD.pdf")

	## Consumption: benchmark vs liquidity provision
    fig, ax = PyPlot.subplots(figsize=(6,6))
    ax[:plot](OMGfall_out[1:end-1,7]-OMGfall_out[1,7], linestyle="-", linewidth=2.5, label="Liquidity policy")
    ax[:plot](OMGfall_bmk[1:end-1,7]-OMGfall_bmk[1,7], color="black", linestyle="--", linewidth=2.5, label="Benchmark")
    ax[:legend](loc=4, frameon=false)
    PyPlot.xticks(collect(0:4))
    PyPlot.grid("on", alpha=0.3)
    PyPlot.tight_layout()
	PyPlot.savefig("figures/fall_omega/fall_omega_Ct.pdf")
end


